Ti.include('/globals.js');


//===================================================================
//	android general utilities.
//===================================================================

if ($.Android==null){
	$.Android = {
		WindowTitleBarHeight             : '44dp',
		WindowToolBarHeight              : '44dp',
		IdentityNumber                   : 0,
		TabBarHeight                     : '49dp',
		TabBarIsOnTop                    : false,
		borderR						    :  App.Config.IS_ANDROID? 5*Titanium.Platform.displayCaps.dpi/160 : 5,
		borderW 					    :  App.Config.IS_ANDROID? 1*Titanium.Platform.displayCaps.dpi/160 : 1,
	};
}

function checkIfButtonWasClickedThenDisableForShortPeriod (button,period) {
	if (button.clicked==true) {
		return true; // already clicked. do nothing
	}
	button.clicked = true;
	setTimeout(function(){button.clicked=false;},period);
	return false;
}

function disableButtonUntilEventOccurs (button,event) {
	Ti.App.addEventListener(event,theFunc=function (evt) {
		Ti.App.removeEventListener(event,theFunc);
		button.setEnabled(true);
	});
	button.setEnabled(false);
}

function _dpToPixel(dp) {
    return ( parseInt(dp) * (Titanium.Platform.displayCaps.dpi / 160));
}

function _pixelToDp (pixel) {
	return ((pixel*160)/Titanium.Platform.displayCaps.dpi) + 'dp';
}

//===================================================================
//	android-based generic window which is light-weight. Should 
//	be wrapped around all other windows.
//===================================================================

function __androidGenericWindowCreate (style, param) {

	//------------------------------------------------------------
	//	Open window and assign own state variables.	
	//	Note that the following params, when present,
	//	creates a heavy-weight window.
	//		- fullscreen
	//		- navBarHidden
	//		- modal
	//		- windowSoftInputMode
	//	also, if 'url' param is present, new context is used.
	//------------------------------------------------------------

	Ti.API.debug ('CTKGW');
    for(key in style){
    	Ti.API.debug(' ctkgw:'+key+': '+style[key]);
    }

	var aStyle = App.Helper.clone(style);
	
	if (aStyle.navBarHidden) {
		win.myNavBarHidden = true;
	}
	
	if (   aStyle.fullscreen!=undefined 
		|| aStyle.navBarHidden!=undefined 
		|| aStyle.modal!=undefined 
		|| aStyle.windowSoftInputMode!=undefined
		|| aStyle.url!=undefined
	){
		Ti.API.info ('CTK-WWWWW!!!');
		if (aStyle.fullscreen)          delete aStyle.fullscreen;
		if (aStyle.navBarHidden)        delete aStyle.navBarHidden;
		if (aStyle.modal)               delete aStyle.modal;
		if (aStyle.windowSoftInputMode) delete aStyle.windowSoftInputMode;
		if (aStyle.url)                 delete aStyle.url;
	}

	var win = Ti.UI.createWindow (aStyle);	
	$.Android.IdentityNumber = 1 + $.Android.IdentityNumber;
	win.myId           = $.Android.IdentityNumber;
	win.myRightNav     = null;
	win.myLeftNav      = null;
	win.myTitleControl = null;
	win.myParent       = null;
	win.myNavBarHidden = false;
	win.relayoutDone   = false;
	win.isOpened       = false;
	
	// Ti.API.info('CTK.win(create):'+win.myId);
	
	//--------------------------------------------------
	//	Essential UI elements.	
	//--------------------------------------------------
	win.myTitleBarContainer = Ti.UI.createWindow ({
		backgroundImage: '/assets/images/ui/titlebar@2x.jpg',
		width:             '100%',
		top:               0,
		height:            $.Android.WindowTitleBarHeight,
		visible:           false,
	});
	/* toolbar container ... probably not used
	win.myToolBarContainer = Ti.UI.createWindow ({
		backgroundColor :  '#808080',
		width:             '100%',
		bottom:               0,
		height:            $.Android.WindowTitleBarHeight,
		visible:           false,
	});
	*/
	win.myTitle = __androidGenericWindowCreateDefaultTitle ();
	win.myTitleBarContainer.add(win.myTitle);
	win.myTitleControl = win.myTitle; 

	win.addEventListener ('postlayout', __androidGenericWindowOnPostLayout);
	win.addEventListener ('open',       __androidGenericWindowOnOpen);
	win.addEventListener ('close',      __androidGenericWindowOnClose);
	
	return win;
}

function __androidGenericWindowOnOpen (e) {
	var win = e.source;
	if (win.isOpened) { return; }
	
	//--------------------------------------------------
	//	codes to mimic nav window,
	//	check if this window has a child view which 
	//	is a base window of a navigation group.
	//--------------------------------------------------	
	for (var n=0; n<win.children.length; n++) {
		var view = win.children[n];
		if (view.hasOwnProperty('isNavGroupRoot')){
			view.myParent = win;  // view is a navGroup, the parent is s.t. parent.add(navGroup)
			win.myNav = view;   // denotes the window is a parent of a navGroup.
			view.window.open();
			break;
		}
	}
		
	__androidGenericWindowIsUpdateLayout (win);
	win.myTitleBarContainer.visible = !win.myNavBarHidden;
	win.myTitleBarContainer.open();
	win.isOpened = true;
}

function __androidGenericWindowOnClose (e) {
	var win = e.source;
	if (!win.isOpened) { return; }
	if (win.hasOwnProperty('myNav')) {
		while (win.myNav.myChildren.length>0){
			var aChild = win.myNav.myChildren.pop();
			aChild.close ();
		}
		win.myNav.window.close();
	}
	win.myTitleBarContainer.close();
	win.isOpened = false;
}

function __androidGenericWindowOnPostLayout (e) {
	var win = e.source;
	if (!win.relayoutDone) {		
		win.startLayout();
		__androidGenericWindowIsUpdateLayout(win);
		win.finishLayout();
		if (__androidGenericWindowIsTitleBarVisible(win)) {
			win.myTitleBarContainer.show();
		} else {
			win.myTitleBarContainer.hide();			
		}
		win.relayoutDone = true;
	}
}

function __androidGenericWindowIsTitleBarVisible (win){
	var titleBarVisible = !win.myNavBarHidden;
	return titleBarVisible;
}

function __androidGenericWindowIsUpdateLayout (win) {
	var tabBarVisible = (win.myParent!=null) && (win.tabBarHidden==false);
	var titleBarVisible = __androidGenericWindowIsTitleBarVisible(win);
	
	win.top = ((
		(titleBarVisible)?parseInt($.Android.WindowTitleBarHeight):0
	)+(
		(tabBarVisible&&($.Android.TabBarIsOnTop==true))?parseInt($.Android.TabBarHeight):0		
	))+'dp';
	win.bottom = (0+
		(tabBarVisible&&($.Android.TabBarIsOnTop!=true))?parseInt($.Android.TabBarHeight):0
	)+'dp';
	win.height = undefined;
	win.myTitleBarContainer.setTop(
		(tabBarVisible&&($.Android.TabBarIsOnTop==true))?($.Android.TabBarHeight):0
	);
}

function __androidGenericWindowCreateDefaultTitle () {
	var title = Ti.UI.createLabel ({
		left:            0,
		right:           0,
		top:             '1dp',
		bottom:          '1dp',
		text:            'WINDOW',
		color:           '#c0800000',
		textAlign:       'center',
		verticalAlign:   'middle',
		backgroundColor: '#40806020',
		visible:         false,
	});
	return title;
}

function __androidGenericWindowCreateDefaultBackButton () {
	var backBtn = Ti.UI.createLabel ({
		left:          '5dp',
		width:         '60dp',
		height:         '30dp',
		top:           '8dp',
		text:			App.Helper.L('BUTTON_BACK'),
		color:         '#FFFFFF',
		font:			{fontSize: '15sp', fontFamily: 'Melbourne-Bold'},
		textAlign:       'center',
		verticalAlign:   'middle',
		backgroundGradient:{
			type:'linear', colors: ['#FFDFA0','#DCA935'],
			startPoint:{x:0,y:0}, endPoint:{x:0,y:'100%'},
			backFillStart:false
		},
		backgroundSelectedColor: '#E5A62B', 
		borderColor:     '#504D4D4D',
		borderRadius:    $.Android.borderR,
		borderWidth:     $.Android.borderW,
	});		
	return backBtn;
}

function __androidGenericWindowOpenChild(parent,win,param) {
	if (win.myLeftNav==null) {
		var backBtn = __androidGenericWindowCreateDefaultBackButton ();
		backBtn.addEventListener ('click', (function(_parent_,_win_){
			return (function(e){
				if (checkIfButtonWasClickedThenDisableForShortPeriod
					(backBtn,1000)) {
						return;
				}
				Ti.UI.Android.hideSoftKeyboard();
				_parent_.close(_win_);
			});
		})(parent,win) );
		backBtn.addEventListener('touchstart', function(e) {
			e.source.backgroundGradient = {
				type:'linear', colors: ['#DCA935','#FFDFA0'],
				startPoint:{x:0,y:0}, endPoint:{x:0,y:'100%'},
				backFillStart:false
			}
		});
		backBtn.addEventListener('touchend', function(e) {
			e.source.backgroundGradient = {
				type:'linear', colors: ['#FFDFA0','#DCA935'],
				startPoint:{x:0,y:0}, endPoint:{x:0,y:'100%'},
				backFillStart:false
			}
		});	
		win.setLeftNavButton(backBtn);
	}
	/// win.showNavBar (); // Need to do this?			
	//----- open and show current window -----
	win.open ();
}


//===================================================================
//	android toolbar
//===================================================================

function createAndroidToolbar (style, param) {
	var toolbar = Ti.UI.createView (style);
	toolbar.isToolbar = true;

	toolbar.backgroundGradient = {
		type:'linear', 
		colors:[
				{color:'#262626', offset:0.02},
				{color:'#4d4d4d', offset:0.04},
				{color:'#303030', offset:0.05},				
				{color:'#000001', offset:1.0},
			], 
		startPoint:{x:0,y:0}, 
		endPoint:  {x:0,y:'100%'}, 
		}; 
	if (param && param.items) {
		var sumItemWidth = 0;
		for (var n=0; n<param.items.length; n++) {
			sumItemWidth += parseInt(param.items[n].width);
		};
		
		var leftPos = 10;
		var horSpacing = (parseInt(toolbar.width) - 20 - sumItemWidth) / (param.items.length - 1);

		if(param.items.length==1){
			var horSpacing = (parseInt(toolbar.width) - 20 - sumItemWidth);
			leftPos = leftPos + horSpacing;
		}
		
		for (var n=0; n<param.items.length; n++) {
			var view = param.items[n];

			view.top = '8dp';
			view.left = leftPos+'dp';
			toolbar.add(view);
			leftPos += horSpacing + parseInt(view.width);

		};		
	}
	return toolbar;	
}


//===================================================================
//	android tabbed bar
//===================================================================

function __androidTabbedBarUpdateUI (tbar, num) {
	
	// Same index, no need to update
	if (tbar.index==num) {
		return;
	}
	
	// previously selected tabbed bar
	if (tbar.index>=0 && tbar.index<tbar.tabs.length){
		tbar.tabs[tbar.index].setOpacity(0.5);		
		tbar.tabs[tbar.index].setBackgroundColor('transparent');	
	}				

	// currently selected selected tabbed bar
	tbar.index = num;
	if (tbar.index>=0 && tbar.index<tbar.tabs.length){
		tbar.tabs[tbar.index].setOpacity(1.0);		
		if(tbar.tabs.length>1){
			tbar.tabs[tbar.index].setBackgroundColor('#40000000');
		}
	}				
}

function __androidTabbedBarOnButtonClick(e) {
	__androidTabbedBarUpdateUI(e.source.myParent,e.source.myIndex);
	e.source.myParent.fireEvent('click',{source:e.source.myParent, type:'click', index:e.source.myIndex});			
}

function createAndroidTabbedBar (style, param) {
	var tbar = Ti.UI.createView (style);
	tbar.layout = 'horizontal';
	tbar.borderRadius =  $.Android.borderR;
	tbar.borderWidth = $.Android.borderW;
	tbar.borderColor = style.borderColor? style.borderColor : '#504D4D4D';
	tbar.tabs = [];
	tbar.index = -1;
	tbar.activeTab = null;
	tbar.style = style;
	tbar.touchEnabled = false;
	if (style.index!=undefined) {
		tbar.index = style.index;
	}
	if (style.labels){
		var count = style.labels.length;
		var widthSizeString = Math.round(''+(100/count))+'%';
		for (var n=0; n<count; n++){
			var lbl = style.labels[n];
			if (typeof lbl == 'string') {
				tbar.tabs[n] = Ti.UI.createLabel({
					height:       '100%',
					width:       lbl.width? lbl.width : widthSizeString,
					font: {fontSize: '14sp', fontFamily: 'Melbourne-Bold'},
					textAlign:    'center',
					textColor:    '#FFFFFF',
					text:         lbl,
					opacity:      0.5,
				});
			}
			else if (lbl.title) {
				tbar.tabs[n] = Ti.UI.createLabel({
					height:			'100%',
					width:			lbl.width? lbl.width : widthSizeString,
					font:			{fontSize: '18sp', fontFamily: 'Melbourne-Bold'},
					textAlign:		'center',
					color:			'#FFFFFF',
					text:			lbl.title,
					opacity:      	0.5,
				});

			} else if (lbl.image){
					tbar.tabs[n] = Ti.UI.createView({
						height:			'100%',
						width:			Math.round(parseInt(style.width)/count)+'dp',
						opacity:      	0.5,
					});
					if(count>1 && (n==0 || n ==count)){
						tbar.tabs[n].width = (parseInt(tbar.tabs[n].width)-1)+'dp';
					};
					tbar.tabs[n].imageView = Ti.UI.createImageView({
						width: 		lbl.width? lbl.width : widthSizeString,
						image:     lbl.image,
						touchEnabled: false,
					});	
					tbar.tabs[n].add(tbar.tabs[n].imageView);
					
				} else {
				tbar.tabs[n] = Ti.UI.createLabel({
					height:       '100%',
					width:			lbl.width? lbl.width : widthSizeString,
					textAlign:		'center',
					color:    '#FFFFFF',
					text:         'B'+n,
					opacity:      0.5,
				});
			}
			tbar.tabs[n].myIndex = n;
			tbar.tabs[n].myParent = tbar;
			tbar.add(tbar.tabs[n]);	
			/*
			tbar.tabs[n].addEventListener ('click', (function(_tbar_){
				return (function(e){
					Ti.API.info('CTKTB:'+e.source.myIndex);
					__androidTabbedBarUpdateUI(e.source.myParent,e.source.myIndex);
					_tbar_.fireEvent('click',{source:e.source.myParent, type:'click', index:e.source.myIndex});
				});
			})(tbar) );
			*/
			tbar.tabs[n].addEventListener ('click',__androidTabbedBarOnButtonClick);
			
		}
	}
	if (tbar.index>=0) {
		var tmp = tbar.index;
		tbar.index = -1;
		__androidTabbedBarUpdateUI(tbar,tmp);
	}
	tbar.setIndex = function(idx) {
		__androidTabbedBarUpdateUI(tbar,idx);
	}
	return tbar;
}


//===================================================================
//	android tab
//===================================================================

function __androidTabShow (tab) { /* declared but not used. */
	if (tab.myChildren.length>0) {
		tab.myChildren[tab.myChildren.length-1].setVisible(true);				
		if (__androidGenericWindowIsTitleBarVisible(tab.myChildren[tab.myChildren.length-1])){
			tab.myChildren[tab.myChildren.length-1].myTitleBarContainer.setVisible(!tab.myNavBarHidden);
		}
	} else {
		tab.window.setVisible(true);
		if (__androidGenericWindowIsTitleBarVisible(tab.window)) {
			tab.window.myTitleBarContainer.setVisible(!tab.myNavBarHidden);
		}
	}
}

function __androidTabHide (tab) { /* declared but not used. */
	if (tab.myChildren.length>0) {
		tab.myChildren[tab.myChildren.length-1].setVisible(false);				
		tab.myChildren[tab.myChildren.length-1].myTitleBarContainer.setVisible(false);						
	} else {
		tab.window.setVisible(false);
		tab.window.myTitleBarContainer.setVisible(false);						
	}
}

function __androidTabOpen (tab,win,param) {
	win.myParent = tab.myParent;
	__androidGenericWindowOpenChild(tab,win,param);
	//----------------------------------------------------
	//	To hide the previously topmost window.
	//	if it is a normal window from the stack, use it
	//	if it is a nav-group, hide the topmost window 
	// 	of the navgroup.
	//----------------------------------------------------
	if (tab.myChildren.length<1) {
		var aChild = tab.window;
	} else {
		var aChild = tab.myChildren[tab.myChildren.length-1];
	}
	if (aChild.hasOwnProperty('myNav')) {
		if (aChild.myNav.myChildren.length<1) {
			var aGrandChild = aChild.myNav.myChildren[aChild.myNav.myChildren.length-1];
		} else {
			var aGrandChild = aChild.myNav.window;				
		}
		aGrandChild.setVisible(false);
		aGrandChild.myTitleBarContainer.setVisible(false);			
	} else {
		aChild.setVisible(false);
		aChild.myTitleBarContainer.setVisible(false);			
	}
	tab.myChildren.push(win);
}

function __androidTabClose (tab,win,param) {
	if (tab.myChildren.length<1) {
		return;   // Cannot close TAB root window
	}

	// Get the 2nd topmost window, which will be shown after the top is closed.
	if (tab.myChildren.length<2) {
		var aChild = tab.window;
	} else {
		var aChild = tab.myChildren[tab.myChildren.length-2];
	}
	if (aChild.hasOwnProperty('myNav')) {
		if (aChild.myNav.myChildren.length<1) {
			var aGrandChild = aChild.myNav.myChildren[aChild.myNav.myChildren.length-1];
		} else {
			var aGrandChild = aChild.myNav.window;				
		}
		aGrandChild.setVisible(true);
		aGrandChild.myTitleBarContainer.setVisible(!aChild.myNavBarHidden);			
	} else {
		aChild.setVisible(true);
		aChild.myTitleBarContainer.setVisible(!aChild.myNavBarHidden);
	}
	
	// Close the top.
	aChild = tab.myChildren.pop();
	aChild.close ();
}

function _getActiveTabWindow (tab) {
	if (tab.myChildren.length>0) {
		var aChild = tab.myChildren[tab.myChildren.length-1];
	} else {
		var aChild = tab.window;
	}
	if (aChild.hasOwnProperty('myNav')) {
		if (aChild.myNav.myChildren.length>1) {
			return aChild.myNav.myChildren[aChild.myNav.myChildren.length-1];
		} else {
			return aChild.myNav.window;
		}
	} else {
		return aChild;
	}
}

function createAndroidTab (style, param) {
	var tab = Ti.UI.createView (style);
	tab.myStyle = style;
	tab.window = null;
	tab.myChildren = [];

	if (style.hasOwnProperty('window')) {
		tab.window = style.window;
		tab.window.myParent = 1;  // to be changed in open (now set to one to display tab bar.)
		tab.window.tabBarHidden = false;		
	}

	tab.setBadge = function (badge) {};
	tab.open = function (win, param) {__androidTabOpen(this,win,param);};
	tab.close = function (win, param) {__androidTabClose(this,win,param);};
	
	return tab;
}



//===================================================================
//	android window
//===================================================================


function __androidWindowSetTitleControl (win,control) {
	// Ti.API.info ('CTKA: win.setTitleControl() : '+win.myId);
	if (win.myTitleControl!=null) {
		win.myTitleBarContainer.remove(win.myTitleControl);
		win.myTitleControl = null;
	}	
	if (control!=null) {
		win.myTitleControl = control;
		win.myTitleBarContainer.add(win.myTitleControl);
		__androidWindowShowNavBar (win);
	} else {
		__androidWindowHideNavBar (win);
	}
}

function __androidWindowHideNavBar (win) {
	// Ti.API.info ('CTKA: win.hideNavBar() : '+win.myId);
	win.myNavBarHidden = true;
	if (win.isOpened){
		win.myTitleBarContainer.hide();
		win.myTitleBarContainer.setVisible(false);
		win.relayoutDone = false;
		win.updateLayout({left:0, right:0});
	} else {
		win.myTitleBarContainer.visible = false;
	}
}

function __androidWindowShowNavBar (win) {
	// Ti.API.info ('CTKA: win.showNavBar() : '+win.myId);
	win.myNavBarHidden = false;
	if (win.isOpened){
		win.myTitleBarContainer.setVisible(!win.myNavBarHidden);
		win.relayoutDone = false;
		win.updateLayout({left:0, right:0});
	} else {
		win.myTitleBarContainer.visible = true;
	}
}

function __androidWindowSetRightNavButton (win,view){
	// Ti.API.info ('CTKA: win.setRightNavButton() : '+win.myId);
	if (win.myRightNav!=null) {
		win.myTitleBarContainer.remove(win.myRightNav);
		win.myRightNav=null;
	}
	if (view!=null) {
		if(view.height == undefined) view.setHeight('30dp');
		if(view.width == undefined) view.setWidth('60dp');
		if(view.right == undefined) view.setRight('5dp');
		view.zIndex = 100;
		if(view.backgroundGradient == undefined) {
			view.backgroundGradient = {
				type:'linear', colors: ['#FFDFA0','#DCA935'],
				startPoint:{x:0,y:0}, endPoint:{x:0,y:'100%'},
				backFillStart:false
			};
			view.addEventListener('touchstart', function(e) {
				e.source.backgroundGradient = {
					type:'linear', colors: ['#DCA935','#FFDFA0'],
					startPoint:{x:0,y:0}, endPoint:{x:0,y:'100%'},
					backFillStart:false
				}
			});
			view.addEventListener('touchend', function(e) {
				e.source.backgroundGradient = {
					type:'linear', colors: ['#FFDFA0','#DCA935'],
					startPoint:{x:0,y:0}, endPoint:{x:0,y:'100%'},
					backFillStart:false
				}
			});					
		};
		win.myTitleBarContainer.add(view);
		win.myRightNav = view;			
	}
}

function __androidWindowSetLeftNavButton (win,view){
	// Ti.API.info ('CTKA: win.setLeftNavButton() : ' + win.myId);
	if (win.myLeftNav!=null) {
		win.myTitleBarContainer.remove(win.myLeftNav);
		win.myLeftNav=null;
	}
	if (view!=null) {
		if(view.height == undefined) view.setHeight('30dp');
		if(view.width == undefined) view.setWidth('60dp');
		if(view.left == undefined) view.setLeft('5dp');
		view.zIndex = 100;
		if(view.backgroundGradient == undefined && view.backgroundImage == undefined) {
			view.backgroundGradient = {
				type:'linear', colors: ['#FFDFA0','#DCA935'],
				startPoint:{x:0,y:0}, endPoint:{x:0,y:'100%'},
				backFillStart:false
			};
			view.addEventListener('touchstart', function(e) {
				e.source.backgroundGradient = {
					type:'linear', colors: ['#DCA935','#FFDFA0'],
					startPoint:{x:0,y:0}, endPoint:{x:0,y:'100%'},
					backFillStart:false
				}
			});
			view.addEventListener('touchend', function(e) {
				e.source.backgroundGradient = {
					type:'linear', colors: ['#FFDFA0','#DCA935'],
					startPoint:{x:0,y:0}, endPoint:{x:0,y:'100%'},
					backFillStart:false
				}
			});					
		};
		win.myTitleBarContainer.add(view);
		win.myLeftNav = view;			
	}
}

function createAndroidWindow (style, param) {
	var win = __androidGenericWindowCreate (style, param);

	win.setRightNavButton = function (view) { __androidWindowSetRightNavButton(this,view); };
	win.setLeftNavButton = function (view) {__androidWindowSetLeftNavButton(this,view); };		
	win.hideNavBar = function () {__androidWindowHideNavBar (this);};
	win.showNavBar = function () {__androidWindowShowNavBar (this);};
	win.setTitleControl = function (title) {__androidWindowSetTitleControl (this,title);};

	return win;
}



//===================================================================
//	android tabGroup
//===================================================================

function __androidTabGroupRefreshTabButtons (tabgrp, index, active){
	if (tabgrp.myTabButtons[index]==undefined) {
		return;
	}
	tabgrp.myTabButtons[index].icn.setOpacity(active?1.0:0.3);
	tabgrp.myTabButtons[index].lbl.setOpacity(active?1.0:0.3);
	tabgrp.myTabButtons[index].setBackgroundColor(active? '#20FFFFFF':'transparent');
}

function __androidTabGroupSetActiveTab(tabGroup,index){
	if (tabGroup.myIndex>=0 && tabGroup.myIndex<tabGroup.tabs.length) {
		if (tabGroup.tabs[tabGroup.myIndex].window.isOpened) {
			__androidTabGroupRefreshTabButtons(tabGroup, tabGroup.myIndex, false);
			if (tabGroup.activeTab!=undefined) {
				var aWin = _getActiveTabWindow(tabGroup.activeTab);
				aWin.setVisible(false);
				aWin.myTitleBarContainer.setVisible(false);
				aWin.fireEvent('blur',{type:'blur', source:aWin, index:tabGroup.myIndex});	
			}
		}
		tabGroup.activeTab = null;
	}
	tabGroup.myIndex = index;
	if (tabGroup.myIndex>=0 && tabGroup.myIndex<tabGroup.tabs.length) {
		if (!tabGroup.tabs[tabGroup.myIndex].window.isOpened) {
			tabGroup.tabs[tabGroup.myIndex].window.open();
		}
		__androidTabGroupRefreshTabButtons(tabGroup, tabGroup.myIndex, true);
		tabGroup.activeTab = tabGroup.tabs[tabGroup.myIndex];
		var aWin = _getActiveTabWindow(tabGroup.activeTab);
		aWin.setVisible(true);
		aWin.myTitleBarContainer.setVisible(!aWin.myNavBarHidden);
		aWin.fireEvent('focus',{type:'focus', source:aWin, index:tabGroup.myIndex});	
	}
}

function createAndroidTabGroup (style, param) {
	
	if (style==undefined) {
		style = {};
	}
	style.exitOnClose = true;
	style.navBarHidden = true;

	var tabGroup = Ti.UI.createWindow (style); 
	tabGroup.tabs = [];
	tabGroup.myTabButtons = [];
	tabGroup.myTabWindows = [];
	tabGroup.myIndex = -1;
	tabGroup.myTabPanel = null;
	tabGroup.myTabPanelIsOnTop = $.Android.TabBarIsOnTop;
	tabGroup.myTabPanelBorderSize = 2;
	tabGroup.navBarHidden = true;
	tabGroup.exitOnClose = true; 

	if (!($.Android.TabBarIsOnTop)) {
		tabGroup.windowSoftInputMode = Ti.UI.Android.SOFT_INPUT_ADJUST_PAN;
	}

	tabGroup.addTab = function (tab) {
		tab.myParent = tabGroup;
		tab.myIndex = tabGroup.tabs.length;
		tabGroup.tabs.push (tab);
	}
	
	tabGroup.setRightNavButton = function (view) {};
	tabGroup.setLeftNavButton = function (view) {Ti.API.info('CTKA: tabGroup.setLeftNavBar()');};	
	tabGroup.hideNavBar = function () {Ti.API.info ('CTKA: tabGroup.hideNavBar()');};
	tabGroup.showNavBar = function () {Ti.API.info ('CTKA: tabGroup.showNavBar()');};
	tabGroup.setBadge = function (badge) {Ti.API.info ('CTKA: tabGroup.setBadge()');};
	
	tabGroup.setTitleControl = function (title) {this.add(title);};
	tabGroup.getActiveTab = function () {return this.activeTab;};
	tabGroup.setActiveTab = function (index) {__androidTabGroupSetActiveTab(this,index);};
	
	tabGroup.addEventListener ('close', function(e){
		var tabGroup = e.source;
		Ti.API.info ('CTK:tabGroup.close.listener');
		if (tabGroup.tabs) {
			for (var n=0; n<tabGroup.tabs.length; n++) {
				var tab = tabGroup.tabs[n];
				if (tab.myChildren) {
					while (0<tab.myChildren.length){
						tab.close();
					}					
				}
				tab.window.close();
			}			
		}
		Ti.API.fireEvent('ApplicationClosed');
	});
	
	tabGroup.addEventListener ('open', function (e){
		var tabGroup = e.source;
		Ti.API.info ('CTK:tabGroup.open.listener');
		var tabCount = tabGroup.tabs.length; 
		if (tabCount<1) {
			return;
		}
		
		
	tabGroup.activity.addEventListener ('create'  , function (e){ Ti.API.info('CTK:TGP--->ONCREATE');})
	tabGroup.activity.addEventListener ('destroy' , function (e){ Ti.API.info('CTK:TGP--->ONDESTROY');})
	tabGroup.activity.addEventListener ('start'   , function (e){ Ti.API.info('CTK:TGP--->ONSTART'); })
	tabGroup.activity.addEventListener ('stop'    , function (e){ Ti.API.info('CTK:TGP--->ONSTOP'); })
	tabGroup.activity.addEventListener ('pause'   , function (e){ Ti.API.info('CTK:TGP--->ONPAUSE'); })
	tabGroup.activity.addEventListener ('resume'  , function (e){ Ti.API.info('CTK:TGP--->ONRESUME'); })

		var winWidth = Ti.Platform.displayCaps.platformWidth;
		var tabWidth = winWidth/tabCount;
		// var tabHeight = _dpToPixel($.Android.TabBarHeight);
		// var tabPannelHeight = tabHeight - tabGroup.myTabPanelBorderSize;

		var tabPannelHeight = $.Android.TabBarHeight;
		var tabHeight = ( parseInt(tabPannelHeight) - parseInt(tabGroup.myTabPanelBorderSize)*2) +'dp';
	
		tabGroup.myTabPanel = Ti.UI.createView ({
			width:              '100%',
			height:             tabPannelHeight,
			backgroundGradient: {
				type:'linear', 
				colors:[
					{color:'#262626', offset:0.02},
					{color:'#4d4d4d', offset:0.04},
					{color:'#303030', offset:0.05},
					{color:'#141414', offset:0.6},
					{color:'#000001', offset:0.6},
					{color:'#000001', offset:1.0},
				], 
				startPoint:{x:0,y:0}, 
				endPoint:  {x:0,y:'100%'}, 
			}, 
		})
		if (tabGroup.myTabPanelIsOnTop) {
			tabGroup.myTabPanel.top = 0;	 
		} else {
			tabGroup.myTabPanel.bottom = 0;
		}
		tabGroup.tabLeft = 0;
		tabGroup.tabRight = 0;
		tabGroup.tabWidth = '100%';
		tabGroup.tabTop = tabGroup.myTabPanelIsOnTop?tabPannelHeight:0;
		tabGroup.tabBottom = tabGroup.myTabPanelIsOnTop?0:tabPannelHeight;
		tabGroup.tabHeight = undefined;
		for (var n=0; n<tabCount; n++) {
			var aWin = tabGroup.tabs[n].window;
			aWin.visible = false;
			aWin.left    = tabGroup.tabLeft;
			aWin.right   = tabGroup.tabRight;
			aWin.width   = tabGroup.tabWidth;
			aWin.top     = tabGroup.tabTop;
			aWin.bottom  = tabGroup.tabBottom;
			aWin.height  = tabGroup.tabHeight;
			
			//----- Assigns dimension back to tab for future 'open's.
			tabGroup.tabs[n].tabParent = tabGroup;

			tabGroup.tabs[n].setVisible(false);
			var aTabBtn = Ti.UI.createView ({
				borderRadius:  $.Android.borderR,
				height:        tabHeight,
				width:         tabWidth,
				left:          tabWidth*n,
			});
			aTabBtn.lbl = Ti.UI.createLabel ({
				width:          '100%',
				height:         '13dp',
				bottom:         0,
				text:           tabGroup.tabs[n].title,
				color:          '#C0C0C0',
				textAlign:      'center',
				touchEnabled:   false,
				font:           {fontSize: '11sp', fontFamily: 'Melbourne-Bold'},
				opacity:        0.3,
			});
			aTabBtn.icn = Ti.UI.createImageView ({
				width:          '30dp',
				height:         '30dp',
				top:            '2dp',
				image:          tabGroup.tabs[n].icon,
				touchEnabled:   false,
				opacity:        0.3,
			});
			aTabBtn.add(aTabBtn.icn);
			aTabBtn.add(aTabBtn.lbl);
			aTabBtn.myIndex = n;
			tabGroup.myTabButtons.push(aTabBtn);
			tabGroup.myTabPanel.add(aTabBtn);
			aTabBtn.addEventListener('click', (function(tg,idx){ 
				return function(e) {
					tg.setActiveTab (idx);
				};
			})(tabGroup,n) );
			aTabBtn.addEventListener('touchstart', function(e) {
				if (e.source.icn) {
					e.source.icn.setOpacity(0.6);
				}
				if (e.source.lbl) {
					e.source.lbl.setOpacity(0.6);
				}
			})
			aTabBtn.addEventListener('touchend', (function(myIndex){ 
				return (function(e) {
					if (e.source.icn) {
						if (e.source.myIndex == myIndex) {
							e.source.icn.setOpacity(1.0);					
						} else {
							e.source.icn.setOpacity(0.3);					
						}
					}
					if (e.source.lbl) {
						if (e.source.myIndex == myIndex) {
							e.source.lbl.setOpacity(1.0);					
						} else {
							e.source.lbl.setOpacity(0.3);					
						}
					}
				});
			})(tabGroup.myIndex) );
		}
		
		tabGroup.add(tabGroup.myTabPanel);
		if (tabGroup.tabs.length>0) {
			if (tabGroup.myIndex<0) {
				tabGroup.setActiveTab(0);					
			} else {
				tabGroup.setActiveTab(tabGroup.myIndex);					
			}
		}
		//Ti.API.fireEvent('ApplicationOpened');
	})
	
	tabGroup.onBack = function () {
		if (tabGroup.currentWindow==tabGroup) {
		Ti.API.info('CTK!-MAINBACK');
			tabGroup.removeEventListener ('android:back', tabGroup.onBack);
			tabGroup.close ();     // tabGroup.close()
		} else {
			tabGroup.currentWindow.close(); // tab.close()
		}
	}
	
	tabGroup.addEventListener ('android:back', (function(tg) {
		return (function (e) {
			Ti.API.info('CTK:ANDROID_BACK');
			if ($.Android.fixEmailDialogOn) { // gmail mailer returns this - must ignore.
				return;
			}
			var theTab = tg.activeTab;
			if (theTab.myChildren.length<1) {
				tg.close ();
			} else {
				var aChild = theTab.myChildren[theTab.myChildren.length-1];
				if (aChild.hasOwnProperty('myNav')) {
					if (aChild.myNav.myChildren.length>0) {
						aChild.myNav.close();
					} else {
						theTab.close();
					}
				} else {
					theTab.close(); // close top window.
				}
			}
		});
	})(tabGroup) );

	return tabGroup;
}


//===================================================================
//	android navGroup
//===================================================================

function __androidNavGroupOpen (nav,win,view){
	win.myParent = nav;
	__androidGenericWindowOpenChild(nav,win,view);
	if (nav.myChildren.length<1) {
		var aChild = nav.window;
	} else {
		var aChild = nav.myChildren[nav.myChildren.length-1];
	}
	aChild.setVisible(false);
	aChild.myTitleBarContainer.setVisible(false);
	nav.myChildren.push(win);
}

function __androidNavGroupClose (nav,win,view){
	if (nav.myChildren.length<1) {
		return; // Not suppose to close the root child!
	}
	if (nav.myChildren.length<2) {
		var aChild = nav.window;
	} else {
		var aChild = nav.myChildren[nav.myChildren.length-2];
	}
	aChild.setVisible(true);
	aChild.myTitleBarContainer.setVisible(!aChild.myNavBarHidden);
	aChild = nav.myChildren.pop ();
	aChild.close();
}

function createAndroidNavGroup (style, param) {

	var nav = Ti.UI.createView ();
	nav.myChildren = [];
	nav.window = null;
	nav.myStyle = style;
	
	if (style.hasOwnProperty('window')) {
		nav.window = style.window;
		nav.isNavGroupRoot = true;
	}
	
	nav.open = function (win, param) {__androidNavGroupOpen(this,win,param);};
	nav.close = function (win, param) {__androidNavGroupClose(this,win,param);};
	nav.setRightNavButton = function (view) {Ti.API.info ('CTKA: nav.setRightNavButton()');};
	nav.setLeftNavButton = function (view) {Ti.API.info ('CTKA: nav.setLeftNavButton()');};
	nav.hideNavBar = function () {Ti.API.info ('CTKA: nav.hideNavBar()');};
	nav.showNavBar = function () {Ti.API.info ('CTKA: nav.showNavBar()');};

	return nav;
}

function AndroidEventHandler (e) {
	Ti.API.info ('CTK!-YES!');
}


function AndroidStartWait (winWait) {
	var style = Ti.UI.ActivityIndicatorStyle.DARK;
	var activityIndicator = Ti.UI.createActivityIndicator({
  		color: 'green',
  		font: {fontFamily:'Helvetica Neue', fontSize:26, fontWeight:'bold'},
  		message: 'Loading...',
  		style:style,
  		top:10,
  		left:10,
  		height:Ti.UI.SIZE,
  		width:Ti.UI.SIZE
	});
	winWait.add(activityIndicator);
	activityIndicator.show();
	return activityIndicator;
}

function AndroidStopWait (winWait, activityIndicator) {
	activityIndicator.hide();
	winWait.remove(activityIndicator);
	activityIndicator = null;
}



//===================================================================
//	android AndroidCacheableImageView
//===================================================================


function _url2hashWithExtension (url) {
	var ext = url.split('.').pop();
	return Ti.Utils.md5HexDigest(url)+'.'+ext;
}

function AndroidCacheableImageView_onError (evt) {
	evt.source.removeEventListener('error'
		,AndroidCacheableImageView_onError);
	// Ti.API.info('CTKCACHE:Load Error - '+evt.source.image);
}

function AndroidCacheableImageView_onLoad (evt) {
	evt.source.removeEventListener('load'
		,AndroidCacheableImageView_onLoad);
	// Ti.API.info('CTKCACHE:Load success - '+evt.source.image);
}

function AndroidCacheableImageView_create  (
	url, style, default_image, cache_location) {
	var aCardView = Ti.UI.createImageView(style);		
	var filename = _url2hashWithExtension(url);
	var cached_file = Ti.Filesystem.getFile(cache_location,filename);
	//---------- params -----------
	aCardView.isCached = false;
	aCardView.isFetching = false;
	aCardView.cachedFile = cached_file;
	aCardView.url = url;
	//---------- params -----------
	if (default_image) {
		aCardView.setDefaultImage(default_image);
	}
	if (aCardView.cachedFile.exists()) {
		Ti.API.info('CTKCACHE:Cached');	
		aCardView.isCached = true;
		aCardView.setImage(aCardView.cachedFile);
	} else {
		Ti.API.info('CTKCACHE:Uncached');	
		if (Ti.Network.online) {
			var c = Titanium.Network.createHTTPClient({
				timeout  : 10000,
				source   : aCardView,
				onload   : function () {
					Ti.API.info('CTKCACHE:Load Success');	
					var self = this;
					var src = self.source;
					src.cachedFile.write(self.responseData);
					src.setImage (src.cachedFile.getNativePath());
					src.isCached = true;
					src.isFetching = false;
				},
				onerror : function () {
					Ti.API.info('CTKCACHE:Load Error - '+evt.source.image);	
				}
			});
			c.open('GET',url);
			c.send();
			aCardView.isFetching = true;
		} else {
			Ti.API.info('CTKCACHE:No network');		
		}
	}
//	aCardView.setImage(url);
	return aCardView;
}

function AndroidCacheableImageView_delete (view) {
	delete view;
}
	


//===================================================================
//	android GenericImageCache
//===================================================================
/**
 * GenericImageCache for Titanium
 *
 * Author: David Geller
 *
 * Copyright (C) 2011.
 *
 * License: You may use or modify this code free without any attribution to its author whatsoever
 */
GenericImageCache = {};
 
GenericImageCache.on = true;                // enable/disable cache functionality
GenericImageCache.SUB_DIR_NAME = "imgCache/";
GenericImageCache.DIRECTORY = Titanium.Filesystem.applicationDataDirectory + GenericImageCache.SUB_DIR_NAME;
 
/**
 * Is our image cached? Easy to check. Just look for the file
 * and if it's there - we've got our answer!
 * @param {Object} name
 */
GenericImageCache.isCached = function (name)
{
    if (!GenericImageCache.on) {
        return false;
    }
 
    Logger.log ("[image_cache.js] Looking for profile file named: " + (GenericImageCache.DIRECTORY + name) + "...");
    var f = Titanium.Filesystem.getFile(GenericImageCache.DIRECTORY, name);
    return f.exists();
};
 
/**
 * Return File object to our cached file
 * 
 * @param {Object} sender_guid
 */
GenericImageCache.getFile = function (name)
{
    var f = Titanium.Filesystem.getFile(GenericImageCache.DIRECTORY, name);
    return f;
};
 
/**
 * Return the nativePath to our cached file, which can be used
 * in the url item for an imageView
 * 
 * @param {Object} sender_guid
 */
GenericImageCache.getNativeURL = function (name)
{
    var f = GenericImageCache.getFile (name);
    return f.nativePath;
};
 
/**
 * Allow users to specify a subdirectory to be created and used off
 * applicationDataDirectory. By default this class will use "gic" for GenericImageCache
 */
GenericImageCache.setDirectory = function (name)
{
    var dir_name = (name.charAt (name.length - 1) != '/') ? (name + '/') : name;
 
    GenericImageCache.SUB_DIR_NAME = dir_name;
    GenericImageCache.DIRECTORY = Titanium.Filesystem.applicationDataDirectory + GenericImageCache.SUB_DIR_NAME;
};
 
/**
 * Does our sub directory we want to use exist?
 */
GenericImageCache.directoryExists = function ()
{
    var dir = Titanium.Filesystem.getFile(Titanium.Filesystem.applicationDataDirectory,GenericImageCache.SUB_DIR_NAME);
    var rc = dir.exists();
 
    Logger.log ("image_cache.js:directoryExists " + rc);
 
    return rc;
};
 
/**
 * Create our directory if it doesn't alerady exist
 */
GenericImageCache.createDirectory = function ()
{
    var dir = Titanium.Filesystem.getFile(Titanium.Filesystem.applicationDataDirectory,GenericImageCache.SUB_DIR_NAME);
    var rc = dir.createDirectory();
 
    Logger.log ("image_cache.js:createDirectory " + rc);
 
    return rc;
};
 
/**
 * Save our blob image to cache by writing it to disk
 * 
 * @param {Object} sender_guid
 * @param {Object} blob
 */
GenericImageCache.save = function (name, blob)
{
    if (!GenericImageCache.on) {
        return;
    }
 
    if (!GenericImageCache.directoryExists()) {
        GenericImageCache.createDirectory ();
    }
 
    var f = Titanium.Filesystem.getFile(GenericImageCache.DIRECTORY, name);
    Logger.log ("[image_cache.js:saveImage] Saving " + blob.length + " bytes to " + name + "...");
    f.write (blob);
};
 
/**
 * Enumerate the cache directory. Return an array of file names
 */
GenericImageCache.enumDirectory = function ()
{
    var dir = Titanium.Filesystem.getFile(Titanium.Filesystem.applicationDataDirectory,GenericImageCache.SUB_DIR_NAME);
    var listing = dir.getDirectoryListing();
    return listing;
};
 
/**
 * Return the number of items in our cache directory
 */
GenericImageCache.itemCount = function ()
{
    return GenericImageCache.enumDirectory().length;
};
 
/**
 * Return the total number of bytes in our cache
 */
GenericImageCache.totalBytes = function ()
{
    var flist = GenericImageCache.enumDirectory();
    var f;
    var total_bytes = 0;
 
    for (var i = 0; i < flist.length; i++) {
        f = Titanium.Filesystem.getFile(GenericImageCache.DIRECTORY, flist[i]);
        if (f.exists() && (f.size > 0)) {
            total_bytes += f.size;
        }
    }
 
    return total_bytes;
};
