/**
 * Implements
 *   1. wrapper class over Ti.UI.View object as event proxy
 * 
 * @author young@waveo.com
 */

//-----------  Constructor  -----------
function WOEventProxy(parent, events){

	this._proxy		= Ti.UI.createView();

	//---  Public Instance Properties ---	
	this.className	= 'WOEventProxy';

	//--- 'inherit' the EVENT declaration of parent object
	this.EVENT = (parent && 'undefined' != typeof parent.EVENT) ? parent.EVENT : {};

	if(events){
		//--- merge event names passed in, overriding if necc
		this.addEvents(events);
	}
}

//--------------------  Public Instance Methods  -------------------------
WOEventProxy.prototype.addEventListener = function(evtName, handler, evtList) {
	this._proxy.addEventListener(evtName, handler);
	if(evtList){
		evtList.push({ evtName: evtName, handler: handler });
	}
};

WOEventProxy.prototype.removeEventListener = function(evtName, handler) {
	this._proxy.removeEventListener(evtName, handler);
};

WOEventProxy.prototype.fireEvent = function(evtName, evtData) {
	this._proxy.fireEvent(evtName, evtData);
	Ti.API.debug('Firing proxy event: '+evtName);
};

WOEventProxy.prototype.addEvents = function(events) {
    for (name in events) {
		this.EVENT[name] = events[name];
    }
};

WOEventProxy.prototype.removeListeners = function(evtList) {
	for(evt in evtList){
		this._proxy.removeEventListener(evtList[evt].evtName, evtList[evt].handler);
		Ti.API.debug('Removed proxy listener: '+evtList[evt].evtName);
	}
};

WOEventProxy.prototype.dispose = function() {
	this._proxy = null;
};

module.exports = WOEventProxy;

//--- End of file: WOEventProxy.js
