/**
 * Configuration Manager: manages various config settings as Key-Value pairs
 *
 * @author dinghua@waveo.com & young@waveo.com
 */
Ti.include('/globals.js');

var ConfigManager = App.Class(App.WOBaseClass, {
	className	: 'ConfigManager',
	alias		: 'config',

	//--- Event Name constants -----
	EVENT		: {
		//--- handles these events:
		doGetLocPermission	: 'config.doGetLocPermission',
		doRefreshLocation	: 'config.doRefreshLocation',

		//--- produces these events:
		ready				: 'config.ready',
    },
    
    CONST : {
    	deviceToken		: 'WOConfig.deviceToken',
    	locationinfo	: 'WOConfig.locationinfo',
    	langRegion		: 'WOConfig.langRegion',
    	language		: 'WOConfig.language',
    },

	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);

		self.carrierInfo = App.loadHelper('carrierInfo').getCarrierInfo();
		self.listenForEvents();
        Ti.App.fireEvent(self.EVENT.ready);		//------ We're Ready! -------
	},

	listenForEvents : function(){
		var self = this;

        //--- doGetLocPermission -----
		Ti.App.addEventListener(self.EVENT.doGetLocPermission, function(evt){
        	self.getLocationPermission(evt);
       	});
        //--- doRefreshLocation -----
		Ti.App.addEventListener(self.EVENT.doRefreshLocation, function(evt){
        	self.refreshLocationInfo(evt);
       	});
       	
        //--- locationInfo.locationUpdated -----
		var locationInfo = App.loadHelper('locationInfo');
		Ti.App.addEventListener(locationInfo.EVENT.locationUpdated, function(evt){
        	self._updateLocation(evt);
       	});
	},
	
	get : function(key, value) {
		var self = this;
		var config = $.dataMgr.newModel('WOSettings', 'config');
		config.fetch();
		if('undefined' == typeof config.data[key]) config.data[key] = null;
		if(null == config.data[key] && 'undefined' != typeof value) config.data[key] = value;
		return config.data[key];
	},
	
	set : function(key, value) {
		var self = this;
		var config = $.dataMgr.newModel('WOSettings', 'config');
		config.fetch();
        config.data[key] = value;
        config.save();
	},
	
	//for device token
	getDeviceToken : function() {
	    var self = this;
        return Ti.App.Properties.getString(self.CONST.deviceToken, '');
    },
    
    setDeviceToken : function(deviceToken) {
        var self = this;
        Ti.App.Properties.setString(self.CONST.deviceToken, deviceToken);
    },
    
    setLanguageRegion : function(isoCountryCode) {
    	var self = this;
    	isoCountryCode = ('MO' == isoCountryCode) ? 'HK' : isoCountryCode;
    	Ti.App.Properties.setString('WOConfig.langRegion', isoCountryCode);
    	App.Config.LANG_RG = isoCountryCode;
		//--- Set language & language Region ---
		if('zh-Hant' == Ti.Locale.currentLanguage || 'zh' == Ti.Locale.currentLanguage) { //iOS
			var langRegion = Ti.App.Properties.getString('WOConfig.langRegion', null);
			if(!langRegion) {
				var currentCountry = Ti.Locale.getCurrentCountry();
				if('MO' == currentCountry || 'HK' == currentCountry || 'TW' == currentCountry || 'CN' == currentCountry || 'SG' == currentCountry){
					langRegion = ('HK' == currentCountry || 'MO' == currentCountry) ? 'HK' : 'TW';
				} else {	
					var carrierInfo = require('/waveo/core/helper/carrierInfo').getCarrierInfo();
					langRegion = ('MO' == carrierInfo['isoCountryCode'] || 'HK' == carrierInfo['isoCountryCode']) ? 'HK' : 'TW';
				}
				Ti.App.Properties.setString('WOConfig.langRegion', langRegion);
			}
			if('HK' == langRegion || 'MO' == langRegion) Ti.App.Properties.setString('WOConfig.language', 'zh-Hant');
			else Ti.App.Properties.setString('WOConfig.language', 'zh-Hant_TW');
		} else {
			Ti.App.Properties.setString('WOConfig.language', Ti.Locale.currentLanguage);
		}
		var LANG = Ti.App.Properties.getString('WOConfig.language', Ti.Locale.currentLanguage);
		if(App.Config.LANG != LANG) {
			App.Config.LANG = LANG;
			//--- Update Mobile No. ---
            var account = new App.Model.WOSettings('account');
            account.fetch();
            account.data.lang = LANG;
            account.save();
			//--- Sync Account Object up to server ---
	        // Ti.App.fireEvent($.syncMgr.EVENT.doSyncObjects, {
	        	// objTypes	: [ 'WOSettings' ]
	        // });
		}
    },
	
    //---------------  Retrieval of extended data payloads --------------------
    getAppInfo : function(){
        var appInfo = {
            appver      : Ti.App.version,
            appid       : Ti.App.id,
            env         : Ti.App.deployType,
        };
        return appInfo;
    },

    getDeviceInfo : function(){       
        var deviceInfo = {
            name        : Ti.Platform.name,
            model       : Ti.Platform.model,
            uuid        : Ti.Platform.id,
            // macAddress  : Ti.Platform.macaddress,       // backup plan, use: Ti.Platform.createUUID(),
            osName      : Ti.Platform.osname,
            osVersion   : Ti.Platform.version,
            locale      : Ti.Platform.locale,
            userName    : Ti.Platform.username,
        };
        return deviceInfo;
    },

    getCarrierInfo : function(){
    	var self = this;
        return  self.carrierInfo;
    },

	//--------------------------  Location Services  --------------------------
	getLocationPermission : function(){
		var self = this;

		var geo = App.loadHelper('locationInfo');
		if(geo.needsPermission()){
			Ti.App.addEventListener(geo.EVENT.locationUpdated, _pushLocation);
			geo.getLocation();
		}

		//----------------  Event Handler  ----------------
		function _pushLocation(evt){
			Ti.App.removeEventListener(geo.EVENT.locationUpdated, _pushLocation);
			if(!evt.error){
				$.syncMgr.pushLocationInfo();
			}
		}
	},

	/**
	 * get refresh location 'silently', ie. without any UI promopt, when we are already authorized
	 */
	refreshLocationInfo : function(){
		var self = this;
		
		var geo = App.loadHelper('locationInfo');
		if(geo.isAuthorized()){
			geo.getLocation();
		}
	},

    getLocationInfo : function(){
    	var self = this;
        return Ti.App.Properties.getObject(self.CONST.locationinfo, {});
    },
    
	//-------------------------  Protected Methods  ------------------------
	_updateLocation : function(evt){
		var self = this;
		
		if('undefined' != typeof evt.coords){
			var locationInfo = App.Helper.clone(evt.coords);
			locationInfo.timezone = (new Date().getTimezoneOffset()/60)*(-1);
			Ti.App.Properties.setObject(self.CONST.locationinfo, locationInfo);
			
			if(!isNaN(locationInfo.latitude) || !isNaN(locationInfo.longitude)) {
				Ti.Geolocation.reverseGeocoder(locationInfo.latitude, locationInfo.longitude, function(evt){
					if(evt.success){
						if(evt.places)		$.trackMgr.track($.app, 'location', evt.places[0]);
					}
				});
			}
		}else{
			Ti.App.Properties.setObject(self.CONST.locationinfo, {});
		}
	},
	
	//-------------------------  Upgrade Methods  ------------------------
	getOld : function(key, versionType){ // version : 1.x / 2.x
		var self = this;
		
		if (App.Config.IS_ANDROID) return;
			
		var dbName = ('1.x' == versionType) ? 'perkpop' : 'account';
		
		var result = App.DB2.getRecord('variable', {name:key}, 'name',null,null, dbName);
		if(!result) return null;
		if(result.length > 0)
			return result[0]['value'];
		else
			return null;
	},
});

module.exports = ConfigManager;

/**
 * End of file: ConfigManager.js
 */