/**
 * Expiry Reminder manager - using Local Notifications (iOS) to implement reminders for any Smart Object type
 * 		relies on NotificationManager for local notification services
 * 
 * Smart Object class requiring reminder support must implement reminderEventTime() method
 * note: see default_config.js for Reminder Types
 * 
 * 
 * @author young@waveo.com
 */

Ti.include('/globals.js');

var ReminderManager = App.Class(App.WOBaseClass, {
	className	: 'ReminderManager',
	
	CONST		: {
		neverRemind		: -1,
	},
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	setCardReminders : function(){
		var self = this;
		App.Helper.requireModel('WOCard');
		App.Helper.requireModel('WOOffer');

        //------- Get Reminder Preferences  -------
        var prefData = $.dataMgr.get('WOSettings', 'preference');
		if(prefData.cardRemindDays == self.CONST.neverRemind)	return;

		//=======  TEST CODE  =========
		// prefData.cardRemindTime = new Date(new Date().getTime() + 20000);	// FIXME [young]  test code

        //------- Set Card Reminders -----------------
        var myCards = new App.Model.WOCard();
        var numCards = myCards.fetchAll();
		
		var reminderParam = {
			reminderType    : App.Const.NOTIFY_CARDEXPIRY,
			reminderDays	: prefData.cardRemindDays,
			timeOfDay		: prefData.cardRemindTime * 1000,
		};
		self._setObjectReminders(myCards, reminderParam);
	},
	
	setOfferReminders : function(){
		var self = this;

        //------- Get Reminder Preferences  -------
        var prefData = $.dataMgr.get('WOSettings', 'preference');
		if(prefData.offerRemindDays == self.CONST.neverRemind)	return;

        //------- Set Card Reminders -----------------
        var myOffers = new App.Model.WOOffer();
        var numOffers = myOffers.fetchAll();
		
		var reminderParam = {
			reminderType    : App.Const.NOTIFY_CARDEXPIRY,
			reminderDays	: prefData.offerRemindDays,
			timeOfDay		: prefData.offerRemindTime * 1000,
		};
		self._setObjectReminders(myOffers, reminderParam);
	},
	
	setRewardReminders : function(){
		// var self = this;
// 		
        // //------- Get Reminder Preferences  -------
        // var prefData = $.dataMgr.get('WOSettings', 'preference');
		// if(prefData.offerRemindDays == self.CONST.neverRemind)	return;
// 
        // //------- Set Card Reminders -----------------
        // var myRewardSets = new App.Model.WORewardset();
        // var numRewardSets = myRewardSets.fetchAll();
// 		
		// var reminderParam = {
			// reminderType    : App.Const.NOTIFY_CARDEXPIRY,
			// reminderDays	: prefData.offerRemindDays,
			// timeOfDay		: prefData.offerRemindTime * 1000,
		// };
		// self._setObjectReminders(myRewardSets, reminderParam);		
	},

    //--------------------  Local Notification Services  ------------------------
    //  wrap iOS & Android specific implementation within these methods
    //  TODO [TK] - Android
		
	getReminders : function(){
	    var self = this;
	    
        // var results = self.notify.returnScheduledNotifications();
        // var test = JSON.stringify(results);
        // Ti.API.info("results stringified" + test);
	},

    /**
     * @param   {object}    of these attributes:
     *          alertBody   - notification message content (string, localized)
     *          date        - time of notification (date)
     *          userInfo    - data payload passed to $.notificationMgr when app launched (JSON string)
     *          badge       - app badge (number, optional)
     */	
    setReminder : function(param){
        var self = this;

		//  FIXME [TK]  Android
		if(! App.Config.IS_ANDROID){ 
	        param.alertAction = "View";    // action button label
	        $.notifyMgr.notify.scheduleLocalNotification(param);
		} else {
			
		}
    },

    cancelReminders : function(){
        var self = this;
		
		//  FIXME [TK]  Android
		if(! App.Config.IS_ANDROID){
        	$.notifyMgr.notify.cancelAllLocalNotifications();			
		}
    },

	//--------------------------  Protected Methods  -----------------------------

	/**
	 * setup reminder notifications for objects
	 * @param	{object}		WOObjects that implements reminder methods
	 * @param	{object}		abstract parameter object:
	 *			reminderDays	- no. of days for reminders, in advance of expiry date. {number}
	 * 			timeOfDay		- time of day for reminders {date} (only HH:MM:SS used, date portion ignored)
	 * 			reminderType	- reminder type {string}, eg. NOTIFY_CARDEXPIRY, see default_config.js
	 */
    _setObjectReminders : function(theObjects, param){
        var self = this;

        var reminderType = param.reminderType;
        var reminderDays = param.reminderDays;
        var timeOfDay = param.timeOfDay;

        // milliseconds between ReminderTime and EventDate
        var reminderOffset = self._calcReminderOffset(reminderDays, timeOfDay);
    
        var reminderCount = 0;
        var eventDate = null;
        var reminderTime = null;
        var timeNow = new Date();
        var numObjects = theObjects.count();
        
        for(var i=0; i < numObjects; i++){
            var anObject = theObjects.nth(i);
            
            eventDate = anObject.reminderEventTime();
            if(eventDate){
                eventDate.setHours(0,0,0,0);        // use only the date
                reminderTime = new Date(eventDate - reminderOffset);
                
                // only set reminders in the future AND not in the Message Queue
                
                var reminderExist = false;		// FIXME  [Zj]
                
                if(! reminderExist && reminderTime >= timeNow){
                    //--- Setup Local Notification ---
                    reminderCount++;
                    var msgParam = {
                        reminderType    : reminderType,
                        duration        : reminderDays,
                    };
                    
                    var alertBody = anObject.reminderMessage(msgParam);
                    alertBody = alertBody.replace('%', '%%'); //[FIXED #41]
                    var notifyParam = {
                        alertBody   : alertBody,     // get message from Object
                        date        : reminderTime,
                        userInfo    : {
                        	messageType	: $.notifyMgr.TYPE.reminder,
                            id          : reminderCount,
                            type        : reminderType,
                            objType     : anObject.className,
                            objID       : anObject.getID(),
                            },
                        badge       : 1,
                        sound 		: 'scan',
                    };
                    
                    self.setReminder(notifyParam);
                    
                    Ti.API.warn('Set Reminder: '+JSON.stringify(notifyParam));
                }
            }
        }
    },

    /**
     * @param   {number}        no. of days of advance expiry notice
     * @param   {number}        time of day for the reminder in milliseconds
     * @return  {number}        pre-expiry reminder time offset in milliseconds
     */
    _calcReminderOffset : function(daysAhead, timeOfDay){
        return (daysAhead * 24*60*60*1000) - timeOfDay;
    },
    	
});

module.exports = ReminderManager;

//---  End of file: ReminderManager.js
