/**
 * App Usage tracking manager
 * 
 * @author dinghua@waveo.com & young@waveo.com
 */
Ti.include('/globals.js');

var TrackManager = App.Class(App.WOBaseClass, {
	className	: 'TrackManager',
	alias		: 'track',

    //--- Event Name constants -----
    EVENT		: {
		//--- responds to these events:
		track           : 'TRACK.track',
    },
    
    ACTION : {
        created : 'created',
        updated : 'updated',
        deleted : 'deleted',
        viewed  : 'viewed',
    },
    
    LOCATION : {
    	'WOCard' : 'view',
    	'WOPerkCode' : 'issueStamp',
    },

	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
		
		App.Helper.requireModel('WOUsage');
		self.listenForEvents();
	},

	listenForEvents : function(){
		var self = this;
		
		//--- track ---
		Ti.App.addEventListener(self.EVENT.track, function(evt){
			self._track(evt);
		});
		
		self.isListenEvents = true;
	},
	
    /**
     *  Wrapper function to make calls to _track() async
     *	@param	{object}	a WOBaseClass object  (must have 'alias' property)
     *  @param  {string}    action
     *  @param  {object}    data object
     */
    track : function(pObject, pAction, pData){
        var self = this;
        pObject = pObject || {};
        pAction = pAction || '';
        pData = pData || {};
        
        Ti.App.fireEvent(self.EVENT.track, {
        	object		: pObject,
            action      : pAction,
            data        : pData,
            stamp		: Math.round(new Date().valueOf() / 1000),      // in seconds
        });
    },

	_track : function(evt){
		var self = this;
	    // evt = { object: object, action: string, data: object
	    var coords = $.configMgr.getLocationInfo();
	    	
		evt.data = evt.data || {};
		var usage = new App.Model.WOUsage();
		usage.data.object	= evt.object.className;
		usage.data.objectID	= 'undefined' != typeof evt.object.id ? evt.object.id : null;
        usage.data.action	= evt.action;
        usage.data.data		= evt.data;
        usage.data.location = {coords:coords};
        usage.data.stamp	= evt.stamp;
        
	    if('undefined' != self.LOCATION[evt.object.className] 
	    && evt.action == self.LOCATION[evt.object.className]) {
	    	if(isNaN(coords.latitude) || isNaN(coords.latitude)) {
	    		usage.data.location = {coords:coords};
	    		usage.save();
	    	} else {
		    	Ti.Geolocation.reverseGeocoder(coords.latitude, coords.longitude, function(result){
					usage.data.location = {coords:coords};
					if(result.success){
						if(result.places) {
							var place = result.places[0];
							place.coords = coords;
							usage.data.location = place;
						}
					}
					usage.save();
				});
			}
	    } else {
	    	usage.save();
	    }
		
		Ti.API.info('track: '+evt.object.className+'::'+evt.action+'  '+JSON.stringify(evt.data));
	},
	
	dispose : function(){
		Ti.App.removeEventListener(self.EVENT.track, self._track);		
	},
	
});

module.exports = TrackManager;

/**
 * End of file: TrackManager.js
 */