/**
 * Provides common Authentication & Login services
 *
 * @author young@waveo.com
 */
Ti.include('/globals.js');

var WOAuthManager = App.Class(App.Controller.WOManager, {
	className	: 'WOAuthManager',
    alias		: 'auth',

    //--- Event Name constants -----
    EVENT		: {
		//--- responds to these events:
		doLogin			: 'auth.doLogin',

		//--- produces these events:
		authed         	: 'auth.authenticated',
		login           : 'auth.loggedIn',
		logout          : 'auth.loggedOut',
		forceReAuth		: 'auth.forceReAuth',
    },
    
	//---  Constructor  --------------- 
	initialize : function(param){
		var self = this;
		self.base(param);
	},

    //----------------------  Authentication Services  ----------------------
    isAuthed : function(){
        return true;
    },
	
    logout : function(param){
    },

    /**
     * provides 2 verification services: (1) verify a given code, (2) request a resend of code
     * @param   {string}        type of verfication, eg. 'registration', 'changeMobileNo'
     * @param   {string}        mobile number
     * @param   {Y/N}           'N' when (1), 'Y' when (2)
     * @param   {string}        the verification code, set as empty string '' when (2)
     */
    verifyMobile : function(param){
        var self = this;

        //---------------  Call Server  -------------
        var payload = {
            type        : param.type,
            mobileNo    : param.mobileNo,
            resend      : param.resend ? 'Y' : 'N', 
            code        : param.resent ? '' : param.code,
        };   
        var callParam = {
            action      : App.Action.VERIFYMOBILE,
            data        : payload,
            report      : true,
            callback    : param.callback,
            errorback   : param.errorback
        };
        self.callServer(callParam);
    },
    
    _preLogoutCleanUp : function(){
        // make sure we push everything to server BEFORE logging out!
    },

    _postLogoutCleanUp : function(){
        // do clean up of any personal data AFTER logging out
    },

});

module.exports = WOAuthManager;

/**
 * End of file: WOAuthManager.js
 */