/**
 * Factory of Card Widgets, ie. mini-controllers with pre-built views
 * 
 * @author young@waveo.com
 */
Ti.include('/globals.js');

var WOCardFactory = App.Class(App.WOFactory, {
	className : 'WOCardFactory',

	//--- All widgets produced by this factory will deal with these events:
	WIDGET_EVENT	: {
		//--- Handles these events:
		doFlipCard		: 'WOCardFactory.doFlipCard',
		setAutoRotate	: 'WOCardFactory.setAutoRotate',
		refresh			: 'WOCardFactory.refresh',
		setMode			: 'WOCardFactory.setMode',
		//--- Produces these events:
		click			: 'WOCardFactory.click',
		longPress		: 'WOCardFactory.longPress',
		inPortrait		: 'WOCardFactory.inPortrait',
		outPortrait		: 'WOCardFactory.outPortrait',
		inLandscape		: 'WOCardFactory.inLandscape',
		outLandscape	: 'WOCardFactory.outLandscape',
    },
	
	MODES	: ['display'],

	FORMAT	: {
		thumbnail	: 'thumbnail',
		standard	: 'standard',
		landscape	: 'landscape',
		cardIntro	: 'cardIntro',
		singleRow	: 'singleRow',
	},
	
	ROTATE	: {
		1	: {
			angle	: { 1:   0,  2:   0,  3:  90,  4: -90 },
			scale	: { 1: 1.0,  2: 1.0,  3: 1.5,  4: 1.5 },
		},
		2	: {
			angle	: { 1:   0,  2:   0,  3:  90,  4: -90 },
			scale	: { 1: 1.0,  2: 1.0,  3: 1.5,  4: 1.5 },			
		},
		3	: {
			angle	: { 1:   0,  2:   0,  3:  90,  4: -90 },
			scale	: { 1: 1.0,  2: 1.0,  3: 1.5,  4: 1.5 },
		},
		4	: {
			angle	: { 1:   0,  2:   0,  3:  90,  4: -90 },
			scale	: { 1: 1.0,  2: 1.0,  3: 1.5,  4: 1.5 },
		},
	},

	//---  Constructor  --------------- 
	initialize : function(param){
		var self = this;
		self.base(param);
		
		if (App.Config.IS_ANDROID) {
//			self.barcodeModule = require('com.bsl.lbarcode');
			self.barcodeModule = require('com.bsl.zxbarcode');
		} else {
			self.barcodeModule = require('com.test');			
		}

		//---  Load & merge Style files  ---
		self.STYLE = App.Helper.loadStyle(param.style);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		var widgetParam = self.base(param, data);
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.properties.cardObj		= param.properties.cardObj;
		widgetParam.properties.format		= ('format' in param.properties) ? param.properties.format : 'standard';
		widgetParam.properties.isCardMaster	= ('isCardMaster' in param.properties) ? param.properties.isCardMaster : false;
		widgetParam.properties.cardFront	= ('cardFront' in param.properties) ? param.properties.cardFront : true;
		widgetParam.properties.cardBack		= ('cardBack' in param.properties) ? param.properties.cardBack : false;
		widgetParam.properties.canFlip		= ('canFlip' in param.properties) ? param.properties.canFlip : false;
		widgetParam.properties.touchEnabled	= ('touchEnabled' in param.properties) ? param.properties.touchEnabled : false;
		widgetParam.properties.showName		= ('showName' in param.properties) ? param.properties.showName : false;
		widgetParam.properties.longPress	= ('longPress' in param.properties) ? param.properties.longPress : false;
		widgetParam.properties.shadow		= ('shadow' in param.properties) ? param.properties.shadow : true;
		widgetParam.properties.landscape	= ('landscape' in param.properties) ? param.properties.landscape : false;
		
		//---  fields is modified directly by rendering methods ---
		widgetParam.properties.fields		= {};
		
		return widgetParam;
	},

	_attachProperties : function(widget){
		var self = this;

		widget.cardFront	= self.widgetParam.properties.cardFront;		
		widget.cardBack		= self.widgetParam.properties.cardBack;
		
		widget.canFlip		= self.widgetParam.properties.canFlip;
		widget.longPress	= self.widgetParam.properties.longPress;
		widget.landscape	= self.widgetParam.properties.landscape;

		widget.fields		= self.widgetParam.properties.fields;
		widget.STYLE		= self.STYLE[self.widgetParam.properties.format];		
		
		if(self.widgetParam.properties.landscape){
			widget.autoRotate = false;
			widget.ROTATE = self.ROTATE;
		}
	},
	
	_attachMethods : function(_Widget_){
		var self = this;	
		self.base(_Widget_);

		//---  attach object behaviors  ---
		if(self.widgetParam.properties.cardFront){
			_Widget_.prototype.updateCardFront = self._updateCardFront_;
		}
		if(self.widgetParam.properties.cardBack){
			_Widget_.prototype.updateCardBack		= self._updateCardBack_;
			_Widget_.prototype.barcodeModule		= self.barcodeModule;
			_Widget_.prototype.renderBarcode		= self._renderBarcode_;
			_Widget_.prototype.highlightExpiryDate	= self._highlightExpiryDate_;
			_Widget_.prototype.codeFormat			= self._codeFormat_;
			_Widget_.prototype.dateString			= self._dateString_;
		}
		if(self.widgetParam.properties.canFlip){
			_Widget_.prototype.doFlip = self._doFlip_;
		}
		if(self.widgetParam.properties.longPress){
			_Widget_.prototype.doLongPress = self._doLongPress_;
		}
		if(self.widgetParam.properties.landscape){
			_Widget_.prototype.setAutoRotate = self._setAutoRotate_;
			_Widget_.prototype.orientHandler = self._orientHandler_;		
		}
	},
			
	//--  View render methods for each supported Mode that subclass MUST implement  --	
	_draw_displayView : function(){
		var self = this;

		var _properties = self.widgetParam.properties;
		var theCard = _properties.cardObj;

		switch(_properties.format){
		case self.FORMAT.thumbnail:
		case self.FORMAT.standard:
		case self.FORMAT.cardIntro:
		case self.FORMAT.landscape:
			var cardView = self._drawCard(theCard, _properties);
			break;
		case self.FORMAT.singleRow:
			var cardView = self._drawSingleRow(theCard, _properties);
			break;
		default:
			Ti.API.error(self.className+'::_draw_displayView - unsupported format ('+_properties.format+')');
		}
		self.widgetParam.modes.display.view = cardView;
		
		return self.widgetParam.modes.display.view;
	},

	//-----  Card Renderer  ------------------------------------------
	_drawCard : function(theCard, properties){
		var self = this;
		
		//---  Enclosing view  -------------------
		var viewStyle = properties.shadow ? self.STYLE[properties.format].shadow : self.STYLE[properties.format].image;
		var cardView = Ti.UI.createView(viewStyle);

		//---  Card Back  ------------------------
		if(properties.cardBack){
			//--- Barcode ---
			var codeType = (theCard.barcodeType == 1) ? 'qrcode' : 'barcode';
			if(theCard.showCardNo && 'qrcode' == codeType) {
				var barcodeStyle = self.STYLE[properties.format].qrcode2.qrcode;
				var cardNumberStyle = self.STYLE[properties.format].qrcode2.cardNumber;
			} else {
				var barcodeStyle = self.STYLE[properties.format][codeType];
				var cardNumberStyle = self.STYLE[properties.format].cardNumber;
			}
			var backView = self._renderCardBack({
				card	: theCard,
				back	: self.STYLE[properties.format].back,
				style	: {
					barcode		: barcodeStyle,
					nocode		: self.STYLE[properties.format].nocode,
					cardNumber	: cardNumberStyle,
					separator	: self.STYLE[properties.format].separator,
					joinDate	: self.STYLE[properties.format].joinDate,
					expiryDate	: self.STYLE[properties.format].expiryDate,
				},
			});
			backView.visible = false;
			cardView.add(backView);
		}
		//---  Card Front  -----------------------
		if(properties.cardFront){
			var imageField = (properties.format == self.FORMAT.thumbnail) ? 'thumbImage' : 'frontImage';
			var imageURL = theCard[imageField]; 
			self.widgetParam.properties.fields.image = imageField;		// add to fields
			
			var frontView = self._renderCardFront({
				card		: theCard,
				URL			: imageURL,
				showName	: properties.showName,
				style		: {
					imageFrame 		: self.STYLE[properties.format].imageFrame,
					image 		: self.STYLE[properties.format].image,
					cardName	: self.STYLE[properties.format].cardName,
					badge		: self.STYLE[properties.format].badge,
				},
			});
			frontView.visible = true;
			cardView.add(frontView);
		}
		
		return cardView;
	},

	_drawSingleRow : function(theCard, properties){
		var self = this;
		//---  Enclosing view  -------------------
		var rowView = Ti.UI.createView(self.STYLE[properties.format].frame);
		
		//--- Card Thumbnail Image ---------
		var cardDefn = {
			name		: theCard.masterName,
			style		: self.STYLE[properties.format].image,
			properties 	: {
				cardObj			: theCard,
				format			: self.FORMAT.thumbnail,
				cardFront		: true,
				shadow			: true,
			},
		};
		var card = $.cardMgr.spareFactory.make(cardDefn, theCard);
		var cardImage = card.getView();		
		
		rowView.add(cardImage);

		//--- Brand Name --------------
        var brandName = Ti.UI.createLabel(self.STYLE[properties.format].brandName);
        brandName.text = theCard.brandName;
		self.widgetParam.properties.fields.brandName = brandName;			// add to fields
        rowView.add(brandName);
        
		//--- Card Name ---------------
        var cardName = Ti.UI.createLabel(self.STYLE[properties.format].cardName);
        cardName.text = theCard.name;
		self.widgetParam.properties.fields.cardName = cardName;				// add to fields
        rowView.add(cardName);
        
		//--- Badge -------------------
        var cardBadge = Ti.UI.createView(self.STYLE[properties.format].badge);
        cardBadge.visible = (theCard.state < 0) ? true : false;
        cardBadge.touchEnabled = false;
		self.widgetParam.properties.fields.cardBadge = cardBadge;			// add to fields
        rowView.add(cardBadge);
        rowView.width = '100%'; //For list order
		
		return rowView;
	},

	//------------  Card Rendering Methods  --------------------------------------
	_renderCardFront : function(param){
		var self = this;

		var viewStyle = param.style.imageFrame ? param.style.imageFrame : param.style.image;

		//---  Card Face Frame  ------
		var frontViewFrame = Ti.UI.createView(viewStyle);

		//---  Card Face Image  ------
		var cardFront = new App.Model.WOImage(param.URL);
		cardFront.initialize();
		
		cardFront.fetch();
		var frontView = cardFront.getView({
			style				: param.style.image,
			activityIndicator	: { show: true },			
		});
		self.widgetParam.properties.cardFront = cardFront;
		self.widgetParam.properties.fields.cardFront = frontViewFrame;		// add to fields
		frontViewFrame.add(frontView);
		
		//---  Card Badge / Alert  ---
        var cardBadge = Ti.UI.createView(param.style.badge);
        cardBadge.visible = (param.card.state < 0) ? true : false;
        cardBadge.touchEnabled = false;
        frontViewFrame.add(cardBadge);
		self.widgetParam.properties.fields.cardBadge = cardBadge;		// add to fields
		
		//---  Card Front Name  ------------
		if(param.showName){
	        var cardFrontName = Ti.UI.createLabel(param.style.cardName);
	        cardFrontName.text = param.card.frontName;
			self.widgetParam.properties.fields.cardFrontName = cardFrontName;		// add to fields
	        frontViewFrame.add(cardFrontName);
		}

		return frontViewFrame;		
	},
	
	_renderCardBack : function(param){
		var self = this;

		var backView = Ti.UI.createView(param.back);

		//--- Barcode & Magstripe  ------------------------
		var codeViews = self._renderBarcode_(param.card, param.style);
		backView.add(codeViews.magstripe);
		backView.add(codeViews.barcode);

		self.widgetParam.properties.fields.magstripe = codeViews.magstripe;		// add to fields		
		self.widgetParam.properties.fields.barcode = codeViews.barcode;			// add to fields		
		self.widgetParam.properties.fields.backView = backView;					// add to fields
		
		//--- Card Number ----------------
		var cardNumber = Ti.UI.createLabel(param.style.cardNumber);
		cardNumber.text = param.card.showCardNo ? self._codeFormat_(param.card.numFormat, param.card.number) : '';
		backView.add(cardNumber);
		self.widgetParam.properties.fields.cardNumber = cardNumber;		// add to fields

		var separator =	Ti.UI.createView(param.style.separator);
		backView.add(separator);
		self.widgetParam.properties.fields.separator = separator;		// add to fields
		
		//--- Join Date  ----------------- 
		var joinView = Ti.UI.createView(param.style.joinDate.view);
		backView.add(joinView);
		self.widgetParam.properties.fields.joinView = joinView;		// add to view
		
		var joinLabel = Ti.UI.createLabel(param.style.joinDate.label);
		joinLabel.text = App.Helper.L('card_join');
		joinView.add(joinLabel);
		self.widgetParam.properties.fields.joinLabel = joinLabel;		// add to fields

		var joinDate = Ti.UI.createLabel(param.style.joinDate.date);
		joinDate.text = self._dateString_(param.card.joined);
		joinView.add(joinDate);
		self.widgetParam.properties.fields.joinDate = joinDate;			// add to fields

		//--- Expiry Date  ---------------
		var expiryView = Ti.UI.createView(param.style.expiryDate.view);
		backView.add(expiryView);
		self.widgetParam.properties.fields.expiryView = expiryView;		// add to view

		var expiryLabel = Ti.UI.createLabel(param.style.expiryDate.label);
		expiryView.add(expiryLabel);
		self.widgetParam.properties.fields.expiryLabel = expiryLabel;	// add to fields
		
		var expiryDate = Ti.UI.createLabel(param.style.expiryDate.date);
		expiryDate.text = self._dateString_(param.card.expiry);
		expiryView.add(expiryDate);
		self.widgetParam.properties.fields.expiryDate = expiryDate;		// add to fields
		
		self._highlightExpiryDate_(param.card.expiry, expiryLabel, expiryDate, param.style.expiryDate);
		
		//Show/Hide Card Number according to card.showCardNo (true/false) 
		cardNumber.visible = param.card.showCardNo;

		//---  Conditionally display of fields  ---
		if(param.card.barcodeType == 1){		// QR Code
			if(joinDate.text == '' && expiryDate.text == '' && !param.card.showCardNo){
				//  vertically center QR Code if no join & expiry dates
				codeViews.barcode.top = null;
			}
		}
		joinLabel.visible = (joinDate.text == '') ? false : true;
		separator.visible = (joinDate.text == '' && expiryDate.text == '') ? false : true;

		return backView;
	},

	
	//------------  Template/Prototype Methods for Field object  -----------

	_setupEventHandlers_ : function(){
		var self = this;
		
		self.base();	// MUST call super-class

		//---  Standard Card behavior with different combinations of TRUE settings:
		//  1. longPress + canFlip:		longPress -> doFlip, click -> throw event, flip-on-event
		//  2. canFlip:					click -> doFlip, flip-on-event  (longPress not supported)
		//  3. longPress:				click -> throw event, longPress -> throw event
		//  4. (none):					click -> throw event (ie.both longPress & canFlip ＝＝ false)		

		if(self.canFlip){
			self._eventProxy.addEventListener(self._eventProxy.EVENT.doFlipCard, function(evt){
				self.doFlip(evt, self.modes[self.currentMode].view);
			});
			if(self.longPress){
				//--- use case #1
				self.VIEW.addEventListener('longpress', _doCardFlip);

				// click event fired by super-class by default, no need to implement
			}else{
				//--- use case #2, override click behavior
				self.VIEW.addEventListener('click', _doCardFlip);
			}
		}else{
			if(self.longPress){
				//--- use case #3
				self.VIEW.addEventListener('longpress', function(evt){
					self._eventProxy.fireEvent(self._eventProxy.EVENT.longPress, evt);
				});
				// click event fired by super-class by default, no need to implement
			}
			// else use case #4:  click event fired by super-class by default, no need to implement
		}
		
		//---  Handle Landscape  ---
		if(self.landscape){
			self._eventProxy.addEventListener(self._eventProxy.EVENT.setAutoRotate, function(evt){
				self.setAutoRotate(evt);
			});
		}
		//---------------  Event Handlers  -----------------
		function _doCardFlip(evt){
			self.doFlip(evt, self.modes[self.currentMode].view);
		}
	},

 	_doFlip_311 : function(evt, cardView){
		var self = this;
		
		var sides = cardView.children;
		if(sides.length >= 2){
			var fromSide = sides[1].visible ? sides[1] : sides[0];
			var toSide = sides[1].visible ? sides[0] : sides[1];
			
			if (App.Config.IS_ANDROID) {
				fromSide.hide();
				  toSide.show();
			} else {
				cardView.remove(toSide);
				if(evt.x<(fromSide.size.width/2)) {
					var direction = Ti.UI.iPhone.AnimationStyle.FLIP_FROM_RIGHT;
				} else {
					var direction = Ti.UI.iPhone.AnimationStyle.FLIP_FROM_LEFT;
				}
				cardView.animate({duration: 500, view: toSide, transition: direction}, function(evt){
					cardView.add(fromSide);
				});
			}
			toSide.visible = true;
			fromSide.visible = false;
			self.isFront = !self.isFront;
		}
	},
	
	_doFlip_ : function(evt, cardView){
		var self = this;
		
		var sides = cardView.children;
		if(sides.length >= 2){
			var fromSide = sides[1].visible ? sides[1] : sides[0];
			var toSide = sides[1].visible ? sides[0] : sides[1];
			
			if (App.Config.IS_ANDROID) {
				fromSide.hide();
				  toSide.show();
			} else {
				var direction = Ti.UI.iPhone.AnimationStyle.FLIP_FROM_LEFT;
				if(evt.x<(fromSide.size.width/2))    direction = Ti.UI.iPhone.AnimationStyle.FLIP_FROM_RIGHT;
				cardView.animate({duration: 500, view: toSide, transition: direction});
			}
			toSide.visible = true;
			fromSide.visible = false;
			self.isFront = !self.isFront;
		}
	},
	
	_doLongPress_ : function(evt){
		var self = this;
		self.doFlip(evt, self.modes[self.currentMode].view);
	},

	_setValue_ : function(card){
		var self = this;

		if(self.cardFront)		self.updateCardFront(card);
		if(self.cardBack)		self.updateCardBack(card);

		self.data = card;
	},

	_setAutoRotate_ : function(evt){
		var self = this;

		if(evt.turnOn == self.autoRotate)	return;		// no state change
		if(evt.turnOn){
			self.autoRotate = true;
			self.rotating = false;
			if('undefined' == typeof self.currentOrient)	self.currentOrient = Ti.UI.PORTRAIT;

			self.evtHandler = _doOrientChange;
			Ti.Gesture.addEventListener('orientationchange', _doOrientChange);
			Ti.Gesture.fireEvent('orientationchange', { orientation: Ti.Gesture.orientation });
		}else{
			Ti.Gesture.removeEventListener('orientationchange', self.evtHandler);
			self.autoRotate = false;
		}
		Ti.API.error('autoRotate='+self.autoRotate);
		
		function _doOrientChange(evt){
			self.orientHandler(evt);
		}
	},

	_updateCardFront_ : function(card){
		var self = this;

		//---  Card Front Image  ---
		if(self.data[self.fields.image] != card[self.fields.image]){		
			// card URL have changed, update cardFront (WOImage object)
			self.cardFront.setURL(card[self.fields.image]);
			self.cardFront.getView();			
		}

		//---  Card Badge  ----
		self.fields.cardBadge.visible = (card.state < 0) ? true : false;
		
		//---  Card Name  -----
		if('undefined' != typeof self.fields.cardName){
			self.fields.cardName.text = card.name;
		}
		//---  Card Front Name  -----
		if('undefined' != typeof self.fields.cardFrontName) {
			self.fields.cardFrontName.text = card.frontName;
		}
		//---  Brand  ---------
		if('undefined' != typeof self.fields.brandName){		
			self.fields.brandName.text = card.brandName;
		}
	},

	_updateCardBack_ : function(card){
		var self = this;

		//---  Card Number  ----
		self.fields.cardNumber.text = card.showCardNo ? self.codeFormat(card.numFormat, card.number) : '';
		
		//---  Join Date  ------
		self.fields.joinDate.text = self.dateString(card.joined);
		self.fields.joinLabel.visible = (self.fields.joinDate.text == '') ? false : true;

		//---  Expiry Date  ----		
		self.fields.expiryDate.text = self.dateString(card.expiry);
		self.highlightExpiryDate(card.expiry, self.fields.expiryLabel, self.fields.expiryDate, self.STYLE.expiryDate);
		
		//---  Separator  ------
		self.fields.separator.visible = (self.fields.joinDate.text == '' && self.fields.expiryDate.text == '') ? false : true;		

		//--- Barcode & Magstripe  ------------------------
		var codeType = (card.barcodeType == 1) ? 'qrcode' : 'barcode';
		if(self.data.number != card.number){
			if('qrcode' == codeType && self.data.showCardNo) {
				var barcodeStyle = self.STYLE.qrcode2.qrcode;
			} else {
				var barcodeStyle = self.STYLE[codeType];
			}
			var style = {
				barcode		: barcodeStyle,
				nocode		: self.STYLE.nocode,
			};
			var codeViews = self.renderBarcode(card, style);

			self.fields.backView.remove(self.fields.magstripe);			//  remove previous magstripe
			self.fields.backView.remove(self.fields.barcode);			//  remove previous barcode

			self.fields.backView.add(codeViews.magstripe);			
			self.fields.backView.add(codeViews.barcode);

			self.fields.magstripe = codeViews.magstripe;
			self.fields.barcode = codeViews.barcode;
			if(self.fields.barcodex) {
				self.fields.backView.remove(self.fields.barcodex);
				self.fields.barcodex = null;
			}
		}
		//---  Conditionally display of fields  ---
		if(codeType == 'qrcode'){
			if(self.fields.joinDate.text == '' && self.fields.expiryDate.text == '' && !card.showCardNo){
				//  vertically center QR Code if no join & expiry dates
				self.fields.barcode.top = null;
			} else {
				if(card.showCardNo) self.fields.barcode.top = self.STYLE.qrcode2.qrcode.top;
				else self.fields.barcode.top = self.STYLE[codeType].top;
			}
		}
		
		//Show/Hide Card Number according to card.showCardNo (true/false) 
		self.fields.cardNumber.visible = card.showCardNo;
	},
	
	_refresh_ : function(data){
		var self = this;
		//--- confirm that refresh request is for me  ---
		if(('undefined' != typeof data.cardID) && (data.cardID == self.data.cardID)){
			self.setValue(data);
		}
	},

	_renderBarcode_ : function(card, style){
		var self = this;
		
		var _views = {};
		//--- Magstripe / No Code placeholder ---
        _views.magstripe = Ti.UI.createView(style.nocode);

		//--- Barcode  --------------------------
		var viewParam = style.barcode;		// style of barcode or qrcode
		
		var isValidCode = (card.barcodeType <= 9);
		if(card.barcodeType > 1 &&  !App.Helper.isASCII(card.number)) isValidCode = false;
		
		if (App.Config.IS_ANDROID) {
			isValidCode = (card.barcodeType <= 9) || (card.barcodeType == 14)
				|| (card.barcodeType == 15) || (card.barcodeType == 16);
		}
		if(!card.number || !card.barcodeType || !isValidCode ){		// FIXME tmp solution for types > 9
			// create placeholder view
			_views.barcode = Ti.UI.createView(viewParam);
			_views.barcode.visible		= false;
			_views.magstripe.visible	= true;
		}else{
			// render actual barcode
			if (App.Config.IS_ANDROID) {
				var scanner = App.loadHelper('Zebra');
				var option = card.barcodeType;
				var code = card.number;
				var valid = scanner.isValidBarcode (code, option);	
				_views.barcode = Ti.UI.createImageView(viewParam);
				Ti.API.info ('CTKCARD='+card.barcodeType);
				if (valid==true) {
					try {
						
						var img = null;
						if (option == 2) {
							//img = self.barcodeModule.encodeCode39(code);
							img = self.barcodeModule.encode(code,"code39");
						} else if (option == 4) {
							//img = self.barcodeModule.encodeCode128(code);
							img = self.barcodeModule.encode(code,"code128");
						} else if (option == 1) {
							//img = self.barcodeModule.encodeQRCode(code);
							img = self.barcodeModule.encode(code,"qrcode");
						} else if (option == 8) {
							//img = self.barcodeModule.encodeEan8(code);
							img = self.barcodeModule.encode(code,"ean8");
						} else if (option == 7) {
							//img = self.barcodeModule.encodeEan13(code);
							img = self.barcodeModule.encode(code,"ean13");
						} else if (option == 5) {
							//img = self.barcodeModule.encodeUpcA(code);
							img = self.barcodeModule.encode(code,"upca");
						} else if (option == 6) {
							//img = self.barcodeModule.encodeUpcE(code);
							img = self.barcodeModule.encode(code,"upce");
						} else if (option == 9) {
	//						img = self.barcodeModule.encodeItf(code);
							img = self.barcodeModule.encode(code,"itf");
						} else if (option == 15) {
							img = self.barcodeModule.encode(code, "codabar");
						} else if (option == 16) {
							img = self.barcodeModule.encode(code, "aztec");
						} else if (option == 14) {
							img = self.barcodeModule.encode(code, "pdf417");
						}
					} catch(err) {	
						Ti.API.info('CTKBAR-Err:'+err+':'+code);					
					}
					_views.barcode.setImage (img);
				}
			} else {
				viewParam.ctype	= card.barcodeType;
				viewParam.code	= card.number;
				_views.barcode = self.barcodeModule.createView(viewParam);
			}
			_views.magstripe.visible	= false;
			_views.barcode.visible		= true;
		}

		return _views;
	},

	_orientHandler_ : function(evt){
		var self = this;

		//--- ignore FaceUP, FaceDown & Unknown events   Ti.UI.FACE_UP, Ti.UI.FACE_DOWN  ---
		if(! (evt.orientation >= 1 && evt.orientation <= 4)){
			return;
		}
		
		self.newOrient = evt.orientation;
		var angle = self.ROTATE[self.currentOrient].angle[self.newOrient];
		var scale = self.ROTATE[self.currentOrient].scale[self.newOrient];
		
		Ti.API.info('Orientation:  current='+self.currentOrient+', new='+self.newOrient+', rotate='+angle);

		if(! self.rotating && self.currentOrient != self.newOrient){
			self.rotating = true;
			
			//---  inform environment that we are leaving current orientation  ---
			if(self.currentOrient == Ti.UI.PORTRAIT || self.currentOrient == Ti.UI.UPSIDE_PORTRAIT){
				self._eventProxy.fireEvent(self._eventProxy.EVENT.outPortrait, {orientation: self.newOrient});
			}else{
				self._eventProxy.fireEvent(self._eventProxy.EVENT.outLandscape, {orientation: self.newOrient});
			}			
			
			self.currentOrient = self.newOrient;
				
			var rot = angle;
			var scale = scale;
			
			switch (self.newOrient){
			case Ti.UI.LANDSCAPE_LEFT:
				var posn = self.STYLE.position.landscapeLeft;
				break;
			case Ti.UI.LANDSCAPE_RIGHT:
				var posn = self.STYLE.position.landscapeRight;
				break;
			default:
				var posn = self.STYLE.position.portrait;
			}

			var matrix = Ti.UI.create2DMatrix();
			matrix = matrix.rotate(rot);
			matrix = matrix.scale(scale, scale);

			var anim = Ti.UI.createAnimation({
				transform	: matrix,
			    duration	: 300,
			    left		: posn.left,
			    top			: posn.top,
			    curve		: Ti.UI.ANIMATION_CURVE_EASE_OUT,
			});
			anim.addEventListener('complete', function(evt){
				self.rotating = false;
				var newOrient = Ti.Gesture.orientation;
				if(newOrient == Ti.UI.FACE_UP)		newOrient = Ti.UI.PORTRAIT;
				
				if(newOrient != self.currentOrient){
					//---  orientation changed again while rotating, let's catch up ---
					Ti.Gesture.fireEvent('orientationchange', { orientation: newOrient });
				}else{
					var card = self.data;
					//---  we are in a rest state, inform environment of new orientation  ---
					if(newOrient == Ti.UI.PORTRAIT || newOrient == Ti.UI.UPSIDE_PORTRAIT){
						if(card.barcodeType > 1) {
							if(self.fields.barcodex) {//barcode only, not qrcode
								self.fields.backView.remove(self.fields.barcodex);			//  remove previous barcode
								self.fields.backView.add(self.fields.barcode);
							}
						}
						self._eventProxy.fireEvent(self._eventProxy.EVENT.inPortrait, {orientation: newOrient});
					}else{
						if(card.barcodeType > 1) {//barcode only, not qrcode
							//--- Barcode & Magstripe  ------------------------
							if(self.fields.barcodex) {
								self.fields.backView.remove(self.fields.barcode);			//  remove previous barcode
								self.fields.backView.add(self.fields.barcodex);
							} else {
								var style = {
									barcode		: self.STYLE.barcodex,
									nocode		: self.STYLE.nocode,
								};
								var codeViews = self.renderBarcode(card, style);
								codeViews.barcode.hide();
								
								var scale = 2/3;
								var matrix = Ti.UI.create2DMatrix();
								matrix = matrix.scale(scale, scale);			
								var anim = Ti.UI.createAnimation({
									transform	: matrix,
								});
								self.fields.barcodex = codeViews.barcode;
								//  remove previous barcode
								self.fields.backView.add(self.fields.barcodex);
								self.fields.barcodex.animate(anim);
								
								setTimeout(function(evt){
									self.fields.backView.remove(self.fields.barcode);
									self.fields.barcodex.show();
								}, 500);
							}
						}
						self._eventProxy.fireEvent(self._eventProxy.EVENT.inLandscape, {orientation: newOrient});		
					}
				}
			});
			
			self.VIEW.animate(anim);
		}
	},

	_highlightExpiryDate_ : function(expiryDate, labelField, displayField, style){
		//---  highlight expiry date if necessary  ---
		labelField.visible = (displayField.text == '') ? false : true;
		if(new Date(Number(expiryDate)*1000) <= new Date()){
			labelField.text = App.Helper.L('card_expired');
			labelField.color = '#79FF0000';
			displayField.color = '#FF0000';
		}else{
			labelField.text = App.Helper.L('card_expire');
			labelField.color = style.label.color; //'#BBBBBB';
			displayField.color = style.date.color; //'#000000';
		}
	},
		
	_codeFormat_ : function(theFormat, theString){
		if(!theString || theString === ''){ 
			return '';
		}
		// if(!theFormat) return util.format('#### #### #### ####', theString);
		if(!theFormat)		return theString;
		
		theString = theString+"";
		var formatLength = theFormat.length;
		var stringLength = theString.length;
		
		var loopLength = formatLength<stringLength ? formatLength : stringLength;
		var result = '';
		var count = 0;
		var countFormat = 0;
		while(true){
			if(count>=stringLength || countFormat>= formatLength) {
				break;
			}
			var formatChar = theFormat.charAt(countFormat++);
			if(formatChar == '#'){ 
				result += theString.charAt(count++);
			}else{
				result += formatChar;
			}
		}
		
		if(stringLength>count){
			result += theString.substring(count);
		}
		return result;	
	},
		
		
		
	_dateString_ : function(dateNum){
		if(Number(dateNum) > 0){
			return App.Moment(new Date(dateNum*1000.0)).format('LL');
		}else{
			return '';
		}		
	},

});

module.exports = WOCardFactory;

//---  End of file: WOCardFactory.js
