/**
 * Provides Card Library & Card services for both Perkd and Merchants
 *
 * @author young@waveo.com
 */
Ti.include('/globals.js');

var WOCardLibManager = App.Class(App.Controller.WOManager, {
    className : 'WOCardLibManager',
    alias     : 'cardLib', 
 
	EVENT		: {
		//--- Handles these events:
		goPerkdIDView		: 'Nav.goPerkdIDView',
		updateCategory		: 'WOCardLibManager.updateCategory',
		//--- Produces these events:
    },

	//---  Constructor  --------------- 
	initialize : function(param){
		var self = this;
		
		self.discoverWinObj = null;
		
		self.base(param);
		self.listenForEvents();
	},

	listenForEvents : function(){
		var self = this;
		
		Ti.App.addEventListener(App.Const.MODEL_CARDM+'.'+$.syncMgr.EVENT.syncUpdate, function(){
			_updateCategory();
		});
		
		Ti.App.addEventListener(self.EVENT.updateCategory, function(){
			_updateCategory();
		});
		
		function _updateCategory(){
			var masters = $.dataMgr.newModel('WOCardMaster');
			var categories = masters.getCategories();
			if(self.discoverWinObj) self.discoverWinObj.updateCategory(categories);
		}
	},

    //=========================  Perkd App Services  ========================
    restore : function(param){
        var self = this;
        
        //later later then signup -> get perkd -> register card -> will popup offer perkd after card list rendered! 
        $.cardMgr.cardListRendered = false;

        //-----------  Setup Data Package  -----------
		var viewParam = {
			navGroupObj		: param.navGroupObj,
		};
        //----  Load & Render View with viewParam ----
 		var winObj = App.Helper.loadView('card/RestoreCard', viewParam);
		var proxy = winObj.getEventProxy();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.cardsRestored, _doCardsRestored);
        //----------  Return Window Object  ---------- 
        winObj.render();
        return winObj;

		//--------------------  Event Handlers  ---------------------
        function _doCardsRestored(){
        	if(!viewParam.navGroupObj) winObj.close();
        	if(param.data.callback){
        		param.data.callback();
        	}else{
	        	Ti.App.fireEvent($.app.EVENT.doStart);
	        	//--- close previous rootWindow when we had a previous login session
	        	//  eg. skipped registration, then login from PerkdID page
				// if($.currentNavGroup){
					// $.currentNavGroup.close({ animated : false });
					// $.currentNavGroup.dispose();
					// $.currentNavGroup = null;
				// }
				// if($.rootWindow){
					// $.rootWindow.close();
					// $.rootWindow.dispose();
					// $.rootWindow = null;
				// }
        	}
        }
    },
	
	//---------------------  Card Master View Controllers  --------------------
	discover : function(param){
		var self = this;

        //-----------  Setup Data Package  -----------
		var masters = $.dataMgr.newModel('WOCardMaster');
		var _categories = masters.getCategories();
		var viewParam = { 
			DATA		: { categories: _categories },
			tabGroupObj	: param.tabGroupObj,
		};
        //----  Load & Render View with viewParam ----
 		var winObj = App.Helper.loadView('discover/DiscoverWindow', viewParam);
 		var proxy = winObj.getEventProxy(self);
        winObj.render();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.viewCategory, _doViewCategory);
		proxy.addEventListener(proxy.EVENT.addCustomCard, _doAddCustomCard);
		proxy.addEventListener(proxy.EVENT.doSearch, _doCardSearch);
		proxy.addEventListener(proxy.EVENT.viewCardMaster, _doViewCardMaster);
		
		self.discoverWinObj = winObj;
		
        //----------  Return rendered View  ---------- 
        return winObj;

		//--------------------  Event Handlers  ------------------------
		function _doViewCategory(evt){
			if(evt.category){
				$.app.route({
					url			: 'cardLib/master',
					tabGroupObj	: param.tabGroupObj,
					tab 		: 1,
					data		: { category: evt.category },
				});
			}
		}
		function _doAddCustomCard(evt){
			var addCardWin = $.app.route({
				url		 	: 'card/addCard',
				tabGroupObj	: param.tabGroupObj,
				tab		 	: 1,
				data	 	: {
					cardMaster: $.dataMgr.get('WOCardMaster', App.Config.CUSTOM_CID),
					authState : $.authMgr.getAuthState(),
				},
			});
			return addCardWin;
		}
		function _doCardSearch(evt){
	    	var keyword = evt.keyword || '';
	    	keyword = keyword.replace(/[\%\_\^\&\<\>]/g,'');
		    keyword = keyword.replace(/\s{2,}/g,' ');
		    if(' ' == keyword.substring(0,1)) keyword = keyword.substring(1, keyword.length);
		    if(' ' == keyword.substring(keyword.length-1)) keyword = keyword.substring(0, keyword.length-1);
		    
		    var searchKeys = [];
			var keywords = keyword.split(' ');
	    	for(var i=0; i<keywords.length-1; i++){
	    		var arr = [];
	    		arr.push(keywords.slice(0, i+1).join(' '));
	    		arr = arr.concat(keywords.slice(i+1));
	    		searchKeys.push(arr);
	    		
	    		var arr = [];
	    		arr = arr.concat(keywords.slice(0, i+1));
	    		arr.push(keywords.slice(i+1).join(' '));
	    		searchKeys.push(arr);
	    	}
	    	searchKeys.pop();
	    	searchKeys.shift();
	    	searchKeys.unshift(keywords);
	    	if(keywords.length>1) searchKeys.push([keyword]);
	    	
	    	Ti.API.error(searchKeys);
	    	
	    	var hasCityKeywords = false;
			var master = $.dataMgr.newModel('WOCardMaster');
			for(var k=0; k<searchKeys.length; k++){
				if(master.hasCityKeywords(searchKeys[k])) {
					hasCityKeywords = true;
					break;
				}
			}
			var searchResult = [];
			for(var k=0; k<searchKeys.length; k++){
				master.searchAll(searchKeys[k], !hasCityKeywords);
				var result = master.p2l();
				searchResult = searchResult.concat(result);
				if(result.length>=30) break;
			}
			
			var cards = [];
			for(var i=0; i<searchResult.length; i++){
				var card = searchResult[i];
				for(var j=0; j<cards.length; j++){
					if(card.id == cards[j].id) {
						card = null;
						break;
					}
				}
				if(card) cards.push(card);
				if(cards.length>=30){
					setTimeout(function(){
						$.app.showHint(App.Helper.L('hint_tooManyCards'), 3000);
						// $.app.notify({ message : 'Too many cards found, only showing first 30.' });
					}, 1000);
					
					break;
				}
			}
			evt.callback(cards);
		}
		function _doViewCardMaster(evt){
			self._doViewCardMaster({
				masterID	: evt.masterID,
				tabGroupObj	: param.tabGroupObj,
			});
		}
	},
	
	/**
	 *  View Category : lists card masters within the given Category
 	 * @param	{string} category		name of Card Category
	 */
	master : function(param){
		var self = this;
        App.Helper.requireModel('WOCardMaster');

        //-----------  Setup Data Package  -----------
		var cardMasters = new App.Model.WOCardMaster();
		cardMasters.fetchCategory(param.data.category);
		var viewParam = { 
			DATA		: {
				category	: App.Helper.L(param.data.category, 'category'),
				cardMasters	: cardMasters.p2l(),
			},
			tabGroupObj	: param.tabGroupObj,
		};
        //----  Load & Render View with viewParam ----
 		var winObj = App.Helper.loadView('discover/MasterListWindow', viewParam);
 		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.viewCardMaster, _doViewCardMaster);
        //----------  Return Window Object  ---------- 
        return winObj;

		//--------------------  Event Handlers  ------------------------        
		function _doViewCardMaster(evt){
			self._doViewCardMaster({
				masterID	: evt.masterID,
				tabGroupObj	: param.tabGroupObj,
			});
		}
	},

	cardMasterIntro : function(param){
		var self = this;
		var _cardMaster = param.data.cardMaster;

        //-----------  Setup Data Package  -----------
		var viewParam = { 
			DATA		: { 
				cardMaster	: _cardMaster,
				authState	: param.data.authState,
			},
			tabGroupObj	: param.tabGroupObj,
		};
        //----  Load & Render View with viewParam ----
 		var winObj = App.Helper.loadView('discover/CardIntroWindow', viewParam);
 		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.addCard, _doAddCard);
        //----------  Return Window Object  ---------- 
        return winObj;

		//--------------------  Event Handlers  ------------------------       
		function _doAddCard(evt){
			if(_cardMaster.isDigital) {
				if(!Ti.Network.online) {
            		$.app.showAlert(App.Helper.L('ALERT_NetworkErrorRequestSignupCard'), App.Helper.L('TITLE_CARDSIGNUP'));
            		return;
        		}
				$.app.showBusy({message:App.Helper.L('hint_wait')});
				self._cardRegisterRequest({
					mid			: _cardMaster.id,
					callback	: _cardRegisterRequested,
					errorback	: function(){
						if(App.Config.IS_ANDROID) {
							Ti.App.fireEvent('SIGNUP_BUTTON_RESET_EVENT');
						}
						$.app.hideBusy();
					},
				});
			} else {
				$.app.route({
					url			: 'card/addCard',
					tabGroupObj	: param.tabGroupObj,
					tab			: 1,
					data		: { 
						cardMaster	: _cardMaster,
						authState	: $.authMgr.getAuthState(),
					},
				});
			}
			
			function _cardRegisterRequested(result) {
				$.app.hideBusy();
				
				if('N' == result['new']){
					$.syncMgr.syncObjects(['WOCard']);
					$.app.showAlert(App.Helper.L('ALERT_CardRestored'), App.Helper.L('TITLE_CARDSIGNUP'));
					$.trackMgr.track(_cardMaster, 'undelete', {
						cardMID		: _cardMaster.id,
						cardID		: result.cardID,
					});
					// need Ti.App.fireEvent('SIGNUP_BUTTON_RESET_EVENT'); ?
				} else {
					
					$.app.route({
						url			: 'card/addCard',
						tabGroupObj	: param.tabGroupObj,
						tab			: 1,
						data		: {
							cardMaster	: _cardMaster,
							form		: result.form.def,
							authState	: $.authMgr.getAuthState(),
							// form		: JSON.decode(result.form.defn),
						},
					});
				}
			}
		}
	},
	
	_cardRegisterRequest : function(param){
	    var self = this;

        var authState = $.authMgr.getAuthState();  
	    var callParam = {
	    	// action : App.Action.ADDCARD,
	    	action : 'card_request_signup',
	    	report : true,
	    	title  : App.Helper.L('TITLE_CARDSIGNUP'),
	    	data:{
	    		mid		: param.mid,
	            token	: authState.token,
	    	},
	    	callback  : param.callback,
	    	errorback : param.errorback
	    };
	    $.syncMgr.callServer(callParam);
	},

	//--------------------  Shared Event Handlers  ------------------------
	_doViewCardMaster : function(param){
		var self = this;
		
		var authState = $.authMgr.getAuthState();
		var cardMaster = $.dataMgr.get('WOCardMaster', param.masterID);
		
		if(cardMaster.appVer && cardMaster.appVer > Ti.App.version) { //need upgrade     [TODO] ZJ
			var dialog = Ti.UI.createAlertDialog({
				title		: App.Helper.L('TITLE_ADDCARD'),
				message		: App.Helper.L('ALERT_RequireNewerPerkd'),
				buttonNames	: [App.Helper.L('BUTTON_CANCEL'), App.Helper.L('BUTTON_UPGRADE')],
			});
			//---  Handle Action button  ---
			dialog.addEventListener('click', function(evt){
				if(1 == evt.index) {
					var config = $.dataMgr.get('WOSettings', 'config');
					var upgradeURL = config['@upgrade'] || 'http://perkd.me/upgrade';
					Ti.Platform.openURL(upgradeURL);
				}
			});
			dialog.show();
			return;
		}
		
		if(cardMaster.hasIntro){
			//--------  Card with introduction page  ------
			var win = $.app.route({
				url			: 'cardLib/cardMasterIntro',
				tabGroupObj	: param.tabGroupObj,
				tab			: 1,
				data		: {
					cardMaster: cardMaster,
					authState 	: authState,
				}
			});
		} else {
			//--------  Card without introduction page  ------
			var win = $.app.route({
				url			: 'card/addCard', 
				tabGroupObj	: param.tabGroupObj,
				tab			: 1,
				data	: {
				    cardMaster	: cardMaster,
				    authState 	: authState,
				},
			});
		}
		if(cardMaster.isDigital && authState.state != App.Const.AUTH_LOGGED_IN) self._showSignupDialog();
		return win;
	},
	
	_showSignupDialog : function() {
		var self = this;
		
		var dialog = Ti.UI.createAlertDialog({
			message:App.Helper.L('popmsg_addDigitalCard'),
			title:App.Helper.L('poptitle_addDigitalCard'),
			buttonNames:[App.Helper.L('BUTTON_CANCEL'), App.Helper.L('BUTTON_SIGNUPNOW')],
		});
		dialog.addEventListener('click', function(evt){
			if(1 == evt.index) Ti.App.fireEvent(self.EVENT.goPerkdIDView);
		});
		dialog.show();
	},

    //========================  Merchants Services  ===========================

    /**
     * Returns Card Master objects that Merchants is authorized to issue
     */
    getIssueCardMasters : function(){
        var self = this;        
        App.Helper.requireModel('WOCardMaster');
        
        var cardMasters = new App.Model.WOCardMaster();
        cardMasters.exCondition = " `deleted`=='N' and `valid`<>'N' ";
        cardMasters.getAll();
        return cardMasters;
    },
    
    getCardIssueForm : function(cardMasterID){
       var self = this;
       App.Helper.requireModel('WOCardMaster');

       //  TODO  [ZJ]:  retrieve form object from card master (replace below)
       //  pls define a standardized definition of Form Object
       var master = new App.Model.WOCardMaster(cardMasterID);
       master.fetch();
       var forms = JSON.parse(master.get('forms'));
       return forms['merchant'];
    },

    issueCard : function(cardInstance){
        var self = this;
        App.Helper.requireModel('WOCardNumber');
        App.Helper.requireModel('WOCardMaster');

        // TODO [ZJ]  please verify the following steps are correct

        //----  Assign Card Number  ----
        var CardNumber = new App.Model.WOCardNumber();
        cardInstance.set('num', CardNumber.getNew(false));
        
        var mid = cardInstance.get('mid');
		var cardMaster = new App.Model.WOCardMaster(mid);
		cardMaster.fetch();
		var expiryTimestampAbs = cardMaster.get('expirydate');
		var expiryDateCalc = cardMaster.get('expirycalc');
		var expiryTimestamp;
		if('' == expiryTimestampAbs) {
			expiryTimestampAbs = 0;
		}
		if('' == expiryDateCalc) {
			expiryTimestamp = expiryTimestampAbs;
		} else {
			expiryDateCalc = JSON.parse(cardMaster.get('expirycalc'));
			var joinDate = new Date();
			var today = new Date();
			var expiryYear;
			var expiryMonth;
			var expiryDay;
			// calc year
			if('join' == expiryDateCalc.year.base) {
				expiryYear = joinDate.getFullYear()+Number(expiryDateCalc.year.value);
			} else if('today' == expiryDateCalc.year.base) {
				expiryYear = today.getFullYear()+Number(expiryDateCalc.year.value);
			} else {
				expiryYear = Number(expiryDateCalc.year.value);
			}
			// calc month
			if('join' == expiryDateCalc.month.base) {
				expiryMonth = joinDate.getMonth()+Number(expiryDateCalc.month.value);
			} else if('today' == expiryDateCalc.month.base) {
				expiryMonth = today.getMonth()+Number(expiryDateCalc.month.value);
			} else {
				expiryMonth = Number(expiryDateCalc.month.value);
			}
			// calc date
			if('join' == expiryDateCalc.date.base) {
				expiryDay = joinDate.getDate()+Number(expiryDateCalc.date.value);
			} else if('today' == expiryDateCalc.date.base) {
				expiryDay = today.getDate()+Number(expiryDateCalc.date.value);
			} else {
				expiryDay = Number(expiryDateCalc.date.value);
			}
	        //----  Calculate Join & Expiry Dates  ----
			var expiryDate = new Date(expiryYear, expiryMonth, expiryDay);
			expiryTimestamp = Math.floor(expiryDate.getTime()/1000)-1;
			// compare with absolute expiry date timestamp
			if(expiryTimestampAbs > 0 && expiryTimestamp > expiryTimestampAbs) {
				expiryTimestamp = expiryTimestampAbs;
			}
		}
		
        cardInstance.set('join', App.Helper.getTimeStamp());      
        cardInstance.set('expiry', expiryTimestamp);
        cardInstance.set('issued_uid', Ti.App.Properties.getString('loginUserId', 0));
        cardInstance.save();

        //----  Sync card instance&number asychronously  -----
        setTimeout(function(){
            $.syncMgr.syncObjects(['WOCard', 'WOCardNumber']);
        }, 0);

        cardInstance.fetchSubObjects();   // make sure we return a complete instance, fetch Card Master, Offers, etc

        return cardInstance;
    },    
});

module.exports = WOCardLibManager;

/**
 * End of file: WOCardLibManager.js
 */