/**
 * Factory of Form Field Widgets, ie. mini-controllers with pre-built views
 * 
 * @author young@waveo.com
 */
Ti.include('/globals.js');

var helper = App.loadHelper('default'); //TODO [ZJ]

var WOFieldFactory = App.Class(App.WOFactory, {
	className : 'WOFieldFactory',

	//--- All widgets produced by this factory will deal with these events:
	WIDGET_EVENT	: {
		//--- Handles these events:
		refresh			: 'WOFieldFactory.refresh',
		setMode			: 'WOFieldFactory.setMode',
		reset			: 'WOFieldFactory.reset',
		blur			: 'WOFieldFactory.blur',
		dispose			: 'WOFieldFactory.dispose',
		update			: 'WOFieldFactory.update',
		//--- Produces these events:
		click			: 'WOFieldFactory.click',
		change			: 'WOFieldFactory.change',
    },
	
	MODES	: ['display', 'edit'],
	
	//---  Constructor  --------------- 
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		param.i18n = param.i18n || false;
		if(param.i18n){
			var langID = App.Config.LanguagesTag[App.Config.LANG];
			i18n(param, langID);
		}
		
		//Locale language
		if('label' in param.properties && 'undefined' != typeof param.properties.label.value) param.properties.label.value = App.Helper.L(param.properties.label.value, 'fieldLabel');
		if('hint' in param.properties && 'undefined' != typeof param.properties.hint.value) param.properties.hint.value = App.Helper.L(param.properties.hint.value, 'fieldLabel');
		if('fieldView' in param.properties && 'undefined' != typeof param.properties.fieldView.title) param.properties.fieldView.title = App.Helper.L(param.properties.fieldView.title, 'fieldLabel');
		
		var widgetParam = self.base(param, data);
		
		//--- Extract Widget Definition ---
		self.hasSubview = ('undefined' != typeof param.hasSubview) ? param.hasSubview : false;
		
		return widgetParam;
	},

	_getWidgetPrototype : function(){

		//----------------  Widget object constructor  -------------
		//  IMPORTANT:  MUST NOT put this as a class method! JClass will interfere

		return function F(param){
			var self = this;

			self.id		= param.id;
			self.name	= param.name;
			self.data	= param.data;
			self.modes	= param.modes;
			self.VIEW	= param._view;
			self._row	= param._row;

			self.EVENT = WOFieldFactory.WIDGET_EVENT;
			self._eventProxy = param._eventProxy;

			self.currentMode = param.defaultMode;
			self.hasSubview = param.hasSubview;
			
			self.rapidFire = new Date();
		};
	},
	
	_attachMethods : function(_Widget_){
		var self = this;	
		self.base(_Widget_);

		//---  attach object behaviors  ---
		if(self.widgetParam.hasSubview){
			_Widget_.prototype.renderFieldView = self._renderFieldView_;
		}
		
		_Widget_.prototype.doChanged = self._doChanged_;
		_Widget_.prototype.reset = self._reset_;
		_Widget_.prototype.update = self._update_;
	},
	
	//------------  Template/Prototype Methods for Field object  -----------
	_setupEventHandlers_ : function(){
		var self = this;
		self.base();

		//----------------- Android-conditional probably can combine. But must test ----------
		if (App.Config.IS_ANDROID) {
			self._eventProxy.addEventListener(self._eventProxy.EVENT.reset, (function(that){
				return (function(){
					that.reset();
				});
			})(self));
			self._eventProxy.addEventListener(self._eventProxy.EVENT.blur, (function(that){
				return (function(){
					that.blur();
				});
			})(self));
			self._eventProxy.addEventListener(self._eventProxy.EVENT.dispose, (function(that){
				return (function(){
					that.dispose();
				});
			})(self));
		} else {
			self._eventProxy.addEventListener(self._eventProxy.EVENT.reset, function(evt){
				self.reset();
			});
			self._eventProxy.addEventListener(self._eventProxy.EVENT.blur, function(evt){
				self.blur();
			});
			self._eventProxy.addEventListener(self._eventProxy.EVENT.dispose, function(evt){
				self.dispose();
			});
			self._eventProxy.addEventListener(self._eventProxy.EVENT.update, function(evt){
				self.update(evt.data);
			});			
		}
	},
	
	_setMode_ : function(mode){
		var self = this;
		
		self.base(mode);
		
		switch (mode){
		case 'display':
			self.VIEW.touchEnabled = false;
			if(self._row){	
				self._row.backgroundColor ='#16000000';
			}
			break;
		case 'edit':
			self.VIEW.touchEnabled = true;
			if(self._row){
				self._row.backgroundColor ='#53000000';
			}
			break;
		}
	},

	_doClick_ : function(evt){
		var self = this;
		
		self.base(evt);
		//---  request for window & render subview ---
		if(self.hasSubview){
			var now = new Date();
			if(now - self.rapidFire > 1000){		// black-out interval between clicks
				//---  Perform Click ---
				self._eventProxy.fireEvent(self._eventProxy.EVENT.openWindow, {
					callback	: function(result){
						self.renderFieldView(result);
					},
				});
			}
			self.rapidFire = now;
		}
	},


	_doChanged_ : function(evt){
		var self = this;
		self._eventProxy.fireEvent(self._eventProxy.EVENT.change, {
			field	: self,
			value	: self.getValue(),
		});
	},

	_renderFieldView_ : function(myWindow){
		Ti.API.error(this.className+'::_renderFieldView_ - not implemented');		
	},
	
	//--- if not use self.data.value, subclasses MUST implement these methods  --------
	_reset_ : function() {
		var self = this;
		self.setValue(self.data.value);
	},
	
	_update_ : function() {
		var self = this;
	},
});

function i18n(object, lang) {
	for(var key in object) {
		var langKey = key+lang;
		if('undefined' != typeof object[langKey]) object[key] = object[langKey];
		else if(helper.isObject(object[key]) || helper.isArray(object[key])) arguments.callee(object[key], lang);
	}
}

module.exports = WOFieldFactory;

//---  End of file: WOFieldFactory.js
