/**
 * Base class of main Core Controllers: ApplicationMgr, SyncMgr, NotificationMgr & other app Controllers
 * 
 * @author young@waveo.com
 */
Ti.include('/globals.js');
if (App.Config.IS_ANDROID) {
	Ti.include('/waveo/core/WOAndroidUtils.js');
}

var WOManager = App.Class(App.WOBaseClass, {
	className : 'WOManager',
	//  add 'alias' property to make controller object Memory Resident, used by route() of ApplicationManager

	//---  Constructor  ---------------    	
	initialize : function(param){
		var self = this;
		self.base(param);
		
		self.isListenEvents = false;
		self.inProgress		= {};			// 'array' of states for manager to prevent re-entrant calls
											//  eg. inProgess.sync = true;   => sync manager is already syncing
		self.loadingWin = null;
		self.isBusy = false;
	},
	
	dispose : function(){
		var self = this;

		if(('alias' in self) && ('alias' in App.ControllerAlias)){
			//--- remove myself from the alias lookup table to be ready for Garbage Collection
			delete App.ControllerAlias[self.alias];
		}
	},
	
	/**
	 * hook up event handlers here, called after init() where appropriate
	 */
	listenForEvents : function(){
		var self = this;
		
		/* IMPORTANT:  event handler methods MUST be wrapped within an anonymous function
			otherwise, 'this' within the event handler method will NOT set correctly

				eg.	Ti.App.addEventListener(eventName, function(){
						self.eventHandlerMethod();
					});
		*/			
		
		self.isListenEvents = true;
	},
	
	showHint : function(message, delay){
		var self = this;
		
		delay = delay || 3000;
		if(self.showingHint){ 
			self.hintMessage.text = message;
			return;
		}
		if(self.hintWin == null){
			
			if (App.Config.IS_ANDROID) {
				self.hintWin = createAndroidWindow({	});
				self.hintWin.hideNavBar();
			} else {
				self.hintWin = Ti.UI.createWindow({	});
			}
			
			var viewBg = Ti.UI.createView({
				width:'80%', height:Ti.UI.SIZE,//App.Config.IS_ANDROID?'160dp':'120dp',
				borderRadius: App.Config.IS_ANDROID? 10*Titanium.Platform.displayCaps.dpi/160 : 10,
				backgroundColor:'#C7000000',
				layout: 'composite',
			});
			
			self.hintMessage = Ti.UI.createLabel({
				color:'#FFFFFF', textAlign:'center',
				font:{fontSize:'21sp', fontFamily:'Melbourne-Bold'},
				height:Ti.UI.SIZE,
				left:'10dp', right:'10dp', top:'15dp', bottom:'15dp'
			});
			viewBg.add(self.hintMessage);
			self.hintWin.add(viewBg);
		}
		
		self.hintMessage.text = message;
		self.hintWin.open();
		self.showingHint = true;
		
		setTimeout(function(){
			if(self.showingHint){
				self.hintWin.close();
			}
			self.showingHint = false;
		}, delay);
	},
	
	showBusy : function(param){
		var self = this;
		
		// CTK - activity indicator usage different for android and iphone.
		/*
		if (App.Config.IS_ANDROID){	// 2.1.4	
			if (self.loadingWin===null){
				self.loadingWin = Ti.UI.createActivityIndicator({});
			}
			if (self.isBusy){
				self.loadingWin.hide();				
			}
			self.loadingWin.message = param.message;
			self.loadingWin.show();
			self.isBusy=true;
			return;			
		}
		*/	
		
		
		if(self.isBusy){ 
			self.labelMessage.text = param.message;
			return;
		}
		if(self.loadingWin == null){
			
			if (App.Config.IS_ANDROID) {
				self.loadingWin = createAndroidWindow({	});
				self.loadingWin.hideNavBar();
			} else {
				self.loadingWin = Ti.UI.createWindow({	});
			}
			
			var viewBg = Ti.UI.createView({
				width:'200dp', height:App.Config.IS_ANDROID?'160dp':'120dp',
				borderRadius: App.Config.IS_ANDROID? 15*Titanium.Platform.displayCaps.dpi/160 : 15,
				backgroundColor:'#C7000000',
				layout: 'composite',
			});
			
			self.labelMessage = Ti.UI.createLabel({
				color:'#FFFFFF', textAlign:'center',
				font:{fontSize:'21sp', fontFamily:'Melbourne-Bold'},
				minimumFontSize:'15sp',
				height:Ti.UI.SIZE,
				left:'10dp', right:'10dp', bottom:'20dp',
			});
			self.intIcon = Titanium.UI.createActivityIndicator({
				top:'25dp',
				style: App.Config.IS_ANDROID ? Ti.UI.ActivityIndicatorStyle.BIG : Titanium.UI.iPhone.ActivityIndicatorStyle.BIG,
			});
			self.intIcon.show();
			viewBg.add(self.intIcon);
			viewBg.add(self.labelMessage);
			self.loadingWin.add(viewBg);
		}
		
		self.labelMessage.text = param.message;
		self.intIcon.show();
		self.loadingWin.open();
		self.isBusy = true;
	},
	
	hideBusy : function(){
		var self = this;

		/*
		if (App.Config.IS_ANDROID) { // 2.1.4
			if (self.loadingWin===null){
				return;
			}
			if (self.isBusy){
				self.loadingWin.hide();
			}
			self.isBusy = false;
			return;
		}
		*/

		if(self.isBusy){ 
			self.loadingWin.close();
		}
		self.isBusy = false;
	},
	
	showAlert : function(message, title){
		var _alertDialog = Ti.UI.createAlertDialog({
			title : title,
			message : message,
			ok : App.Helper.L('BUTTON_OK'),
		});
		_alertDialog.show();
	},
	
	callServer : function(param){
		var self = this;
		var ajaxParam = {};
		
		if(!param.url){
			ajaxParam.url = Ti.App.Properties.getString('defaultAPI', '');
		} else {
			ajaxParam.url = param.url;
		}
		
		if('' == ajaxParam.url) {// TODO [ZJ]
			// $.app.showAlert('API Config Error', 'Merchant');
			// return;
			ajaxParam.url = App.Config.API;
		}
		
		if(!param.method){
			ajaxParam.method = 'POST';
		} else {
			ajaxParam.method = param.method;
		}
		
		ajaxParam['report']       	= param.report;
	    ajaxParam['data']       	= param.data;
	    ajaxParam['data']['action'] = param.action+'_v2';
        ajaxParam['data']['apiv']   = App.Config.API_VERSION;
        ajaxParam['data']['rqsid']  = Math.round(new Date().valueOf());  // unique request id. 
        ajaxParam['data']['lang']   = App.Config.LANG; //Ti.Locale.currentLanguage;
        
        var authState = $.authMgr.getAuthState();
        ajaxParam['data']['xtoken']  = authState.token;
        
        var callstarted = Math.round(new Date().valueOf());

	    ajaxParam['callback']   = function(r){
	    	var callbacked = Math.round(new Date().valueOf());
	    	var callUsed = callbacked - callstarted;
	    	Ti.API.error('*********************************** Request ID: '+ajaxParam['data']['rqsid']+',  CallServer: '+ajaxParam['data']['action']+', Used: '+callUsed+' milliseconds ***********************************');
	    	
	    	// Error handle
//			if(r.result && r.result.code != 0){
			if(r.flag != 'Y'){
				if(param.report){
					if(!r) r = {};
					var code 	= r.code;        // r.result.code
					if(!code){
						var message = 'Unknown Error';
					}else{
						var message = self.getErrorString(code);
					}
			    		
		    		var title 	= param.title || false;
		    		title ? $.app.showAlert(message, title) : $.app.showAlert(message);
				}
				if(param.errorback) param.errorback(r);
        		return;
			}
			if(param.callback) param.callback(r);
	    }
	    ajaxParam['errorback'] = param.errorback;
	    
	    Ti.API.error('*********************************** Request ID: '+ajaxParam['data']['rqsid']+',  CallServer: '+ajaxParam['data']['action']+' ***********************************');
	    Ti.API.error('Start at: '+(new Date()).toString());
	    Ti.API.error('Parameters:');
	    Ti.API.error(ajaxParam);
	    
	    App.Helper.ajax(ajaxParam);
	},

	getErrorString : function(errorCode, param){
		var self = this;
		var key = 'err_'+errorCode;
		var string = self._getLangStr(key, param);
		if(string == key || string == key+'_TW' || string == key+'_HK') {
			return errorCode;
		} else {
			return string;
		}
	},
	
	_getLangStr : function(key, param){
		if(param){
			return result = String.format(App.Helper.L(key), param);
		}else{
			return result = App.Helper.L(key);
		}
	},
	

	// curl : function(param){
		// var ajaxParam = {
			// url 		: param.url,
			// method		: param.method || 'POST',
			// data		: param.data,
			// report		: param.report,
			// callback	: param.callback,
			// errorback	: param.errorback,
		// }
		// App.Helper.ajax(ajaxParam);
	// },

});

module.exports = WOManager;

//---  End of file: WOManager.js
