/**
 * WONotificationManager:
 *
 * @author young@waveo.com
 */
Ti.include('/globals.js');
var helper = App.loadHelper('default'); //TODO [ZJ]

var WONotificationManager = App.Class(App.Controller.WOManager, {
	className	: 'WONotificationManager',
	alias		: 'notify',

	EVENT		: {
		//--- handles these events:
		doGetPushPermission	: 'notify.getPushPermission',
		doSetupReminders	: 'notify.doSetupReminders',
		doMessages			: 'notify.doMessages',
		doPerkd				: 'notify.doPerkd',
		doNotifyCallback	: 'notify.doNotifyCallback',
		messagesUpdated		: 'WOMessage.sync.update',

		//--- produces these events:
		ready				: 'notify.ready',
		pushEnabled			: 'notify.pushEnabled',
		localNotifyEnabled	: 'notify.localNotifyEnabled',
		deviceTokenUpdated	: 'notify.deviceTokenUpdated',		// mirrored in SyncMgr
		messageReceived		: 'notify.messageReceived',
    },

	TYPE	: {
		//---  Local Notifications  ---
		reminder	: 1,
		//---  Push Notifications  ----
		pushMsg		: 10,
		//---  Perk Message  ----------
		perkMsg		: 11,
		//--- Pull Notifications  -----
		pullMsg		: 20,
		perkMail	: 21,			// not implemented
	},
	
	SUBTYPE		: {
		reminder	: {
		},
		perkd		: {
			issueCard	: 'perkd_card',
			issueOffer	: 'perkd_offer',
			issueStamp	: 'perkd_stamp',
		},
		perkMail	: {
		},
	},

	PUSH	: {
		type	: 't',
		data	: 'd',
		options	: 'o',
	},
	
	DATA	: {
		cardID	: 'cid',
		perkMsg	: 'pid',
		URL		: 'url',
		buttons	: 'btn',
	},
	
	OPTION	: {
		reset	: 8,		// not implemented
		sync	: 4,
		echo	: 2,		// override skipping over of startup msg, ie. will echo msg already seen outside app
		mute	: 1,		// supress dialog if received in app
	},
	
	ACTION	: {
		viewCard	: 'cardView',
		viewPerk	: 'perkView',
		editCard	: 'cardEdit',
		viewOffer	: 'offerView',
		viewReward	: 'rewardsetView',
		gotoURL		: 'go2URL',
	},

	CALLBACKtag		: 'perkMsgCallback',
	
	PUSH_State	: {
		notGranted	: 0,
		optIN		: 1,
		optOUT		: -1,
	},

	//---  Constructor  ---------------	
	initialize : function(param){
		var self = this;
		self.base(param);
		
		// self.UrbanAirship = require('ti.urbanairship');
		
		self.notifyLaunchedApp = true;				// for detecting if app launch was trigger by notification
		self.pushRegistered = false;
		self.inProgress.getPermission = false;		// whether is in state of requesting Push Permission
		self.isShowingMsg = false;					// in the midst of displaying a Message
		
		self.isActive = false;						// IMPORTANT: only start displaying Messages when Active
		
		self.listenForEvents();
		
		var notifyModule = App.Config.IS_ANDROID ? 'bencoding.alarmmanager' : 'bencoding.localnotify';
        self.notify = require(notifyModule);
		
        Ti.App.fireEvent(self.EVENT.ready);		//------ We're Ready! -------
	},

	listenForEvents : function(){
		var self = this;

		if(self.isListenEvents)		return;

        //--- doSetupReminders -----
        Ti.App.addEventListener(self.EVENT.doSetupReminders, function(evt){
        	self.setupReminders(evt);
       	});
        //--- doGetPushPermission -----
        Ti.App.addEventListener(self.EVENT.doGetPushPermission, function(evt){
        	self.getPushPermission({ isInterrupt : true });
       	});
        //--- doMessages -----
        Ti.App.addEventListener(self.EVENT.doMessages, function(evt){
        	self.showNextMessage(evt);
       	});

        //--- messageReceived -----
        Ti.App.addEventListener(self.EVENT.messageReceived, function(evt){
        	self.showNextMessage(evt);
       	});
		//--- messagesUpdated  (SyncMgr event) ---
		Ti.App.addEventListener(self.EVENT.messagesUpdated, function(evt){
			self.showNextMessage(evt);
		});
      	       	
        //--- doPerkd -----
        Ti.App.addEventListener(self.EVENT.doPerkd, function(evt){
        	self.showNextMessage({ msgID: evt.msgID, callback: evt.callback });
       	});
       	//--- doNotifyCallback  (Private) ----
        Ti.App.addEventListener(self.EVENT.doNotifyCallback, function(evt){
        	self._notificationCallback(evt.data);
       	});

		//--- ready  (System event) ---
		Ti.App.addEventListener($.app.EVENT.ready, function(evt){
			self.isActive = true;
			setTimeout(function(e){
				self.notifyLaunchedApp = false;
			}, 1000);
		});
		//--- resume  (System event) ---
		Ti.App.addEventListener($.app.EVENT.resume, function(evt){
			self.notifyLaunchedApp = true;
			self.isShowingMsg = false;
		});
		
		self.isListenEvents = true;
	},

	/**
	 * Manage & display Messages
	 * @param	{object}		param
	 * 			msgID			if provided, will be used instead of getting next msg
	 * 			msgTypes		message types to select for display, one {number} or more {array}
	 * 			callback		(optional) function to call with results, after completion
	 * 	NOTE: pass in EITHER msgID OR msgTypes, not both
	 */
	showNextMessage : function(param){
		var self = this;

		//---  Check if we should show Message now  ------------
		
		if(!self.isActive){
			Ti.API.warn('showNextMessage: is INACTIVE');
			return;
		}else if(self.isShowingMsg || !$.app.canDisturb() || !$.cardMgr.cardListRendered){
			Ti.API.warn('showNextMessage'+(self.isShowingMsg ? ': is BUSY' : ': in DND'));
			// setTimeout(function() {
				// Ti.App.fireEvent(self.EVENT.doMessages);
			// }, 3000);
			return false;
		}
		self.isShowingMsg = true;
		
		// App.loadHelper('playSound').play('scan');
		Ti.API.warn('showNextMessage >>>>>>>');
		
		//--- Extract MessageObj or MsgType from param  ---------
		var messageObj = false;
		var msgTypes = [self.TYPE.reminder, self.TYPE.pushMsg];		// Priority for startup Push & Reminder

		if(param && 'undefined' != typeof param.msgID){
			messageObj = $.dataMgr.newModel('WOMessage', param.msgID);	// message ID provided
			messageObj.fetch();
		}else if(param && 'undefined' != typeof param.msgTypes){
			msgTypes = param.msgTypes;			// message TYPE provided
		}
		
		if(! messageObj){
			//---  handle Startup messages first  ---
			messageObj = self._getStartupMessageObj(msgTypes);
			if(! messageObj){
				messageObj = self._getNextMessageObj();
			}
			if(! messageObj){		//  no unread message in queue
				self.isShowingMsg = false;
				Ti.API.warn('<<<<<<<< NO messages');
				return false;
			}
		}

		//---  Show Message  -------------------------------------
		var message = messageObj.p2l();
		Ti.API.warn('******* Showing message: '+JSON.stringify(message));

		self._bindButtonActions(message);		// modifies 'message' object

		switch(message.type){
		//---  Push Message  ---------------
		case self.TYPE.pushMsg:
			var actionHandled = self._show_pushMsg(message, _doMessageDone);
			break;
		//---  Perk Message  ---------------
		case self.TYPE.perkMsg:
			var actionHandled = self._show_perkMsg(message, _doMessageDone);
			break;
		//---  Reminder Message  -----------
		case self.TYPE.reminder:
			var actionHandled = self._show_reminder(message, _doMessageDone);
			break;
		//---  Pull Message  ---------------
		case self.TYPE.pullMsg:
			var actionHandled = self._show_pullMsg(message, _doMessageDone);
			break;
		//-------------  Perk Mail  -----------------
		case self.TYPE.perkMail:
			var actionHandled = self._show_perkMail(message, _doMessageDone);
			break;

		default:
			$.watch.dog(self, 'showNextMessage', 'unrecognized Message Type: '+message.type);
		}

		//----  Perform Action  ------------------------------------
		// if not handled within dialog, eg. mute mode
		if(! actionHandled && message.action.length){
			// NOTE:  Action ALWAYS associated with last (rightmost) button
			var i = message.action.length-1;

			//--- Notification callback to Server  ---
			if('undefined' != typeof message.action[i].op){
				Ti.App.fireEvent(self.EVENT.doNotifyCallback, {
					data: { msgID: message.id,  op: message.action[i].op }
				});
			}
			//---  perform Action  -------------------
			if(message.action[i].method){
				message.action[i].method(message.action[i].data);
			}

			//---  Message Read & not busy now  ------
			_doMessageDone({ hasRead: true });
			self.isShowingMsg = false;
		}

		return true;

		//---  Post-message Read Callback ------------
		// 	result =	{ hasRead: {boolean} }

		function _doMessageDone(result){
			if(result.hasRead){
				messageObj.updateStatus(messageObj.STATUS.read);
				Ti.UI.iPhone.appBadge = null;
			}
			self.isShowingMsg = false;
			$.app.doNotDisturb(false);

			//---  Show next Unread Message after a pause  ---
			if(self._countUnreadMessage()>0 && $.app.isStarted()){
				setTimeout(function(evt){
					Ti.App.fireEvent(self.EVENT.doMessages);
				}, 3000);
			}

			//---  make callback, { hasRead: {boolean } ----
			if(param.callback)		param.callback(result);
		}
	},

	_bindButtonActions : function(message){
		var self = this;

		//---  Bind Button Actions to actual functions  ---
		for(var i=0; i<message.action.length; i++){
			
			switch(message.action[i].name){
			case self.ACTION.viewCard:
				message.action[i].method = _goViewCard;
				break;
			case self.ACTION.editCard:
				message.action[i].method = _goEditCard;
				break;
			case self.ACTION.viewOffer:
				message.action[i].method = _goViewOffer;
				break;
			case self.ACTION.viewReward:
				message.action[i].method = _goViewReward;
				break;
			case self.ACTION.gotoURL:
				// substitute URL if necc
				message.action[i].data.url = _substituteURL(message.action[i].data.url);
				message.action[i].method = _gotoURL;
				break;
			default:
				message.action[i].method = null;
			}
		}

		//--------------  Private Methods  -------------
		function _substituteURL(url){
			var newURL = url;
			if(url.search('://')==-1){
				newURL = $.configMgr.get(url, '');
			}
			return newURL;
		}
		//---------------  Actions  --------------
		function _goViewCard(data){
			Ti.App.fireEvent($.cardMgr.EVENT.goCardView, { cardID: data.cardID });
		}
		function _goEditCard(data){
			Ti.App.fireEvent($.cardMgr.EVENT.goCardView, { cardID: data.cardID });
		}
		function _goViewOffer(data){
			Ti.App.fireEvent($.cardMgr.EVENT.goCardView, { cardID: data.cardID });
		}
		function _goViewReward(data){
			Ti.App.fireEvent($.cardMgr.EVENT.goCardView, { cardID: data.cardID });
		}
		function _gotoURL(data){
			Ti.Platform.openURL(data.url);
		}
	},
		
	_showMessageDialog : function(message, callback){
		var self = this;

		var param = {
			title		: message.content.title,
			message		: message.content.alertBody,
			action		: message.action,				// array of {label, name, op }
			msgID		: message.id,
			callback	: callback,
			// return {result} of dialog to callback (result.hasRead = true => msg read, false => cancel)
		};

		//---  setup buttons  ---
		if('undefined' == typeof param.action){
			var _buttons = [App.Helper.L('BUTTON_OK')];
		}else{
			var _buttons = [];
			for(var i=0; i<param.action.length; i++) _buttons.push(App.Helper.L(param.action[i].label, 'notifyButton'));
		}

		var dialog = Ti.UI.createAlertDialog({
			title		: param.title,
			message		: param.message,
			buttonNames	: _buttons,
		});

		//---  Handle Action button  ---
		dialog.addEventListener('click', function(evt){
			//--- make Notification callback to Server if required  ---
			if('undefined' != typeof param.action[evt.index].op){
				Ti.App.fireEvent(self.EVENT.doNotifyCallback, {
					data: { msgID: param.msgID, op: param.action[evt.index].op }
				});
			}
			//---  perform Action  ---
			if(param.action[evt.index].method){
				param.action[evt.index].method(param.action[evt.index].data);
			}
			//--- do Callback with read status -------
			if(param.callback)		param.callback({ hasRead: true });
		});

		//---  Display Alert Dialog  ---------
		var sound = ('undefined' != typeof param.sound) ? param.sound : 'notify';
		App.loadHelper('playSound').play(sound);
		dialog.show();
	},

	//--------------------------  Views for Message Types  -------------------------
	_show_pushMsg : function(message, callback){
		//---  Sub-class to implement  ------
		return true;	// true => action handled within, false => need post-dialog action (mute)
	},

	_show_perkMsg : function(message, callback){
		//---  Sub-class to implement  ------
		return true;
	},

	_show_reminder : function(message, callback){
		//---  Sub-class to implement  ------
		return true;
	},
	
	_show_pullMsg : function(message, callback){
		//---  Sub-class to implement  ------
		return true;
	},

	_show_perkMail : function(message, callback){
		//---  Sub-class to implement  ------
		return true;
	},

    //----------------------------  View Controllers  -------------------------------
    pushRequest : function(param){
        var self = this;

        //-----------  Setup Data Package  -----------
        var viewParam = {
        	navGroupObj		: param.navGroupObj,
        };	
        //----  Load & Render View with viewParam ----
		var winObj = App.Helper.loadView('notify/PushRequestWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.requestDone, _doRequestDone);
        //----------  Return Window Object  ---------- 
        return winObj;
        
		//-------------------  Event Handlers  -------------------        
        function _doRequestDone(evt){
            if(param.data.close){
                winObj.close({ animated : true });
                $.app.doNotDisturb(false);
            }
            if(param.data.callback)		param.data.callback();
        }
    },

    //-----------------------  Remote Notification Services  -----------------------
	/**
	 * supports 2 UI modes:  1. startup (navi), 2. interrupt (flip)
	 * @param	{object}		abstract parameter object
	 * 			isInterrupt		boolean (optional), when absent = false = startup mode
	 * 			navGroupObj		only when isInterrupt = false
	 * 			callback		function
	 */
    getPushPermission : function(param){
        var self = this;
		param = param || {};

        if(!self.inProgress.getPermission && !self.remoteNotifyEnabled()){
            //---  request Push Notification permission  ---
            self.inProgress.getPermission = true;
            var _effect = '';
            var _closeWin = false;
			var interruptMode = ('undefined'==typeof param.isInterrupt) ? false : param.isInterrupt;

			if(interruptMode){
				if($.app.canDisturb()){
					$.app.doNotDisturb(true);
					_effect = App.Const.WINDOW.effect.flip;
					_closeWin = true;
				} else {
					self.inProgress.getPermission = false;
					return;		// app not to be disturbed
				}
			}
            $.app.route({
                url     	: 'notify/pushRequest',
                navGroupObj	: param.navGroupObj,
                effect		: _effect,
                data    	: {
                	callback	: function(){
                		self.inProgress.getPermission = false;
                		if(param.callback)    param.callback();
                	},
                	errorback	: function(){
                		self.inProgress.getPermission = false;
                		$.app.doNotDisturb(false);
                	},
                close		: _closeWin,
                },
            });
        }else{
            if(param.callback)		param.callback();
        }
    },

	remoteNotifyEnabled : function(param){
		var self = this;
		
		if(App.Config.IS_SIMULATOR)     return true;

		var isEnabled = false;
		if(App.Config.IS_ANDROID){
			isEnabled = true;
			self.checkRemoteNotify();
		}else if(Ti.Network.remoteNotificationsEnabled){
			if(App.Helper.inArray(Ti.Network.NOTIFICATION_TYPE_ALERT, Ti.Network.remoteNotificationTypes)){
				isEnabled = true;
				self.checkRemoteNotify();
			}
		}
		return isEnabled;
	},
	
	/**
	 * setup callback to listen for Remote Notifications & update Device Token if necessary
	 */
	checkRemoteNotify : function(){
		var self = this;

		if(self.pushRegistered) return;
		
		if(App.Config.IS_ANDROID){
			var gcm = require('com.activate.gcm');
			gcm.registerC2dm({
				success		: function(evt){
					_updateDeviceToken({ deviceToken: evt.registrationId });
				},
				error		: _deviceTokenError,
				callback	: _doHandlePush,
			});
		}else{
			Ti.Network.registerForPushNotifications({
				types		: [
					Ti.Network.NOTIFICATION_TYPE_BADGE,
					Ti.Network.NOTIFICATION_TYPE_ALERT,
					Ti.Network.NOTIFICATION_TYPE_SOUND,
				],
				success		: _updateDeviceToken,
				callback	: _doHandlePush,
				error		: _deviceTokenError,
			});
		}
		
		//--------------  Callback Handlers  --------------
		function _updateDeviceToken(result){
			self.pushRegistered = true;
			
			var newToken = result.deviceToken;
			if(newToken == '' || newToken==null){
				$.watch.dog(self, 'checkRemoteNotify', 'device token is null');
				return;
			}

			// if(!App.Config.IS_ANDROID) self.UrbanAirship.registerDevice(newToken);
			
			Ti.App.fireEvent(self.EVENT.pushEnabled);
			
			var currentToken = $.configMgr.getDeviceToken();
			if(newToken != currentToken){
				//--- store new token ---
				$.configMgr.setDeviceToken(newToken);
				Ti.App.fireEvent(self.EVENT.deviceTokenUpdated, { deviceToken: newToken });
			}
		}
		function _deviceTokenError(result){
			$.watch.dog(self, 'checkRemoteNotify', 'failed to register for Push Notification ('+result.error+')');
		}
		function _doHandlePush(result){
			// self.UrbanAirship.handleNotification(result.data);
			self.handlePushNotification(result.data);
		}
	},

	/**
	 * Remote Push Notifications callback handler, called by OS when push messages are received
	 * @param {Object}	param		abstract data object  (OS dependent)
	 */
	handlePushNotification : function(param){
		var self = this;

// App.loadHelper('playSound').play('notify');

		param.isStartUp = self.notifyLaunchedApp;	// safe to modify system object ?
		self.notifyLaunchedApp = false;		// only true for 1st message, false for all subsequent

		var result = self._createMessageFromPush(param);
		self._postPushMessageProcess(result.message);
		Ti.App.fireEvent(self.EVENT.messageReceived, { messageID: result.msgID });

		Ti.API.warn('REMOTE received '+(param.isStartUp ? '<STARTUP>' : '')+': '+JSON.stringify(param));
	},

	//---  translate Push message & save to Message Queue  ---
	/**
	 * @param {Object}	param		abstract data object
	 * 					alert (iOS)	{string} message content, field name is 'message' for Android
	 * 					sound		{string} sound to play
	 */
	_createMessageFromPush : function(param){
		var self = this;
		
		var result = {};
		result.message = $.dataMgr.newModel('WOMessage');
		//---  sub-class to implement actual method to extract message data from Push Msg
		result.msgID = message.save();
		return result;
	},

	_postPushMessageProcess : function(message){
		//--- Perform any post-receive Actions here  ---
	},

	//------------------------  Local Notification Services  ------------------------
	
	/**
	 * setup callback to listen for Local Notifications
	 */
	checkLocalNotify : function(){
		var self = this;

		if (App.Config.IS_ANDROID){
			//CTK
		}else{
			Ti.App.iOS.addEventListener('notification',function(evt){
				self.handleLocalNotification(evt);
			});
			Ti.App.fireEvent(self.EVENT.localNotifyEnabled);
		}
	},

	/**
	 * Local Notifications callback handler, called by OS on trigger time of message
	 * @param {Object} param
	 */
	handleLocalNotification : function(param){
		var self = this;

		param.isStartUp = self.notifyLaunchedApp;
		self.notifyLaunchedApp = false;			// only true for 1st message, false for all subsequent
		
		var result = self._createMessageFromLocalNotify(param);
		self._postLocalMessageProcess(result.message);
		Ti.App.fireEvent(self.EVENT.messageReceived, { messageID: result.msgID });
		
		Ti.API.warn('LOCAL received '+(param.isStartUp ? '<STARTUP>' : '')+': '+JSON.stringify(param));
	},

	//---  translate Local notify message & save to Message Queue  ---
	_createMessageFromLocalNotify : function(param){
		var self = this;
		
		var result = {};
		result.message = $.dataMgr.newModel('WOMessage');
		//---  sub-class to implement actual method to extract message data from Push Msg
		result.msgID = message.save();
		return result;
	},

	_postLocalMessageProcess : function(message){
		//--- Perform any post-receive Actions here  ---
	},
	
	//-------------------------  Notification Callback  -------------------------
	_notificationCallback : function(param){
		var self = this;

        //---------------  Call Server  ------------- 
        var callParam = {
            // url			: $.configMgr.get(self.CALLBACKtag),
            action      : 'perkevt_cbk',
            data        : param,
            callback	: function(){},
            errorback	: function(){},
        };
        self.callServer(callParam);      
	},

	//-------------------------  Message Queue Methods  -------------------------
	_countUnreadMessage : function(type, subType){
		var self = this;
		
		type = type || '%';
		subType = subType || '%';
		var timeNow = Math.round(new Date().valueOf() / 1000);
		var message = $.dataMgr.newModel('WOMessage');
		message.fetchAll({
    		where : '( ('+timeNow+' BETWEEN valid_begin AND valid_end) AND (status=0 or status is NULL) AND type LIKE ? AND sub_type LIKE ? )',
    		vararg : [type, subType],
    	});
    	return message.list.length;
	},
		
	_getNextMessageObj : function(type, subType){
		var self = this;
		
		type = ('undefined' == typeof type) ? '%' : type+'';
		subType = subType || '%';
		
		var timeNow = Math.round(new Date().valueOf() / 1000);
		var message = $.dataMgr.newModel('WOMessage');
		message.fetchAll({
    		where : '( ('+timeNow+' BETWEEN valid_begin AND valid_end) AND (status=0 or status is NULL) AND type LIKE ? AND sub_type LIKE ? )',
    		vararg : [type, subType],
    		order : {
    			updated : 'ASC',
    			rowid	: 'ASC',
    		},
    		limit : 'limit 1',
    	});
    	if(message.list.length)	return message.nth(0);
    	return false;
	},
	
	_getStartupMessageObj : function(type, subType) {
		var self = this;
		
		var timeNow = Math.round(new Date().valueOf() / 1000);
		if(helper.isArray(type)) {
			if(!type.length) return false;
			
			var types = []; 
			for(var i=0; i < type.length; i++) {
				types.push('?');
			}
			
			var message = $.dataMgr.newModel('WOMessage');
			message.fetchAll({
		    	where : '( ('+timeNow+' BETWEEN valid_begin AND valid_end) AND startup=1 AND (status=0 or status is NULL )  AND type IN ('+types+'))',
		    	vararg : type,
		    	order : {
		    		updated : 'DESC',
		    		rowid	: 'DESC',
		    	},
		    });
		    
		} else {
			
			type = ('undefined' == typeof type) ? '%' : type+'';
			subType = subType || '%';
			
			var message = $.dataMgr.newModel('WOMessage');
			message.fetchAll({
	    		where : '(  ('+timeNow+' BETWEEN valid_begin AND valid_end) AND startup=1 AND (status=0 or status is NULL )  AND type LIKE ? AND sub_type LIKE ? )',
	    		vararg : [type, subType],
	    		order : {
	    			updated : 'DESC',
	    			rowid	: 'DESC',
	    		},
	    	});
    	}
    	if(message.list.length) {
	    	 for(var i=1; i<message.list.length; i++) {
    			var msgObj = message.nth(i);
    			msgObj.data.startup = false;
    			msgObj.save();
    		}	
	    	return message.nth(0);
	    }
		return false;
	},

    // _showAlert : function(message, title){
        // var _alertDialog = Ti.UI.createAlertDialog({
            // title : title,
            // message : message
        // });
        // _alertDialog.show();
    // },
	   
});

module.exports = WONotificationManager;

//---  End of file: WONotificationManager.js
