/**
 * Error Log Manager - logs critical runtime errors on device
 * 
 * @author young@waveo.com
 */
Ti.include('/globals.js');

var WatchDog = App.Class(App.Controller.WOManager, {
    className   : 'WatchDog',

    //--- Event Name constants -----
    EVENT  : {
       log           : 'watch.log',
    },
    
    SEVERITY : {
    	debug 		: 1, 
    	info 		: 2, //default level
    	warning 	: 3,
    	error	 	: 4,
    	critical    : 5,
    },

	//---  Constructor  ---------------    
    initialize : function(param){
        var self = this;
        self.base(param);
        
        //--- install event listen ---
         Ti.App.addEventListener(self.EVENT.log, function(evt){
            self._log(evt);
        });
    },
    
    /**
     *  Wrapper function to make calls to log() async
     *  @param  {object}    a WOObject
     *  @param  {string}    name of the method / function
     *  @param  {string}    message to log
     */
    dog : function(pObject, pMethod, pMessage, severity) {
        var self = this;
        
        pObject = pObject || {};
        severity = severity  || self.SEVERITY.info;

        Ti.App.fireEvent(self.EVENT.log, {
            className   : ('undefined' != typeof pObject.className) ? pObject.className : '',
            method      : pMethod,
            message     : pMessage,
            severity	: severity,
        });
    },

    _log : function(evt){
        // evt = { className: object, method: string, message: string }
        var self = this;

        var theDog = {
            className   : evt.className,
            method      : evt.method || '',
            message     : evt.message || '',
            stamp       : Math.round(new Date().valueOf() / 1000),      // in seconds
            severity    : evt.severity,
        };

		// FIXME [ZJ]   Must READ settings during Bootstrap, NOT here, will impact performance
		       
        App.Helper.requireModel('WOSettings');
        var preference = new App.Model.WOSettings('preference');
        preference.fetch();

        //--- just echo to Console if using Simulator ---
        if(App.Config.IS_SIMULATOR){
            var methodStr = ('' == theDog.className) ? theDog.method : theDog.className + '::' + theDog.method;
            methodStr = (methodStr == '' || methodStr == '::') ? '' : methodStr + (('' == theDog.message) ? '' : ' -  ');
            Ti.API.error('[WatchDOG] '+methodStr+theDog.message);
            return;
        }
                   
        if(preference.data.logRealtime) {
            //---------------  Call Server  -------------
            var payload = {
                objects    : JSON.stringify([ theDog ]),
                //--- extended payloads ---
                deviceInfo  : JSON.stringify($.configMgr.getDeviceInfo()),
            };
            var callParam = {
                action      : 'sync_watchdog',
                data        : payload,
                errorback   : _offlineMode,
            };
            self.callServer(callParam);
        } else {
            _offlineMode();
        }

        //---------------  Callback Handlers  --------------
        function _offlineMode(){
            App.Helper.requireModel('WOLog');
        	var log = new App.Model.WOLog({data:theDog});
        	log.save();
        }
    },
});

module.exports = WatchDog;

/**
 * End of file: WatchDog.js
 */