
//-----------------  Public Static properties  ------------------
Ti.include('/globals.js');

//-------------------------  The Class  -------------------------
function scanClass(){
}

//--- Event Name constants -----
scanClass.EVENT = {
	//--- handles these events:
	
	//--- produces these events:
	scannedCode			: 'scan.scannedCode',
};

scanClass.RESULT = {
	success : 1,
	failed : -1,
	cancelled : 0,
};

scanClass.barcodeConfig = {
	"QR-Code"		: { code : 1,	scan : true },
	"CODE-39"		: { code : 2,	scan : true },
//	"ECODE-39		: { code : 3,	scan : true },
	"CODE-128"		: { code : 4,	scan : true },
	"UPC-A"			: { code : 5,	scan : true },
	"UPC-E"			: { code : 6,	scan : true },
	"EAN-13"		: { code : 7,	scan : true },
	"EAN-8"			: { code : 8,	scan : true },
	"I25"			: { code : 9,	scan : true },
	"I2/5"			: { code : 9,	scan : false },		// returned value is not 'I25', due to bug in Ti.bar
		
	"DataBar"		: { code : 10,	scan : true },
	"DataBar-Exp"	: { code : 11,	scan : true },
	"ISBN-13"		: { code : 12,	scan : true },
	"ISBN-10"		: { code : 13,	scan : true },
	"PDF417"		: { code : 14,	scan : false },


	// Android
	"QR_CODE"		: { code :  1,	scan : true },
	"CODE_39"		: { code :  2,	scan : true },
	"CODE_128"		: { code :  4,	scan : true },
	"UPC_A"			: { code :  5,	scan : true },
	"UPC_E"			: { code :  6,	scan : true },
	"EAN_13"		: { code :  7,	scan : true },
	"EAN_8"			: { code :  8,	scan : true },
	"ITF"	  	    : { code :  9,	scan : true },
	"RSS_14"		: { code : 10,	scan : true },	
	"PDF_417"		: { code : 14,	scan : true },
	"CODABAR"		: { code : 15,	scan : true },
	"AZTEC"		    : { code : 16,	scan : true },


};

//-----------------  Public Static Methods  ----------------------

/**
 * @return	{boolean} or {string}		true if barcode is valid, else error message
 */
scanClass.isValidBarcode = function(code_number, code_type){
    
    if(!code_number || code_number == '') return true;
    if(code_type<=0) return true;

    var result = true;
    switch(code_type){
    case 2://'CODE-39':
        result = _validateCode39(code_number);
        break;
    case 4://'CODE-128':
        result = _validateCODE128(code_number);
        break;
    case 5://'UPC-A':
        result = _validateUPCA(code_number);
        break;
    case 6://'UPC-E':
        result = _validateUPCE(code_number);
        break;
    case 7://'EAN-13':
        result = _validateEAN13(code_number);
        break;
    case 8://)'EAN-8':
        result = _validateEAN8(code_number);
        break;
    case 9://'I2/5':
        result = _validateI25(code_number);
        break;
    }
    return result;
}

/**
 * @param	{Object}		abstract parameter object
 * 			symbols			{object}	symbols that we want to scan (optional)
 */
scanClass.scan = function(param){
	var self = this;
	param = param || {};
	
	if (App.Config.IS_ANDROID) {
//		var TiBar = require("com.bsl.lbarcode");
		var TiBar = require("com.bsl.zxbarcode");
	} else {
		var TiBar = require('tibar');	
	}
  
	//--- Symbologies that we will scan, override defaults with param.symbols if specified ---
	var configSymbol = {};
	if('undefined' == typeof param.symbols){
		for(sym in scanClass.barcodeConfig){
			configSymbol[sym] = scanClass.barcodeConfig[sym].scan;
		}
	} else {
		for(sym in param.symbols){
			configSymbol[sym] = param.symbols[sym];
		}
	}
	
	//--- Scan it ---
	var _classType	= App.Config.IS_SIMULATOR ? 'ZBarReaderController' : 'ZBarReaderViewController';
	var _sourceType	= App.Config.IS_SIMULATOR ? 'Album' : 'Camera';
	var _cameraMode	= App.Config.IS_SIMULATOR ? 'Default' : 'Sampling';

	TiBar.scan({
	    configure	: {
	        classType	: _classType,
	        sourceType	: _sourceType,
	        cameraMode	: _cameraMode,
	        symbol		: configSymbol,
	        config:{
		        //'showsCameraControls'	: true,		// (VC)
		        'showsZBarControls'		: true,
		        'tracksSymbols'			: true,		// the tracking rectangle that highlights barcodes
		        // 'enableCache'			: true,
		        'showsHelpOnFail'		: false,
		        // 'takesPicture'			: false
		    },
	    },
	    success		: _doSuccess,
		cancel		: _doCancel,
		error		: _doError,
	});
	
	//---------------  Callback Handlers  -------------------
	function _doSuccess(data){

		data = data || {};
		
		App.loadHelper('playSound').play('scan');

		if (App.Config.IS_ANDROID) {
			var BC = App.Helper.clone(data);
			
			BC.code = BC.barcode;
			BC.type = BC.format;
			Ti.API.info('CTK BARCODE:');
			for (var names in BC) {
				Ti.API.info(' CTK BARCODE: '+ names + ' ' + BC[names]);
			}
			data.symbology = BC.type;
			
			var code_type = ('undefined' != scanClass.barcodeConfig[data.symbology]) ? scanClass.barcodeConfig[data.symbology].code : -1; // -1 means unsupport barcode type.
			var code_data = BC.code;
			// TODO test code
			Ti.API.info(' CTK BARCODES: '+ code_type + ' ' + code_data);
			$.watch.dog(null, 'Zebra.scan', data.barcode + ' [' + data.symbology + ' / ' + code_type + ']');
			
			Ti.App.fireEvent(scanClass.EVENT.scannedCode, {
				result	: scanClass.RESULT.success,	// success
				number	: code_data,
				code	: code_type,
			});

		} else {
			
			var code_type = ('undefined' != scanClass.barcodeConfig[data.symbology]) ? scanClass.barcodeConfig[data.symbology].code : -1; // -1 means unsupport barcode type.
			// TODO test code
			$.watch.dog(null, 'Zebra.scan', data.barcode + ' [' + data.symbology + ' / ' + code_type + ']');
			
			Ti.App.fireEvent(scanClass.EVENT.scannedCode, {
				result	: scanClass.RESULT.success,	// success
				number	: data.barcode || '',
				code	: code_type,
			});
		}
	}
	function _doCancel(){
		Ti.App.fireEvent(scanClass.EVENT.scannedCode, {
			result	: scanClass.RESULT.cancelled,	// cancelled
		});
	}
	function _doError(errResult){
		Ti.App.fireEvent(scanClass.EVENT.scannedCode, {
			result	: scanClass.RESULT.failed,	// error,
			error	: errResult,
		});
	}
};


module.exports = scanClass;

//-----------------  Private Static Methods  ----------------------
    
var _helper_getString = function(origin, index){
    return Number(origin.substr(index-1, 1));
};

var _helper_checkNumeric = function(param){
    var patrn=/[^0-9]/;
    if (patrn.exec(param)) return false;
    return true;
};

var _helper_cleanFormat = function(theFormat, theString){    
    var sepratorChars = {};
    for(var i=0;i<theFormat.length;i++){
        var theChar = theFormat.charAt(i);
        if(theChar!='#' && !sepratorChars[theChar]){
            sepratorChars[theChar] = true;
        }
    }
    var result = '';
    for(var m=0;m<theString.length;m++){ 
        var theChar = theString.charAt(m);
        var valid = true;
        
        for(var seprator in sepratorChars){
            if(theChar == seprator){
                valid = false;
                break;
            }
        } 
        if(valid) {
            result+=theChar;
        }
    }
    return result;
};

var _helper_format = function(theFormat, theString){
	if(!theString || theString == '') return '';
    // if(!theFormat) return util.format('#### #### #### ####', theString);
    if(!theFormat) return theString;
    theString = theString+"";
    
    var formatLength = theFormat.length;
    var stringLength = theString.length;
    
    var loopLength = formatLength<stringLength ? formatLength : stringLength;
    var result = '';
    var count = 0;
    var countFormat = 0;
    while(true){
        if(count>=stringLength || countFormat>= formatLength) {
            break;
        }
        var formatChar = theFormat.charAt(countFormat++);
        if(formatChar == '#'){ 
            result += theString.charAt(count++);
        }else{
            result += formatChar;
        }
    } 
    if(stringLength>count){
        result += theString.substring(count);
    }
    return result;  
};
    
//---------------- Barcodes validation --------------
    
var _validateCode39 = function(param){
    var patrn=/^[A-Za-z0-9  \$\*\-\/\.\%\+]*$/; 
    if (!patrn.exec(param)){
        var errorCode = "'";
        for(var m=0; m<param.length;m++){
            var theChar = param.charAt(m);
            if(!patrn.exec(theChar)){
                errorCode += theChar + ' ';
            }
        }
        errorCode = errorCode.substring(0, errorCode.length - 1);
        errorCode += "'";
        return errorCode+App.Helper.L('code_error_char');
    }
    return true;
};
    
var _validateEAN8 = function(param){
    if(!_helper_checkNumeric(param)){
            return App.Helper.L('code_only_number');
    }
    if(param.length != 8 && param.length != 7){
            return App.Helper.L('code_error_length')+8+' or '+7;
    }
    
    if (param.length==8) {
	    var b1 = _helper_getString(param, 1);
	    var b2 = _helper_getString(param, 2);
	    var b3 = _helper_getString(param, 3);
	    var b4 = _helper_getString(param, 4);
	    var b5 = _helper_getString(param, 5);
	    var b6 = _helper_getString(param, 6);
	    var b7 = _helper_getString(param, 7);
	    var b8 = _helper_getString(param, 8);
	    var c = (10-(b2 + b4 + b6 + (b1 + b3 + b5 + b7)*3)%10)%10;
	    if(c!=b8){
	        return App.Helper.L("code_invalid");
	    }    	
    }
    return true;
};
    
var _validateEAN13 = function(param){
    if(!_helper_checkNumeric(param)){
        return App.Helper.L('code_only_number');
    }
    if(param.length != 13 && param.length != 12){
        return App.Helper.L('code_error_length') + 13 + ' or ' + 12;
    }
    
    if (param.length == 13) {
	    var b1 = _helper_getString(param, 1);
	    var b2 = _helper_getString(param, 2);
	    var b3 = _helper_getString(param, 3);
	    var b4 = _helper_getString(param, 4);
	    var b5 = _helper_getString(param, 5);
	    var b6 = _helper_getString(param, 6);
	    var b7 = _helper_getString(param, 7);
	    var b8 = _helper_getString(param, 8);
	    var b9 = _helper_getString(param, 9);
	    var b10 = _helper_getString(param, 10);
	    var b11 = _helper_getString(param, 11);
	    var b12 = _helper_getString(param, 12);
	    var b13 = _helper_getString(param, 13);
	    var c = (10 - ( (b2 + b4 + b6 + b8 + b10 + b12)*3 + (b1 + b3 + b5 + b7+ b9 + b11) )%10 );
	    if(c!=b13){
	        return App.Helper.L('code_invalid');
	    } 	
    } 
    return true;
};
    
var _validateUPCA = function(param){
    if(!_helper_checkNumeric(param)){
        return App.Helper.L('code_only_number');
    }
    if(param.length != 12){
        return App.Helper.L('code_error_length') + 12;
    }
    return true;
};
    
var _validateUPCE = function(param){
    if(!_helper_checkNumeric(param)){
        return App.Helper.L('code_only_number');
    }
    if(param.length != 8){
        return App.Helper.L('code_error_length') + 8;
    }
    return true;
};
    
var _validateI25 = function(param){
    if(!_helper_checkNumeric(param)){
        return App.Helper.L('code_only_number');
    }
    return true;
};
    
var _validateCODE128 = function(param){
	var patrn=/^[\x21-\x7e ]*$/; 
    if (!patrn.exec(param)){
        var errorCode = "'";
        for(var m=0; m<param.length;m++){
			var theChar = param.charAt(m);
			if(!patrn.exec(theChar)){
				errorCode += theChar + ' ';
			}
        }
        errorCode = errorCode.substring(0, errorCode.length - 1);
        errorCode += "'";
        return errorCode+App.Helper.L('code_error_char');
    }
    return true;
};

//--- End of file: Zebra.js

