/**
 * Core helper functions that are loaded at bootstrap time
 *
 * @author young@waveo.com
 */

Ti.include('/globals.js');

/**
 * Check the str is only Ascii character
 */
exports.isASCII = function(str){
    if(typeof(str)!=='string'){
        return false;
    }
    for(var i=0;i<str.length;i++){
        if(str.charCodeAt(i)>127){
            return false;
        }
    }
    return true;
}

/**
 * Get display name according to nameOrderID
 */
exports.getDisplayName = function(nameOrderID, familyName, givenName) {
	var display = '';
	switch(nameOrderID) {
		case 1:
		case '1':
			display = givenName+' '+familyName;
			break;
		case 2:
		case '2':
			display = givenName;
			break;
		case 0:
		case '0':
			default: 
			display = familyName+' '+givenName;
	}
	return display;
}

/**
 * For dynamic multi-language
 */
exports.L = function(string, type) {
	if(type) { //i18n from DB.
		var strings = $.dataMgr.get('WOSettings', type+'.i18n', false);
		if(strings && 'undefined' != typeof strings.data[App.Config.LANG] && 'undefined' != typeof strings.data[App.Config.LANG][string]) {
			return strings.data[App.Config.LANG][string];
		} else {
			if(type in App.Config.i18n && App.Config.LANG in App.Config.i18n[type]) {
				return 'undefined' != typeof App.Config.i18n[type][App.Config.LANG][string] ? App.Config.i18n[type][App.Config.LANG][string] : string;
			} else {
				return string;
			}
		}
	} else { //i18n from L
		if('zh-Hant_TW' == App.Config.LANG) string += '_TW';
		else if('zh-Hant' == App.Config.LANG) string += '_HK';
		return L(string);
	}
};

/**
 * Add "dedupe" behavior to Array  (JS object)
 * @author zhoujun@waveo.com
 */
// if (!Array.prototype.dedupe) {
    // Array.prototype.dedupe = function() {
        // var tmp={}, len=this.length;
        // for(var i=0, j=0; i<len; i++) if(!tmp.hasOwnProperty(this[i])) tmp[this[i]] = j++;
        // this.length = j;
        // for(var item in tmp) this[tmp[item]] = item;
    // };
// }

exports.dedupe = function(array) {
    var tmp={}, len=array.length;
    for(var i=0, j=0; i<len; i++) if(!tmp.hasOwnProperty(array[i])) tmp[array[i]] = j++;
    var newArray = [];
    for(var item in tmp) newArray.push(item);
    return newArray;
};

/**
 * Add "forEach" behavior to Array  (JS object)
 */
exports.dateToString = function (aDate) {
	if (Ti.Platform.osname=='android') {
		Ti.API.info('CTK -- '+String.formatDate(aDate,'medium'));
		return String.formatDate(aDate,'medium');			
	} else {
		return aDate.toString('MM dd yyyy');
	}
}

exports.humaneDate = function(date, days){
	var dateStr = '';
	var now = new Date();
	// within these days, show relative dates, -1 => no relative dates, default 7 days
	var relativeDays = ('undefined' != typeof days) ? days * 86400000 : 604800000;   // milliseconds
	if((days != -1) && (now - date) <= relativeDays){
		dateStr = App.Moment(date).fromNow();
	}else if(now.getYear() == date.getYear()){
        // if(Ti.Platform.osname=='android'){
        	// dateStr = App.Moment(date).format('LL');
        // }
	 	// else{
	 		dateStr = App.Moment(date).format('l');
	 	// }
	}else{
	 	dateStr = App.Moment(date).format('LL');
	}
	return dateStr;
};

/**
 * Add "forEach" behavior to Array  (JS object)
 */
if (!Array.prototype.forEach)
{
  Array.prototype.forEach = function(fun /*, thisp*/)
  {
    var len = this.length;
    if (typeof fun != "function")
      throw new TypeError();

    var thisp = arguments[1];
    for (var i = 0; i < len; i++)
    {
      if (i in this)
        fun.call(thisp, this[i], i, this);
    }
  };
}

/**
 * Generate a UUID
 * @param   {number}      length of UUID to generate
 * @return  {string}
 */
exports.generateUUID = function(length){
	if(!length) length = 7;
	var x	= "0123456789QWERTYUIOPASDFGHJKLZXCVBNMabcdefghijklmnopqrstuvwxyz";
	var UUID = "";
	for(var  i= 1; i <= length; i ++)  {
		UUID += x.charAt(Math.ceil(Math.random()*100000000)%x.length);
		if(i==8 || i==12 || i==16 || i==20){
			UUID += '-';
		}
	}
	return UUID;
};

/**
 * Determind whether the object is an array
 * 
 * @param   {object}    object to determine if is Array
 */
exports.isArray = function(value){
    if(Object.prototype.toString.apply(value) === '[object Array]'){
        return true;
    }else{
        return false;
    }
};

/**
 * Check if a string or number is in an array
 * 
 * @param   {string or number}      item to look up in the array
 * @param   {array}                 the array
 */
exports.inArray = function(item, arr) {
    var type = typeof item;
    if(type == 'string' || type =='number') {
        for(var i in arr) {
            if(arr[i] == item) return true;
        }
    }
    return false;
};

exports.isAlphabet = function(theChar){
    return exports.inArray(theChar, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ');
}

/**
 * RECURSIVELY Extend & Override properties of target with those of options
 * 
 * @param {Object} target		NOTE: target will be modified & returned
 * @param {Object} options
 */
exports.extend = function(target, options){
    for (var p in options) {
        try {
          // Property in destination object set; update its value.
            if ( options[p].constructor==Object ) {
                target[p] = exports.extend(target[p], options[p]);
            } else {
                target[p] = options[p];
            }
        } catch(e) {
            // Property in destination object not set; create it and set its value.
            target[p] = options[p];
        }
    }
    return target;
};

/**
 * Simple merge & override of 1st level properties from options to target
 * 
 * @param {Object} target		NOTE: target & options NOT modified
 * @param {Object} options
 */
exports.merge = function(target, options){
	var merged = exports.clone(target);
	for(p in options){
		merged[p] = options[p];
    }
    return merged;
};

exports.clone = function(obj){
    if(typeof(obj) != 'object' || obj === null){ 
        return obj;
    }
    var newObj = new obj.constructor();  
    for(key in obj){
        newObj[key] = exports.clone(obj[key]); 
    } 
    return newObj;
};

exports.getTimeStamp = function(aDate){
    if(!aDate){
        aDate = new Date();
    }
    return Math.floor(aDate.valueOf()/1000) + '';
};

//---------------  Network Utilities  -----------------------

var USER_AGENT = Ti.App.id + "/" + Ti.App.version+" ("+Ti.Platform.model+"/"+Ti.Platform.version+";"+Ti.Platform.locale+")";

exports.ajax = function(param){
    if(!Ti.Network.online){
        if(param.report){
            _showAlert(App.Helper.L('net_required'), App.Helper.L('net_error_title'));
        }
        if(param.errorback){
            param.errorback({ code: -1 });   // not connected to network
        }
        return;
    }
    if(!param['url']){
        Ti.API.error('App.Helper.ajax: URL missing in parameter');
        param['error'] = 'no url';
        if(param.errorback) param.errorback(param);
        return;
    }
    
    var xhr = Ti.Network.createHTTPClient();
    xhr.onload = function(e) {
        if(xhr.status!=200){
            if(param.report){
                _showAlert(String.format(App.Helper.L('sever_error'), xhr.status));
            }
            if(param.errorback){ 
                param.errorback(e);
            }
            return;
        }
        if(param.noJSON) {
            var header = xhr.getResponseHeader('Content-Type');
            var length = xhr.getResponseHeader('Content-Length');
            if(header.substr(0, 5)!='image' || length<=0){
                if(param.errorback) param.errorback();
                return;
            }
            if(param.callback) param.callback(xhr.responseData);
            return;
        }
        try{
            Ti.API.info('ajax.js: Result'+xhr.responseText);
            var result = JSON.parse(xhr.responseText);
            
        }catch(E){
            Ti.API.info(E);
            
            if(param.report){
                _showAlert(App.Helper.L('service_fail'));
            }
            if(param.errorback){
                param.errorback();
            }
            return;
        }
        if(param.callback) param.callback(result);
    };
    
    xhr.onerror = function(e) {
        if(param.report){
            _showAlert(App.Helper.L('service_fail'));
        }
        if(param.errorback) param.errorback(e);
        return;
    };
    if(!param.method) param.method = 'POST';
    xhr.open(param.method, param['url']);

    if(!param.timeout){
        param.timeout = 60000;
    }
    xhr.setTimeout(param.timeout);

    xhr.setRequestHeader('User-Agent', USER_AGENT);
    if(!param['data']){
        param['data'] = {};
    }
    //param['data']['apiv'] = app.API_VERSION;
    //param['data']['appver'] = app.APP_VERSION;
    xhr.send(param['data']);
    
    
    //----------  Private methods  --------------
    function _showAlert(message, title){
        var _alertDialog = Ti.UI.createAlertDialog({
            title : title,
            message : message,
			ok : App.Helper.L('BUTTON_OK'),
        });
        _alertDialog.show();
    } 
};

exports.getFuncName = function(callee){
    var _callee = callee.toString().replace(/[\s\?]*/g,"");
    var comb = _callee.length >= 50 ? 50 :_callee.length;
    _callee = _callee.substring(0,comb);
    var name = _callee.match( /^function([^\(]+?)\(/ );
    if(name && name[1]){
        return name[1];
    }
    var caller = callee.caller, _caller = caller.toString().replace(/[\s\?]*/g,"");
    var last = _caller.indexOf(_callee), str = _caller.substring(last-30,last);
    name = str.match(/var([^\=]+?)\=/);
    if(name && name[1]){
        return name[1];
    }
    return "anonymous";
};


/**
 * Deep copy for simple objects, used for clone abstract parameter objects
 * 
 * @param   {object}        object to clone
 */
exports.clone = function(obj){
    if(typeof(obj) != 'object' || obj === null){ 
        return obj; 
    }
    var newObj = new obj.constructor();  
    for(key in obj){
        newObj[key] = this.clone(obj[key]); 
    } 
    return newObj;
};

/**
 * Get country code
 */
exports.getCountryISO = function(id) {
	var countryISO = [
	    { id :'GE', name: 'Abkhazia', code: 995, header : 'A' },
	    { id :'AF', name: 'Afghanistan', code: 93 },
	    { id :'AX', name: 'Aland', code: 35818 },
	    { id :'AL', name: 'Albania', code: 355 },
	    { id :'DZ', name: 'Algeria', code: 213 },
	    { id :'AS', name: 'American Samoa', code: 1684 },
	    { id :'AD', name: 'Andorra', code: 376 },
	    { id :'AO', name: 'Angola', code: 244 },
	    { id :'AI', name: 'Anguilla', code: 1264 },
	    // { id :'AQ', name: 'Antarctica', code: null },
	    { id :'AG', name: 'Antigua and Barbuda', code: 1268 },
	    { id :'AR', name: 'Argentina', code: 54 },
	    { id :'AM', name: 'Armenia', code: 374 },
	    { id :'AW', name: 'Aruba', code: 297 },
	    { id :'AC', name: 'Ascension', code: 247 },
	    { id :'AU', name: 'Australia', code: 61 },
	    { id :'AT', name: 'Austria', code: 43 },
	    { id :'AZ', name: 'Azerbaijan', code: 994 },
	    { id :'BH', name: 'Bahrain', code: 973, header : 'B' },
	    // { id :'UM', name: 'Baker Island', code: null },
	    { id :'BD', name: 'Bangladesh', code: 880 },
	    { id :'BB', name: 'Barbados', code: 1246 },
	    { id :'BY', name: 'Belarus', code: 375 },
	    { id :'BE', name: 'Belgium', code: 32 },
	    { id :'BZ', name: 'Belize', code: 501 },
	    { id :'BJ', name: 'Benin', code: 229 },
	    { id :'BM', name: 'Bermuda', code: 1441 },
	    { id :'BT', name: 'Bhutan', code: 975 },
	    { id :'BO', name: 'Bolivia', code: 591 },
	    { id :'BA', name: 'Bosnia and Herzegovina', code: 387 },
	    { id :'BW', name: 'Botswana', code: 267 },
	    // { id :'BV', name: 'Bouvet Island', code: null },
	    { id :'BR', name: 'Brazil', code: 55 },
	    { id :'IO', name: 'British Indian Ocean Territory', code: 246 },
	    { id :'VG', name: 'British Virgin Islands', code: 1284 },
	    { id :'BN', name: 'Brunei', code: 673 },
	    { id :'BG', name: 'Bulgaria', code: 359 },
	    { id :'BF', name: 'Burkina Faso', code: 226 },
	    { id :'BI', name: 'Burundi', code: 257 },
	    { id :'KH', name: 'Cambodia', code: 855, header : 'C' },
	    { id :'CM', name: 'Cameroon', code: 237 },
	    { id :'CA', name: 'Canada', code: 1 },
	    { id :'CV', name: 'Cape Verde', code: 238 },
	    { id :'KY', name: 'Cayman Islands', code: 1345 },
	    { id :'CF', name: 'Central African Republic', code: 236 },
	    { id :'TD', name: 'Chad', code: 235 },
	    { id :'CL', name: 'Chile', code: 56 },
	    { id :'CN', name: 'China', code: 86 },
	    { id :'CX', name: 'Christmas Island', code: 61 },
	    { id :'CC', name: 'Cocos (Keeling) Islands', code: 61 },
	    { id :'CO', name: 'Colombia', code: 57 },
	    { id :'KM', name: 'Comoros', code: 269 },
	    { id :'CG', name: 'Congo', code: 242 },
	    { id :'CK', name: 'Cook Islands', code: 682 },
	    { id :'CR', name: 'Costa Rica', code: 506 },
	    { id :'CI', name: 'Cote d\'Ivoire (Ivory Coast)', code: 225 },
	    { id :'HR', name: 'Croatia', code: 385 },
	    { id :'CU', name: 'Cuba', code: 53 },
	    { id :'CY', name: 'Cyprus', code: 357 },
	    { id :'CZ', name: 'Czech Republic', code: 420 },
	    { id :'CD', name: 'Democratic Republic of the Congo', code: 243, header : 'D' },
	    { id :'DK', name: 'Denmark', code: 45 },
	    { id :'DJ', name: 'Djibouti', code: 253 },
	    { id :'DM', name: 'Dominica', code: 1767 },
	    { id :'DO', name: 'Dominican Republic', code: 1829 },
	    { id :'EC', name: 'Ecuador', code: 593, header : 'E' },
	    { id :'EG', name: 'Egypt', code: 20 },
	    { id :'SV', name: 'El Salvador', code: 503 },
	    { id :'GQ', name: 'Equatorial Guinea', code: 240 },
	    { id :'ER', name: 'Eritrea', code: 291 },
	    { id :'EE', name: 'Estonia', code: 372 },
	    { id :'ET', name: 'Ethiopia', code: 251 },
	    { id :'FK', name: 'Falkland Islands', code: 500, header : 'F' },
	    { id :'FO', name: 'Faroe Islands', code: 298 },
	    { id :'FJ', name: 'Fiji', code: 679 },
	    { id :'FI', name: 'Finland', code: 358 },
	    { id :'FR', name: 'France', code: 33 },
	    { id :'GF', name: 'French Guiana', code: 594 },
	    { id :'PF', name: 'French Polynesia', code: 689 },
	    // { id :'TF', name: 'French Southern and Antarctic Lands', code: null },
	    { id :'GA', name: 'Gabon', code: 241, header : 'G' },
	    { id :'DE', name: 'Germany', code: 49 },
	    { id :'GH', name: 'Ghana', code: 233 },
	    { id :'GI', name: 'Gibraltar', code: 350 },
	    { id :'GR', name: 'Greece', code: 30 },
	    { id :'GL', name: 'Greenland', code: 299 },
	    { id :'GD', name: 'Grenada', code: 1473 },
	    { id :'GP', name: 'Guadeloupe', code: 590 },
	    { id :'GU', name: 'Guam', code: 1671 },
	    { id :'GT', name: 'Guatemala', code: 502 },
	    { id :'GG', name: 'Guernsey', code: 44 },
	    { id :'GW', name: 'Guinea-Bissau', code: 245 },
	    { id :'GN', name: 'Guinea', code: 224 },
	    { id :'GY', name: 'Guyana', code: 592 },
	    { id :'HT', name: 'Haiti', code: 509, header : 'H' },
	    // { id :'HM', name: 'Heard Island and McDonald Islands', code: null },
	    { id :'HN', name: 'Honduras', code: 504 },
	    { id :'HK', name: 'Hong Kong', code: 852 },
	    { id :'HU', name: 'Hungary', code: 36 },
	    { id :'IS', name: 'Iceland', code: 354, header : 'I' },
	    { id :'IN', name: 'India', code: 91 },
	    { id :'ID', name: 'Indonesia', code: 62 },
	    { id :'IR', name: 'Iran', code: 98 },
	    { id :'IQ', name: 'Iraq', code: 964 },
	    { id :'IE', name: 'Ireland', code: 353 },
	    { id :'IM', name: 'Isle of Man', code: 44 },
	    { id :'IL', name: 'Israel', code: 972 },
	    { id :'IT', name: 'Italy', code: 39 },
	    { id :'JM', name: 'Jamaica', code: 1876, header : 'J' },
	    { id :'JP', name: 'Japan', code: 81 },
	    { id :'JE', name: 'Jersey', code: 44 },
	    { id :'JO', name: 'Jordan', code: 962 },
	    { id :'KZ', name: 'Kazakhstan', code: 7, header : 'K' },
	    { id :'KE', name: 'Kenya', code: 254 },
	    { id :'KI', name: 'Kiribati', code: 686 },
	    { id :'CS', name: 'Kosovo', code: 381 },
	    { id :'KW', name: 'Kuwait', code: 965 },
	    { id :'KG', name: 'Kyrgyzstan', code: 996 },
	    { id :'LA', name: 'Laos', code: 856, header : 'L' },
	    { id :'LV', name: 'Latvia', code: 371 },
	    { id :'LB', name: 'Lebanon', code: 961 },
	    { id :'LS', name: 'Lesotho', code: 266 },
	    { id :'LR', name: 'Liberia', code: 231 },
	    { id :'LY', name: 'Libya', code: 218 },
	    { id :'LI', name: 'Liechtenstein', code: 423 },
	    { id :'LT', name: 'Lithuania', code: 370 },
	    { id :'LU', name: 'Luxembourg', code: 352 },
	    { id :'MO', name: 'Macau', code: 853, header : 'M' },
	    { id :'MK', name: 'Macedonia', code: 389 },
	    { id :'MG', name: 'Madagascar', code: 261 },
	    { id :'MW', name: 'Malawi', code: 265 },
	    { id :'MY', name: 'Malaysia', code: 60 },
	    { id :'MV', name: 'Maldives', code: 960 },
	    { id :'ML', name: 'Mali', code: 223 },
	    { id :'MT', name: 'Malta', code: 356 },
	    { id :'MH', name: 'Marshall Islands', code: 692 },
	    { id :'MQ', name: 'Martinique', code: 596 },
	    { id :'MR', name: 'Mauritania', code: 222 },
	    { id :'MU', name: 'Mauritius', code: 230 },
	    { id :'YT', name: 'Mayotte', code: 262 },
	    { id :'MX', name: 'Mexico', code: 52 },
	    { id :'FM', name: 'Micronesia', code: 691 },
	    { id :'MD', name: 'Moldova', code: 373 },
	    { id :'MC', name: 'Monaco', code: 377 },
	    { id :'MN', name: 'Mongolia', code: 976 },
	    { id :'ME', name: 'Montenegro', code: 382 },
	    { id :'MS', name: 'Montserrat', code: 1664 },
	    { id :'MA', name: 'Morocco', code: 212 },
	    { id :'MZ', name: 'Mozambique', code: 258 },
	    { id :'MM', name: 'Myanmar', code: 95 },
	    { id :'AZ', name: 'Nagorno-Karabakh', code: 37497, header : 'N' },
	    { id :'NA', name: 'Namibia', code: 264 },
	    { id :'NR', name: 'Nauru', code: 674 },
	    { id :'NP', name: 'Nepal', code: 977 },
	    { id :'AN', name: 'Netherlands Antilles', code: 599 },
	    { id :'NL', name: 'Netherlands', code: 31 },
	    { id :'NC', name: 'New Caledonia', code: 687 },
	    { id :'NZ', name: 'New Zealand', code: 64 },
	    { id :'NI', name: 'Nicaragua', code: 505 },
	    { id :'NE', name: 'Niger', code: 227 },
	    { id :'NG', name: 'Nigeria', code: 234 },
	    { id :'NU', name: 'Niue', code: 683 },
	    { id :'NF', name: 'Norfolk Island', code: 672 },
	    { id :'KP', name: 'North Korea', code: 850 },
	    { id :'MP', name: 'Northern Mariana Islands', code: 1670 },
	    { id :'NO', name: 'Norway', code: 47 },
	    { id :'OM', name: 'Oman', code: 968, header : 'O' },
	    { id :'PK', name: 'Pakistan', code: 92, header : 'P' },
	    { id :'PW', name: 'Palau', code: 680 },
	    { id :'PS', name: 'Palestinian Territories', code: 970 },
	    { id :'PA', name: 'Panama', code: 507 },
	    { id :'PG', name: 'Papua New Guinea', code: 675 },
	    { id :'PY', name: 'Paraguay', code: 595 },
	    { id :'PE', name: 'Peru', code: 51 },
	    { id :'PH', name: 'Philippines', code: 63 },
	    // { id :'PN', name: 'Pitcairn Islands', code: null },
	    { id :'PL', name: 'Poland', code: 48 },
	    { id :'PT', name: 'Portugal', code: 351 },
	    { id :'PR', name: 'Puerto Rico', code: 1939 },
	    { id :'QA', name: 'Qatar', code: 974, header : 'Q' },
	    { id :'RE', name: 'Reunion', code: 262, header : 'R' },
	    { id :'RO', name: 'Romania', code: 40 },
	    { id :'RU', name: 'Russia', code: 7 },
	    { id :'RW', name: 'Rwanda', code: 250 },
	    { id :'SH', name: 'Saint Helena', code: 290, header : 'S' },
	    { id :'KN', name: 'Saint Kitts and Nevis', code: 1869 },
	    { id :'LC', name: 'Saint Lucia', code: 1758 },
	    { id :'PM', name: 'Saint Pierre and Miquelon', code: 508 },
	    { id :'VC', name: 'Saint Vincent and the Grenadines', code: 1784 },
	    { id :'WS', name: 'Samoa', code: 685 },
	    { id :'SM', name: 'San Marino', code: 378 },
	    { id :'ST', name: 'Sao Tome and Principe', code: 239 },
	    { id :'SA', name: 'Saudi Arabia', code: 966 },
	    { id :'SN', name: 'Senegal', code: 221 },
	    { id :'RS', name: 'Serbia', code: 381 },
	    { id :'SC', name: 'Seychelles', code: 248 },
	    { id :'SL', name: 'Sierra Leone', code: 232 },
	    { id :'SG', name: 'Singapore', code: 65 },
	    { id :'SK', name: 'Slovakia', code: 421 },
	    { id :'SI', name: 'Slovenia', code: 386 },
	    { id :'SB', name: 'Solomon Islands', code: 677 },
	    { id :'SO', name: 'Somalia', code: 252 },
	    { id :'ZA', name: 'South Africa', code: 27 },
	    // { id :'GS', name: 'South Georgia and the South Sandwich Islands', code: null },
	    { id :'KR', name: 'South Korea', code: 82 },
	    { id :'ES', name: 'Spain', code: 34 },
	    { id :'LK', name: 'Sri Lanka', code: 94 },
	    { id :'SD', name: 'Sudan', code: 249 },
	    { id :'SR', name: 'Suriname', code: 597 },
	    { id :'SJ', name: 'Svalbard', code: 47 },
	    { id :'SZ', name: 'Swaziland', code: 268 },
	    { id :'SE', name: 'Sweden', code: 46 },
	    { id :'CH', name: 'Switzerland', code: 41 },
	    { id :'SY', name: 'Syria', code: 963 },
	    { id :'TW', name: 'Taiwan', code: 886, header : 'T' },
	    { id :'TJ', name: 'Tajikistan', code: 992 },
	    { id :'TZ', name: 'Tanzania', code: 255 },
	    { id :'TH', name: 'Thailand', code: 66 },
	    { id :'BS', name: 'The Bahamas', code: 1242 },
	    { id :'GM', name: 'The Gambia', code: 220 },
	    { id :'TL', name: 'Timor-Leste (East Timor)', code: 670 },
	    { id :'TG', name: 'Togo', code: 228 },
	    { id :'TK', name: 'Tokelau', code: 690 },
	    { id :'TO', name: 'Tonga', code: 676 },
	    { id :'TT', name: 'Trinidad and Tobago', code: 1868 },
	    { id :'TA', name: 'Tristan da Cunha', code: 290 },
	    { id :'TN', name: 'Tunisia', code: 216 },
	    { id :'TR', name: 'Turkey', code: 90 },
	    { id :'TM', name: 'Turkmenistan', code: 993 },
	    { id :'TC', name: 'Turks and Caicos Islands', code: 1649 },
	    { id :'TV', name: 'Tuvalu', code: 688 },
	    { id :'VI', name: 'U.S. Virgin Islands', code: 1340, header : 'U' },
	    { id :'UG', name: 'Uganda', code: 256 },
	    { id :'UA', name: 'Ukraine', code: 380 },
	    { id :'AE', name: 'United Arab Emirates', code: 971 },
	    { id :'GB', name: 'United Kingdom', code: 44 },
	    { id :'US', name: 'United States', code: 1 },
	    { id :'UY', name: 'Uruguay', code: 598 },
	    { id :'UZ', name: 'Uzbekistan', code: 998 },
	    { id :'VU', name: 'Vanuatu', code: 678, header : 'V' },
	    { id :'VA', name: 'Vatican City', code: 379 },
	    { id :'VE', name: 'Venezuela', code: 58 },
	    { id :'VN', name: 'Vietnam', code: 84 },
	    { id :'WF', name: 'Wallis and Futuna', code: 681, header : 'W' },
	    { id :'EH', name: 'Western Sahara', code: 212 },
	    { id :'YE', name: 'Yemen', code: 967, header : 'Y' },
	    { id :'ZM', name: 'Zambia', code: 260, header : 'Z' },
	    { id :'ZW', name: 'Zimbabwe', code: 263 },
	];
	if(id) {
		for(var i=0; i<countryISO.length; i++){
			if(countryISO[i].id == id) return countryISO[i];
		}
		return { id: id, name: '', code: '' };
	} else {
		return countryISO;	
	}
	
}

/**
 * End of file: coreHelpers.js
 */