
Ti.include('/globals.js');

//---------------------------  Loaders  ---------------------------

exports.loadHelper = function(helperName){
    var helper = require(_getPath('helper', helperName));
    return helper;
};

exports.loadModule = function(path, param){
    param = param || {};
    var ModuleClass = require(path);
    return new ModuleClass(param);
};

//-----------------------  MODEL methods  ---------------------------
exports.requireModel = function(modelName){
    if(App.Model[modelName]){
       return App.Model[modelName];
    } else {
        var modelClass = require(_getPath('model', modelName));
        App.Model[modelName] = modelClass;
        return modelClass;
    }
};

//-----------------------  IMAGE methods  ---------------------------
exports.fetchRemoteImage = function(param){
	var ajaxParam 		= {
		method		: 'GET',
		url			: param.url,
		noJSON		: true,
		timeout		: App.Config.IMAGEFETCH_TIMEOUT,
		callback 	: function(result){
			App.Helper.requireModel('WOImage');
			var image = new App.Model.WOImage(param.url);
			image.initialize();
			
			image.save(result);
			image.fetch();
			if(param.callback)
				param.callback({
					filepath	: image.getFilepath(),
					originalURL	: param.url,
				});
		},
		errorback	: param.errorback,	
	};
	App.Helper.ajax(ajaxParam);
};

//----------------------  CONTROLLERS methods  -----------------------
exports.requireController = function(controllerName){

    var searchDir = [
        App.Config.APP_PATH+'controllers/',
        App.Config.CORE_PATH+'controllers/'
    ];
    var controllerClass = _searchRequire(searchDir, controllerName);
    
    if(null !== controllerClass){
        App.Controller[controllerName] = controllerClass;
        return controllerClass;
    } else {
        Ti.API.error("Loaders::requireController - failed to load controller: "+controllerName+". File not found.");
        return null;
    }
};

exports.newController = function(controllerName, param){
    param = param || {};
    var controllerClass = exports.requireController(controllerName);
    if(controllerClass){
        var theController = new controllerClass(param);
        //--- call the real constructor, to overcome compaibility issue between JClass & 'require'
        if(theController.initialize) 	theController.initialize(param);
        
        // add controller to Alias lookup if it has one
        // WARNING: having an alias will make it STAY in memory !!
        
        if('alias' in theController){
            App.ControllerAlias[theController.alias] = theController;
        }
        return theController;
    } else {
        return null;
    } 
};

//------------------------  VIEW methods  ----------------------------
exports.requireView = function(viewName){
    if(App.View[viewName]){
        return App.View[viewName];
    }else{    	
        var viewClass = require(_getPath('view', viewName));
        return viewClass;        

    	// var viewPath = _getPath('view', '');
    	// var extendedPath = '';
		// var separator = viewName.lastIndexOf('/');
		// if(separator>-1){
			// extendedPath = viewName.substr(0, separator+1);
			// viewName = viewName.substr(separator+1);
		// }
		// var searchDir = [ viewPath+extendedPath, viewPath+extendedPath+Ti.Platform.osname+'/'];
    	// var viewClass = _searchRequire(searchDir, viewName);
        // return viewClass;
    }   
};

exports.loadView = function(viewName, param){
    param = param || {};
    var viewClass = exports.requireView(viewName);    
    var viewObj = new viewClass(param);
	//--- call the real constructor, to overcome compaibility issue between JClass & 'require'
	if(viewObj.initialize) 	viewObj.initialize(param);
    return viewObj;
};

/**
 * @param {string} or {array}	single or array of style filenames
 */
exports.loadStyle = function(styleNames){
	if('' == styleNames)	return {};

	//--- deal with parameter variations ---
	switch(typeof styleNames){
	case 'undefined':
		return {};
		break;
	case 'string':
		var styleFiles = [styleNames];
		break;
	default:
		var styleFiles = styleNames;
		break;
	}

	//--- merge styles in style files ---
	var stylePath = _getPath('style');
	var baseStyle = require(stylePath + styleFiles[0]);
	var mergedStyle = App.Helper.clone(baseStyle.STYLE);

	for(var i=1; i<styleFiles.length; i++){
		var style = require(stylePath + styleFiles[i]);
		//--- merge styles ---
		for (s in style.STYLE) {
			mergedStyle[s] = style.STYLE[s];
	    }
	}
	return mergedStyle;
};

//------------------------  FORM methods  ----------------------------
exports.newForm = function(formName, param){
    param = param || {};
    var formClass = require(_getPath('form', formName));
	//--- call the real constructor, to overcome compaibility issue between JClass & 'require'
	var formObj = new formClass(param);
	if(formObj.initialize) 	formObj.initialize(param);
    return formObj;
};

exports.requireField = function(fieldName){
    return  require(_getPath('field', fieldName));
};
exports.newField = function(fieldName, param){
    param = param || {};
    var fieldClass = exports.requireField(fieldName);
	//--- call the real constructor, to overcome compaibility issue between JClass & 'require'
	var fieldObj = new fieldClass(param);
	if(fieldObj.initialize) 	fieldObj.initialize(param);
    return fieldObj;
};

//------------------------  Private methods  --------------------------------
/**
 * Encapsulates directories and paths manipulation
 * @param   {string}        type of file, eg. 'controller', 'image', etc
 * @param   {string}        name of file  (optional)
 * @return  {string}        path for fileType
 */

function _getPath(fileType, fileName){

    fileType.toLowerCase();
    fileName = fileName || '';
    var dirPath = '';
    
    switch(fileType){
    case 'image':
        dirPath = App.Config.IMAGE_DIR;
        break;
    case 'view':
        // dirPath = App.Config.APP_PATH+'views/'+Ti.Platform.osname+'/';
        dirPath = App.Config.APP_PATH+'views/';
        break;
    case 'style':
        dirPath = App.Config.APP_PATH+'styles/';
        break;
    case 'form':
        dirPath = App.Config.APP_PATH+'forms/';
        break;
    case 'field':
    	dirPath = App.Config.APP_PATH+'fields/'+Ti.Platform.osname+'s/';
    	break;    	
    case 'controller':
        dirPath = App.Config.APP_PATH+'controllers/';
        break
    case 'model':
        dirPath = App.Config.CORE_PATH+'models3/';
        break;
    case 'helper':
        dirPath = App.Config.CORE_PATH+'helper/';
        break;
    }
    
    //====  do extra checks during development  ====
    if(App.Config.DEBUG_LEVEL > 0){
    	if(fileName){
	        var filePath = (dirPath.charAt(0) == '/') ? dirPath.slice(1) : dirPath;
	        var file = Ti.Filesystem.getFile(Ti.Filesystem.resourcesDirectory+filePath+fileName+'.js');
	        if(!file.exists()){
	            Ti.API.error("Loader - failed to load "+fileType+": "+fileName+". File not found at ("+dirPath+")");
	        }
	        Ti.API.info('--- '+fileType.toUpperCase()+'::'+fileName+' loaded  ---');    		
    	}
    }   //===========
    
    return dirPath + fileName;
}

/**
 * Attempts to load module from an array of directories
 * @param   {array}         array of (string) search paths
 * @param   {string}        name of file
 * @return  {object}        module object  (null if not found)
 */
function _searchRequire(searchDir, fileName){
  
    for(var i=0; i < searchDir.length; i++){
        var filePath = searchDir[i] + fileName;
        try{
            var theModule = require(filePath);
        } catch(err){
            continue;
        }
        return theModule;
    }
    return null;
}


