
Ti.include('/globals.js');

//-----------------  Public Static properties  ------------------
	
//--- Event Name constants -----
locationInfo.EVENT = {
	//--- handles these events:
	
	//--- produces these events:
	locationUpdated			: 'locationInfo.locationUpdated',
};

//-----------------  Public Static Methods  ----------------------

/**
 * @return	{boolean}		true if we have permission to get location information
 * permission is: AUTHORIZATION_RESTRICTED, AUTHORIZATION_AUTHORIZED, AUTHORIZATION_UNKNOWN or AUTHORIZATION_DENIED
 */
locationInfo.hasPermission = function(){
	var self = this;
	
	var hasPermission = false;
	var permission = Ti.Geolocation.getLocationServicesAuthorization();
	if((permission == Ti.Geolocation.AUTHORIZATION_AUTHORIZED) || (permission == Ti.Geolocation.AUTHORIZATION_UNKNOWN)){
		hasPermission = true;
	}
	return  hasPermission;
}

locationInfo.isAuthorized = function(){
	if(App.Config.IS_ANDROID){
		return true;
	}else{
		return (Ti.Geolocation.getLocationServicesAuthorization() == Ti.Geolocation.AUTHORIZATION_AUTHORIZED);	
	}
}

locationInfo.needsPermission = function(){
	if(App.Config.IS_ANDROID){
		return false;
	}else{
		return (Ti.Geolocation.getLocationServicesAuthorization() == Ti.Geolocation.AUTHORIZATION_UNKNOWN);
	}
}

locationInfo.isDenied = function(){
	return  (Ti.Geolocation.getLocationServicesAuthorization() == Ti.Geolocation.AUTHORIZATION_DENIED);;
}

/**
 * @return	{object}		abstract parameter object:
 * 			msg				{string} optional message to display when requesting for location permission
 */
locationInfo.getLocation = function(param){
	var self = this;
	param = param || {};
	
	Ti.Geolocation.preferredProvider = 'gps';	
	if (App.Config.IS_ANDROID)
		Ti.Geolocation.accuracy = Ti.Geolocation.ACCURACY_HIGH;
	else
		Ti.Geolocation.accuracy = Ti.Geolocation.ACCURACY_BEST;
	Ti.Geolocation.purpose = ('undefined' == typeof msg) ? App.Helper.L('ALERT_ShowForCards') : msg;
	Ti.Geolocation.distanceFilter = 10;

	Ti.Geolocation.getCurrentPosition(function(result){
		if(!result.success || result.error){
			Ti.App.fireEvent(locationInfo.EVENT.locationUpdated, {
				error	: _translateErrorCode(result.code),
			});
			$.watch.dog(self, 'getLocation', _translateErrorCode(result.code));
		}else{
			var locInfo	= {
				longitude			: result.coords.longitude,
				latitude			: result.coords.latitude,
				altitude			: result.coords.altitude,
				heading				: result.coords.heading,
				accuracy			: result.coords.accuracy,
				speed				: result.coords.speed,
				timestamp			: result.coords.timestamp,
				altitudeAccuracy	: result.coords.altitudeAccuracy,
			};
			Ti.App.fireEvent(locationInfo.EVENT.locationUpdated, {
				coords	: locInfo,
			});
		}
	});
}

//-----------------  Private Static Methods  ----------------------
    
var _translateErrorCode = function(code){
	if (code == null) {
		return null;
	}
	var errMsg = '';
	switch (code) {
	case Ti.Geolocation.ERROR_LOCATION_UNKNOWN:
		errMsg = "Location unknown";
		break;
	case Ti.Geolocation.ERROR_DENIED:
		errMsg = "Access denied";
		break;
	case Ti.Geolocation.ERROR_NETWORK:
		errMsg = "Network error";
		break;
	case Ti.Geolocation.ERROR_HEADING_FAILURE:
		errMsg = "Failure to detect heading";
		break;
	case Ti.Geolocation.ERROR_REGION_MONITORING_DENIED:
		errMsg = "Region monitoring access denied";
		break;
	case Ti.Geolocation.ERROR_REGION_MONITORING_FAILURE:
		errMsg = "Region monitoring access failure";
		break;
	case Ti.Geolocation.ERROR_REGION_MONITORING_DELAYED:
		errMsg = "Region monitoring setup delayed";
		break;
	}
	return errMsg;
}

//-------------------------  The Class  ---------------------------
function locationInfo(){
	var self = this;

}

module.exports = locationInfo;

//--- End of file: locationInfo.js

