/**
 * WOImage
 * 
 * @author zhoujun@waveo.com & young@waveo.com
 */

Ti.include('/globals.js');
var helper = App.loadHelper('default'); //TODO [ZJ]

var WOImage = App.Class(App.Model.WOObject, {
    className 	: 'WOImage',
    alias	  	: 'image',
    
	EVENT		: {
		//--- handles these events:
		updated			: 'WOImage.updated.',		// to concatenate with imageID
		//--- produces these events:
    },
    
    tableName	: 'image',
    fileDir		: App.Config.IMAGE_DIR,
    
	DEFAULT_PLACEHOLDER	: App.Config.IS_ANDROID ? App.Config.IMAGE_DIR+'default_thumbAndroid.png' : '/assets/images/ui/default_thumb.png' ,
	DEFAULT_IMAGE		: App.Config.IS_ANDROID ? App.Config.IMAGE_DIR+'default_faceAndroid.png' : '/assets/images/ui/default_face.png',

	init : function(param){
		var self = this;
		self.base(param);
		
        self._isLocal		= false;
        self.VIEW			= null;			// view generated on-demand when getView() is called
        self.viewRendered	= false;		// is our view properly rendered with the image?

		self.activityIndicator = null;
		self.placeholder = (param && ('undefined' != typeof param.placeholder)) ? param.placeholder : self.DEFAULT_PLACEHOLDER;
		self.defaultImage = (param && ('undefined' != typeof param.defaultImage)) ? param.defaultImage : self.DEFAULT_IMAGE;
		
		self.fetchCallback	= null;
		self.ImageFactory = null;
		
		self.initProperties(param);
	},

    initialize : function(param){
        var self = this;
        
		if (App.Config.IS_ANDROID) self.ImageFactory = require('ti.imagefactory');
		//------------------  Event Handlers  ---------------------
		// Ti.App.addEventListener(self.EVENT.updated+self.id, function(evt){
			// self.updateView(evt.filepath);
		// });
    },
    
    initProperties : function(param){
    	var self = this;
    	if('string' == typeof param || 'number' == typeof param) param = {id:param,data:{},master:{}};
    	param = param || {id:'NoUrls'};
    	self.url = param.id;
    	param.id = Ti.Utils.md5HexDigest(self.url);
        self.base(param);
    },
    
    save : function(filedata){
    	var self = this;
    	if(self._isLocal) return;
    	
    	// if (App.Config.IS_ANDROID) { //resize card big image for android platform
    		// var pattern = /app\/card\/image\/\d+\/frt/; //makesure only resize card big image
    		// if(pattern.test(self.url)) {
    			// filedata = self.ImageFactory.imageAsResized(filedata, { width: App.Config.CardBigImageW, height: App.Config.CardBigImageH });
    		// }
    	// }
    	
        self.data.filename = self._getFileName();
        self.data.url = self.url;
        var file = Ti.Filesystem.getFile(self.fileDir, self.data.filename);
        self.data.filesize = 0;
        if(filedata && !file.write(filedata)) self.data.filesize = file.getSize();
        
        self.base();
    },
    
    fetch : function(fetchImage) {
    	var self = this;
    	fetchImage = fetchImage || false;
    	if(self.url.substring(0,4) == 'http') {
    		var rt = self.base();
    		
    		if(fetchImage && !self._imageFileExist()) {
		        App.Helper.fetchRemoteImage({
		        	url			: self.url,
		        });
    		}    		
    		return rt;
    	}
    	//else is local application file
    	self._isLocal = true;
    	self.data.id = self.id;
    	self.data.url = self.url;
    	self.data.filename = self.url.split('/').pop();
    	self._fetched = true;
    	return self._fetched;
    },
    
    getFilepath : function() {
    	var self = this;
    	if(self._isLocal) return self.url;
    	if(self._fetched) return self.fileDir + self.data.filename;
    	return self.placeholder;
    },
    
    remove : function(){
        var self = this;
        if(self._isLocal) return;
        if(self._imageFileExist())
        	file.deleteFile();
        self.base(true);
    },
    
    getAllImageIDs : function(){
    	var self = this;
        return App.DB2.fetch2ColsToHashArray("SELECT id AS key, 0 AS value FROM "+self.tableName, [], 'key', 'value');
    },
    
    setURL : function(URL) {
    	var self = this;

        self.initProperties({id:URL,data:{},master:{}});
        self.fetch();
        self.viewRendered = false;
    },

	getView : function(param){
		var self = this;
		param = param || {};
		
		var imageFileExist = self._imageFileExist();
			
		//----------------  Deal with View creation & update  ------------
		if(!self.VIEW){
			//--- no view yet, create on-demand ---
			var viewParam = ('undefined' != typeof param.style) ? param.style : {};
			viewParam.defaultImage = self.placeholder;

			if(imageFileExist){
				//--- use local cached image ---
				self.viewRendered = true;
				viewParam.image = self.getFilepath();
			}else{
				//--- image file NOT localized, return placeholder image first
				var needRemoteFetch = true;
				viewParam.image = self.placeholder;
			}
			self.VIEW = Ti.UI.createImageView(viewParam);
		} else if(!self.viewRendered) {
			if(imageFileExist){
				//--- use local cached image ---
				self.viewRendered = true;
				self.VIEW.setImage(self.getFilepath());				
			}else{
				//--- image file NOT localized, return placeholder image first
				var needRemoteFetch = true;
				if('undefined' != typeof param.activityIndicator) param.activityIndicator = undefined;
			}
		}

		//----------------  Fetch remote image when necessary  ------------			
		if(needRemoteFetch){
			
			function _fetchCallback(evt){
				self.updateView(evt.filepath);
			}			
			//---  Get notified when image is fetched ---
			Ti.App.addEventListener(self.EVENT.updated+self.id, _fetchCallback);

			// remember it, so that we can dispose it later
			self.fetchCallback = {
				evtName		: self.EVENT.updated+self.id,
				handler		: _fetchCallback,
			};
			
			//--- show activity indicator if required --
			if(('undefined' != typeof param.activityIndicator) && param.activityIndicator.show && !App.Config.IS_ANDROID){
				if('undefined' != typeof param.activityIndicator.properties){
					var properties = param.activityIndicator.properties;
				}else{
					if(parseInt(self.VIEW.width) > 110){
						var properties = {style: Ti.UI.iPhone.ActivityIndicatorStyle.BIG};
					}else{
						var properties = {style: Ti.UI.iPhone.ActivityIndicatorStyle.PLAIN};
					}
				}
				if (self.activityCount==undefined) {
					self.activityCount=0;				
					self.activityIndicator = Ti.UI.createActivityIndicator(properties);
					self.VIEW.add(self.activityIndicator);
					self.activityIndicator.show();
				}
				self.activityCount++;
			}
			//--- Fetch image from remote asynchronously and update our view automatically
	        App.Helper.fetchRemoteImage({
	        	url			: self.url,
	        	callback	: _completeRemoteFetch,
	        	errorback	: _remoteFetchError,
	        });
		}
		
		return self.VIEW;
		
		//---------------------  Callback Handler  ---------------------
		function _completeRemoteFetch(result){
			if('undefined' != typeof result.filepath){
				Ti.App.fireEvent(self.EVENT.updated+self.id, { filepath: result.filepath });
			}
		}
		function _remoteFetchError(result){
			self._removeActivityIndicator();
			if(self.VIEW){
				self.VIEW.setImage(self.defaultImage);
			}
		}
	},

	updateView : function(filepath){
		var self = this;

		if(self.fetchCallback){
			Ti.App.removeEventListener(self.fetchCallback.evtName, self.fetchCallback.handler);
			self.fetchCallback = null;
		}
		self._removeActivityIndicator();
		if(self.VIEW){
			self.viewRendered = true;
			self.VIEW.setImage(filepath);
		}
	},
	
	dispose : function(param){
		var self = this;
		
		if(self.fetchCallback){
			Ti.App.removeEventListener(self.fetchCallback.evtName, self.fetchCallback.handler);
			self.fetchCallback = null;
		}
	},
	
    _getFileName : function(){
        var self = this;
        return self.id+'.'+self.url.split('.').pop();
    },
    
    _imageFileExist : function(){
    	var self = this;
    	if(self._isLocal) return true;
    	if(self._fetched && 'undefined' != typeof self.data.filename) {
    		var file = Ti.Filesystem.getFile(self.fileDir, self.data.filename);
    		return file.exists();
    	} else {
    		return false;
    	}
    },
    
    _removeActivityIndicator : function(){
    	var self = this;
    	
		if(self.activityIndicator){

			self.activityCount--;
			if (self.activityCount<1){
				self.activityIndicator.hide();
				if (App.Config.IS_ANDROID) {
					
				} else {
					self.VIEW.remove(self.activityIndicator);			
				}
			}
			self.activityIndicator = null;
		}
    }
});

module.exports = WOImage;

//--- End of file: WOImage.js

