/**
 * Smart object base class
 * 
 * @author zhoujun@waveo.com
 */

Ti.include('/globals.js');

var helper = App.loadHelper('default'); //TODO [ZJ]
var base64 = App.loadHelper('base64');

var WOObject = App.Class({
    className : 'WOObject',
    alias : 'object',
    encryptData : false && App.Config.MODEL_ENCRYPT,
    encryptMaster : false && App.Config.MODEL_ENCRYPT,
    hasMaster : false,
    removeSynced : false,
    related : {},
    refered : {},
    
    init : function() {
        var self = this;
        
        self.list = [];
        self.data = {};
        self.master = {};
        self.id = false;
        self._isNew = true;
        self._fetched = false;
        self._propMap = {};
        self._propMap['id'] = 'id';
        self._propMap['data'] = 'data';
        self._propMap['created'] = 'created';
        self._propMap['changed'] = 'changed';
        self._propMap['updated'] = 'updated';
        self._propMap['deleted'] = 'deleted';
        self._propMap['isDeleted'] = 'is_deleted';
        self._propMap['isHidden'] = 'is_hidden';
        self._propMap['SYNC'] = 'sync';
        
        self._LANG = App.Config.LanguagesTag[App.Config.LANG];
    },
    
    initProperties : function(param) {
    	var self = this;
    	if('string' == typeof param || 'number' == typeof param) param = {id:param,data:{},master:{}};
    	param = param || {id:false,data:{},master:{}};
    	self.id = param.hasOwnProperty('id') ? param.id : false;
    	self.data = param.hasOwnProperty('data') ? param.data : {};
    	self.master = param.hasOwnProperty('master') ? param.master : {};
    },
    
    p2l : function() {
    	var self = this;
    	if(!self.id) {
    		var list = [];
    		for(var i=0; i < self.list.length; i++) list.push(self.data);
    	} else {
    		var list = self.data;
    	}
    	return list;
    },
    
    getClass : function() {
        return this.constructor;
    },

    getID : function() {
        var self = this;
        return self.id;  
    },
    
    get : function(prop) {
    	var self = this;
    	return self.data[prop];
    },
    
    set : function(prop, value) {
    	var self = this;
    	self.data[prop] = value;
    },
    
    setProperties : function(props) {
    	var self = this;
    	for(var key in props) self.data[key] = props[key];
    },
    
    p2l : function() {
    	var self = this;
    	if(!self.id) {
    		var list = [];
    		for(var i=0; i < self.list.length; i++) list.push(self.list[i].data);
    	} else {
    		var list = self.data;
    		list.className = self.className;
    	}
    	return list;
    },
    
    fetch : function() {
    	var self = this;
    	if(self.hasMaster) {
    		var instance_table = self.tableName;
    		var master_table = instance_table+'_master';
    		var sql = "\
SELECT instance.sync AS SYNC, instance.id AS id, instance.data AS data, master.id AS mid, master.data AS master \
FROM "+instance_table+" instance, "+master_table+" master \
WHERE instance.mid = master.id AND instance.id=? LIMIT 1";
    	} else {
    		var sql = "SELECT sync AS SYNC, id AS id, data AS data FROM "+self.tableName+" WHERE id=? LIMIT 1";
    	}
    	var record = App.DB2.fetchOneRowToHashArray(sql, [self.id]);
    	for(var prop in record) {
    		if('data' == prop || 'master' == prop) {
    			if(self.encryptData) record[prop] = decrypt(record[prop]);
    			try { self[prop] = JSON.parse(record[prop]); } catch(E) { self[prop] = {}; }
    			if(null == self[prop]) self[prop] = {};
    			if('undefined' != typeof self[prop].i18n && self[prop].i18n && self._LANG) lang(self[prop], self._LANG);
    		}
    		else self[prop] = record[prop];
    	}
    	for(var prop in record) { self._isNew = false; self._fetched = true; return true; } return false;
    },
    
    fetchAll : function(param) {
    	var self = this;
    	
    	param 	= param || {};
    	where 	= param.where 	|| 1;
    	vararg 	= param.vararg 	|| [];
    	order 	= param.order 	|| {};
    	limit   = param.limit   || '';
    	
    	var order_list = [];
    	for(var field in order) order_list.push(field+' '+order[field]);
    	order = '';
    	if(order_list.length) order = "ORDER BY "+order_list.join(', ');
    	
    	var myClass = self.getClass();
    	if(self.hasMaster) {
    		var instance_table = self.tableName;
    		var master_table = instance_table+'_master';
    		var sql = "\
SELECT instance.sync AS SYNC, instance.id AS id, instance.data AS data, master.id AS mid, master.data AS master \
FROM "+instance_table+" instance, "+master_table+" master \
WHERE (instance.mid = master.id AND instance.is_deleted=0 AND instance.is_hidden=0) AND "+where+' '+order+' '+limit;
	        self.list = App.DB2.fetchRowsToArray(sql, vararg);
	    	for(var i=0; i < self.list.length; i++) {
	    		if(self.encryptData) self.list[i].data = decrypt(self.list[i].data);
	    		try { self.list[i].data = JSON.parse(self.list[i].data); } catch(E) { self.list[i].data = {}; }
	    		if(null == self.list[i].data) self.list[i].data = {};
	    		if('undefined' != typeof self.list[i].data.i18n && self.list[i].data.i18n && self._LANG) lang(self.list[i].data, self._LANG);
	    		if(self.encryptMaster) self.list[i].master = decrypt(self.list[i].master);
	    		try { self.list[i].master = JSON.parse(self.list[i].master); } catch(E) { self.list[i].master = {}; }
	    		if(null == self.list[i].master) self.list[i].master = {};
	    		if('undefined' != typeof self.list[i].master.i18n && self.list[i].master.i18n && self._LANG) lang(self.list[i].master, self._LANG);
	    		self.list[i] = new myClass({id:self.list[i].id, data:self.list[i].data, master:self.list[i].master});
	    	}
    	} else {
    		var sql = "\
SELECT instance.sync AS SYNC, instance.id AS id, instance.data AS data \
FROM "+self.tableName+" instance \
WHERE (instance.is_deleted=0 AND instance.is_hidden=0) AND "+where+' '+order+' '+limit;
	    	self.list = App.DB2.fetchRowsToArray(sql, vararg);
	    	for(var i=0; i < self.list.length; i++) {
	    		if(self.encryptData) self.list[i].data = decrypt(self.list[i].data);
	    		try { self.list[i].data = JSON.parse(self.list[i].data); } catch(E) { self.list[i].data = {}; }
	    		if(null == self.list[i].data) self.list[i].data = {};
	    		if('undefined' != typeof self.list[i].data.i18n && self.list[i].data.i18n && self._LANG) lang(self.list[i].data, self._LANG);
	    		self.list[i] = new myClass({id:self.list[i].id, data:self.list[i].data});
	    	}
    	}
    	return self.list.length;
    },
    
    fetchRelated : function(classNames, param) {
    	var self = this;
    	
    	param 	= param || {};
    	param.where 	= param.where 	|| 1;
    	param.vararg 	= param.vararg 	|| [];
    	param.order 	= param.order 	|| {};
    	param.limit   = param.limit   || '';
    	
    	classNames = classNames || [];
    	if(!helper.isArray(classNames)) classNames = [classNames];
    	
    	for(var i=0; i < classNames.length; i++) {
    		var className = classNames[i];
    		if('undefined' == typeof self.related[className]) {
    			Ti.API.error('undefined related className : '+className);
    			continue;
    		}
    		var field = self.related[className];
    		App.Helper.requireModel(className);
    		var model = new App.Model[className]();
    		// model.fetchAll(param);//'instance.'+field+'=?', self.id);               //TODO [ZJ] : for test only!
    		var vararg = param.vararg;
    		vararg.push(self.id);
    		model.fetchAll({
    			where : param.where + ' AND instance.'+field+'=?',
    			vararg : vararg,
    			order : param.order,
    			limit : param.limit,
    		})
    		self[className] = model;
    	}
    },
    
    fetchRefered : function(classNames) {
    	var self = this;
    	classNames = classNames || [];
    	if(!helper.isArray(classNames)) classNames = [classNames];
    	
    	for(var i=0; i < classNames.length; i++) {
    		var className = classNames[i];
    		if('undefined' == typeof self.refered[className]) {
    			Ti.API.error('undefined refered className : '+className);
    			continue;
    		}
    		var prop = self.refered[className];
    		App.Helper.requireModel(className);
    		var model = new App.Model[className](self.data[prop]);
    		model.fetch();
    		self[className] = model;
    	}
    },
    
    nth : function(index) {
    	var self = this;
    	if('undefined' != self.list[index]) return self.list[index];
    	else return new myClass();
    },
    
    save : function(syncDown) {
    	var self = this;
    	var timestamp = Math.round(new Date().valueOf() / 1000);
    	syncDown = syncDown || false;
    	if(!syncDown) {
	    	if(!self.id || self._isNew) {
	    		self.id = self.id || App.Helper.generateUUID(7);
	    		self.data.id = self.id;
	    		self.data.created = timestamp;
	    		self.data.changed = timestamp;
	    		self.data.updated = timestamp;
	    		self.data.deleted = 0;
	    		self.data.isDeleted = 0;
	    		self.data.isHidden  = 0;
	    		self.SYNC = 1;
	    	} else {
	    		self.data.changed = timestamp;
	    		self.data.updated = timestamp;
	    		self.SYNC += 1;
	    	}
	    	self._fillData();
    	} else {
    		if('undefined' == typeof self.data.id || null == self.data.id) return null;
    		if(!self.data.hasOwnProperty('created')) self.data.created = timestamp;
    		if(!self.data.hasOwnProperty('changed')) self.data.changed = timestamp;
    		if(!self.data.hasOwnProperty('updated')) self.data.updated = timestamp;
    		if(!self.data.hasOwnProperty('deleted')) self.data.deleted = 0;
    		if(!self.data.hasOwnProperty('isDeleted')) self.data.isDeleted = 0;
    		if(!self.data.hasOwnProperty('isHidden')) self.data.isHidden = 0;
    		self.SYNC = 0;
    	}
    	if(self.data.hasOwnProperty('forceDel') && self.data.forceDel) {
    		$.watch.dog(self, 'forceDel', 'Object ID: '+self.data.id);
    		App.DB2.deleteTableRows(self.tableName, {where:{id:self.id}});
    	} else {
	    	var record = {};
	    	self.data.SYNC = self.SYNC;
	    	for(var prop in self._propMap) if(self.data.hasOwnProperty(prop)) record[self._propMap[prop]] = ('mid'==self._propMap[prop]) ? self.data[prop]+'' : self.data[prop];
	    	record.id = self.id;
	    	record.sync = self.SYNC;
	    	record.data = JSON.stringify(self.data);
	    	if(self.encryptData) record.data = encrypt(record.data);
	    	App.DB2.replaceTableRows(self.tableName, [record]);
    	}
    	self._isNew = false;
    	
    	$.dataMgr.resetCache(self.className, self.id);
    	return self.id;
    },
    
    remove : function(forceDel) {
    	var self = this;
    	if(self._isNew) self.fetch();
    	if(self._isNew) return;
    	forceDel = forceDel || false;
    	if(forceDel) self.data.forceDel = true;
    	self.data.deleted = Math.round(new Date().valueOf() / 1000);
    	self.data.isDeleted = 1;
    	self.save();
    },
    
    saveSyncDown : function(objects) {
    	var self = this;
    	objects = objects || [];
    	for(var i=0, j=0; i < objects.length; i++) {
    		try { var data = JSON.parse(objects[i]) } catch (E) { data = false };
    		if(false === data || !data.hasOwnProperty('id')) continue;
    		var myClass = self.getClass();
    		var anObj = new myClass({id:data.id,data:data});
    		anObj.save(true);
    		j++;
    	}
    	return j;
    },
    
    getSyncUp : function() {
    	var self = this;
    	var sql = "SELECT data AS data FROM "+self.tableName+" WHERE sync<>0 LIMIT 1000"; //TODO [ZJ] : sync up LIMIT 1000 each time.
    	var rows = App.DB2.fetchRowsToArray(sql);
    	var objects = [];
    	for(var i=0; i < rows.length; i++) {
    		if(self.encryptData) rows[i].data = decrypt(rows[i].data);
    		objects.push(rows[i].data);
    	}
    	return objects;
    },
    
    syncDone : function(callbacks) {
    	var self = this;
    	callbacks = callbacks || [];
		if(self.removeSynced) {
	        var objectIDs = [];
	        for(var objID in callbacks) objectIDs.push(objID);
	        App.DB2.deleteTableRows(self.tableName, {where:{id:objectIDs}});
	        return objectIDs.length;
        } else {
	        var length=0;
	        var sync = {};
	        for(var objID in callbacks) {
	            var value = callbacks[objID];
	            if(!sync.hasOwnProperty(value)) sync[value] = [];
	            sync[value].push(objID);
	            length++;
	        }
	        for(var value in sync) {
	            var param = {
	                set   :{sync:0},
	                where :{sync:value},
	            }
	            param.where.id = sync[value];
	            App.DB2.updateTableRows(self.tableName, param);
	        }
        }
        return callbacks.length;
    },
    
    count : function() {
        var self = this;
        return self.list.length;
    },
    
    removeAll : function() {
        var self = this;
        App.DB2.deleteTableRows(self.tableName);
    },
    
    reset : function() {
    	var self = this;
    	self.removeAll();
    },
    
    _getDefaultData : function() {
    	var self = this;
    	return {};
    },
    
    _fillData : function() {
    	var self = this;
    	
    	var defaultData = self._getDefaultData();
    	for(var key in defaultData) if(!(key in self.data)) self.data[key] = defaultData[key];
    },
});

function encrypt(str) {
	return base64.encrypt(str);
}

function decrypt(str) {
	return base64.decrypt(str);
}

function lang(object, lang) {
	for(var key in object) {
		var langKey = key+lang;
		if('undefined' != typeof object[langKey]) object[key] = object[langKey];
		else if(helper.isObject(object[key]) || helper.isArray(object[key])) arguments.callee(object[key], lang);
	}
}

module.exports = WOObject;
/**
 * End of file: WOObject.js
 */