/**
 * WOOffer
 * 
 * @author zhoujun@waveo.com
 */

Ti.include('/globals.js');
App.Helper.requireModel('WOOfferMaster');

var WOOffer = App.Class(App.Model.WOObject, {
    className : 'WOOffer',
    alias : 'offerins',
    tableName : 'offer',
    encryptData : true && App.Config.MODEL_ENCRYPT,
    encryptMaster : true && App.Config.MODEL_ENCRYPT,
    hasMaster : true,
    STATUS : {
		valid 			:   'valid',
		redeemed		: 	'redeemed',
    	locked 			: 	'locked',
		unstart			: 	'unstart',
    	expiry 			: 	'expiry',
    	fullyRedeemed   :   'fullyRdm',
    },
    
    init : function(param) {
        var self = this;
        self.base();
        self.initProperties(param);
    },
    
    initProperties : function(param) {
    	var self = this;
        self._propMap['masterID']	= 'mid';
        self._propMap['cardID']	= 'card_id';
        self.base(param);
    },
    
    p2l : function() {
    	var self = this;
    	if(!self.id) {
    		var list = [];
    		for(var i=0; i < self.list.length; i++) list.push(self._p2l(self.list[i].master, self.list[i].data));
    	} else {
    		var list = self._p2l(self.master, self.data);
    	}
    	return list;
    },
    
    _p2l : function(master, instance) {
    	var self = this;
		var list = {};
		list.className			= self.className;
		list.id 				= instance.id;
		list.masterID 			= master.id;
		list.cardID 			= instance.cardID;
		list.brandName			= master.brandName || '';
		list.tc					= master.tc || '';
		list.issued				= instance.issued;
		list.validBegin			= instance.validBegin || master.validBegin;
		list.validEnd			= instance.validEnd || master.validEnd;
		list.barcodeValue		= instance.barcodeValue;
		list.barcodeType		= master.barcodeType;
		list.displayCode		= instance.displayCode;	
		list.maxRedeem			= master.maxRedeem || 1;
		list.totalRedeem     	= instance.totalRedeem;
		list.canRedeemTimes		= (-1 == master.maxRedeem) ? -1 : list.maxRedeem - list.totalRedeem;
		list.redeemInfo			= instance.redeemInfo;
		list.status				= instance.status;
		list.showName			= 'undefined' != typeof master.showName ? master.showName : true;
		list.showRdmBtn			= ('undefined' == typeof master.showRdmBtn) ? true : master.showRdmBtn;
		list.passwords			= {};
		
		list.STATUS				= self.STATUS;

		if(master.isMystery && instance.isLocked) {
			list.name				= master.mystery.name;
			list.desc 				= master.mystery.desc;
			list.shortDesc			= master.mystery.shortDesc;
			list.image				= master.mystery.image;
			list.isLocked			= true;
			list.status				= self.STATUS.locked;
			
			if(master.password) {
				var password = master.password.replace(/(^\s*)|(\s*$)/g, "");
				var passwords = password.split(',');
				for(var i=0; i < passwords.length; i++) list.passwords[passwords[i]] = passwords[i];
			}
		} else {
			list.name				= master.name;
			list.desc 				= master.desc;
			list.shortDesc			= master.shortDesc;
			list.image				= master.image;
			list.isLocked			= false;
			
			if(list.canRedeemTimes > 0 || -1 == list.canRedeemTimes) {
				var timeNow = Math.round(new Date().valueOf() / 1000);
				if(master.fullyRedeemed) list.status = self.STATUS.fullyRedeemed;
				else if(timeNow < list.validBegin) list.status = self.STATUS.unstart;
				else if(timeNow > list.validEnd) list.status = self.STATUS.expiry;
				else list.status = self.STATUS.valid;
			} else {
				list.status 		= self.STATUS.redeemed;
			}
		}
		
		return list;
    },
    
    _getDefaultData : function() {
    	var self = this;
    	
    	var masterData = $.dataMgr.get('WOOfferMaster', self.data.masterID);    	
    	
    	var defaultData = {
    		masterID : self.data.masterID,
    		cardID : self.data.cardID,
    		issued : Math.round(new Date().valueOf() / 1000),
    		validBegin : masterData.validBegin,
    		validEnd : masterData.validEnd,
    		barcodeValue : masterData.barcodeValue,
    		displayCode : masterData.displayCode,
    		totalRedeem : 0,
    		redeemInfo : [],
    	};
    	
    	return defaultData;
    },
    
    doUnlock : function() {
    	var self = this;
    	
    	if(!self._fetched && !self.fetch()) return;
    	
    	self.data.isLocked = 0;
    	self.save();
    },
    
    doRedeem : function() {
    	var self = this;
    	
    	if(!self._fetched && !self.fetch()) return;
    	
    	var location = $.configMgr.getLocationInfo();
    	
    	self.data.totalRedeem++;
    	self.data.redeemInfo.push({
    		when : Math.round(new Date().valueOf() / 1000),
    		where : {coords:location},
    		who : null,
    	});
    	self.save();
    },
    
    doReturn : function() {
    	var self = this;
    	
    	if(!self._fetched && !self.fetch()) return;
    	
    	self.data.isReturn = 1;
    	self.data.returned = Math.round(new Date().valueOf() / 1000);
    	self.data.deleted = Math.round(new Date().valueOf() / 1000);
    	self.data.isDeleted = 1;    	
    	self.save();
    },
    
    //---------  Reminder Manager support methods  --------------
        
    reminderEventTime : function(reminderType) {
        var self = this;
        var offerExpiryDate = null;
        var offerExpiry = Number(self.data.validEnd);
        if(0 != offerExpiry) {
            offerExpiryDate = new Date(offerExpiry*1000);  // convert to milliseconds
        }
        return offerExpiryDate;
    },

    /**
     * @param   {object}    of these attributes:
     *          reminderType   - see default_config.js for reminderTypes
     *          duration       - time to Event (in milliseconds)
     * @return  {string}    notification message (localized)
     */
    reminderMessage : function(reminderParam) {
        var self = this;
        
        var offerData = self.p2l();
        var message = '';
    	if(reminderParam.duration==0){
    		message = String.format(App.Helper.L('ReminderMsg_OfferExpiringToday'), offerData.name);
    	}else if(reminderParam.duration==1){
    		message = String.format(App.Helper.L('ReminderMsg_OfferExpiringTomorrow'), offerData.name);
    	}else{
    		message = String.format(App.Helper.L('ReminderMsg_OfferExpiringInNDays'), offerData.name, reminderParam.duration);
    	}          
        return message;
    },    
});

module.exports = WOOffer;
/**
 * End of file: WOOffer.js
 */