/**
 * WORewardset
 * 
 * @author zhoujun@waveo.com
 */

Ti.include('/globals.js');
App.Helper.requireModel('WORewardsetMaster');

var WORewardset = App.Class(App.Model.WOObject, {
    className : 'WORewardset',
    alias : 'rwdsetins',
    tableName : 'rewardset',
    hasMaster : true,
    
    init : function(param) {
        var self = this;
        self.base();
        self.initProperties(param);
    },
    
    initProperties : function(param) {
    	var self = this;
        self._propMap['masterID']	= 'mid';
        self._propMap['cardID']		= 'card_id';
        self.base(param);
    },

    p2l : function() {
    	var self = this;
    	if(!self.id) {
    		var list = [];
    		for(var i=0; i < self.list.length; i++) list.push(self._p2l(self.list[i].master, self.list[i].data));
    	} else {
    		var list = self._p2l(self.master, self.data);
    	}
    	return list;
    },
    
    _p2l : function(master, instance) {
    	var self = this;

		var list = {};
		list.className			= self.className;
		list.id 				= instance.id;
		list.masterID 			= master.id;
		list.cardID 			= instance.cardID;
		list.cardMID 			= master.cardMID;
		list.name				= master.name;
		list.desc 				= master.desc;
		list.shortDesc			= master.shortDesc;
		list.tc					= master.tc;
		list.brandLogo          = master.brandLogo;
		list.status				= instance.status;
		list.isFinished			= 1 == instance.status ? true : false;
		list.notStarted         = new Date() < new Date(Number(instance.validBegin || master.validBegin)*1000) ? true : false;
		list.isExpired 			= new Date() > new Date(Number(instance.validEnd || master.validEnd)*1000) ? true : false;
		list.stampsDailyLimit	= master.stampsDailyLimit || 0;
		list.validBegin			= instance.validBegin || master.validBegin;
		list.validEnd			= instance.validEnd || master.validEnd;
		list.level				= [];
		for(var i=0; i < instance.level.length; i++) {
			var aLevelIns = instance.level[i];
			var aLevelMas = master.level[i];
			list.level[i] = {
				id				: aLevelIns.id,
				name			: aLevelMas.name,
				showName        : 'undefined' != typeof aLevelMas.showName ? aLevelMas.showName : true,
				desc			: aLevelMas.desc,
				image			: aLevelMas.image,
				stampSize		: aLevelMas.stampSize,
				tc				: aLevelMas.tc,
				offerID			: aLevelIns.offerID,
				validBegin		: aLevelIns.validBegin || aLevelMas.validBegin,
				validEnd		: aLevelIns.validEnd || aLevelMas.validEnd,
				stampsDailyLimit : aLevelMas.stampsDailyLimit || 0,
				notStarted      : new Date() < new Date(Number(aLevelIns.validBegin || aLevelMas.validBegin)*1000) ? true : false,
				isExpired		: new Date() > new Date(Number(aLevelIns.validEnd || aLevelMas.validEnd)*1000) ? true : false,
				isFinished		: 1 == aLevelIns.status ? true : false,
				offer           : aLevelMas.offer,
				stamp			: [],
			};
			var collectedStamps = 0;
			for(var k=0; k < aLevelIns.stamp.length; k++) {
				
				var aStampIns = aLevelIns.stamp[k];
				var aStampMas = aLevelMas.stamp[k];
				list.level[i].stamp[k] = {
					id			: aStampMas.id,
					name 		: aStampMas.name,
					desc		: aStampMas.desc,
					image		: aStampMas.statesImage[aStampIns.stateID],
					stateID		: aStampIns.stateID,
					when		: aStampIns.when,
					where		: aStampIns.where,
					who			: aStampIns.who,
					drill		: aStampMas.drill,
					statesImage : aStampMas.statesImage,
				};
				if(aStampIns.stateID > 0) collectedStamps++;
			}
			list.level[i].collectedStamps = collectedStamps;
			if(list.level[i].offer && list.level[i].offer.collectStamp) list.level[i].isChecking = (collectedStamps >= list.level[i].offer.collectStamp && !list.level[i].isFinished);
			else list.level[i].isChecking = (collectedStamps >= list.level[i].stamp.length && !list.level[i].isFinished);
		}
        return list;
    },
    
    _getDefaultData : function() {
    	var self = this;
    	
    	var masterData = $.dataMgr.get('WORewardsetMaster', self.data.masterID);
    	var cardData = $.dataMgr.get('WOCard', self.data.cardID);
    	    	
    	if('undefined' == typeof masterData.beginRule) masterData.beginRule = {};
    	if('undefined' == typeof masterData.endRule) masterData.endRule = {};
    	
    	var validBegin = _getUnixTimeByRule(masterData.validBegin, masterData.beginRule, cardData);
    	if('undefined' == typeof masterData.endRule.base || 'validBegin' == masterData.endRule.base) masterData.endRule.base = Number(validBegin)*1000;
    	var validEnd = _getUnixTimeByRule(masterData.validEnd, masterData.endRule, cardData);
    	
    	var defaultData = {
    		masterID 	: masterData.id,
    		cardID 		: self.data.cardID,
			validBegin	: validBegin,
			validEnd	: validEnd,
    		level : [],
    	};
    	
    	for(var i=0; i < masterData.level.length; i++) {
    		var curMasterLevel = masterData.level[i];
    		
    		if('undefined' == typeof curMasterLevel.beginRule) curMasterLevel.beginRule = {};
    		if('undefined' == typeof curMasterLevel.endRule) curMasterLevel.endRule = {};
    		
    		var levelValidBegin = _getUnixTimeByRule(curMasterLevel.validBegin, curMasterLevel.beginRule, cardData);
    		if('undefined' == typeof curMasterLevel.endRule.base || 'validBegin' == curMasterLevel.endRule.base) curMasterLevel.endRule.base = Number(levelValidBegin)*1000;
    		var levelValidEnd = _getUnixTimeByRule(curMasterLevel.validEnd, curMasterLevel.endRule, cardData);
    		defaultData.level[i] = {
    			id : i,
    			validBegin : levelValidBegin,
    			validEnd : levelValidEnd,
    			stamp : [],
    			offerID : null,
    		};
    		for(var k=0; k < curMasterLevel.stamp.length; k++) {
    			var curMasterStamp = curMasterLevel.stamp[k];
    			defaultData.level[i].stamp[k] = {
    				id : curMasterStamp.id,
    				stateID : 0,
    				when : null,
    				where : null,
    				who : null,
    			};
    		}
    	}
    	
    	return defaultData;
    },
    
    issueStamp : function(stampID, stateID, quantity){
    	var self = this;
    	
    	var result = {
    		type : 'issueStamp',
    		remain 			: quantity,
    		offerIDs 		: [],
    		stampIDs 		: [],
    		checkOnline 	: false,    
    		dailyLimited    : false,		
    	};

    	var remain = quantity;
    	if(!self._fetched && !self.fetch()) return result;
    	var rewardsetData = self.p2l();
    	
    	//check isStarted or isExpired
    	if(rewardsetData.notStarted) {
    		$.app.showAlert(String.format(App.Helper.L('ALERT_RewardsetNotStarted'), App.Helper.humaneDate(new Date(Number(rewardsetData.validBegin)*1000), 366)));
    		return result;
    	}
    	if(rewardsetData.isExpired) {
    		$.app.showAlert(App.Helper.L('ALERT_REWARDSETEXPIRED'));
    		return result;
    	}
    	
    	var today = new Date();
    	today = today.getFullYear()+''+today.getMonth()+''+today.getDate();
    	
    	var todayCollect = 0;
    	var issuedOfferIDs = [];
    	var issuedStampIDs = [];
    	var checkOnline = false;
    	var dailyLimited = false;
    	for(var i=0; i<rewardsetData.level.length; i++){
    		var curLevel = rewardsetData.level[i];
    		if(curLevel.offerID) continue;								//TODO [ZJ] if(curLevel.offerID) continue;
    		var collectedStamps = 0;
    		for(var k=0; k<curLevel.stamp.length; k++) if(curLevel.stamp[k].stateID > 0) collectedStamps++;
    		curLevel.offer.collectStamp = curLevel.offer.collectStamp || curLevel.stamp.length;
    		if(collectedStamps >= curLevel.offer.collectStamp) continue;  								//TODO [ZJ] issueOffer ?
    		for(var x=0; x<curLevel.stamp.length; x++){
    			var curStamp = curLevel.stamp[x];
    			if(stampID == curStamp.id && stateID != curStamp.stateID){
    				//check level valid begin date
    				if(curLevel.notStarted) {
    					$.app.showAlert(String.format(App.Helper.L('ALERT_RewardsetNotStarted'), App.Helper.humaneDate(new Date(Number(curLevel.validBegin)*1000), 366)));
    					return result;
    				}
    				//check level valid end date
    				if(curLevel.isExpired) {
    					$.app.showAlert(App.Helper.L('ALERT_REWARDSETEXPIRED'));
    					return result;
    				}
    				//daily limit check : 
    				var rewardsetStampsDailyLimit = rewardsetData.stampsDailyLimit || 0;
    				if(rewardsetStampsDailyLimit > 0) {
    					for(var v=0; v<rewardsetData.level.length; v++) {
    						var aLevel = rewardsetData.level[v];
    						for(var y=0; y<aLevel.stamp.length; y++) {
    							var aStamp = aLevel.stamp[y];
    							if(aStamp.stateID > 0) {
    								var collected = new Date(aStamp.when * 1000);
    								if(!collected) continue;
    								var collectedDate = collected.getFullYear()+''+collected.getMonth()+''+collected.getDate();
    								if(collectedDate == today) todayCollect++;
    							}
    						}
    						if(todayCollect >= rewardsetStampsDailyLimit) {
    							$.app.showAlert(App.Helper.L('ALERT_STAMPDAILYLIMIT'));
    							result.dailyLimited = true;
    							return result;
    						}
    					}
    				}
    				var levelTodayCollect = 0;
    				var levelStampsDailyLimit = curLevel.stampsDailyLimit || 0;
    				if(levelStampsDailyLimit > 0) {
    					for(var y=0; y<curLevel.stamp.length; y++) {
    						var aStamp = curLevel.stamp[y];
    						if(aStamp.stateID > 0) {
    							var collected = new Date(aStamp.when * 1000);
    							if(!collected) continue;
    							var collectedDate = collected.getFullYear()+''+collected.getMonth()+''+collected.getDate();
    							if(collectedDate == today) levelTodayCollect++;
    						}
    					}
    					if(levelTodayCollect >= levelStampsDailyLimit) {
    						$.app.showAlert(App.Helper.L('ALERT_STAMPDAILYLIMIT'));
    						result.dailyLimited = true;
    						return result;
    					}
    				}
    				
    				self.data.level[i].stamp[x].stateID = stateID;
    				self.data.level[i].stamp[x].when = Math.round(new Date().valueOf() / 1000);
    				var stampImageID = rewardsetData.level[i].stamp[x].statesImage[stateID];
    				issuedStampIDs.push(stampImageID);
    				remain--;
    				if(stateID > 0) {
    					collectedStamps++;
    					if(collectedStamps >= curLevel.offer.collectStamp) { 		//TODO [ZJ] issueOffer ?
    						if(!curLevel.offer.online) {
    							self.data.level[i].status = 1; //1 : level is completed
	    						var offerID = self._issueOffer(rewardsetData, curLevel);
	    						self.data.level[i].offerID = offerID;
	    						issuedOfferIDs.push(offerID);
	    						break;
    						} else {
    							checkOnline = true;
    						}
    					}
    				}
    			}
    			if(!remain) break;
    		}
    		if(!remain) break;
    	}
    	if(remain != quantity) {
    		self.data.status = 1; // assume rewardset is completed
    		for(var i=0; i<self.data.level.length; i++) if(0 == self.data.level[i].status) { self.data.status = 0; break; }
    		self.save();
    	}
    	var result = {
    		type : 'issueStamp',
    		remain 			: remain,
    		offerIDs 		: issuedOfferIDs,
    		stampIDs 		: issuedStampIDs,
    		checkOnline 	: checkOnline,
    		dailyLimited	: dailyLimited,
    	};
    	
    	return result;
    },
   
	deductStamp : function(stampID, stateID, quantity){
    	var self = this;
    	
    	var result = {
    		type 		: 'deductStamp',
    		remain 		: quantity,
    		offerIDs 	: [],
    		stampIDs 	: [],
    		checkOnline : false,    		
    	};
    	    	
    	var remain = quantity;
    	if(!self._fetched && !self.fetch()) return result;
    	var rewardsetData = self.p2l();
    	
    	var returnedOfferIDs = [];
    	var returnedStampIDs = [];
    	var checkOnline = false;
    	for(var i=rewardsetData.level.length-1; i>=0; i--){
    		var curLevel = rewardsetData.level[i];
    		
    		for(var k=curLevel.stamp.length-1; k>=0; k--){
    			var curStamp = curLevel.stamp[k];
    			if(stampID == curStamp.id && stateID != curStamp.stateID){
    				self.data.level[i].stamp[k].stateID = stateID;
    				self.data.level[i].stamp[k].when = Math.round(new Date().valueOf() / 1000);
    				var stampImageID = rewardsetData.level[i].stamp[k].statesImage[stateID];
    				returnedStampIDs.push(stampImageID);
    				remain--;
    			}
    			if(!remain) break;
    		}
    		//----- check offer return ------
    		var collectedStamps = 0;
    		curLevel.offer.collectStamp = curLevel.offer.collectStamp || curLevel.stamp.length;
    		for(var x=0; x<curLevel.stamp.length; x++) if(self.data.level[i].stamp[x].stateID > 0) collectedStamps++;
    		if(collectedStamps < curLevel.offer.collectStamp) {
				self.data.level[i].status = 0; //0 : level not completed
				self.data.status = 0; //0 : rewardset not completed
				if(curLevel.offerID) { // return related offer
    				self._returnOffer(curLevel.offerID);
    				self.data.level[i].offerID = null;
    				returnedOfferIDs.push(curLevel.offerID);
    				checkOnline = true;
    			}
    		}
    		if(!remain) break;
    	}
    	if(remain != quantity) {
    		self.data.status = 1; // assume rewardset is completed
    		for(var i=0; i<self.data.level.length; i++) if(0 == self.data.level[i].status) { self.data.status = 0; break; }
    		self.save();
    	}
    	
    	var result = {
    		type 		: 'deductStamp',
    		remain 		: remain,
    		offerIDs 	: returnedOfferIDs,
    		stampIDs 	: returnedStampIDs,
    		checkOnline : checkOnline,
    	};
    	
    	return result;
    },
    
    _issueOffer : function(rewardsetData, aLevel) {
    	var self = this;
    	if(!self._fetched) return;

    	var offerID = $.cardMgr.issueOffer({
    		data : {
    			offerMID 	:  aLevel.offer.offerMID,
    			cardID		: rewardsetData.cardID,
    			rewardsetID : rewardsetData.id,
    			levelID 	: aLevel.id,
    		}
    	});
    	
    	return offerID;
    },
    
    _returnOffer : function(offerID) {
    	var self = this;
    	
    	var offer = $.dataMgr.newModel('WOOffer', offerID);
    	offer.doReturn();
    },
    
});

function _getUnixTimeByRule(time, rule, card) {
	
    	if(!isNaN(time)) { // is numberic, ok, fixed date, use it.
    		return time;
    	} else if('cardJoin' == time) {
    		return card.joined;
    	} else if('cardExpiry' == time) {
    		return card.expiry;
    	} else if('realtime' == time) {
    		return App.Moment().unix();
    	} else if('ruleBased' == time) {
    		if('undefined' == typeof rule.base) rule.base = 'realtime';
    		if(!isNaN(rule.base)) {
    			var base = rule.base; 
    		} else if('cardJoin' == rule.base) {
    			var base = Number(card.joined)*1000;
    		} else if('cardExpiry' == rule.base) {
    			var base = Number(card.expiry)*1000;
    		} else {
    			var base = new Date();
    		}
    		if('undefined' == typeof rule.eom) rule.eom = false;
			if(rule.eom) return App.Moment(base).add(rule.key, Number(rule.value)+1).date(0).endOf('day').unix();
			else return App.Moment(base).add(rule.key, Number(rule.value)).endOf('day').unix();
    	}
}

module.exports = WORewardset;
/**
 * End of file: WORewardset.js
 */