/**
 * WORewardsetMaster
 * 
 * @author zhoujun@waveo.com
 */

Ti.include('/globals.js');

var WORewardsetMaster = App.Class(App.Model.WOObject, {
    className : 'WORewardsetMaster',
    alias : 'rwdsetmas',
    tableName : 'rewardset_master',
    related : {
    	'WORewardset' 	: 'mid',
    },
    
    init : function(param) {
        var self = this;
        self.base();
        self.initProperties(param);
    },
    
    initProperties : function(param) {
    	var self = this;
        self._propMap['cardMID']	= 'card_mid';
        self.base(param);
    },
    
    save : function(syncDown) {
    	var self = this;
    	self.base(syncDown);
    	
    	//Sync Image
    	var images = [];
    	if(self.data.brandLogo) images.push(self.data.brandLogo);
    	
    	if('undefined' != typeof self.data.level) {
	    	for(var i=0; i<self.data.level.length; i++) {
	    		if(self.data.level[i].image) images.push(self.data.level[i].image);
	    		if('undefined' != typeof self.data.level[i].stamp) {
	    			for(var j=0; j<self.data.level[i].stamp.length; j++) {
	    				for(var k=0; k<self.data.level[i].stamp[j].statesImage.length; k++) {
	    					if(self.data.level[i].stamp[j].statesImage[k]) {
	    						images.push(self.data.level[i].stamp[j].statesImage[k]);
	    					}
	    				}
	    			}
	    		}
	    	}
    	}
		$.syncMgr.queueImages({ urls: images });
    },
    
    deductStamp : function(stampID, stateID, quantity){
    	var self = this;
    	
    	var deductResult = {
    		type : 'deductStamp',
    		total : quantity,
    		remain : quantity,
    		offerIDs : [],
    		stampIDs : [],
    		checkOnline : false,
    	};
    	
 		if(!self._fetched && !self.fetch()) return deductResult;
    	var masData = self.p2l();
    	
    	self.fetchRelated('WORewardset', {
    		order : { 'instance.created' : 'DESC' }
    	});
    	
    	//check card instance
    	var cardMaster = $.dataMgr.newModel('WOCardMaster', masData.cardMID);
    	cardMaster.fetchRelated('WOCard');
    	if(!cardMaster.WOCard.list.length) return deductResult;
    	
    	// if(!self.WORewardset.list.length) return remain;
    	for(var i=0; i<self.WORewardset.list.length; i++) {
    		var aRewardset = self.WORewardset.nth(i).p2l();
    		if(aRewardset.isExpired) continue; // skip expired rewardset.
    		var result = self.WORewardset.list[i].deductStamp(stampID, stateID, deductResult.remain);
    		deductResult.offerIDs = deductResult.offerIDs.concat(result.offerIDs);
    		deductResult.stampIDs = deductResult.stampIDs.concat(result.stampIDs);
       		deductResult.remain = result.remain;
    		deductResult.checkOnline = deductResult.checkOnline || result.checkOnline;
    		if(!deductResult.remain) break;
    	}
    	
    	return deductResult;
    },
    
    issueStamp : function(stampID, stateID, quantity, dailyLimit){
    	var self = this;
    	
    	dailyLimit = 'undefined' == typeof dailyLimit ? 0 : dailyLimit;
    	
    	var issueResult = {
    		type : 'issueStamp',
    		total : quantity,
    		remain : quantity,
    		offerIDs : [],
    		stampIDs : [],
    		checkOnline : false,
    		dailyLimited : false,
    	};    	
    	    	
    	if(!self._fetched && !self.fetch()) return issueResult;
    	var masData = self.p2l();
    	masData.notStarted = isNaN(masData.validBegin) ? false : ((new Date() < new Date(Number(masData.validBegin)*1000)) ? true : false);
    	if('undefined' == typeof masData.isExpired) masData.isExpired = isNaN(masData.validEnd) ? false : ((new Date() > new Date(Number(masData.validEnd)*1000)) ? true : false);

    	self.fetchRelated('WORewardset', {
    		order : { 'instance.created' : 'ASC' }
    	});
    	
    	//check card instance
    	var cardMaster = $.dataMgr.newModel('WOCardMaster', masData.cardMID);
    	cardMaster.fetchRelated('WOCard');
    	if(!cardMaster.WOCard.list.length) {
    		_cardMaster = cardMaster.p2l();
    		$.app.showAlert(String.format(App.Helper.L('ALERT_NeedAddCardForReward'), _cardMaster.getFullName));
    		
    		return issueResult;
    	} else {
    		cardID = cardMaster.WOCard.list[0].id;
    		issueResult.cardID = cardID;
    	}
    	
    	if(dailyLimit > 0) {
	    	// Check Daily limit
	    	var today = new Date();
	    	today = today.getFullYear()+''+today.getMonth()+''+today.getDate();
	    	var stampCollectedToday = 0;
	    	for(var i=0; i<self.WORewardset.list.length; i++) {
	    		var aRewardset = self.WORewardset.nth(i).p2l();
	    		for(var j=0; j<aRewardset.level.length; j++) {
	    			var aLevel = aRewardset.level[j];
	    			for(var k=0; k<aLevel.stamp.length; k++) {
	    				var aStamp = aLevel.stamp[k];
	    				if(aStamp.stateID > 0 && aStamp.id == stampID) {
	    					var collected = new Date(aStamp.when * 1000);
		    				if(!collected) continue;
		    				var collectedDate = collected.getFullYear()+''+collected.getMonth()+''+collected.getDate();
		    				if(collectedDate == today) stampCollectedToday++;
	    				}
	    			}
	    		}
	    	}
	    	if(stampCollectedToday >= dailyLimit) {
	    		issueResult.dailyLimited = true;
	    		
	    		$.app.showAlert(App.Helper.L('ALERT_STAMPDAILYLIMIT'));
	    		return issueResult;
	    		
	    	} else {
	    		var todayLimit = dailyLimit - stampCollectedToday;
	    		issueResult.remain = issueResult.total > todayLimit ? todayLimit : issueResult.total;
	    	}
    	}
    	
    	// if(!self.WORewardset.list.length) return remain;
    	for(var i=0; i<self.WORewardset.list.length; i++) {
    		var aRewardset = self.WORewardset.nth(i).p2l();
    		if(aRewardset.notStarted || aRewardset.isExpired) continue; // skip notStarted or expired rewardset.
    		var result = self.WORewardset.list[i].issueStamp(stampID, stateID, issueResult.remain);
    		issueResult.offerIDs = issueResult.offerIDs.concat(result.offerIDs);
    		issueResult.stampIDs = issueResult.stampIDs.concat(result.stampIDs);
       		issueResult.remain = result.remain;
    		issueResult.checkOnline = issueResult.checkOnline || result.checkOnline;
    		issueResult.dailyLimited = result.dailyLimited;
    		if(!issueResult.remain || issueResult.dailyLimited) break;
    	}
    	if(0 == stateID || issueResult.dailyLimited) return issueResult;
    	
    	if(!masData.isExpired && !masData.notStarted) {
			if(issueResult.remain && (masData.maxsets > self.WORewardset.list.length)) {
				var newInsLimit = masData.maxsets-self.WORewardset.list.length;
				var safeValue = issueResult.remain;
				while(safeValue-- && issueResult.remain && newInsLimit--) {
					//create rewardset instance
					var rewardsetInsObj = $.dataMgr.newModel('WORewardset');
					rewardsetInsObj.data.masterID = masData.id;
					rewardsetInsObj.data.cardID = cardID;
					rewardsetInsObj.save();
					
					var result = rewardsetInsObj.issueStamp(stampID, stateID, issueResult.remain);
					issueResult.offerIDs = issueResult.offerIDs.concat(result.offerIDs);
	    			issueResult.stampIDs = issueResult.stampIDs.concat(result.stampIDs);
	    			issueResult.checkOnline = issueResult.checkOnline || result.checkOnline;
	    			issueResult.remain = issueResult.remain > result.remain ? result.remain : issueResult.remain;
	    			issueResult.dailyLimited = result.dailyLimited;
	    			
	    			if(issueResult.remain != result.remain || issueResult.dailyLimited) {
	    				break;
	    			}
				}
			} else if(issueResult.remain) $.app.showAlert(App.Helper.L('ALERT_MAXSTAMPEREACHED'));
		} else if(masData.isExpired) {
			//Rewardset is expired!
			$.app.showAlert(App.Helper.L('ALERT_REWARDSETEXPIRED'));
		} else if(masData.notStarted) {
			//Rewardset is not started!
			$.app.showAlert(String.format(App.Helper.L('ALERT_RewardsetNotStarted'), App.Helper.humaneDate(new Date(Number(masData.validBegin)*1000), 366)));
		}
		
		return issueResult;
    },
});

module.exports = WORewardsetMaster;
/**
 * End of file: WORewardsetMaster.js
 */