/**
 * Produces Form Objects from: form definition + form data + styles
 * 
 * @author young@waveo.com
 */
Ti.include('/globals.js');
if (App.Config.IS_ANDROID) {
	Ti.include('/waveo/core/WOAndroidUtils.js');
}

var WOForm = App.Class({
	className : 'WOForm',
	
	EVENT	: {
		//--- handles these events:
		refresh			: 'WOForm.refresh',
		setMode			: 'WOForm.setMode',
		reset			: 'WOForm.reset',
		update          : 'WOForm.update',
		webViewLoaded	: 'App:webview.load',
		//--- produces these events:
		submit			: 'WOForm.submit',
		cancel			: 'WOForm.cancel',
		change			: 'WOForm.change',
	},

	FIELD_EVENT	: {
		//--- handles these events:
		openWindow		: 'WOForm.openWindow',
		closeWindow		: 'WOForm.closeWindow',
	},
	
	FIELD_CLASS	: {
		text		: 'textField',
		info		: 'webViewField',
		date		: 'dateField',
		tabbar		: 'tabbedBarField',
		barcodeScan	: 'barcodeScanField',
		slider		: 'sliderField',
		switcher 	: 'switchField',
		emailDialog	: 'emailDialogField',
	//	webView		: 'webViewField',
		html		: 'webViewField',
		phone		: 'phoneField',
		link		: 'linkField',
		url			: 'linkField',
		nameOrder   : 'nameOrderField',
		city        : 'cityField',
		lang        : 'langField',
		terms		: 'tcField',
	},
    
	MODES	: ['display', 'edit'],

	init : function(param){
	},

	//-----------  Constructor  -----------
	initialize : function(param){
		var self = this;

		self.formDefn	= param.formDefn;
		self.formData	= _hasProperty(param.formData) ? param.formData : {};
		self.DATA		= _hasProperty(param.DATA) ? param.DATA : {};
		self.styleNames	= _hasProperty(param.stylePath) ? param.stylePath : '';

		self.tabGroup	= _hasProperty(param.tabGroup) ? param.tabGroup : null;
		self.navGroup	= _hasProperty(param.navGroup) ? param.navGroup : null;

		self.defaultMode = _hasProperty(param.defaultMode) ? param.defaultMode : 'display';
		
		//---  setup EXTERNAL event handling  ---
		self._eventProxy = _hasProperty(param.eventProxy) ? param.eventProxy : null;
		if(self._eventProxy) self._eventProxy.addEvents(self.EVENT);
		
		//---  setup INTRA-FORM event handling with Fields  ---
		self._formEventProxy = new App.WOEventProxy();
		self._formEventProxy.addEvents(self.FIELD_EVENT);
			
		self.VIEW		= null;
		self.sections	= [];
		self.fields		= [];		
	},

	setupEventHandlers : function(param){
		var self = this;
		
		//---  Define how we handle incoming events  -----
		if (App.Config.IS_ANDROID) {
			self._eventProxy.addEventListener(self._eventProxy.EVENT.refresh, (function(that){
				return (function(evt){
					that.refresh(evt.data);
				});
			})(self) );
			self._eventProxy.addEventListener(self._eventProxy.EVENT.setMode, (function(that){
				return (function(evt){
					that.setMode(evt.mode);
				});
			})(self) );			
		} else {
			self._eventProxy.addEventListener(self._eventProxy.EVENT.refresh, function(evt){
				self.refresh(evt.data);
			});		
			self._eventProxy.addEventListener(self._eventProxy.EVENT.setMode, function(evt){
				self.setMode(evt.mode);
			});
			self._eventProxy.addEventListener(self._eventProxy.EVENT.update, function(evt){
				self.update(evt.data);
			});
		}

		//---  Handle events from Fields  ---
		if (App.Config.IS_ANDROID) {
			self._formEventProxy.addEventListener(self.FIELD_EVENT.openWindow, (function(that){
				return (function(evt){
					that.openFieldWindow(evt);
				});
			})(self) );			
			self._formEventProxy.addEventListener(self.FIELD_EVENT.closeWindow, (function(that){
				return (function(evt){
					that.closeFieldWindow(evt);
				});
			})(self) );			
			self._formEventProxy.addEventListener(self._formEventProxy.EVENT.change, (function(that){
				return (function(evt){
					that._eventProxy.fireEvent(that.EVENT.change, evt);
				});
			})(self) );			
		} else {
			self._formEventProxy.addEventListener(self.FIELD_EVENT.openWindow, function(evt){
				self.openFieldWindow(evt);
			});
			self._formEventProxy.addEventListener(self.FIELD_EVENT.closeWindow, function(evt){
				self.closeFieldWindow(evt);
			});
			self._formEventProxy.addEventListener(self._formEventProxy.EVENT.change, function(evt){
				Ti.API.debug('field changed: '+evt.field.name+' value='+evt.value);
				self._eventProxy.fireEvent(self.EVENT.change, evt);
			});
		}
	},

	render : function(param){
		var self = this;
	
		//--- Load Styles & substitute style names in Form Definition with actual styles
		self.STYLE = App.Helper.loadStyle(self.styleNames);
		self.substituteStyleNames(self.formDefn, self.STYLE);

		self.VIEW = Ti.UI.createTableView(self.formDefn.form.style);

		//---  Header  ------------------------------------
		if(_hasProperty(self.formDefn.header)){
			self.VIEW.setHeaderView(self.getHeaderView(self.formDefn.header));
		}
		
		//---  Footer  ------------------------------------
		if(_hasProperty(self.formDefn.footer)){
			self.VIEW.setFooterView(self.getFooterView(self.formDefn.footer));
		}
	
		//---  Sections  ----------------------------------
		self.sections = [];
		self.formDefn.sections = self.formDefn.sections || [];
		var numSections = self.formDefn.sections.length;
		for(var sect=0; sect < numSections; sect++){
			
			var aSection = self.formDefn.sections[sect];
	
			//---  section header & footer  ---
			var sectionParam = {};
			if(_hasProperty(aSection.header)) sectionParam.headerView = self.getSectionHeaderView(aSection.header);
			if(_hasProperty(aSection.footer)) sectionParam.footerView = self.getSectionFooterView(aSection.footer);

			if(App.Config.IS_ANDROID) {
				if(!_hasProperty(aSection.header)) sectionParam.headerView = Ti.UI.createView({
					height: '20dp',
				});
			}
			
			var sectionView = Ti.UI.createTableViewSection(sectionParam);
	
			//---  fields  --------------------
			var numFields = aSection.fields.length;
			for(var i=0; i < numFields; i++){
	
				//--- Make field object ---
				var name = aSection.fields[i].type;
				var className = ('undefined' != typeof self.FIELD_CLASS[name]) ? self.FIELD_CLASS[name] : null;
				if(! className){
					Ti.API.error(self.className+'::render - unsupported field type: '+name);
					continue;
				}

				//--- Field Definition ---------
				var fieldDefn = aSection.fields[i];
				fieldDefn.eventProxy = self._formEventProxy;	//--- Remember: we are using formEventProxy here
				fieldDefn.useRow = true;
				fieldDefn.defaultMode = self.defaultMode;

				//--- Get Data for the field ---
				var fieldData = _hasProperty(self.formData[fieldDefn.name]) ? self.formData[fieldDefn.name] : null;
	
				//---  Make the field, pass in it's Data  ---
				var fieldFactory = App.Helper.newField(className);
				var aField = fieldFactory.make(fieldDefn, fieldData);
				self.fields.push(aField);
	
				sectionView.add(aField.getRow());
			}
			self.sections.push(sectionView);
		}
		if(numSections) self.VIEW.setData(self.sections);

		if(numSections) self.setupEventHandlers();

		return self.VIEW;
	},

	refresh : function(formData){
		var self = this;
		//--- ask our fields to change mode ---
		self._formEventProxy.fireEvent(self._formEventProxy.EVENT.refresh, { data: formData });
	},
	reset : function(){
		var self = this;
		self._formEventProxy.fireEvent(self._formEventProxy.EVENT.reset, {});
	},
	
	update : function(formData){
		var self = this;
		self._formEventProxy.fireEvent(self._formEventProxy.EVENT.update, { data: formData });
	},
	
	setMode : function(newMode){
		var self = this;
		if(newMode == 'edit'){
			$.app.doNotDisturb(true);
		}else{
			$.app.doNotDisturb(false);
		}

		//--- ask our fields to change mode ---
		self._formEventProxy.fireEvent(self._formEventProxy.EVENT.setMode, { mode: newMode });
		
		//---  subclass to inherit & implement any change to form's display mode  ---
	},
		
	getFieldValues : function(){
		var self = this;

		var formData = {};
		for(var i=0; i<self.fields.length; i++){
			formData[self.fields[i].name] = self.fields[i].getValue();
		}
		return formData;
	},

	/**
	 * Handles Field's request to provide a window to render it's extended subview
	 */
	openFieldWindow : function(evt){
		var self = this;
		
		var windowStyle = '';
		if(self.formDefn.form.properties.fieldWindow.style){
			windowStyle =self.formDefn.form.properties.fieldWindow.style;
		}

		if (App.Config.IS_ANDROID) {
			self.fieldWindow = createAndroidWindow(windowStyle);
		} else {
			self.fieldWindow = Ti.UI.createWindow(windowStyle);
		}
		//--- Pass window to callback method of WOField
		evt.callback(self.fieldWindow);		// NOTE:  field must fireEvent 'closeFieldWindow' after use		
		
		//--- Open the appropriate type of window ---
		if(self.navGroup){
			self.navGroup.open(self.fieldWindow);
		}else if(self.tabGroup){
			self.fieldWindow.tabBarHidden = true;
        	self.tabGroup.activeTab.open(self.fieldWindow);
		}else {
			self.fieldWindow.open({modal: true});
		}
	},

	closeFieldWindow : function(evt){
		var self = this;

		if(self.navGroup){
			self.navGroup.close(self.fieldWindow);
		}else if(self.tabGroup){
        	self.tabGroup.activeTab.close(self.fieldWindow, {animated:true});
		}else {
			self.fieldWindow.close({animated:true});
		}
	},

	//---  mirrors method of the same name in TableView  ---
	scrollToTop : function(top, animation){
		var self = this;

		if (App.Config.IS_ANDROID) {
			self.VIEW.scrollToIndex(top);
		} else{
			self.VIEW.scrollToTop(top, animation);		
		}
	},

	blur : function(param){
		var self = this;		
		self._formEventProxy.fireEvent(self._formEventProxy.EVENT.blur, {});
	},

	dispose : function(){
		var self = this;
		self._formEventProxy.fireEvent(self._formEventProxy.EVENT.dispose, {});
		
		//  need to dispose sections?    self.sections = [];
	},

	/**
	 * Substitute (recursively) all 'style' names under the fieldObj (tree) with the actual style
	 * definitions looked up from STYLES
	 * 
	 * @param {Object} fieldObj
	 * @param {Object} STYLES
	 */
	substituteStyleNames : function(fieldObj, STYLES){
		var self = this;
			
		for(property in fieldObj){
			if('style' == property){
				if('object' == typeof fieldObj[property]){
					continue;	// have already been substituted, skip
				}
				//--- substitute style's name with actual style object ---
				if(fieldObj[property] in STYLES){
					//Ti.API.info('Style substituted: '+fieldObj[property]+'->'+JSON.stringify(STYLES[fieldObj[property]]));
					fieldObj[property] = STYLES[fieldObj[property]];
				}else{
					Ti.API.error(self.className+'::substituteStyleNames - style ('+fieldObj[property]+') used but not declared');
				}
			}else{
				if('object' == typeof fieldObj[property]){
					//--- recursive call if property is an object ---
					self.substituteStyleNames(fieldObj[property], STYLES);
				}
			}
		}
	},
	
	getHeaderView : function(header){
		var self = this;
		return self._getSimpleTextAreaView(header);
	},
	
	getFooterView : function(footer){
		var self = this;
		return self._getSimpleTextAreaView(footer);
	},
	
	getSectionHeaderView : function(header){
		var self = this;
		return self._getSimpleTextAreaView(header);
	},
	
	getSectionFooterView : function(footer){
		var self = this;
		return self._getSimpleTextAreaView(footer);
	},
	
	_getSimpleTextAreaView : function(area){
		var self = this;
	
		var viewStyle	= area.style || {};
		var labelStyle	= (_hasProperty(area.properties) && _hasProperty(area.properties.style)) ? area.properties.style : {};
		labelStyle.text	= (_hasProperty(area.properties) && _hasProperty(area.properties.value)) ? App.Helper.L(area.properties.value, 'fieldLabel') : '';
	
		var areaView = Ti.UI.createView(viewStyle);
		var label = Ti.UI.createLabel(labelStyle);
		areaView.add(label);
		
		return areaView;
	},

	//---  FIXME   Put here for convenience first, to relocate later
	/**
 	* @param {Object} param
 	* 		html			URL or html text
 	* 		style			style for webview 	MUST not set the html or url properties
 	* 		callback		optional callback function
	*/
	getWebView : function(param){
		var self = this;
		var webLoadEvent = 'App:webview.load';
		
		if(('undefined' == typeof param.html) || (param.html == ''))	return null;
		
		var webviewStyle = param.style || {};
		var webview = Ti.UI.createWebView(param.style);
	
		//--- determine if our html is external (URL) or embedded
		var viewID = new Date().getTime();
		var isExternal = (param.html.slice(0,5)=='http:') ? true : false;
		if(isExternal){
			webview.addEventListener('load', _postLoadExternal);
			webview.url = param.html;
		}else{
			//---  embedded html  ---
			var EventName = webLoadEvent+'.'+viewID;
			Ti.App.addEventListener(EventName, _postLoadEmbedded);
			webview.html =  _injectScript(param.html, EventName);
		}
	
		return webview;
	
		//--------------  Private Methods --------------
		function _injectScript(originalHtml, eventName){
		
			var SCRIPT_ONLOAD = ' onLoad="getHeight()" ';
			var TAG_SCRIPT = '</head>';
			var TAG_ONLOAD = '<body';
	
			var SCRIPT_GETHEIGHT = '<script>function getHeight(){var D = document;var height1 = Math.max(D.body.scrollHeight, D.documentElement.scrollHeight);var height2 = Math.max(D.body.offsetHeight, D.documentElement.offsetHeight);var height3 = Math.max(D.body.clientHeight, D.documentElement.clientHeight);var documentHeight =  Math.max(	Math.max(D.body.scrollHeight, D.documentElement.scrollHeight),	Math.max(D.body.offsetHeight, D.documentElement.offsetHeight),	Math.max(D.body.clientHeight, D.documentElement.clientHeight));  Ti.App.fireEvent(\'';
			SCRIPT_GETHEIGHT += eventName+'\' , { height: documentHeight}); }; </script>';	
			
			var newHtml = originalHtml.replace(TAG_ONLOAD, TAG_ONLOAD + SCRIPT_ONLOAD);
			newHtml = newHtml.replace(TAG_SCRIPT, SCRIPT_GETHEIGHT + TAG_SCRIPT);
			
			return newHtml;
		}
		//--------------  Event Handlers  --------------
		function _postLoadExternal(evt){
			webview.removeEventListener(evt.type, _postLoadExternal);
			var newHeight = parseInt(webview.evalJS('document.height'));
			if(parseInt(webview.height) != newHeight){
				webview.updateLayout({ height: newHeight });
			}
			webview.touchEnabled = false;
			if(param.callback)	param.callback({ height: evt.height });		
		}
		function _postLoadEmbedded(evt){
			Ti.App.removeEventListener(evt.type, _postLoadEmbedded);
			if(parseInt(webview.height) != evt.height){
				webview.updateLayout({ height: evt.height });
			}
			webview.touchEnabled = false;
			if(param.callback)	param.callback({ height: evt.height });
		}
	},

});

//----------------------------  Private Methods  -------------------------------
function _hasProperty(obj){
	return ('undefined' == typeof obj) ? false : true;
}

module.exports = WOForm;

//---  End of file: WOForm.js
