/**
 * WOList View
 */
Ti.include('/globals.js');

var WOList = App.Class(App.View.WOViewBase, {
	className : 'WOList',

	EVENT		: {
		//--- Handles these events:
		//--- Produces these events:
		rowClicked		: 'WOList.rowClicked',
		rowDeleted		: 'WOList.rowDeleted',
    },

	//-----------  Constructor  -----------
	initialize : function(param){
		var self = this;
		self.base(param);

		self.locked			= false;

		self.ITEM_KEY		= 'id';
		self.ALTERNATE		= false;
		
		self.insertIndex	= 1;
		
		self.locked			= false;
		self.tableView		= null;
		
		self.renderbyTimer	= false;
		self.renderInterval	= 10;
		
		self.dirty			= true;
		
		self.showLoadingIndicator = false;
	},

	/**
	 * Row render
	 * Should be implement by sub-class
	 */
	renderRow : function(param, index){
		var row = Ti.UI.createTableViewRow({
			data:param,
		});
		return row;
	},
	
	renderLastRow : function(index){
		return null;
	},
	
	/**
	 * Row alternate style render
	 * Should be implement by sub-class
	 */
	alternateUpdateRow : function(row, index){
		
	},
	
	draw: function(param){
		var self = this;

		self.tableView = Ti.UI.createTableView(self.STYLE.tableView);
		
		var header = self.getHeader(param);
		var footer = self.getFooter(param);
		if(header)
			self.tableView.headerView = header;
		if(footer)
			self.tableView.footerView = footer;

		self.tableView.addEventListener('click', _tableClickHandler);
		self.tableView.addEventListener('delete', _tableDeleteHandler);

		self.VIEW.add(self.tableView);
		
		//-------------------  Event Handlers  -----------------------
		function _tableClickHandler(evt){

			// Should use row instead of rowData for both iphone and android. to be converted after thorough testing.
			if (App.Config.IS_ANDROID) {
				var rowData = evt.row;			
			} else {
				var rowData = evt.rowData;
			}
			if(!rowData || self.locked){
				Ti.API.debug('Invalid click');
			}else{
				if(self.locked || !self._eventProxy) return;
				self._eventProxy.fireEvent(self.EVENT.rowClicked, {data:rowData});
			}
		}
		
		function _tableDeleteHandler(evt){
			var rowData = evt.rowData;
			if(!rowData || self.locked){
				Ti.API.debug('Invalid click');
			}else{
				if(self._eventProxy){
					self._eventProxy.fireEvent(self.EVENT.rowDeleted, {data:rowData});
				}
			}
		}
	},
	
	getHeader : function(parm) {
		return null;
	},

	getFooter : function(param) {
		return null;
	},
	
	setData : function(param){
		var self = this;
		
		if(self.showLoadingIndicator) self.startLoading();
		self.DATA 		= param;
		var rows 		= [];
		self.total 		= param.length;
		self.locked 	= true;

		if(self.renderbyTimer){
			var counter = 0;
			var timer = setInterval(function(e){
				if(counter>=self.total){
					clearInterval(timer);
					_finishSetData();
					return;
				}
				var row = self.renderRow(self.DATA[counter], counter);
				rows.push(row);
				counter++;
			}, self.renderInterval);
			
		}else{
			for(var counter=0;counter<self.total;counter++){
				var row = self.renderRow(self.DATA[counter], counter);
				rows.push(row);
			}
			_finishSetData();
		}
		
		function _finishSetData(){
			Ti.API.info('finish '+counter+' / '+self.total);
			var lastRow = self.renderLastRow(counter);
			if(lastRow){
				rows.push(lastRow);
			}
			self.tableView.setData(rows);
			self.hookSetData();
			self.stopLoading(); //Just in case.
			self.locked 	= false;
		}
	},
	
	hookSetData:function(){
		
	},
	
	removeRow : function(param){
		var self = this;
		
		var index = self.getIndex(param);
		if(null!=index){
			self.locked = true;
			setTimeout(function(){
				self.tableView.deleteRow(index);
				self.insertIndex	= 1;
				if(self.ALTERNATE){
					self.alternateUpdate(index);
				}
				self.total --;
				self.hookSetData();
				self.locked = false;
			}, 500);
		}else{
			Ti.API.error('Not a valid row to delete');
		}
	},
	
	getIndex : function(param){
		var self = this;
		
		var datas = self.tableView.data[0].rows;
		for(var i=0;i<datas.length;i++){
			if(param[self.ITEM_KEY] == datas[i][self.ITEM_KEY]){
				return i;
			}
		}
		return null;
	},
	
	alternateUpdate : function(index){
		var self = this;
		
		setTimeout(function(){
			var datas = self.tableView.data[0].rows;
			if(null == index){
				index = 0;
			}
			for(var i= index; i<datas.length; i++){
				self.alternateUpdateRow(datas[i], i);
			}
		}, 500);
	},

	//---  mirrors method of the same name in TableView  ---
	scrollToTop : function(top, animation){
		var self = this;

		self.tableView.scrollToTop(top, animation);
	},

	insertRow : function(param, position){
		var self = this;
		Ti.API.info(self.tableView.data[0]);
		var row = self.renderRow(param, 0);
		self.total ++;
		self.hookSetData();
			
		if(!self.tableView.data[0]){
			self.tableView.appendRow(row);
			return;
		}
		var datas = self.tableView.data[0].rows;
		var total = datas.length;
		
		Ti.API.info('CTK:Card-length='+total);
		
		if(total == 0){
			self.tableView.appendRow(row);
		}else{
			self.tableView.insertRowBefore(0, row);
		}
		
		if(self.ALTERNATE){
			self.alternateUpdate();
		}
		
		if (App.Config.IS_ANDROID) {
			var target_total = total + 1;
			/*
			function monitor_target () {
				if (target_total!=total) {
					setTimeout(monitor_target,500);
					return false;
				}
				if (self.refreshRow) {
					for (var n=0; n<target_total; n++) {
						self.refreshRow(n);
					}			
				}
				return true;
			};	
			monitor_target();		
			*/
			if (self.refreshRow) {
				for (var n=0; n<target_total; n++) {
					self.refreshRow(n);
				}			
			}
			var data = self.tableView.getData();
			self.tableView.setData([]);
			self.tableView.setData(data);
		}
	},
	
	updateRow : function(param){
		var self = this;
		var index = self.getIndex(param);
		var row = self.tableView.data[0].rows[index];
		self.updateOneRow(row, param, index);
	},

	updateOneRow : function(row, param, index){
		
	},

	appendRow : function(param){
		var self = this;
		
		var row = self.renderRow(param);
		self.tableView.appendRow(row);
		
		if(self.ALTERNATE){
			self.alternateUpdate();
		}
	},
	
	getEmptyView : function(){
		var label = Ti.UI.createLabel({
			text:'No data'
		});
		return label;
	},
	
	dispose : function(){
		var self = this;
		
		self.tableView.data = null;
		self.base();
	},

	showNoResult : function(){
		var self = this;
		
		self.stopLoading();
		if(!self.noResultView){
			self.noResultView = self.renderNoResult();
			self.VIEW.add(self.noResultView);
		}
		self.noResultView.show();
		if (App.Config.IS_ANDROID) {
			
		} else {
			self.tableView.setScrollable(false);		
		}
	},
	
	hideNoResult : function(){
		var self = this;
		
		self.stopLoading();
		if(self.noResultView){
			self.noResultView.hide();
		}
		if (App.Config.IS_ANDROID) {
			
		} else {
			self.tableView.setScrollable(true);		
		}
	},

	renderNoResult : function(){
		return Ti.UI.createView();
	}
});

module.exports = WOList;

//---  End of file: WOList.js
