/**
 * WOList2 View
 */
Ti.include('/globals.js');

var WOList2 = App.Class(App.View.WOViewBase, {
	className : 'WOList2',

	EVENT		: {
		//--- Handles these events:
		//--- Produces these events:
		rowClicked		: 'WOList2.rowClicked',
		rowDeleted		: 'WOList2.rowDeleted',
		rowMoved 		: 'WOList2.rowMoved',
		listRendered	: 'WOList2.listRendered',
    },

	//-----------  Constructor  -----------
	initialize : function(param){
		var self = this;
		self.base(param);
		
		self.tableView = null;
		self.headerPullView = null;
		self.headerView = null;
		self.footerView = null;
		self.noResultView = null;
		
		self.rowsBackgroundColorEnabled = false;
		
		self.locked = false;
		
		self.dynamicMode = false;
		self.dynamicFirstRows = 10; //only valid when dynamicMode is true;		
		self.dynamicRows = 10; //only valid when dynamicMode is true;
		self.dynamicLoading = false;
		self.dynamicLoadingListened = false;
		self.dynamicDistance = 0;
		
		self.activityIndicator = true;
		
		self.editable = false;
		self.dialogShowing = false;
	},
	
	listenForEvents : function(tableView){
		var self = this;
		
		tableView.addEventListener('click', _clickHandler);
		tableView.addEventListener('move', _moveHandler);
		tableView.addEventListener('delete', _deleteHandler);
		
		if(App.Config.IS_ANDROID) tableView.addEventListener('longpress', function(evt){
			if(!self.editable || self.dialogShowing) return;
			self.dialogShowing = true; //because sometimes pops up twice when longpressing.
			var dialog = Ti.UI.createAlertDialog({
				title		: App.Helper.L('TITLE_DELETE'),
				// message		: 'Will delete...'+evt.index,//App.Helper.L('Delete'),
				buttonNames	: [App.Helper.L('BUTTON_CANCEL'), App.Helper.L('BUTTON_OK')],
			});
			//---  Handle Action button  ---
			dialog.addEventListener('click', function(e){
				if(1 == e.index) {
					tableView.deleteRow(evt.index);
					_deleteHandler(evt);
				}
				self.dialogShowing = false;
			});
			dialog.show();
		});
		
		function _clickHandler(evt){
			if(!evt.rowData || self.locked) return;
			self.clickHandler(evt);
		}
		function _moveHandler(evt){
			evt.index > evt.fromIndex ? self.updateRowsBackgroundColor(evt.fromIndex) :  self.updateRowsBackgroundColor(evt.index) ;
		}
		function _deleteHandler(evt){
			if(!evt.rowData || self.locked) return;
			self.deleteHandler(evt);
		}
		
		self.listenForDynamicLoading(tableView);
	},
	
	listenForDynamicLoading : function(tableView){
		var self = this;
		
		if(self.dynamicMode && !self.dynamicLoadingListened) {
			tableView.addEventListener('scroll', _scrollHandler);
			self.dynamicLoadingListened = true;
		}
		
		function _scrollHandler(evt){
			if(self.dynamicMode && !self.locked && self.DATA.length > self.getRowsLength()){
				self._dynamicLoading(evt);
				if(self.DATA.length <= self.getRowsLength()) {
					tableView.removeEventListener('scroll', _scrollHandler);
					self.dynamicLoadingListened = false;
				}
			}
		}
	},
	
	clickHandler : function(evt){
		var self = this;
		if(self._eventProxy) self._eventProxy.fireEvent(self.EVENT.rowClicked, evt);
	},
	
	deleteHandler : function(evt){
		var self = this;
		if(self._eventProxy) self._eventProxy.fireEvent(self.EVENT.rowDeleted, evt);
		return self.getRowsLength();
	},
	
	draw : function(param){
		var self = this;
		
		var tableView = self.drawTableView(param);
		self.listenForEvents(tableView);
		
		self.VIEW.add(tableView);
	},
	
	drawNoResultView : function(param){
	},
	
	drawTableView : function(param){
		var self = this;
		self.tableView = Ti.UI.createTableView(self.STYLE.tableView);
		self.tableView.deleteButtonTitle = App.Helper.L('BUTTON_DELETE');
		
		self.headerPullView = self.drawHeaderPullView(); if(self.headerPullView) self.tableView.setHeaderPullView(self.headerPullView);
		self.headerView = self.drawHeaderView(self.tableView); if(self.headerView) self.tableView.setHeaderView(self.headerView);
		self.footerView = self.drawFooterView(self.tableView); if(self.footerView) self.tableView.setFooterView(self.footerView);
		if(self.DATA.length) {
			setTimeout(function(){
				self.setData();
			}, 200);
		}
		return self.tableView;
	},
	
	drawHeaderPullView : function(){
		return null;
	},
	
	drawHeaderView : function(){
		return null;
	},
	
	drawFooterView : function(){
		return null;
	},	
	
	renderRow : function(data, index){
		//Should be implement by sub-class
	},
	
	setRowBackgroundColor : function(row, index){
		//Should be implement by sub-class
		return false;
	},
	
	setEditable : function(editable){
		var self = this;
		self.editable = editable;
		if(!App.Config.IS_ANDROID) self.tableView.setEditable(editable);
	},
	
	updateRowsBackgroundColor : function(fromIndex){
		var self = this;
		if(!self.rowsBackgroundColorEnabled) return;
		var rows = self.getRows();
		for(var i=fromIndex; i<rows.length; i++) self.setRowBackgroundColor(rows[i], i);
	},
	
	getRowsLength : function(){
		var self = this;
		return self.getRows().length;
	},
	
	getRows : function(){
		var self = this;
		if('undefined' != typeof self.tableView.data[0] && 'undefined' != typeof self.tableView.data[0].rows){
			return self.tableView.data[0].rows;
		} else {
			return [];
		}
	},
	
	removeRow : function(index){
		var self = this;
		if(null==index) return;
		if(self.getRowsLength() >= index+1) {
			self.locked = true;
			setTimeout(function(){
				self.tableView.deleteRow(index);
				if(self.rowsBackgroundColorEnabled){
					self.updateRowsBackgroundColor(index);
				}
				self.locked = false;
			},800);
		}
	},
	
	insertRow : function(data, index){
		var self = this;
		if('undefined' == typeof index || !self.getRowsLength()) {
			self.appendRow(data);
		} else if(self.getRowsLength() >= index+1) {
			self.tableView.insertRowBefore(index, self.renderRow(data, index));
			if(self.rowsBackgroundColorEnabled){
				setTimeout(function(){
					self.updateRowsBackgroundColor(index+1);
				},200);
			}
		}
	},
	
	updateRow : function(data, index){
		var self = this;
	},
	
	moveRow : function(fromIndex, toIndex){
		var self = this;
		var rowsLength = self.getRowsLength();
		if(rowsLength >= fromIndex+1){
			self.tableView.deleteRow(fromIndex);
		}
		var row = self.renderRow(self.DATA[fromIndex], fromIndex);
		
		if(rowsLength >= toIndex+1) {
			toIndex ? self.tableView.insertRowAfter(toIndex-1, row) : self.tableView.insertRowBefore(toIndex, row);	
		}
		
		if(self.rowsBackgroundColorEnabled){
			setTimeout(function(){
				toIndex > fromIndex ? self.updateRowsBackgroundColor(fromIndex) : self.updateRowsBackgroundColor(toIndex);
			},200);
		}
	},
	
	appendRow : function(data){
		var self = this;
		var rowsNum = self.getRowsLength()
		self.tableView.appendRow(self.renderRow(data, rowsNum));
		return rowsNum+1;
	},
	
	setData : function(rowsData, animation){
		var self = this;
		self.locked = true;
		if(!self.dynamicMode && self.activityIndicator) self.startLoading();
		if(rowsData) self.DATA = rowsData;
		
		length = self.DATA.length;
		if(true === self.dynamicMode){
			length = (self.DATA.length > self.dynamicFirstRows) ? self.dynamicFirstRows : self.DATA.length;
			if(!self.dynamicLoadingListened) self.listenForDynamicLoading(self.tableView);
		}
		var rows = [];
		for(var index=0; index<length; index++){
			var row = self.renderRow(self.DATA[index], index);
			if(row) rows.push(row);
		}
		// if(self.getRowsLength()) self.tableView.setData([], animation);
		self.tableView.setData(rows, animation);
		
		if(self.dynamicMode){
			self.dynamicLoading = false;
			self.dynamicDistance = 0;
		} else {
			self.stopLoading();
		}
		self.locked = false;
		if(self._eventProxy) self._eventProxy.fireEvent(self.EVENT.listRendered, {});
		return self.DATA.length;
	},
	
	// orderRows : function(rowsData, animation){
		// var self = this;
		// if(rowsData) self.DATA = rowsData;
		// var rows = self.getRows();
		// var newRows = [];
		// for(var i=0; i<self.DATA.length; i++){
			// var objectID = self.getRowObjectID(self.DATA[i]);
			// for(var j=0; j<rows.length; j++){
				// if(rows[j].objectID == objectID){
					// newRows[i] = rows[j];
					// break;
				// }
			// }
		// }
		// self.tableView.setData(newRows, animation);
		// setTimeout(function(){
			// self.updateRowsBackgroundColor(0);
		// }, 200);
	// },
	
	// getRowObjectID : function(data){
		// var self = this;
		// //Sub-class do
	// },
	
	//---  mirrors method of the same name in TableView  ---
	scrollToTop : function(top, animation){
		var self = this;
		self.tableView.scrollToTop(top, animation);
	},
	
	getRowsObjectData : function(){
		var self = this;
		var rowsObjectData = [];
		var rows = self.getRows();
		for(var i=0; i<rows.length; i++) rowsObjectData.push(rows[i].objectData);
		return rowsObjectData;
	},
	
	searchIndex : function(objectID){
		var self = this;
		var rows = self.getRows();
		for(var i=0; i<rows.length; i++){
			if(objectID == rows[i].objectID){
				return i;
			}
		}
		return null;
	},
	
	showNoResult : function(){
		var self = this;
		if(!self.noResultView) {
			self.noResultView = self.drawNoResultView();
			self.VIEW.add(self.noResultView);
		}
		self.noResultView.show();
	},
	
	hideNoResult : function(){
		var self = this;
		if(self.noResultView) self.noResultView.hide();
	},
	
	_dynamicLoading : function(evt){
		var self = this;
		var offset = evt.contentOffset.y;
		var height = evt.size.height;
		var total = offset + height;
		var theEnd = evt.contentSize.height;
		var distance = theEnd - total;
		// going down is the only time we dynamically load,
		// going up we can safely ignore -- note here that
		// the values will be negative so we do the opposite
		if (distance < self.dynamicDistance){
			// adjust the % of rows scrolled before we decide to start fetching
			var nearEnd = theEnd * .5;
			if (!self.dynamicLoading && (total >= nearEnd)) _loadingMore();
		}
		self.dynamicDistance = distance;
		
		function _loadingMore(){
			self.dynamicLoading = true;
			var n=0;
			var index = self.getRowsLength();
			while(index < self.DATA.length && index >= 0){
				self.tableView.appendRow(self.renderRow(self.DATA[index], index++));
				if(++n >= self.dynamicRows) break;
			}
			self.dynamicLoading = false;
		}
	}
});

module.exports = WOList2;

//---  End of file: WOList2.js
