/**
 * View Base
 */
Ti.include('/globals.js');

var WOViewBase = App.Class(App.WOBaseClass, {
	className	: 'WOViewBase',

	//-----------  Constructor  -----------
	initialize : function(param){
		var self = this;
		self.base(param);

		// self.parent			= ('undefined' != typeof param.parent) ? param.parent : null;		
		self.DATA 			= ('undefined' != typeof param.DATA) ? param.DATA : {};
		self._eventProxy	= ('undefined' != typeof param.eventProxy) ? param.eventProxy : null;

		//--- automatically add my EVENT declarations to the event proxy ---
		if(self._eventProxy && ('undefined' != typeof self.EVENT)){
			self._eventProxy.addEvents(self.EVENT);
		}
		
		self.VIEW		= null;				/* Ti UI view object */
		self.initParam  = param;

		self.isRendered	= false;
		self.lazyRender = false;
		self.loading	= false;
	},

	getEventProxy : function(parent){
		var self = this;

		if(!self._eventProxy){
			var initEvents = (parent && 'undefined' != typeof parent.EVENT) ? parent.EVENT : null;
			self._eventProxy = new App.WOEventProxy(self, initEvents);
		}
		return self._eventProxy;
	},

	/**
	 * Get the UI component
	 * if not ready, render it first
	 */
	render : function(param){
		var self = this;
		
		Ti.API.debug(self.className+'::render');
		
		//---  Load & merge Style files  ---
		self.STYLE = App.Helper.loadStyle(self.stylePath);

		self.viewFrame = self._createView(self.initParam);	// NOTE:  self.VIEW created inside _createView() !			

		if(!self.isRendered){
			self.preDraw(param);
			
			if(self.lazyRender){
				//---  do Draw() after view/window opened
				self.startLoading();
				self.viewFrame.addEventListener('open', _doDelayedDraw);
			}else{
				//---  do Draw() immediately
				self._doDraw(param);
			}
		}
		self.isRendered = true;
		return self.viewFrame;

		//---------------  Event Handlers  ----------------
		function _doDelayedDraw(){
			if(self.lazyRender){
				self.viewFrame.removeEventListener('open', _doDelayedDraw);
				self.stopLoading();
			}
			self._doDraw(param);
		}
	},

	/**
	 * update display when condition changed
	 */
	update : function(data){
		
	},
	
	getView : function(){
		var self = this;
		return self.VIEW;	
	},
	
	show : function(){
		var self = this;
		if(self.VIEW) 	self.VIEW.show();
	},

	hide : function(){
		var self = this;
		if(self.VIEW) 	self.VIEW.hide();
	},
	
	/**
	 * Execute when the view is closed of removed
	 * Garbage collection
	 */
	dispose : function(){
		var self = this;
//		self.removeEventListener();
		Ti.API.info(self.className+'::dispose  ID='+self.uuid);
		self.VIEW = null;
	},

	changeStyle : function(obj, style){
		var self = this;
		
		for(var key in style){
			obj[key] = style[key];
		}
	},

	//----------------------  Protected Methods  ---------------------
	_createView : function(param){
		var self = this;
		
		self.VIEW = Ti.UI.createView(self.STYLE.view);
		return self.VIEW;
	},

	//------  3 phases of view rendering  -------	
	preDraw : function(param){
		//--- called AFTER view created, BEFORE draw() of actual view content
		//  mainly used when lazyRender is ON to draw 'skeleton' view/window
	},

	_doDraw : function(param){
		var self = this;

		self.draw(param);
		self.postDraw(param);
		self.afterPostDraw(param);
	},

	draw : function(param){
		//--- main renderer of view/window content
		
		// REMEMBER to add rendered view to self.VIEW
	},

	/**
	 * Hook afer after draw, for some special requirements
	 */
	postDraw : function(param){
	},

	afterPostDraw : function(param){
	},
	
	
	startLoading : function(){
		var self = this;
		
		if(self.loading) return;
		if(!self.loadingView){
			self.loadingView = self._getLoadingView();
		}
		if (! App.Config.IS_ANDROID) {
			self.VIEW.add(self.loadingView);
		}
		self.loadingView.show();
		self.loading = true;
	},
	
	stopLoading : function(){
		var self = this;
				
		if(!self.loading) return;
		if(self.loadingView){
			self.loadingView.hide();
			if (! App.Config.IS_ANDROID) {
				self.VIEW.remove(self.loadingView);
			}
		}
		self.loading = false;
	},
	
	_getLoadingView : function(){
		var self = this;

		if(App.Config.IS_ANDROID){
			var loadingView = Ti.UI.createActivityIndicator({
				message: 'loading...',
				font: {fontFamily: 'Melbourne-Bold', fontSize:'24sp'},
			});
		}else{
			var theStyle = (self.VIEW.backgroundColor == '#FFFFFF') ? Ti.UI.iPhone.ActivityIndicatorStyle.DARK : Ti.UI.iPhone.ActivityIndicatorStyle.BIG;
			var loadingView = Ti.UI.createActivityIndicator({
				style: theStyle,
			});
	}
		return loadingView;
	},

});

module.exports = WOViewBase;

//---  End of file: WOViewBase.js
