/**
 * Window View
 */
Ti.include('/globals.js');
if (App.Config.IS_ANDROID) {
	Ti.include('/waveo/core/WOAndroidUtils.js');
}

var WOWindow = App.Class(App.View.WOViewBase, {
	className : 'WOWindow',
	stylePath : 'WOStyle',

	EVENT : {
		//--- Handles these events:
		setBadge : 'WOWindow.setBadge',
		webViewLoaded : 'App:webview.load',
		//--- Produces these events:
	},

	/**
	 * Window attributes that can be set via the parameters
	 * @param	{object}			abstract parameter object defines window attributes/behaviors
	 * 			close	{boolean}	close window when done?
	 * 			isDND	{boolean}	specifies if window operates in Do-Not-Disturb mode
	 * 			<custom>			optionally, additional custom values may be passed in
	 */

	//-----------  Constructor  -----------
	initialize : function(param) {
		var self = this;
		param = param || {};
		self.base(param);

		self.WINDOW = {
			title : 'Window',
			type : ('undefined' != typeof param.type) ? param.type : App.Const.WINDOW.type.auto,
			hideNavBar : false,
			hideTabBar : false,
			effect : ('undefined' != typeof param.effect) ? param.effect : null,
			isDND : ('undefined' == typeof param.isDND) ? false : param.isDND,
		};
		self.navGroupObj = ('undefined' != typeof param.navGroupObj) ? param.navGroupObj : null;
		self.tabGroupObj = ('undefined' != typeof param.tabGroupObj) ? param.tabGroupObj : null;
		self.tabsMap = [];
		self.windows = [];

		self.savedNavGroupObj = null;
		// if we are navGroup Type, used to restore
		// previous navGroup when we close
		// --------------------------------------------------------
		self.viewing = true;
		self.toolbar = null;

		self.rapidFire = {};
		// storage used by notRapidFire()
	},

	_createView : function(param) {
		var self = this;

		//---  create based on window types  ---
		switch(self.WINDOW.type) {
			case App.Const.WINDOW.type.tabGroup:
				var myWindow = self._createTabgroup(param);
				self.VIEW = myWindow;
				$.currentTabGroup = self;
				break;

			case App.Const.WINDOW.type.navGroup:
				var myWindow = self._createNavGroup(param);
				self.navGroupObj = self;
				// preserve existing navGroup, restore when close
				self.savedNavGroupObj = ('undefined' != typeof $.currentNavGroupObj) ? $.currentNavGroupObj : self;
				$.currentNavGroupObj = self;
				// NOTE:  self.VIEW created inside _createNavGroup() & is DIFFERENT from myWindow
				break;

			default:
				if (App.Config.IS_ANDROID) {
					self.VIEW = createAndroidWindow(self.STYLE.window);
				} else {
					self.VIEW = Ti.UI.createWindow(self.STYLE.window);
				}
				var myWindow = self.VIEW;
				break;
		}

		return myWindow;
	},

	preDraw : function(param) {
		var self = this;

		self.setWindowTitle(self.WINDOW.title);
		if (App.Config.IS_ANDROID) {
			self.VIEW.myNavBarHidden = self.WINDOW.hideNavBar;
		} else {
			self.VIEW.navBarHidden = self.WINDOW.hideNavBar;
		}
		if (self.tabGroupObj) {
			self.VIEW.tabBarHidden = self.WINDOW.hideTabBar;
		}

		//---  Install Event Handlers  -------
		self.VIEW.addEventListener('focus', function() {
			self.viewing = true;
			Ti.API.warn('>>> ' + self.className + ' focused');
			self.onFocus();
		});
		self.VIEW.addEventListener('blur', function() {
			self.viewing = false;
			Ti.API.warn('<<< ' + self.className + ' blurred');
			self.onBlur();
		});
	},

	draw : function(param) {
		// REMEMBER to add rendered view to self.VIEW
	},

	_createTabgroup : function(param) {
		var self = this;

		if (App.Config.IS_ANDROID) {
			var aStyle = App.Helper.clone(self.STYLE.tabGroup);
			self.tabGroup = createAndroidTabGroup(aStyle);
		} else {
			self.tabGroup = Ti.UI.createTabGroup(self.STYLE.tabGroup);
		}

		var tabCount = param.tabDefine.length;

		for (var i = 0; i < tabCount; i++) {
			var tabItem = param.tabDefine[i];

			//---  get Controller to render the Window  ---
			var route = $.app.parseRoute(param.tabDefine[i].url);
			var winObj = route.controller[route.action]({
				tabGroupObj : self
			});

			if (App.Config.IS_ANDROID) {
				var _tab = createAndroidTab({
					title : tabItem.title,
					icon : tabItem.icon,
					window : winObj.getView(),
				});
			} else {
				var _tab = Ti.UI.createTab({
					title : tabItem.title,
					icon : tabItem.icon,
					window : winObj.getView(),
				});
			}

			self.tabGroup.addTab(_tab);
			self.tabsMap[tabItem.id] = _tab;

			_tab.addEventListener('focus', function(evt) {
				evt.source.setBadge(null);
			});
		}

		//---  Setup TabGroup Event Handlers  ---
		// Ti.App.addEventListener(self.EVENT.showNavBar, _doShowNavBar);
		Ti.App.addEventListener(self.EVENT.setBadge, _doSetBadge);

		return self.tabGroup;

		//-----------------------  Event Handlers  -----------------------
		// function _doShowNavBar(evt){
		// //			self.VIEW.activeTab.window.setTabBarHidden(!evt.show);
		// self.VIEW.activeTab.window.hideTabBar();
		// }
		function _doSetBadge(evt) {
			self.tabGroup.tabs[evt.tab].setBadge(evt.badge);
		}

	},

	_createNavGroup : function(param) {
		var self = this;

		if (App.Config.IS_ANDROID) {
			self.VIEW = createAndroidWindow(self.STYLE.window);
		} else {
			self.VIEW = Ti.UI.createWindow(self.STYLE.window);
		}

		/* Note: Do not assign extra param to navParam as this is a reference copy */
		if (Ti.Platform.osname == 'android') {
			var navParam = App.Helper.clone(self.STYLE.navGroup);
			navParam.window = self.VIEW;
			self.navGroup = createAndroidNavGroup(navParam);
			var baseWindow = createAndroidWindow(self.STYLE.baseWindow);
			baseWindow.myNavBarHidden = true;
		} else {
			var navParam = App.Helper.clone(self.STYLE.navGroup);
			navParam.window = self.VIEW;
			self.navGroup = Ti.UI.iPhone.createNavigationGroup(navParam);
			var baseWindow = Ti.UI.createWindow(self.STYLE.baseWindow);
			baseWindow.navBarHidden = true;
		}
		baseWindow.add(self.navGroup);

		return baseWindow;
	},

	afterPostDraw : function() {
		var self = this;

		// self.viewFrame.addEventListener('close', function(e){
		// self.dispose();
		// });
		self.VIEW.addEventListener('close', function(e) {
			self.dispose();
		});
	},

	open : function(param) {
		var self = this;
		param = param || {};

		//---  determine window Open effect  ---
		var effects = {
			style : {}
		};
		if ('undefined' != typeof param.effect) {
			self.WINDOW.effect = param.effect;
		}
		var theWin = self.VIEW;
		Ti.API.info('Open: my window type' + self.WINDOW.type);
		//---  open according to window type  ---
		switch(self.WINDOW.type) {

			case App.Const.WINDOW.type.auto:
				if (self.tabGroupObj) {
					//---  open the window as part of it's tabGroup  ---
					if (!param.tab) {
						self.myTab = self.tabGroupObj.tabGroup.getActiveTab();
						// tabGroup.activeTab.open(self.VIEW);
					} else {
						self.myTab = self.tabGroupObj.tabGroup.tabs[param.tab];
						// tabGroup.tabs[param.tab].open(self.VIEW);
					}
					self.myTab.open(self.VIEW);
					self.tabGroupObj.windows.push(self.VIEW);

					Ti.API.error(self.className + '::Auto Tab Window opened >>> ID=' + self.uuid);
				} else if (self.navGroupObj) {
					//---  open the window as part of it's navGroup  ---
					self.navGroupObj.navGroup.open(self.VIEW);
					self.navGroupObj.windows.push(self.VIEW);

					Ti.API.error(self.className + '::Auto Nav Window opened >>> ID=' + self.uuid);
				} else {
					//---  open a Standalone window  ---
					effects = self._getOpenEffects(self.VIEW, self.WINDOW.effect);
					self.VIEW.open(effects.style);
					Ti.API.error(self.className + '::Standalone Window opened >>> ID=' + self.uuid);
				}
				break;

			case App.Const.WINDOW.type.tabGroup:
				effects = self._getOpenEffects(self.viewFrame, self.WINDOW.effect);
				self.viewFrame.open(effects.style);
				theWin = self.viewFrame;

				Ti.API.error(self.className + '::TabGroup opened >>> ID=' + self.uuid);
				break;

			case App.Const.WINDOW.type.navGroup:
				if (self.tabGroupObj) {
					//        		var tabGroup = self.tabGroupObj.tabGroup;
					self.myTab = self.tabGroupObj.tabGroup.getActiveTab();
					self.myTab.open(self.viewFrame);
					// tabGroup.activeTab.open(self.viewFrame);
				} else {
					effects = self._getOpenEffects(self.viewFrame, self.WINDOW.effect);
					self.viewFrame.open(effects.style);
				}
				Ti.API.error(self.className + '::NavGroup opened >>> ID=' + self.uuid);
				break;

			case App.Const.WINDOW.type.standalone:
				effects = self._getOpenEffects(self.VIEW, self.WINDOW.effect);
				self.VIEW.open(effects.style);
				Ti.API.error(self.className + '::Standalone Window opened >>> ID=' + self.uuid);
				break;
		}

		//--- animate window if necc  ---
		if ('undefined' != typeof effects.animation) {
			theWin.animate(effects.animation);
		}

		//--- set Do-Not-Disturb state if this is a DND window  ---
		if (self.WINDOW.isDND) {
			$.app.doNotDisturb(true);
		}
	},

	close : function(param) {
		var self = this;
		param = param || {};

		//---  determine window Close effect  ---
		var toAnimate = ('undefined' != typeof param.animated) ? param.animated : true;
		var closeEffect = ('undefined' != typeof param.effect) ? param.effect : self.WINDOW.effect;
		var effects = {
			style : {}
		};

		//---  close according to window type  ---
		Ti.API.info('WOWindow::close - window type: ' + self.WINDOW.type);
		switch(self.WINDOW.type) {
			case App.Const.WINDOW.type.tabGroup:
				//CTK 			self.VIEW.close();
				self.viewFrame.close();
				Ti.API.error(self.className + '::TabGroup closed <<< ID=' + self.uuid);
				break;

			case App.Const.WINDOW.type.navGroup:

				if (toAnimate) {
					effects = self._getCloseEffects(self.viewFrame, closeEffect);
				}
				//--- animate window if necc  ---

				if ('undefined' != typeof effects.animation) {
					self.viewFrame.animate(effects.animation, function(evt) {
						if (self.tabGroupObj) {
							// 		self.viewFrame.close();
							self.myTab.close(self.viewFrame);
						} else {
							if (self.toolbar) {
								if (App.Config.IS_ANDROID) {
									self.removeToolbar();
								} else {
									self.viewFrame.remove(self.toolbar);
								}
							}
							self.viewFrame.close(effects.style);
						}
						// setTimeout(function(evt){
						//
						// /// CTK BUG! self.navGroup.close();
						// self.viewFrame.close();
						//
						// }, 2000);
					});
				} else {
					if (self.tabGroupObj) {
						self.myTab.close(self.viewFrame);
					} else {
						if (self.toolbar) {
							if (App.Config.IS_ANDROID) {
								self.removeToolbar();
							} else {
								self.viewFrame.remove(self.toolbar);
							}
						}
						self.viewFrame.close(effects.style);
					}
					// setTimeout(function(evt){
					//
					// //CTK bug!!! self.navGroup.close();
					// self.viewFrame.close ();
					// }, 2000);
				}

				///	        self.removeToolbar ();

				//--- restore previous navGroup
				$.currentNavGroupObj = self.savedNavGroupObj;

				Ti.API.error(self.className + '::NavGroup closed <<< ID=' + self.uuid);
				break;

			default:
				/*
				 //---  check if this window belongs to a navGroup  ---
				 if(self.navGroupObj){
				 ///		        self.removeToolbar ();
				 self.navGroupObj.navGroup.close(self.VIEW);
				 Ti.API.error(self.className+'::NavGroup closed <<< ID='+self.uuid);
				 }else{
				 Ti.API.error(self.className+'::Window closed. ID='+self.uuid);
				 if(toAnimate){
				 effects = self._getCloseEffects(self.VIEW, closeEffect);
				 }
				 //--- animate window if necc  ---
				 if('undefined' != typeof effects.animation){
				 effects.animation.addEventListener('complete', function(evt){
				 if (self.tabGroupObj) {
				 self.myTab.close(self.VIEW, effects.style);
				 } else {
				 self.VIEW.close(effects.style);
				 }
				 });
				 self.VIEW.animate(effects.animation);
				 }else{
				 if (self.tabGroupObj) {
				 self.myTab.close(self.VIEW, effects.style);
				 } else {
				 self.VIEW.close(effects.style);
				 }
				 }
				 }

				 */

				if (toAnimate) {
					effects = self._getCloseEffects(self.VIEW, closeEffect);
				}
				//---  check if this window belongs to a navGroup  ---
				Ti.API.info('CtkCTK:' + self.tabGroupObj + ' ' + self.navGroupObj);
				if (self.tabGroupObj) {

					Ti.API.error(self.className + '::Window closed. ID=' + self.uuid);
					if ('undefined' != typeof effects.animation) {
						effects.animation.addEventListener('complete', function(evt) {
							self.myTab.close(self.VIEW, effects.style);
						});
						self.VIEW.animate(effects.animation);
					} else {
						self.myTab.close(self.VIEW, effects.style);
					}
				} else if (self.navGroupObj) {
					self.navGroupObj.navGroup.close(self.VIEW);
					Ti.API.error(self.className + '::NavGroup closed <<< ID=' + self.uuid);
				} else {
					if ('undefined' != typeof effects.animation) {
						effects.animation.addEventListener('complete', function(evt) {
							self.VIEW.close(effects.style);
						});
						self.VIEW.animate(effects.animation);
					} else {
						self.VIEW.close(effects.style);
					}
				}

				break;
		}

		//--- set Do-Not-Distrub state if this is a DND window  ---
		if (self.WINDOW.isDND) {
			$.app.doNotDisturb(false);
		}
	},

	_getOpenEffects : function(win, openEffect) {
		var self = this;

		var effects = {
			style : {}
		};
		if ('undefined' == typeof openEffect)
			return effects;

		switch(openEffect) {
			case App.Const.WINDOW.effect.fade:
				effects.animation = Ti.UI.createAnimation({
					opacity : 1,
					duration : 300,
				});
				win.opacity = 0.1;
				break;

			case App.Const.WINDOW.effect.zoom:
				var t = Ti.UI.create2DMatrix();
				t = t.scale(0.2);
				var t1 = Ti.UI.create2DMatrix();
				t1 = t1.scale(1);
				effects.animation = Ti.UI.createAnimation({
					transform : t1,
					duration : 350,
					opacity : 1,
				});
				win.transform = t;
				win.opacity = 0;
				break;

			case App.Const.WINDOW.effect.flip:
				effects.style = {
					duration : 350,
					transition : Ti.UI.iPhone.AnimationStyle.FLIP_FROM_LEFT
				};
				break;

			case App.Const.WINDOW.effect.modal:
				effects.style = {
					modal : true
				};
				break;

			case App.Const.WINDOW.effect.modalFlip:
				effects.style = {
					duration : 350,
					modal : true,
					modalTransitionStyle : Ti.UI.iPhone.MODAL_TRANSITION_STYLE_FLIP_HORIZONTAL,
				};
				break;
		}

		return effects;
	},

	_getCloseEffects : function(win, closeEffect) {
		var self = this;

		var effects = {
			style : {}
		};
		if ('undefined' == typeof closeEffect)
			return effects;

		switch(closeEffect) {
			case App.Const.WINDOW.effect.fade:
				effects.animation = Ti.UI.createAnimation({
					opacity : 0,
					duration : 300,
				});
				win.opacity = 1;
				break;

			case App.Const.WINDOW.effect.zoom:
				var t = Ti.UI.create2DMatrix();
				t = t.scale(0.25);
				var t1 = Ti.UI.create2DMatrix();
				t1 = t1.scale(1);
				effects.animation = Ti.UI.createAnimation({
					transform : t,
					duration : 400,
					opacity : 0,
				});
				win.transform = t1;
				win.opacity = 1;
				break;

			case App.Const.WINDOW.effect.flip:
				effects.style = {
					duration : 450,
					transition : Ti.UI.iPhone.AnimationStyle.FLIP_FROM_RIGHT,
				};
				break;

			case App.Const.WINDOW.effect.modal:
			case App.Const.WINDOW.effect.modalFlip:
			default: 
				effects.style = {
					animated : true,
				};
		}

		return effects;
	},

	//-------------------------  Event Handlers  --------------------------
	onFocus : function() {
	},

	onBlur : function() {
	},

	//-------------------------  Navigation Methods  ----------------------
	home : function(param) {
		var self = this;

		var toAnimate = (param && ('undefined' != typeof param.animated)) ? param.animated : false;

		if (self.tabGroupObj) {
			Ti.API.info('tabGroupObj: closing all tab windows, total :' + self.tabGroupObj.windows.length);
			if (App.Config.IS_ANDROID) {
				for (var i = 0; i < self.tabGroupObj.windows.length; i++) {
					self.tabGroupObj.tabGroup.activeTab.close(self.tabGroupObj.windows[i]);
				}

			} else {
				for (var i = 0; i < self.tabGroupObj.windows.length; i++) {//Note: change to length, not length-1.
					self.tabGroupObj.windows[i].close();
				}
			}
			/* not needed?
			 while(self.tabGroupObj.windows.length>0){
			 self.myTab.close(self.tabGroupObj.windows.pop());
			 // self.tabGroupObj.tabGroup.activeTab.close(self.tabGroupObj.windows.pop());
			 }
			 */
		} else if (self.WINDOW.type == App.Const.WINDOW.type.tabGroup) {
			Ti.API.info('tabGroup: closing all tab windows, total :' + self.windows.length);
			var numWin = self.windows.length;
			for (var i = 0; i < numWin; i++) {
				if (App.Config.IS_ANDROID) {
					//self.myTab.close(self.windows[i]);
					self.tabGroup.activeTab.close(self.windows[i]);
				} else {
					self.windows[i].close();
				}
			}
			// self.myTab.close(self.windows[numWin]);
			//
			// Ti.API.error('=========  GOT IT  ============');
			// while(self.windows.length>0){
			// self.tabGroup.activeTab.close(self.windows[numWin-1]);
			// }
		} else if (self.navGroupObj) {
			Ti.API.info('navGroup: closing all tab windows, total :' + self.navGroupObj.windows.length);
			while (self.navGroupObj.windows.length > 0) {
				self.navGroupObj.navGroup.close(self.navGroupObj.windows.pop(), {
					animated : toAnimate
				});
			}
			// self.navGroupObj.navGroup.close();
			//
			// // for(var i=numWindows-1; i >= 0; i--){
			// // self.navGroupObj.navGroup.close(self.navGroupObj.windows[i]);
			// // }
			// // self.navGroupObj.navGroup.close(self.navGroupObj.windows[numWindows-1]);
			// self.navGroupObj.windows = [];

		}
	},

	back : function(param) {
		var self = this;

		if (self.tabGroupObj) {
			var win = self.tabGroupObj.windows.pop();

			if (App.Config.IS_ANDROID) {

				var myTab = self.tabGroupObj.tabGroup.getActiveTab();
				myTab.close(win);

			} else {
				win.close();
			}

		} else if (self.navGroupObj) {
			var win = self.navGroupObj.windows.pop();
			self.navGroupObj.navGroup.close(win);
		}
	},

	//--------------------  Window Attributes Methods  -------------------
	setWindowTitle : function(title) {
		var self = this;

		var windowTitle = Ti.UI.createLabel(self.STYLE.windowTitle);
		windowTitle.text = title;

		if (App.Config.IS_ANDROID) {
			self.VIEW.setTitleControl(windowTitle);
		} else {
			self.VIEW.setTitleControl(windowTitle);
		}
	},

	setToolbar : function(toolbar) {
		var self = this;

		Ti.API.info('CTK-setToolbar');
		if (self.navGroupObj) {
			self.navGroupObj.toolbar = toolbar;
			if (App.Config.IS_ANDROID) {

				self.navGroupObj.viewFrame.TBW = createAndroidWindow({
					left : 0,
					right : 0,
					width : '100%',
					height : '40dp',
					bottom : 0,
					zIndex : 16,
				});
				self.navGroupObj.viewFrame.TBW.updated = false;
				self.navGroupObj.viewFrame.TBW.addEventListener('postlayout', function(e) {
					if (!e.source.updated) {
						e.source.updated = true;
						//						alert(' '+self.navGroupObj.viewFrame.TBW.rect.x+' '+
						//						self.navGroupObj.viewFrame.TBW.rect.y+' '+self.navGroupObj.viewFrame.TBW.rect.width+' '
						//						+self.navGroupObj.viewFrame.TBW.rect.height);
						e.source.left = 0;
						e.source.right = 0;
						e.source.width = e.source.rect.width;
						e.source.bottom = 0;
						e.source.top = e.source.rect.height - _dpToPixel('44');
					}
				});
				self.navGroupObj.viewFrame.TBW.add(toolbar);
				self.navGroupObj.viewFrame.TBW.open();

			} else {
				// self.navGroupObj.toolbar = toolbar;
				self.navGroupObj.viewFrame.add(toolbar);
			}
		} else
			Ti.API.error('WOWindow::setToolbar - only support windows with NavGroup');
	},

	hideToolbar : function(toHide) {
		var self = this;

		if (self.navGroupObj && self.navGroupObj.toolbar) {
			if (App.Config.IS_ANDROID) {
				self.navGroupObj.viewFrame.TBW.setVisible(self.toolbar.visible = ('undefined' == typeof toHide) ? false : !toHide);
			} else {
				self.navGroupObj.toolbar.visible = ('undefined' == typeof toHide) ? false : !toHide;
			}
		}
	},

	removeToolbar : function() {
		var self = this;

		Ti.API.info('CTK-removeToolbar');
		///		if(self.navGroupObj && self.navGroupObj.toolbar){
		if (App.Config.IS_ANDROID) {

			self.viewFrame.TBW.remove(self.toolbar);
			self.viewFrame.TBW.close();
			self.viewFrame.TBW = null;
			self.toolbar = null;

		} else {
			self.viewFrame.remove(self.toolbar);
			self.toolbar = null;
		}
		///		}
	},

	//---------------------  TabGroup Window Methods  ---------------------
	setBadge : function(param) {
		var self = this;

		var index = param.tab;
		var action = param.action;
		var count = param.count;
		var tab = $.rootWindow.tabGroup.tabs[index];

		var currentBadge = tab.badge;
		if (!currentBadge)
			currentBadge = 0;

		switch(action) {
			case 'add':
				currentBadge += count;
				break;

			case 'reduce':
				currentBadge -= count;
				break;

			case 'set':
				currentBadge = count;
				break;
		}

		tab.badge = currentBadge;
	},

	setActiveTab : function(index) {
		var self = this;
		if (App.Config.IS_ANDROID) {
			self.VIEW.setActiveTab(index);
		} else {
			self.VIEW.setActiveTab(self.VIEW.tabs[index]);
		}
	},

	//---  Utility function to prevent rapid fire Events within interval  -----
	notRapidFire : function(signature, interval) {
		var self = this;

		var millisecInterval = ('undefined' != typeof interval) ? interval : 1000;
		var now = new Date();
		if ('undefined' == typeof self.rapidFire[signature]) {
			self.rapidFire[signature] = now;
		} else {
			if (now - self.rapidFire[signature] < millisecInterval) {
				self.rapidFire[signature] = now;
				return false;
			}
		}
		self.rapidFire[signature] = now;
		return true;
	},

	//---  FIXME   Put here for convenience first, to relocate later
	/**
	 * @param {Object} param
	 * 		html			URL or html text
	 * 		style			style for webview 	MUST not set the html or url properties
	 * 		callback		optional callback function
	 */
	getWebView : function(param) {
		var self = this;
		var webLoadEvent = 'App:webview.load';

		if (('undefined' == typeof param.html) || (param.html == ''))
			return null;

		var webviewStyle = param.style || {};
		var webview = Ti.UI.createWebView(param.style);

		//--- determine if our html is external (URL) or embedded
		var viewID = new Date().getTime();
		var isExternal = (param.html.slice(0, 5) == 'http:') ? true : false;
		if (isExternal) {
			webview.addEventListener('load', _postLoadExternal);
			webview.url = param.html;
		} else {
			//---  embedded html  ---
			var EventName = webLoadEvent + '.' + viewID;
			Ti.App.addEventListener(EventName, _postLoadEmbedded);
			webview.html = _injectScript(param.html, EventName);
		}

		return webview;

		//--------------  Private Methods --------------
		function _injectScript(originalHtml, eventName) {

			var SCRIPT_ONLOAD = ' onLoad="getHeight()" ';
			var TAG_SCRIPT = '</head>';
			var TAG_ONLOAD = '<body';

			var SCRIPT_GETHEIGHT = '<script>function getHeight(){var D = document;var height1 = Math.max(D.body.scrollHeight, D.documentElement.scrollHeight);var height2 = Math.max(D.body.offsetHeight, D.documentElement.offsetHeight);var height3 = Math.max(D.body.clientHeight, D.documentElement.clientHeight);var documentHeight =  Math.max(	Math.max(D.body.scrollHeight, D.documentElement.scrollHeight),	Math.max(D.body.offsetHeight, D.documentElement.offsetHeight),	Math.max(D.body.clientHeight, D.documentElement.clientHeight));  Ti.App.fireEvent(\'';
			SCRIPT_GETHEIGHT += eventName + '\' , { height: documentHeight}); }; </script>';

			var newHtml = originalHtml.replace(TAG_ONLOAD, TAG_ONLOAD + SCRIPT_ONLOAD);
			newHtml = newHtml.replace(TAG_SCRIPT, SCRIPT_GETHEIGHT + TAG_SCRIPT);

			return newHtml;
		}

		//--------------  Event Handlers  --------------
		function _postLoadExternal(evt) {
			webview.removeEventListener(evt.type, _postLoadExternal);
			var newHeight = parseInt(webview.evalJS('document.height'));
			if (parseInt(webview.height) != newHeight) {
				webview.updateLayout({
					height : newHeight
				});
			}
			webview.touchEnabled = false;
			if (param.callback)
				param.callback({
					height : evt.height
				});
		}

		function _postLoadEmbedded(evt) {
			Ti.App.removeEventListener(evt.type, _postLoadEmbedded);
			if (parseInt(webview.height) != evt.height) {
				webview.updateLayout({
					height : evt.height
				});
			}
			webview.touchEnabled = false;
			if (param.callback)
				param.callback({
					height : evt.height
				});
		}

	},

	dispose : function(param) {
		var self = this;

		if (self.toolbar)
			self.removeToolbar();

		self.base(param);
	},
});

module.exports = WOWindow;

//---  End of file: WOWindow.js
