/**
 * Perkd-specific Account & Profile Controller
 *
 * @author young@waveo.com
 */
Ti.include('/globals.js');
App.Helper.requireController('WOAccountManager');

var AccountController = App.Class(App.Controller.WOAccountManager, {
	className : 'AccountController',
	
	CONST : {
		authToken 	: 'WOAccount.authToken',
		authState 	: 'WOAccount.authState',
		regMobile 	: 'WOAccount.regMobile',
		regCountry 	: 'WOAccount.regCountry',
	},

	//---  Constructor  ---------------	
	initialize : function(param){
		var self = this;
		self.base(param);
	},

    //----------------------  Account Services  ----------------------
    getRegisterCountryCode : function() {
    	var self = this;
    	return Ti.App.Properties.getString(self.CONST.regCountry, '');
    },
    setRegisterCountryCode : function(countryCode) {
    	var self = this;
    	return Ti.App.Properties.setString(self.CONST.regCountry, countryCode);
    },
    getRegisterMobileNum : function() {
    	var self = this;
    	return Ti.App.Properties.getString(self.CONST.regMobile, '');
    },
    setRegisterMobileNum : function(mobileNum) {
    	var self = this;
    	return Ti.App.Properties.setString(self.CONST.regMobile, mobileNum);
    },
    getRegisterMobileNo : function() {
    	var self = this;
    	return self.getRegisterCountryCode()+self.getRegisterMobileNum();
    },
    //for auth token
    getAuthToken : function() {
        var self = this;
        return Ti.App.Properties.getString(self.CONST.authToken, null);
    },
    
    setAuthToken : function(authToken) {
        var self = this;
        Ti.App.Properties.setString(self.CONST.authToken, authToken);
    },
    
    getAuthState : function() {
        var self = this;
        return Ti.App.Properties.getString(self.CONST.authState, null);
    },
    
    setAuthState : function(authState) {
        var self = this;
        Ti.App.Properties.setString(self.CONST.authState, authState);
    },
    
    resetAccount : function(param){
        var self = this;
        Ti.App.Properties.removeProperty(self.CONST.authState);
        Ti.App.Properties.removeProperty(self.CONST.authToken);
        Ti.App.Properties.removeProperty(self.CONST.regMobile);
        $.syncMgr.resetUserObjects();
    },
    
    exist : function(param){
    	var self = this;
    	if(false === $.dataMgr.get('WOSettings', 'account', false)) {
    		return false;
    	} else {
    		return true;
    	}
    },

    setProfile : function(param){
        var self = this;
        var account = $.dataMgr.newModel('WOSettings', 'account');
        account.fetch();
        for(var key in param) account.data[key] = param[key];
        account.save();
    },
    
    getProfile : function(){
    	var self = this;
        var account = $.dataMgr.newModel('WOSettings', 'account');
        account.fetch();
        return account.p2l();
    },
    
    getPerkdCardID : function(){
    	var self = this;
    	var perkdCardMID = '000001';
    	var cards = $.dataMgr.newModel('WOCard');
    	cards.fetchAll({
    		where : "( mid=? )",
    		vararg : [perkdCardMID],
    	});
    	if(cards.list.length>0) return cards.list[0].id;
    	else return null;
    },

    account : function(param){
        var self = this;
        
        //-----------  Setup Data Package  -----------
		var viewParam = {
			tabGroupObj	: param.tabGroupObj,
		};
        //----  Load & Render View with viewParam ----
        var winObj = App.Helper.loadView('account/AccountWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.logout, _doLogout); 
        //----------  Return Window Object  ---------- 
        return winObj;
        
		//--------------------  Event Handlers  ------------------------
		function _doLogout(evt){
			if(!Ti.Network.online){
	        	$.app.showAlert(App.Helper.L('net_required'), App.Helper.L('net_error_title'));
	        	return;
        	}
        	        
			$.app.isDND = true;
			$.notifyMgr.isActive = false;			// FIXME [young]  tmp solution
			
            $.app.showBusy({ message: App.Helper.L('SHOW_LOGGINGOUT') });

            $.authMgr.logout({
                callback    : function(result){
                    $.app.hideBusy();
					Ti.App.fireEvent($.app.EVENT.doRestart);
                },
                errorback	: function(result){
					$.watch.dog(self, 'logout', 'Encountered error, result: '+result);
                    $.app.hideBusy();
					Ti.App.fireEvent($.app.EVENT.doRestart);
                },
            });
        }
    },

    updatePassword : function(param){
        var self = this;

        //-----------  Setup Data Package  -----------
        var viewParam = {
        	tabGroupObj	: param.tabGroupObj,
        };
        //----  Load & Render View with viewParam ----
        var winObj = App.Helper.loadView('account/UpdatePasswordWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.changePassword, _doChangePassword);
        //----------  Return Window Object  ---------- 
        return winObj;
			
		//--------------------  Event Handlers  ------------------------
        function _doChangePassword(evt){
            // evt = { oldPassword : string, newPassword : string }

            $.app.showBusy({message:App.Helper.L('hint_requesting')});
            self._changePswd({
                old         : evt.oldPassword,
                pswd        : evt.newPassword,
                callback    : function(){
                    $.app.hideBusy();
                    $.app.showAlert(App.Helper.L('change_pswd_ok') , App.Helper.L('account_title_changePassword'));
                    winObj.back();
                },
                errorback   : function(){
                    $.app.hideBusy();
                }
            });
        }
    },
    
    updateMobile : function(param){
        var self = this;
        App.Helper.requireModel('WOSettings');

        //-----------  Setup Data Package  -----------
        var carrierInfo = $.configMgr.getCarrierInfo();
        var country = App.Helper.getCountryISO(carrierInfo.isoCountryCode);        
        var viewParam = { 
			DATA		: {
				countryCode : country.code,
				countryName : country.name,
			},
			tabGroupObj	: param.tabGroupObj,
			navGroupObj : param.navGroupObj,
		};
        //----  Load & Render View with viewParam ----
        var winObj = App.Helper.loadView('account/UpdateMobileWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.updateMobileNo, _doUpdateMobileNo);
		proxy.addEventListener(proxy.EVENT.verifyCode, _doVerifyCode);
		proxy.addEventListener(proxy.EVENT.resendCode, _doResendCode);
		proxy.addEventListener(proxy.EVENT.searchCountry, _goSearchCountry);
        //----------  Return Window Object  ---------- 
        return winObj;
        
		//--------------------  Event Handlers  ------------------------
        function _doUpdateMobileNo(evt){
            // evt = { mobileNo : string, resend : boolean }
            
            $.app.showBusy({message:App.Helper.L('hint_requesting')});
            
            self._changeMobileNumber({
                mobileNo    : evt.mobileNo,
                callback    : function(result){
                    $.app.hideBusy();
                    proxy.fireEvent(proxy.EVENT.mobileUpdated, {
                    	mobileNum	: evt.mobileNum,
                    });
                },
                errorback   : function(){
                    $.app.hideBusy();
                }
            })
        }
        function _doVerifyCode(evt){
            // evt = { mobileNo : string, code : string }

            $.app.showBusy({message:App.Helper.L('hint_requesting')});
            
            $.authMgr.verifyMobile({
                type        : 'changeMobileNo',
                mobileNo    : evt.mobileNo,
                resend      : evt.resend,
                code        : evt.resend ? '' : evt.code,
                callback    : function(result){
                    $.app.hideBusy();
                    
                    if(!evt.resend){
                        $.app.showAlert(App.Helper.L('change_mobile_ok') , App.Helper.L('account_title_mobileNumber'));
                        proxy.fireEvent(proxy.EVENT.mobileVerified, {
                        	mobileNo : evt.mobileNo,
                        });
                        // winObj.back();
                        winObj.close({animated:true});
                        
                        Ti.App.addEventListener($.syncMgr.EVENT.doSyncObjects, _doSyncObjects);
                        
                        //--- Update Mobile No. ---
                        var account = new App.Model.WOSettings('account');
                        account.fetch();
                        account.data.mobileNo = evt.mobileNo;
                        account.save();
                        
                        //--- Sync Account Object up to server ---
                        Ti.App.fireEvent($.syncMgr.EVENT.doSyncObjects, {
                        	objTypes	: [ 'WOSettings' ]
                        });
                        
                        
                        function _doSyncObjects(){
                        	Ti.App.removeEventListener($.syncMgr.EVENT.doSyncObjects, _doSyncObjects);
                        }
                    }
                    //--- trigger callback if any ---
                    if(evt.callback)    evt.callback();
                },
                errorback   : function(){
                    $.app.hideBusy();
                }
            });
        }
		function _doResendCode(evt){
			$.app.showBusy({message:App.Helper.L('hint_requesting')});
			
			$.authMgr.verifyMobile({
			    type        : 'changeMobileNo',
				mobileNo    : evt.mobileNo,
				resend      : true,
				callback    : function(result){
					$.app.hideBusy();
					proxy.fireEvent(proxy.EVENT.codeSent);
				},
				errorback   : function(result){
					$.app.hideBusy();
					proxy.fireEvent(proxy.EVENT.codeSent, {
						error	: true,
					});
				}
			});			
		}
		function _goSearchCountry(evt){
			var _winObj = $.app.route({
				url			: 'RegistrationController/searchCountry',
				eventProxy  : proxy,
				tabGroupObj	: winObj.tabGroupObj || undefined,
				navGroupObj	: winObj.navGroupObj || undefined,
			});
			var _proxy = _winObj.getEventProxy();
			_proxy.addEventListener(_proxy.EVENT.countrySelected, function(evt){
				winObj.updateCountry(evt);
				_winObj.close({animated:true});
			});
		}
    },

    //----------------------  Protected Methods  ----------------------
    
    _changePswd : function(param){
        var self = this;

        var payload = {
            old_pswd    : param.old,
            pswd        : param.pswd        
        };
        var callParam = {
            action      : App.Action.CHANGEPSWD,
            data        : payload,
            report      : true,
            title       : App.Helper.L('account_title_changePassword'),
            callback    : param.callback,
            errorback   : param.errorback
        };
        self.callServer(callParam);
    },
    
    _changeMobileNumber : function(param){
        var self  = this;

        var payload = {
            mobileNo    : param.mobileNo, 
        };       
        var callParam = {
            action      : App.Action.CHANGEMOBILE,
            data        : payload,
            report      : true,
            title       : App.Helper.L('account_title_mobileNumber'),
            callback    : param.callback,
            errorback   : param.errorback
        };
        self.callServer(callParam);
    },
    
});

module.exports = AccountController;

//---  End of file: AccountController.js
