/**
 * The main controller of Perkd app
 *
 * @author young@waveo.com
 */
Ti.include('/globals.js');
App.Helper.requireController('WOApplicationManager');

var ApplicationController = App.Class(App.Controller.WOApplicationManager, {
	className	: 'ApplicationController',

	//---  Constructor  ---------------	
	initialize : function(param){
		var self = this;
		self.base(param);

	    if (App.Config.IS_ANDROID) {
	    	Ti.Android.currentActivity.setRequestedOrientation(Titanium.Android.SCREEN_ORIENTATION_PORTRAIT);
	    	$.dummyroot = null;
	    }

	},

	initManagers : function(param){
		var self = this;
		self.base(param);

		//---- load application-specific (memory-resident) managers ----
		$.authMgr     = App.Helper.newController('AuthenticationController');
		$.notifyMgr	  = App.Helper.newController('NotificationController');
		$.perkMgr     = App.Helper.newController('PerkController');
		$.cardMgr	  = App.Helper.newController('CardController');
		$.prefMgr     = App.Helper.newController('PreferencesController');
		$.rewardMgr   = App.Helper.newController('RewardsController');

		//--- Usage tracking: record App launch time ---
	    Ti.App.Properties.setDouble(self.CONST.lastLaunched, new Date().valueOf());
		
		self.listenForEvents();
	},

	listenForEvents : function(){
		// NOTE:	event handler methods MUST be wrapped within an anonymous function
		//			see WOManager.js for explanations
		var self = this;
		if(self.isListenEvents)    return;

		//--- doStart ---
        Ti.App.addEventListener(self.EVENT.doStart, function(evt){
        	self.start(evt);
        });
		//--- doRun ---
        Ti.App.addEventListener(self.EVENT.doRun, function(evt){
        	self.run(evt);
        });
		//--- doRestart ---
        Ti.App.addEventListener(self.EVENT.doRestart, function(evt){
        	self.restart(evt);
        });

		// if (App.Config.IS_ANDROID) { // Trigger message backlogs.
			// Ti.App.addEventListener(self.EVENT.DNDchange, function(evt){
				// setTimeout(function(){
					// Ti.App.fireEvent($.notifyMgr.EVENT.doMessages);
				// }, 3000);
			// });
		// }

        //--- pause: system event ---
		Ti.App.addEventListener(self.EVENT.pause, function(){
			self.pause();
		});
		//--- resume: system event ---
		Ti.App.addEventListener(self.EVENT.resume, function(){
			self.resume();
		});

		//--- authMgr.forceReAuth --- 

		// //=============  TEST CODE  ==============
		// Ti.Gesture.addEventListener('shake', function(evt){
			// if($.rootWindow.VIEW.visible){
				// $.rootWindow.VIEW.setVisible(false);
			// }else{
				// $.rootWindow.VIEW.setVisible(true);
			// }
// 			
			// self.isDND = false;
			// $.notifyMgr.isShowingMsg = false;
// 
			// Ti.API.error('=====  Messages ENABLED ======');
		// });
		// //=========================================

		$.notifyMgr.checkLocalNotify();		// install local notification event listener
		
		self.isListenEvents = true;
	},
	
	start : function(){
		var self = this;

		Ti.API.info ('CTK>>>>>START');

		//---- things to do PRE-AUTH?  eg. handle resume, forced logout, etc ???

        //------  Authenticate User  ------
		if($.authMgr.isAuthed()){
            //--- we are logged in ---
            self.isLogin = true;

			//----- Fresh launch tasks, not for restarts, only executed ONCE per app launch or restart ----
			if(! self.hasStarted) {
				self.hasStarted = true;

				//--- FIXME  -> if Push previously not granted handle separately, random?

				//--- Get Push Notification Permission ---
				Ti.App.fireEvent($.notifyMgr.EVENT.doGetPushPermission);

				Ti.App.fireEvent(self.EVENT.ready);		// listened by syncMgr, notifyMgr, etc
			}
			
			//---- Do it -----
			self.run();
		}
	},

	/**
	 * 
	 */
    run : function(){
		var self = this;

		Ti.API.info ('CTK>>>>>RUN');

		self.hideBusy();
		var previousNavObj = $.currentNavGroupObj;

    	//--- close previous rootWindow when we had a previous login session
    	//  eg. skipped registration, then login from PerkdID page
		

		function runMain () {

	        $.rootWindow = self.route({
	            url     : 'MainController/main',
	        });
			Ti.API.info ('CTK>>>>>RUN has removed');
			if(previousNavObj){
		        setTimeout(function(){
		        	previousNavObj.close({ animated : false });
					// previousNav.dispose();
					// previousNav = null;
		        }, 3000);
			}
			//--- Get Location Permission ---
			setTimeout(function(){
				Ti.App.fireEvent($.configMgr.EVENT.doGetLocPermission);
			}, 10000);
			//---  silent update of location  ---
			var geo = App.loadHelper('locationInfo');
			if(geo.isAuthorized()){
				//if relaunch app, and authorized, to refresh location immediately!
				Ti.App.fireEvent($.configMgr.EVENT.doRefreshLocation);
			} else {
				setTimeout(function(){
					Ti.App.fireEvent($.configMgr.EVENT.doRefreshLocation);
				}, 30000);
			}

			self.isDND = false;		//---  start accepting messages
		}

		if($.rootWindow){
			
			Ti.API.info ('CTK>>>>>RUN has rootWindow');
			
			if (App.Config.IS_ANDROID) {

				if (!$.dummyRoot) {
					$.dummyRoot = Ti.UI.createWindow({
						exitOnClose:true,
						fullscreen:true,
						zIndex:1,
						// backgroundImage:'/assets/images/ui/bg.png',
					});
	//				var view = Ti.UI.createView({backgroundColor:'#FABA36'});
					// var view = Ti.UI.createView({backgroundImage:'/assets/images/ui/bg.png'});
					var view = Ti.UI.createView();
					$.dummyRoot.add(view);
					$.dummyRoot.open();
				}
	    		Ti.Android.currentActivity.setRequestedOrientation(Titanium.Android.SCREEN_ORIENTATION_PORTRAIT);

		  		('CTK>>>>>RESTART');

				function DelayedRun () {			
					$.rootWindow = null; // cleaned up.
					runMain ();
						if ($.dummyRoot) {
							$.dummyRoot.close();
							$.dummyRoot = null;			
						}
				 	Ti.API.removeEventListener ('ApplicationClosed', DelayedRun);					
				}
				Ti.API.addEventListener ('ApplicationClosed', DelayedRun);

				$.rootWindow.close();
				return;
			}

			// Setting timeout
			$.rootWindow.close();		
			$.rootWindow = null;
			
		}
		
		runMain ();

    },

	restart : function(){
		var self = this;

		Ti.API.info ('CTK>>>>>RESTART');

		//  close windows of sub-views here for smoother visual transitions
		// if($.currentNavGroupObj){
			// $.currentNavGroupObj.navGroup.close({ animated : false });
			// $.currentNavGroupObj.dispose();
			// $.currentNavGroupObj = null;
		// }

		self.hideBusy();

		//--- start with a new root window ---
		
		if($.rootWindow){
			
			if (App.Config.IS_ANDROID) {

				$.dummyRoot = Ti.UI.createWindow({exitOnClose:true,fullscreen:true});
				var view = Ti.UI.createView();
				$.dummyRoot.add(view);
				$.dummyRoot.open();
			
				Ti.API.info ('CTK>>>>>RESTART 1');
	
				function DelayedRestart () {
					 Ti.API.removeEventListener ('ApplicationClosed', DelayedRestart);					
					//---  reset states  ---
					self.isDND			= false;
					self.viewingCardID 	= null;
					self.isLogin		= false;
					self.hasStarted		= false;
			
			        self.start();
				}
				Ti.API.addEventListener ('ApplicationClosed', DelayedRestart);
				$.rootWindow.close();	
				return;
			}
	
			$.rootWindow.close();	
			$.rootWindow = null;
		}

		//---  reset states  ---
		self.isDND			= false;
		self.viewingCardID 	= null;
		self.isLogin		= false;
		self.hasStarted		= false;

        self.start();
	},

	stop : function(){
		var self = this;

		Ti.API.info ('CTK>>>>>STOP');

		if(self.hasStarted){
	        //--- Remove Event Handlers -----

		}
		self.hasStarted = false;
	},

    /**
     * handler when app being switched to background
     */
    pause : function(param){
    	var self = this;

		Ti.API.info ('CTK>>>>>PAUSE');

		//--- Schedule all the reminders before we leave the app  ---
		Ti.App.fireEvent($.notifyMgr.EVENT.doSetupReminders);

		//--- Usage tracking: calculate session time
    	var endTime = new Date().valueOf();
    	var startTime = Ti.App.Properties.getDouble(self.CONST.lastLaunched, endTime);
    	var sessionTime = endTime - startTime;		// in milliseconds

		Ti.App.Properties.setDouble(self.CONST.lastPaused, endTime);
		
		//---  Session Tracking  --------------
		var sessionInfo = {
			startTime	: startTime,
			endTime		: endTime,
			duration	: (sessionTime / 1000).toFixed(0),
		};
		$.trackMgr.track(self, 'session', sessionInfo);
    },

    /**
     * handler when app resume to foreground
     */
    resume : function(param){
    	var self = this;
					
		Ti.API.info ('CTK>>>>>RESUME');

		//--- Usage tracking: record App launch time ---
		var now = new Date().valueOf();
	    Ti.App.Properties.setDouble(self.CONST.lastLaunched, now);

    	if(self.isLogin){
	    	var lastPaused = Ti.App.Properties.getDouble(self.CONST.lastPaused, now);
	    	var timeLapsed = now - lastPaused;		// in milliseconds

			//--- reset key states for safety ---
			self.isDND = false;

			// Ti.App.fireEvent($.notifyMgr.EVENT.doMessages);

			if(timeLapsed >= self.CONST.sessionTime){
				// paused longer than sessionTime, considered a new session, re-auto-login
				Ti.App.fireEvent($.authMgr.EVENT.doLogin);
			}
			
			Ti.App.fireEvent($.configMgr.EVENT.doRefreshLocation);	// silient update of location
			Ti.UI.iPhone.appBadge = null;

			Ti.App.fireEvent(self.EVENT.ready);
    	}
    },

	errorPage : function(param){
		var self = this;
		var winObj = App.Helper.loadView('ErrorWindow', param);
		winObj.render();
		return winObj;
	},

});

module.exports = ApplicationController;

//---  End of file: ApplicationController.js
