/**
 * Implements Authentication, Registration, Login and Account management
 *
 * @author dinghua@waveo.com & young@waveo.com
 */
Ti.include('/globals.js');
App.Helper.requireController('WOAuthManager');

var AuthenticationController = App.Class(App.Controller.WOAuthManager, {
	className	: 'AuthenticationController',

	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
		
		self.autoLoggedIn = false;
	},

	listenForEvents : function(){
		var self = this;

		if(self.isListenEvents)    return;

		//--- doLogin ---
        Ti.App.addEventListener(self.EVENT.doLogin, function(evt){
        	self.login();
        });
		
		self.isListenEvents = true;
	},

	isSkippedReg : function(){
		var self = this;
		return (self.getAuthState().state == App.Const.AUTH_SKIPPED);
	},

	isLoggedIn : function(){
		var self = this;
		return (self.getAuthState().state == App.Const.AUTH_LOGGED_IN);
	},
   	
	//----------------------  Authentication Services  -----------------------
    
    /**
     * check if logged in. If not, start authentication flow, else do auto-login
     */
    isAuthed : function(){
        var self = this;

        var authState = self.getAuthState();
        var hadLoggedIn = (authState.state == App.Const.AUTH_LOGGED_IN) || (authState.state == App.Const.AUTH_SKIPPED);
        
        if(authState.token != null && authState.token != '' && hadLoggedIn){
	        
	        if(!$.accountMgr.exist()){
	        	$.syncMgr.resetUserObjects();
	        	$.app.route({
	                url		: 'cardLib/restore',
	                effect		: App.Const.WINDOW.effect.flip,
	            });
	            return false;
	        }
        	//----- Auto-login ------
        	if(! self.autoLoggedIn) {
        	    setTimeout(function(){
        			self.login();
        		}, 500);
        	}
            return true;
        }
        
        Ti.API.info('CTK>>>>>>IsAuth '+authState.state);
        
        if(authState.state == App.Const.AUTH_REGISTER){
        	//--- Registered but NOT verified ---
            $.app.route({
                url		: 'auth/startup',
            }); 
        } else {
            //----  Not Authenticated, start Authentication sequence
            $.app.route({
                url			: 'auth/authenticate',
                effect		: App.Const.WINDOW.effect.flip,
            });
        }
        return false;
    },

    /**
     * Authenticate with server. used by authenticate._doLogin()
     */
    auth : function(param){
        var self = this;

        var authResult = {};
        
        //---------------  Call Server  -------------
        var payload = {
            mobileNo    : param.mobileNo,
            password    : param.password,
            //--- extended payloads ---
            deviceToken : $.configMgr.getDeviceToken(),
            appInfo     : JSON.stringify($.configMgr.getAppInfo()),
            deviceInfo  : JSON.stringify($.configMgr.getDeviceInfo()),
            carrierInfo : JSON.stringify($.configMgr.getCarrierInfo()),
        };
        var callParam = {
            action      : App.Action.AUTHENTICATE,
            data        : payload,
            report      : true,
            title       : App.Helper.L('TITLE_LOGIN'),
            callback    : _authSuccess,
            errorback   : param.errorback,
        };
        self.callServer(callParam);
        
        //------------------  Callback Handlers  ----------------
        function _authSuccess(result){
            $.accountMgr.resetAccount();
            
            self.setAuthState({
                state   : App.Const.AUTH_LOGGED_IN,
                token   : result.token,
            });
            $.syncMgr.updateSyncDownObjects(result.objects);
            
            //--- Set language & language Region ---
			if('zh-Hant' == Ti.Locale.currentLanguage || 'zh' == Ti.Locale.currentLanguage) { //iOS
				var langRegion = $.prefMgr.get('langRegion', App.Config.LANG_RG);
				if(langRegion != '') Ti.App.Properties.setString('WOConfig.langRegion', langRegion);
				
				if('HK' == langRegion || 'MO' == langRegion) Ti.App.Properties.setString('WOConfig.language', 'zh-Hant');
				else Ti.App.Properties.setString('WOConfig.language', 'zh-Hant_TW');
				
				App.Config.LANG 	= Ti.App.Properties.getString('WOConfig.language', Ti.Locale.currentLanguage);
				App.Config.LANG_RG 	= Ti.App.Properties.getString('WOConfig.langRegion', '');
			}
            
            //--- already logged in at server, no need to do login
            self.autoLoggedIn = true;

            Ti.App.fireEvent(self.EVENT.authed);
            if(param.callback)		param.callback(result);
        }
    },

    getAuthState : function(){
        var authState = {
            state   : $.accountMgr.getAuthState(),
            token   : $.accountMgr.getAuthToken(),
        };
        return authState;
    },

    setAuthState : function(newState){
        if('undefined' != typeof newState.state) $.accountMgr.setAuthState(newState.state);
        if('undefined' != typeof newState.token) $.accountMgr.setAuthToken(newState.token);
    },

    /**
     * Login to server
     */
    login : function(param){    
        var self = this;
        param = param || {};

        var authState = self.getAuthState();
        if(!authState.token){
            $.watch.dog(self, self.className, 'failed to retrieve authToken');
            Ti.App.fireEvent(self.EVENT.forceReAuth);
            return;
        }

        //----------  Call Server  -----------
        var payload = {
            token           : authState.token,
            //--- extended payloads ---
            deviceToken     : $.configMgr.getDeviceToken(),
            appInfo         : JSON.stringify($.configMgr.getAppInfo()),
            deviceInfo      : JSON.stringify($.configMgr.getDeviceInfo()),
            carrierInfo     : JSON.stringify($.configMgr.getCarrierInfo()),
        };
        var callParam = {
            action      : App.Action.LOGIN,
            data        : payload,
            callback    : _loginSuccess,
            errorback   : param.errorback
        };
        self.callServer(callParam);

        //-----------------  Callback Handlers  -----------------
        function _loginSuccess(result) {
            self.autoLoggedIn = true;
            
            //---  Update account with latest info from server
            $.syncMgr.updateSyncDownObjects(result.objects);
            Ti.App.fireEvent(self.EVENT.login);
            if(param.callback)		param.callback(result);
        }
    },

    logout : function(param) {
        var self = this;

//		var benchmark = App.loadHelper('benchmark');
//		benchmark.mark('logout_start');
        self._preLogoutCleanUp();
//      benchmark.mark('logout_preCleanup');

        //----------  Call Server  -----------        
        var callParam = {
            action      : App.Action.LOGOUT,
            title       : App.Helper.L('Logout'),
            data        : {},
            callback    : _logoutDone,
            errorback   : _logoutDone,
        };
		self.callServer(callParam);

        //--------------  Callback Handlers  --------------
        function _logoutDone(result){

			if(!result.flag || ('Y' != result.flag)){
				//--- error with logout request to server, track in usage & proceed with logout anyway
				$.trackMgr.track(self, 'logout');
			}
//			benchmark.mark('logout_callserver');

            self._postLogoutCleanUp();
//            benchmark.mark('logout_postCleanup');

            // benchmark.elapsedTime('logout_start', 'logout_preCleanup');
            // benchmark.elapsedTime('logout_preCleanup', 'logout_callserver');
            // benchmark.elapsedTime('logout_callserver', 'logout_postCleanup');
            
            self.autoLoggedIn = false;
            $.app.hideBusy();
            if(param.callback) 		param.callback(result);
        }
    },
    
    //---------------------  Authentication View Controllers  --------------------

    authenticate : function(param){
        var self = this;

        //-----------  Setup Data Package  -----------
		var carrierInfo = $.configMgr.getCarrierInfo();
		var viewParam = { 
			DATA		: {
				mobilePrefix	: '+' + carrierInfo.mobileCountryCode,
			},
			navGroupObj	: param.navGroupObj,
		};
        //----  Load & Render View with viewParam ----
        if(param.data.inApp){

            //--- login within PerkID ---
            var winObj = App.Helper.loadView('auth/PerkdIDLoginView', viewParam);
			var proxy = winObj.getEventProxy();
            winObj.render();
        }else{

            //--- normal Login screen ---
            var winObj = App.Helper.loadView('auth/LoginView', viewParam);
			var proxy = winObj.getEventProxy();
            winObj.render();
            
            //------ Setup Event handlers -------
            proxy.addEventListener(proxy.EVENT.later, _showLaterView);
            proxy.addEventListener(proxy.EVENT.signup, _showSignUpView);
        }      
        //----------  Setup Event handlers  ----------
        proxy.addEventListener(proxy.EVENT.login, _doLogin);
        proxy.addEventListener(proxy.EVENT.resendPswd, _resendPassword);
        //----------  Return rendered View  ---------- 
        return winObj;

		//--------------------  Event Handlers  ------------------------
        function _doLogin(evt){
            $.app.showBusy({message:App.Helper.L('hint_logining')});   
            self.auth({
                mobileNo    : evt.mobileNo,
                password    : evt.pswd,
                callback    : function(param){
                    $.app.hideBusy();
                    
                    // show Push Notify page if necc, then proceed to RestoreCard page
                    $.notifyMgr.getPushPermission({
                    	navGroupObj	: winObj.navGroupObj,
                        callback	: _showRestoreCards,
                    });
                },
                errorback   : function(result){ $.app.hideBusy(); }
            });
        };
        function _showLaterView(evt){
            $.app.route({
                url     	: 'RegistrationController/later',
                navGroupObj	: winObj.navGroupObj,
            });
        };
        function _showSignUpView(evt){
            $.app.route({
                url     	: 'RegistrationController/signup',
                navGroupObj	: winObj.navGroupObj,
            });
        };
        function _resendPassword(evt){
            $.app.showBusy({message:App.Helper.L('hint_requesting')});

			//----------  Call Server  -----------            
			var callParam = {
                action      : App.Action.RESENDPSWD,
                report      : true,
                data        : { mobileNo  : evt.mobileNo },
                callback    : function(){
                    $.app.hideBusy();
                    $.app.showAlert(App.Helper.L('resend_ok'), App.Helper.L('title_login'));
                },
                errorback   : function(){
                    $.app.hideBusy();
                }
            };
            self.callServer(callParam);
        };
        function _showRestoreCards(){
            $.app.route({
                url     	: 'cardLib/restore',
                navGroupObj	: winObj.navGroupObj,
            });
        };
    },
    
    startup : function(param){
    	var self = this;
    	var viewParam = {
    		navGroupObj : param.navGroupObj,
    	}
    	//----  Load & Render View with viewParam ----
		var winObj = App.Helper.loadView('auth/StartupWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.goNext, _goVerify);
        //----------  Return Window Object  ---------- 
        return winObj;
        
        function _goVerify(){
        	$.app.route({
                url     	: 'auth/verify',
                navGroupObj	: winObj.navGroupObj,
                data		: Ti.App.Properties.getObject('tmp_reg_form', {}),
            });
        }
    },
    
	verify : function(param){
		var self = this;
		 App.Helper.requireModel('WOSettings');
		 
        //-----------  Setup Data Package  -----------
		var viewParam = { 
			DATA		: param.data,
			navGroupObj	: param.navGroupObj,
		};	
        //----  Load & Render View with viewParam ----
		var winObj = App.Helper.loadView('auth/VerifyWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.resendCode, _goResendCode);
		proxy.addEventListener(proxy.EVENT.verifyCode, _doVerifyCode);
		proxy.addEventListener(proxy.EVENT.searchCountry, _goSearchCountry);
        //----------  Return Window Object  ---------- 
        return winObj;
        
		//--------------------  Event Handlers  ------------------------
        function _doVerifyCode(evt){
            $.app.showBusy({message:App.Helper.L('hint_verifying')});
            
            self.verifyMobile({
                type        : 'register',
                mobileNo    : evt.mobileNo,
                resend      : false,
                code        : evt.code,
                callback    : _doCardRestore,
                errorback   : function(){
                	// invalid verification code, let user re-enter
                    $.app.hideBusy();
                }
            });
		}
		function _goSearchCountry(evt){
			var _winObj = $.app.route({
				url			: 'RegistrationController/searchCountry',
				eventProxy  : proxy,
				navGroupObj	: winObj.navGroupObj,
			});
			var _proxy = _winObj.getEventProxy();
			_proxy.addEventListener(_proxy.EVENT.countrySelected, function(evt){
				winObj.updateCountry(evt);
				_winObj.close({animated:true});
			});
		}
		function _goResendCode(){
			var _winObj = $.app.route({
				url			: 'auth/resendCode',
				eventProxy  : proxy,
				navGroupObj	: winObj.navGroupObj,
				data		: Ti.App.Properties.getObject('tmp_reg_form', {countryCode:'65', countryName:'Singapore'}),
			});
			var _proxy = _winObj.getEventProxy();
			_proxy.addEventListener(_proxy.EVENT.codeSent, function(evt){
				_winObj.close({animated:true});
			});
		}
		//--------------------  Callback Handlers  ------------------------
		function _doCardRestore(result){
			$.app.hideBusy();
        	//--- successful registration, setup card library ---
            self.setAuthState({ state : App.Const.AUTH_LOGGED_IN });
            $.app.route({
                url     	: 'cardLib/restore',
                navGroupObj	: winObj.navGroupObj,
                data		: {
                	callback	: _doRestoreDone,
                },
            });
		}
        function _doRestoreDone(){	
			//---  show Start Up Perk if any  ---
			$.notifyMgr.showStartUpPerk();

        	//--- close previous rootWindow when we had a previous login session
        	//  eg. skipped registration, then login from PerkdID page
			// if($.currentNavGroupObj){
				// $.currentNavGroupObj.navGroup.close({ animated : false });
				// $.currentNavGroupObj.dispose();
				// $.currentNavGroupObj = null;
			// }
			// if($.rootWindow){
				// $.rootWindow.close();
				// $.rootWindow.dispose();
				// $.rootWindow = null;
			// }
        }
	},
	
	resendCode : function(param){
		var self = this;
    	var viewParam = {
    		DATA		: param.data,
			navGroupObj	: param.navGroupObj,
		};
    	var winObj = App.Helper.loadView('auth/ResendCodeWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        
        proxy.addEventListener(proxy.EVENT.searchCountry, _goSearchCountry);
       proxy.addEventListener(proxy.EVENT.resendCode, _doResendCode);
        
		function _goSearchCountry(evt){
			var _winObj = $.app.route({
				url			: 'RegistrationController/searchCountry',
				eventProxy  : proxy,
				navGroupObj	: winObj.navGroupObj,
			});
			var _proxy = _winObj.getEventProxy();
			_proxy.addEventListener(_proxy.EVENT.countrySelected, function(evt){
				winObj.updateCountry(evt);
				_winObj.close({animated:true});
			});
		}
		function _doResendCode(evt){
			$.app.showBusy({message:App.Helper.L('hint_requesting')});
			
			self.verifyMobile({
			    type        : 'register',
				mobileNo    : evt.mobileNo,
				resend      : true,
				callback    : function(result){
					$.app.hideBusy();
					proxy.fireEvent(proxy.EVENT.codeSent);
				},
				errorback   : function(result){
					$.app.hideBusy();
				}
			});			
		}
        return winObj;
	},
	
	/**
	 * make sure we push everything to server BEFORE logging out!
	 */
	_preLogoutCleanUp : function(){
        $.syncMgr.pushUserObjects();
	},

	/**
	 * purge all local personalized data AFTER logout
	 */	
	_postLogoutCleanUp : function(){
        $.accountMgr.resetAccount();
	},

});

module.exports = AuthenticationController;

//---  End of file: AuthenticationController.js
