/**
 * CardController
 */
Ti.include('/globals.js');
if (App.Config.IS_ANDROID) {
	Ti.include('/waveo/core/WOAndroidUtils.js');
}

var CardController = App.Class(App.Controller.WOManager, {
	className : 'CardController',
    alias     : 'card',

	EVENT		: {
		//--- Handles these events:
		goPerkdIDView	: 'Nav.goPerkdIDView',
		goCardView		: 'Nav.goCardView',
		// displayNameChanged : 'ProfileController.displayNameChanged', //From Profile Controller.
		//--- Produces these events:
		cardAdded		: 'CardController.cardAdded',
		cardUpdated		: 'CardController.cardUpdated',
		cardDeleted		: 'CardController.cardDeleted',
		cardRemoved     : 'CardController.cardRemoved',
		cardAccepted	: 'CardController.cardAccepted',
		cardRelatedUpdated : 'CardController.cardRelatedUpdated',
    },

	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);

		self.cardListWinObj	= null;						// created by list() method
		self.viewingCardID	= null;						// card that's in view now
		self.viewingCardDetailWinObj = null;
		self.cardListRendered = false;

		var factoryParam = {
			style	: 'CardFactoryStyle',
		};
		self.factory = new App.WOCardFactory();			// this is publicly accessible
		self.factory.initialize(factoryParam);
		self.spareFactory = new App.WOCardFactory();	// need a 2nd factory to render 'singleRow' formats
		self.spareFactory.initialize(factoryParam);

		//---  register object sync services with SyncMgr  ---
		// so that Card Objects are sync'd with server IMMEDIATELY on these events
		
		var eventList = [ self.EVENT.cardAdded, self.EVENT.cardUpdated, self.EVENT.cardRemoved];//self.EVENT.cardDeleted ];
		$.syncMgr.registerObjectSync(eventList, 'WOCard');
		
		self.listenForEvents();
	},

	listenForEvents : function(){
		var self = this;

        //--- goPerkdIDView -----
		Ti.App.addEventListener(self.EVENT.goPerkdIDView, function(evt){
			$.app.route({
				url			: 'card/detail',
				data		: { cardID: $.accountMgr.getPerkdCardID() },
				tabGroupObj	: $.rootWindow,
			});
		});
		//--- goCardView --------
		Ti.App.addEventListener(self.EVENT.goCardView, function(evt){
			$.rootWindow.tabGroup.setActiveTab(0);
			$.rootWindow.home();
			$.app.route({
				url			: 'card/detail',
				data		: {
					cardID	: evt.cardID,
					noFlip	: ('undefined' != typeof evt.noFlip) ? evt.noFlip : true,
				},
				tabGroupObj	: $.rootWindow,
			});
		});

		//----------------  Card Events  ------------------
		//--- cardAdded --------
		Ti.App.addEventListener(self.EVENT.cardAdded, function(evt){
			self.cardListWinObj.addCard($.dataMgr.get(App.Const.MODEL_CARD, evt.id));
		});
		//--- cardDeleted ------
		Ti.App.addEventListener(self.EVENT.cardDeleted, function(evt){
			self.cardListWinObj.deleteCard({ cardID: evt.cardID });
		});
		//--- cardUpdated------
		Ti.App.addEventListener(self.EVENT.cardUpdated, function(evt){
			if(self.cardListWinObj){
				var proxy = self.cardListWinObj._eventProxy;
				proxy.fireEvent(proxy.EVENT.refresh, { data: evt.data });
			}
		});
		//--- cardDisplayChanged --------
		// Ti.App.addEventListener(self.EVENT.displayNameChanged, function(evt){
			// /*
			 // * evt : {
			 // * 	 data : {
			 // * 		oldName : {
			 // * 			display : 'xxx',
			 // * 			fname : '',
			 // * 			gname : '',
			 // * 		},
			 // * 		newName : {
			 // * 			display : 'yyy',
			 // * 			fname : '',
			 // * 			gname : '',
			 // * 		},
			 // *   }
			 // * }
			 // */
			// var _cardMgr = $.dataMgr.newModel('WOCard');
			// _cardMgr.fetchAll();
			// for(var i=0; i < _cardMgr.list.length; i++) {
				// var _card = _cardMgr.nth(i);
				// var _cardData = _card.p2l();
				// if(_cardData.canModDisplay && _cardData.display.toLowerCase() == evt.data.oldName.display.toLowerCase()) {
					// if('undefined' != typeof _card.data.profile.display_name
					// || ('undefined' != typeof _card.data.profile.fname && 'undefined' != typeof _card.data.profile.gname)) {
						// _card.data.display = evt.data.newName.display;
						// if('undefined' != typeof _card.data.profile.display_name) _card.data.profile.display_name = evt.data.newName.display;
						// if('undefined' != typeof _card.data.profile.fname && 'undefined' != typeof _card.data.profile.gname) {
							// _card.data.profile.fname = evt.data.newName.fname;
							// _card.data.profile.gname = evt.data.newName.gname;
						// }
						// _card.save();
					// }
				// }
			// }
		// });

		//----------------  Sync Mgr Events  --------------
        //--- Card Updates -----
		Ti.App.addEventListener('WOCard.'+$.syncMgr.EVENT.syncUpdate, function(evt){
			if(self.cardListWinObj){
				var _cardsList = self._getCardListdata();
				self.cardListWinObj.setListData({ cards: _cardsList });
			}
		});
        //--- Card Master Updates -----
		Ti.App.addEventListener('WOCardMaster.'+$.syncMgr.EVENT.syncUpdate, _refreshCardList);

       	function _refreshCardList(evt){
       		if(self.cardListWinObj){
				var cardListData = self._getCardListdata();
				var proxy = self.cardListWinObj._eventProxy;
				for(card in cardListData){
					proxy.fireEvent(proxy.EVENT.refresh, { data: cardListData[card] });
				}
			}
       	}
       	
       	Ti.App.addEventListener('WOOffer.'+$.syncMgr.EVENT.syncUpdate, function(evt){
       		Ti.App.fireEvent(self.EVENT.cardRelatedUpdated);
       	});
       	
       	Ti.App.addEventListener('WORewardset.'+$.syncMgr.EVENT.syncUpdate, function(evt){
       		Ti.App.fireEvent(self.EVENT.cardRelatedUpdated);
       	});
       	
		//--- resume  (System event) ---
		Ti.App.addEventListener($.app.EVENT.resume, function(evt){
       		self.viewingCardID = null;
       		self.viewingCardDetailWinObj = null;
       	});

		//---------------  Perk Mgr Events  --------------
		//--- doCardIssue -------
		Ti.App.addEventListener($.perkMgr.EVENT.issueCard, function(evt){
			// alert('issueCard... CardController hear you!');
			// alert(evt)
			self.issueCard(evt);
		});
		//--- doStampIssue ------
		Ti.App.addEventListener($.perkMgr.EVENT.issueStamp, function(evt){
			// alert('issueStamp... CardController hear you!');
			// alert(evt);
			self.issueStamp(evt);
		});
		//--- doStampDeduct ------
		Ti.App.addEventListener($.perkMgr.EVENT.deductStamp, function(evt){
			// alert('issueStamp... CardController hear you!');
			// alert(evt);
			self.deductStamp(evt);
		});
		//--- doOfferIssue -----
		Ti.App.addEventListener($.perkMgr.EVENT.issueOffer, function(evt){
			// alert('issueOffer... CardController hear you!');
			// alert(evt);
			self.issueOffer(evt);
		});
	},
	
	issueCard : function(param){
		var self = this;
		
		var message = $.dataMgr.newModel('WOMessage');
		message.data = {
			type	: $.notifyMgr.TYPE.perkMsg,
			subType : $.notifyMgr.SUBTYPE.perkd.issueCard,
			source	: '',
			sourceAction : '',
			action	: [],
			content	: {
				source	: '',
				sourceAction : '',	
				title : '',
				desc : '',
				//------ param -----
				cardMID : param.data.cardMID,
				cardID	: param.data.cardID,
			}
		}
		message.save();
		
		// Ti.App.fireEvent($.notifyMgr.EVENT.doPerkd, { msgObj: message.fetch() });			
		Ti.App.fireEvent($.notifyMgr.EVENT.doMessages);
	},
	
	deductStamp : function(param){
		var self = this;
		
		var rewardsetMaster = $.dataMgr.newModel('WORewardsetMaster', param.data.rewardsetMID);
		var result = rewardsetMaster.deductStamp(param.data.stampID, param.data.stateID, param.data.quantity);
		
		var numStampsDeducted = result.total - result.remain;
		var numOffersDeducted = result.offerIDs.length;
		$.app.showAlert(String.format( App.Helper.L('ALERT_STAMPANDOFFERDEDUCTED'), numStampsDeducted, numOffersDeducted));
		
		if(numOffersDeducted > 0) Ti.App.fireEvent(self.EVENT.cardRelatedUpdated);
		
		if(result.remain == result.total) return ;
		
		// var message = $.dataMgr.newModel('WOMessage');
		// message.data = {
			// type : $.notifyMgr.TYPE.perkMsg,
			// subType : $.notifyMgr.SUBTYPE.perkd.issueStamp,
			// action : [],
			// content : {
				// source : param.data.source, // the stamp issued by ...
	    		// sourceAction : 2, // issue
				// title : 'Deduct Stamp',
				// desc : '',
				// //------ param -----	
				// cardID 		: result.cardID,
				// rewardsetMID: param.data.rewardsetMID,
				// stampIDs 	: result.stampIDs,
// 				
				// // ---- no use after check with model definition -----
				// // offerIDs	: result.offerIDs,
				// // checkOnline : result.checkOnline,
				// // brand 		: param.data.brand || '',
			// }
		// }
		// var msgID = message.save();
// 		
		// //--- fire event, notifyMgr should handle me ---
		// if(msgID) Ti.App.fireEvent($.notifyMgr.EVENT.doPerkd, { msgID : msgID });
		
		if(result.checkOnline) {
			Ti.App.fireEvent($.syncMgr.EVENT.doSyncObjects, {
        		objTypes	: [ 'WORewardset', 'WOOffer' ]
        	});
       	}
		
		return result;
	},
	
	issueStamp : function(param){
		var self = this;
		
		if('undefined' == param.data.dailyLimit) param.data.dailyLimit = 0;
		
		var rewardsetMaster = $.dataMgr.newModel('WORewardsetMaster', param.data.rewardsetMID);
		var _rewardsetMaster = rewardsetMaster.p2l();
		var result = rewardsetMaster.issueStamp(param.data.stampID, param.data.stateID, param.data.quantity, param.data.dailyLimit);
		
		if(result.remain == result.total) return ;

		var message = $.dataMgr.newModel('WOMessage');
		message.data = {
			type : $.notifyMgr.TYPE.perkMsg,
			subType : $.notifyMgr.SUBTYPE.perkd.issueStamp,
			action : [],
			content : {
				source : param.data.source || '', // the stamp issued by ...
	    		sourceAction : 2, // issue
				title : '',
				desc : '',
				//------ param -----	
				cardID 		: result.cardID,
				rewardsetMID: param.data.rewardsetMID,
				stampIDs 	: result.stampIDs,
				
				// ---- no use after check with model definition -----
				// offerIDs	: result.offerIDs,
				// checkOnline : result.checkOnline,
				// brand 		: param.data.brand || '',
			}
		}
		var msgID = message.save();
		
		//--- fire event, notifyMgr should handle me ---
		if(msgID) Ti.App.fireEvent($.notifyMgr.EVENT.doPerkd, { msgID : msgID });
		
		if(result.checkOnline) {
			Ti.App.fireEvent($.syncMgr.EVENT.doSyncObjects, {
        		objTypes	: [ 'WORewardset', 'WOOffer', 'WOMessage']
        	});
        }
        
        Ti.App.fireEvent(self.EVENT.cardRelatedUpdated);
		return result;
	},
	
	issueOffer : function(param){
		var self = this;
		
    	var offer = $.dataMgr.newModel('WOOffer');
    	offer.data.masterID = param.data.offerMID;
    	
    	var cards = $.dataMgr.getRelated('WOCardMaster', param.data.cardMID, 'WOCard');
    	if(!cards.length) {
    		var cardMaster = $.dataMgr.get('WOCardMaster', param.data.cardMID);
    		$.app.showAlert(String.format( App.Helper.L('ALERT_NeedAddCardForOffer'), cardMaster.fullName));
    		return;
    	}
    	var cardID = cards[0].id;
    	offer.data.cardID = cardID;
    	
    	var offerID = offer.save();
    	var offerData = $.dataMgr.get('WOOffer', offerID);
    	
    	var message = $.dataMgr.newModel('WOMessage');
		message.data = {
			type : $.notifyMgr.TYPE.perkMsg,
			subType : $.notifyMgr.SUBTYPE.perkd.issueOffer,
			action : [],
			content : {
				source : offerData.brandName,
				sourceAction : 2,
				title		: '',
				desc 		: '',
				//------ param -----
				cardID		: cardID,
				offerIDs	: [offerID],
			}
		}
		var msgID = message.save();
		
		if(msgID) Ti.App.fireEvent($.notifyMgr.EVENT.doPerkd, { msgID : msgID });
		
		Ti.App.fireEvent(self.EVENT.cardRelatedUpdated);
		return offerID;
	},
	
    //----------------------  Card Services  -----------------------------	
	submitRegister : function(param){
		var self = this;
		$.app.doNotDisturb(true);
		var callParam = {
	    	action		: 'card_submit_registration',
	    	report		: param.report,
	    	data		: {
				cardID		: param.cardID,
				formData	: JSON.stringify(param.formData)
	    	},
	    	callback	: function(result){
	    		if('undefined' != typeof result.objects)
	    			$.syncMgr.updateSyncDownObjects(result.objects);

				if(param.callback) 		param.callback();
				// Ti.App.fireEvent(self.EVENT.cardUpdated, { data: _card.p2l() });     //TODO [ZJ] double check!!!
			},
	    	errorback	: param.errorback
	    };
	    
	    $.syncMgr.callServer(callParam);
	},

	// registerCard : function(param){
		// var self = this;
// 		
    	// var callParam = {
	    	// action : App.Action.REGISTERCARD,
	    	// report : true,
	    	// data:{
				// mid         : param.mid,
				// register	: JSON.stringify(param.register)
	    	// },
	    	// callback  : param.callback,
	    	// errorback : param.errorback
	    // };
	    // $.syncMgr.callServer(callParam);
	// },

	deleteCard : function(cardID){
		var self = this;
        App.Helper.requireModel('WOCard');
		
		Ti.API.debug('CardController::deleteCard  ID='+cardID);
		
		if(cardID){
			var card = $.dataMgr.get('WOCard', cardID);
			var _card = new App.Model.WOCard(cardID);
			_card.remove();
			
			Ti.App.fireEvent(self.EVENT.cardDeleted, { cardID : cardID });
			
			if(card.isDigital) {//only Digital card , sync immediately 
				setTimeout(function(evt){
					Ti.App.fireEvent(self.EVENT.cardRemoved, { cardID : cardID });
				}, 3000);
			}
		}
	},

	doShare : function(param){
		var self = this;
		
		var total = param.total;
		var callParam = {
	    	action		: App.Action.SHARECARD,
	    	data		: {
	    		share_method	: param.share_mode,
				cins_id			: param.cardID,
				mobiles			: param.mobiles
	    	},
	    	callback	: function(result){
	    		if(1 == total) var message = App.Helper.L('shareCard_cardShared1');
	    		else var message = String.format(App.Helper.L('shareCard_cardShared2'), total);
	    		$.app.notify({ message: message});
	    		
				if(param.callback) param.callback();
	    	}
	    };
	    $.syncMgr.callServer(callParam);
	},
			
    //-----------------------  Card View Controllers  ------------------------	
	list : function(param){
		var self = this;
		App.Helper.requireModel('WOCard');

        //-----------  Setup Data Package  -----------
        var cardsList = self._getCardListdata();
		var viewParam = { 
			DATA		: { cards: cardsList },
			tabGroupObj	: param.tabGroupObj || undefined,
			navGroupObj	: param.navGroupObj || undefined,
		};
		
        //----  Load & Render View with viewParam ----
 		self.cardListWinObj = App.Helper.loadView('card/CardListWindow', viewParam);
		var proxy = self.cardListWinObj.getEventProxy();
        self.cardListWinObj.render();
        
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.addCard, _goDiscoverTab);
		proxy.addEventListener(proxy.EVENT.viewCard, _doViewCard);
		proxy.addEventListener(proxy.EVENT.goPerkdIDView, _doViewPerkdID);
		proxy.addEventListener(proxy.EVENT.fetchData, _doFetchData);
		// proxy.addEventListener('data.order', function(event){		// TODO  not implemented
			// var cardsData = self._getCardListdata(event.order);
			// view.update({cards:cardsData});
		// });		
        //----------  Return Window Object  ---------- 
        return self.cardListWinObj;

		//------------  Event Handlers  --------------
		function _goDiscoverTab(evt){
			$.rootWindow.tabGroup.setActiveTab(1);
		}
		function _doViewCard(evt){
			var _cardID = evt.cardID;
			if(_cardID){
				var detailWin = $.app.route({
					url			: 'card/detail', 
					tabGroupObj	: param.tabGroupObj || undefined,
					navGroupObj	: param.navGroupObj || undefined,
					tab			: 0,
					data		: { cardID: _cardID }
				});
			}
		}
		function _doViewPerkdID(evt){
			Ti.App.fireEvent(self.EVENT.goPerkdIDView);
		}
		function _doFetchData(evt){
			$.app.doNotDisturb(true);
			self.cardListRendered = false;
			var _cardsList = self._getCardListdata();
			self.cardListWinObj.setListData({ cards: _cardsList });
			$.app.doNotDisturb(false);
			self.cardListRendered = true;
		}
	},
	
	/*
	 * param.cardID
	 */	
	detail : function(param){
		var self = this;

		self.viewingCardID = param.data.cardID;
		self.viewingCardDetailWinObj = null;

		var _card = $.dataMgr.get('WOCard', param.data.cardID);
		$.trackMgr.track(_card, 'view', { 
			cardMID : _card.masterID,
			cardID : param.data.cardID,
		});

		if(_card.masterID == App.Config.PERKD_ID){
			//---  Card is Perkd ID  --------------------------
			if($.authMgr.isSkippedReg()){
				//--  skipped registration  ---
				
		        //-----------  Setup Data Package  -----------
				App.Helper.requireModel('WOSettings');
		        var account = new App.Model.WOSettings('account');
		        account.fetch();
	
	 			var viewParam = {
					DATA	: { isSkipped	: true },
				};
		        //----  Load & Render View with viewParam ----	
		        var winObj = App.Helper.loadView('account/PerkIDSkippedWindow', viewParam);
				var proxy = winObj.getEventProxy();
		        winObj.render();
		        //----------  Setup Event handlers  ----------
				proxy.addEventListener(proxy.EVENT.signup, _doSignUp);
				proxy.addEventListener(proxy.EVENT.login, _doLogin);
				proxy.addEventListener(proxy.EVENT.later, _doLater);
		        //----------  Return Window Object  ---------- 
				return winObj;
			}else{
				//-----------  Setup Data Package  -----------
				var viewParam = {
					DATA	: {
						aCard : _card,
					},
					tabGroupObj	: param.tabGroupObj || undefined,
					navGroupObj	: param.navGroupObj || undefined,
				};		        
		        //----  Load & Render View with viewParam ----
		 		var winObj = App.Helper.loadView('card/PerkdIDDetailWindow', viewParam);
				var proxy = winObj.getEventProxy();
		        winObj.render();
		        //----------  Setup Event handlers  ----------
				proxy.addEventListener(proxy.EVENT.manageAccount, _doAccountMgt);
		        //----------  Return Window Object  ---------- 
		        return winObj;
			}
		}else{
			//-----------  Setup Data Package  -----------
			var viewParam = {
				DATA	: {
					aCard : _card,
					rewardsets : $.dataMgr.getRelated('WOCard', param.data.cardID, 'WORewardset'),
					offers : $.dataMgr.getRelated('WOCard', param.data.cardID, 'WOOffer'),
					points : $.dataMgr.getRelated('WOCard', param.data.cardID, 'WOPoints', {order:{'instance.transacted':'DESC'}}),
				},
				tabGroupObj	: param.tabGroupObj || undefined,
				navGroupObj	: param.navGroupObj || undefined,
			};
	        //----  Load & Render View with viewParam ----
	 		var winObj = App.Helper.loadView('card/CardDetailWindow', viewParam);
			var proxy = winObj.getEventProxy();
	        winObj.render();
	        //----------  Setup Event handlers  ----------
			proxy.addEventListener(proxy.EVENT.registerCard, _goCardRegister);
			proxy.addEventListener(proxy.EVENT.editCard, _goCardEdit);
			proxy.addEventListener(proxy.EVENT.toShareCard, _goCardShare);
			proxy.addEventListener(proxy.EVENT.viewAbout, _goCardAbout);
			proxy.addEventListener(proxy.EVENT.viewRewardset, _goRewardsetView);
			proxy.addEventListener(proxy.EVENT.viewOffer, _goOfferView);
			proxy.addEventListener(proxy.EVENT.viewPoints, _goPointsView);
			proxy.addEventListener(proxy.EVENT.closedCardDetailWindow, _closedCardDetailWindow);
			
			Ti.App.addEventListener(self.EVENT.cardUpdated, _doUpdateCard);
			Ti.App.addEventListener(self.EVENT.cardRelatedUpdated, _doUpdateDashboard);
			
			self.viewingCardDetailWinObj = winObj;
	        //----------  Return Window Object  ---------- 
	        return winObj;
		}
		
		//---------------------  Event Handlers  --------------------
		function _closedCardDetailWindow(evt){
			self.viewingCardID = null;
			self.viewingCardDetailWinObj = null;
			Ti.App.removeEventListener(self.EVENT.cardUpdated, _doUpdateCard);
			Ti.App.removeEventListener(self.EVENT.cardRelatedUpdated, _doUpdateDashboard);
		}
		function _doUpdateDashboard(evt){
			var cardID = _card.id;
			winObj.updateDashboard({
				aCard 		: $.dataMgr.get('WOCard', cardID),
				rewardsets 	: $.dataMgr.getRelated('WOCard', cardID, 'WORewardset'),
				offers 		: $.dataMgr.getRelated('WOCard', cardID, 'WOOffer'),
				points 		: $.dataMgr.getRelated('WOCard', cardID, 'WOPoints', {order:{'instance.transacted':'DESC'}}),
			});
		}
		function _doUpdateCard(evt){
			proxy.fireEvent(proxy.EVENT.refresh, { data: evt.data });
			winObj.updateCard(evt.data);
		}
 		function _doSignUp(evt){
			$.app.route({
				url			: 'RegistrationController/signup',
				navGroupObj	: winObj.navGroupObj,
			});
		}
		function _doLogin(evt){
			$.app.route({
				url			: 'auth/authenticate',
				navGroupObj	: winObj.navGroupObj,
				data		: { inApp : true },
			});
		}
		function _doLater(evt){
            $.app.route({
				url			: 'RegistrationController/later',
				navGroupObj	: winObj.navGroupObj,
				data		: { isSkipped: evt.isSkipped },
			});
		}
		function _goCardAbout(evt){
			$.app.route({
				url			: 'card/about',
				tabGroupObj	: winObj.tabGroupObj,
				data		: {
					cardID 	: evt.cardID,
					about 	: evt.about,
				}
			});
		}
		function _goCardEdit(evt){
			$.app.route({
				url			: 'card/edit',
				tabGroupObj	: winObj.tabGroupObj,
				// effect	: App.Const.WINDOW.effect.flip,
				data		: { cardID : evt.cardID }
			});
		}
		function _goCardRegister(evt){
			$.app.route({
				url		: 'card/register', 
				tabGroupObj	: winObj.tabGroupObj,
				// effect	: App.Const.WINDOW.effect.flip,
				data	: { cardID : evt.cardID }
			});
		}
		function _goCardShare(evt){
			if(!Ti.Network.online) {
		        $.app.showAlert(App.Helper.L('ALERT_NetworkErrorShareCard'), App.Helper.L('TITLE_SHARE'));
		        return;
		    }
			var authState = $.authMgr.getAuthState();
			if(authState.state != App.Const.AUTH_LOGGED_IN){
				var dialog = Ti.UI.createAlertDialog({
					message: App.Helper.L('ALERT_SignupBeforeShare'),
					title: App.Helper.L('TITLE_SHARE'),
					buttonNames:[ App.Helper.L('BUTTON_CANCEL'), App.Helper.L('BUTTON_SIGNUPNOW')],
				});
				dialog.addEventListener('click', function(e){
					if(e.index==1){
						Ti.App.fireEvent(self.EVENT.goPerkdIDView);
					}
				});
				dialog.show();
				return;
			}
			//---  iOS 6 compatibility (when compiled with iOS 5 SDK)  ---
			try{
				$.app.route({
					url			: 'card/share',
					tabGroupObj	: winObj.tabGroupObj,
					data		: { cardID : evt.cardID },
				});			
			}catch(err){
				//--- Didn't get permission to access Contacts info ---
    			$.app.showAlert( App.Helper.L('ALERT_AllowToShare'));
			}
		}
		function _goRewardsetView(evt){
			$.app.route({
				url			: 'reward/list',
				tabGroupObj	: winObj.tabGroupObj,
				data		: { cardID : evt.cardID },
			});
		}
		function _goOfferView(evt){
			$.app.route({
				url			: 'reward/offerList',
				tabGroupObj	: winObj.tabGroupObj,
				data		: { cardID : evt.cardID },
			});
		}
		function _goPointsView(evt){
			$.app.route({
				url			: 'reward/points',
				tabGroupObj	: winObj.tabGroupObj,
				data		: { cardID : evt.cardID },
			});
		}
		function _doAccountMgt(evt){
			$.app.route({
				url			: 'account/account',
				tabGroupObj	: winObj.tabGroupObj,
			});
		}
	},
	
	/*
	 * param.cardID
	 */	
	edit : function(param){
		var self = this;

		//-----------  Setup Data Package  -----------
		var _card = $.dataMgr.get('WOCard', param.data.cardID);
		var cardMaster = $.dataMgr.newModel('WOCardMaster', _card.masterID);
		cardMaster.fetch();
		
		var viewParam = {
			DATA		: {
				aCard : _card,
				form : cardMaster.getCardEditFormDefn(cardMaster.p2l()),
			},
			tabGroupObj	: param.tabGroupObj || undefined,
			navGroupObj : param.navGroupObj || undefined,
		};
        //----  Load & Render View with viewParam ----
 		var winObj = App.Helper.loadView('card/CardEditWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.formSubmit, _doEditSubmit);
		proxy.addEventListener(proxy.EVENT.deleteCard, _doCardDelete);
        //----------  Return Window Object  ---------- 
        return winObj;

		//------------------  Event Handlers  -------------------
		function _doCardDelete(evt){
			//---  Confirmation Dialog  ---
			var dialog = Ti.UI.createAlertDialog({
				message		: App.Helper.L('ALERT_SureToRemoveCard'),
				title		: App.Helper.L('TITLE_CONFIRM'),
				buttonNames	:[ App.Helper.L('BUTTON_CANCEL'), App.Helper.L('BUTTON_DELETE')],
			});
			dialog.addEventListener('click', function(e){
				if(e.index==1){
					//---  Proceed to delete  ---
					self.deleteCard(evt.cardID);
					$.trackMgr.track(_card, 'delete', { cardMID: _card.masterID, cardID: _card.id });
					winObj.home();
				}
			});
			// toReEnable a button needed?
			dialog.show();
		}
		function _doEditSubmit(evt){
			var _formdata = evt.formData;	
			var _card = $.dataMgr.newModel('WOCard', evt.cardID);
			_card.fetch();
			var _cardData = _card.p2l();
			
			if(typeof(_formdata.join) != 'undefined') _card.set('joined', _formdata.join);
			if(typeof(_formdata.expiry) != 'undefined') _card.set('expiry', _formdata.expiry);
			if(typeof(_formdata.card_number) != 'undefined' && typeof(_formdata.card_number.number) != 'undefined') {
				var theNumber = _formdata.card_number.number;
				Ti.API.info('CTK(Number): '+theNumber);
				if (theNumber!=null) {
	    			theNumber = theNumber.replace(/(^\s*)|(\s*$)/gi,"");
    				theNumber = theNumber.replace(/\n /,"\n");
					Ti.API.info('CTK(Number?): '+theNumber);		
					_formdata.card_number.number = theNumber;
				}
 				_card.set('number', _formdata.card_number.number);
 				//Patch for card number format. will complted in v2.3.x - TODO [ZJ]
 				if(typeof(_formdata.card_number.type) == 'undefined' || !_formdata.card_number.type) {
 					if(typeof(_card.data.profile) != 'undefined' && typeof(_card.data.profile.card_number) != 'undefined' && '' != _formdata.card_number.number) {
 						if(_card.data.profile.card_number.type) _formdata.card_number.type = _card.data.profile.card_number.type;
 					}
 				}
			}
			if(typeof(_formdata.display_name) != 'undefined') {
				$.prefMgr.set('cardDisplayName', _formdata.display_name);
				_card.set('display', _formdata.display_name);
			} else if(_cardData.canModDisplay && 'undefined' != typeof _formdata.fname && 'undefined' != typeof _formdata.gname){
				_card.set('display', App.Helper.getDisplayName($.accountMgr.getProfile().nameOrder, _formdata.fname, _formdata.gname));
			}
			_card.set('profile', 	_formdata);
			_card.set('register', 	_formdata);
			
			var valid = _card.validate();
			if(true != valid){
				$.app.showAlert(valid[0]['msg'], valid[0]['field']);
				// view.fireEvent('reset');						//  FIXME [young]  what is this????
				return;
			} else {
				_card.save();
				Ti.API.error($.dataMgr.get('WOCard', self.viewingCardID));
				if(self.viewingCardDetailWinObj && self.viewingCardID) self.viewingCardDetailWinObj.update($.dataMgr.get('WOCard', self.viewingCardID));
				$.trackMgr.track(_card, 'edit', { cardMaster: _card.masterID, cardID: _card.id });
				winObj.close({animated : true});
				Ti.App.fireEvent(self.EVENT.cardUpdated, { data: $.dataMgr.get('WOCard', evt.cardID) });
			}
		}
	},
	
	/*
	 * param.cardID
	 * param.about
	 */
	about : function(param) {
		var self = this;
		
		//-----------  Setup Data Package  -----------
		// var _card = $.dataMgr.get('WOCard', param.data.cardID);
		var viewParam = {
			DATA		: {
				about	: param.data.about,
			},
			tabGroupObj	: param.tabGroupObj || undefined,
			navGroupObj	: param.navGroupObj || undefined,
		};
        //----  Load & Render View with viewParam ----
 		var winObj = App.Helper.loadView('card/CardAboutWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
        //----------  Return Window Object  ---------- 
        return winObj;

		//------------------  Event Handlers  -------------------		
	},
	
	addCard : function(param){
		var self = this;
		
		var _cardMaster = param.data.cardMaster || $.dataMgr.get('WOCardMaster', param.data.cardMID);
		
		var _form = false;
		if(param.data.form) for(var key in param.data.form) { _form = param.data.form ; break;}
		if(!_form) {
			var cardMaster = $.dataMgr.newModel('WOCardMaster');
			_form = cardMaster.getCardFormDefn(_cardMaster, 'add');
		}
		
		var account = $.dataMgr.get('WOSettings', 'account');
		var defaultDisplay = $.prefMgr.get('cardDisplayName', '');
		var defaultFormData = {
			fname : account.familyName || null,
			gname : account.givenName || null,
			gender : account.gender || 0,
			birth : account.birth || null,
			email_opt : 'Y',
			email : account.email || null,
			mobile_opt : 'Y',
			mobile : account.mobileNo || null,
			display_name : defaultDisplay || account.displayName,
		};
		
        //-----------  Setup Data Package  -----------
		var viewParam = { 
			DATA		: {
				authState	: param.data.authState,
				cardMaster	: _cardMaster,
				form		: _form,
				formData	: defaultFormData,
			},
			tabGroupObj	: param.tabGroupObj || undefined,
			navGroupObj	: param.navGroupObj || undefined,
		};

        //----  Load & Render View with viewParam ----
 		var winObj = App.Helper.loadView('card/CardAddWindow', viewParam);
		var proxy = winObj.getEventProxy(self);
        winObj.render();

        //----------  Setup Event handlers  ----------
        proxy.addEventListener(proxy.EVENT.formSubmit, _doFormSubmit);
		// proxy.addEventListener(proxy.EVENT.addCard, function(event){
			// $.rootWindow.setActiveTab(1);
		// });
        
        //----------  Return rendered View  ---------- 
        return winObj;

		//------------  Event Handlers  --------------
		function _doFormSubmit(evt){
		/*
		 * evt.isDigital 
		 * evt.masterID
		 * evt.formData
		 * evt.cardDefn
		 */
			$.app.doNotDisturb(true);
			if('undefined' != typeof _form.resource && 'undefined' != typeof _form.resource.messages){
				var stupidReport = false;
			} else {
				var stupidReport = true;
			}

			if(evt.isDigital){
				$.app.showBusy({message: App.Helper.L('SHOW_REGISTERINGCARD')});
				self._cardRegisterSubmit({
					mid			: evt.masterID,
					register	: evt.formData,
					callback	: _cardRegisterDone,
					errorback	: function(r){
						$.app.hideBusy();
						// view.fireEvent('reset');			//  FIXME  [young]  What is this???
						if(!stupidReport){
							if(-1 == r.code){
								$.app.showAlert(App.Helper.L('net_required'), App.Helper.L('net_error_title'));								
							} else {
								var messages = _form.resource.messages;
								var langID = App.Config.LanguagesTag[App.Config.LANG];
								var msgKey = r.code+langID+'';
								if('undefined' != typeof messages[msgKey]){
									var msg =  messages[msgKey];
									$.app.showAlert(msg);
								} else if('undefined' != typeof messages[r.code]){
									var msg =  messages[r.code];
									$.app.showAlert(msg);
								} else {
									var msg = self.getErrorString(r.code);
									$.app.showAlert(msg);
								}
							}
						}
					},
					report 		: stupidReport, //yes. we need report
				});
				
				if (evt.viewToUnclick) {
					evt.viewToUnclick.setEnabled(true);
				}
				
			} else {
				if(App.Config.IS_ANDROID) $.app.showBusy({message: App.Helper.L('SHOW_ADDINGCARD')});
				var _card = $.dataMgr.newModel('WOCard');
				_card.set('masterID', evt.masterID);
				
				var _formdata = evt.formData;
				if(typeof(_formdata.join) != 'undefined') _card.set('joined', _formdata.join);
				if(typeof(_formdata.expiry) != 'undefined') _card.set('expiry', _formdata.expiry);
				if(typeof(_formdata.card_number) != 'undefined') _card.set('number', _formdata.card_number.number);
				if(typeof(_formdata.display_name) != 'undefined') {
					$.prefMgr.set('cardDisplayName', _formdata.display_name);
					_card.set('display', _formdata.display_name);
				}
				_card.set('profile', 	_formdata);
				_card.set('register', 	_formdata);
				
				if (evt.viewToUnclick) {
					evt.viewToUnclick.setEnabled(true);
				}

				var valid = _card.validate();
				if(true != valid){
					$.app.showAlert(valid[0]['msg'], valid[0]['field']);
					// view.fireEvent('reset');						//  FIXME [young]  what is this????
					$.app.hideBusy();
					$.app.doNotDisturb(false);
					return;
				}
				_card.save();
				
				//  WARNING:	the sequence below is VERY delicate.
				// 				Do NOT change unless absolutely necessary
			    self.animateCardAdd(evt.animateCard);
				// winObj.close();
				winObj.home();
				if(param.data.callback)		param.data.callback({animated: false});
				Ti.App.fireEvent(self.EVENT.cardAdded, { id: _card.getID() });
				setTimeout(function(evt){
					$.app.doNotDisturb(false);
					$.app.hideBusy();
				}, 2000);
			}

			function _cardRegisterDone(result){
				$.app.doNotDisturb(true, 3333);
				if('undefined' != typeof result.objects) $.syncMgr.updateSyncDownObjects(result.objects);
				$.app.hideBusy();
				
				if (evt.viewToUnclick) evt.viewToUnclick.setEnabled(true);
				
				//  WARNING:	the sequence below is VERY delicate.
				// 				Do NOT change unless absolutely necessary
				self.animateCardAdd(evt.animateCard);
				winObj.home();				
				if(param.data.callback)		param.data.callback({ animated: false });
				
				setTimeout(function(evt){
					var total = $.syncMgr.imagesQueued();
					if(total > 0) Ti.App.fireEvent($.syncMgr.EVENT.doSyncImages);
						$.app.hideBusy();					
						$.app.doNotDisturb(false);
				}, 2000);
				
				setTimeout(function(evt){
					Ti.App.fireEvent($.syncMgr.EVENT.doSyncObjects, {
		        		objTypes	: [ 'WOMessage' ]
		        	});
	        	}, 3000);
			}
		}
	},
	
	/*
	 * param.cardID
	 */
	register : function(param){
		var self = this;
		$.app.doNotDisturb(true);
		var _card = $.dataMgr.get('WOCard', param.data.cardID);
		var _cardMaster = $.dataMgr.get('WOCardMaster', _card.masterID);
		
		var _form = false;
		if(param.data.form) for(var key in param.data.form) { _form = param.data.form; break;}
		if(!_form) {
			var cardMaster = $.dataMgr.newModel('WOCardMaster');
			_form = cardMaster.getCardFormDefn(_cardMaster, 'reg');
		}
		
		//-----------  Setup Data Package  -----------
		var viewParam = {
			DATA	: {
				cardMaster 	: _cardMaster,
				aCard 		: _card,
				form 		: _form,
				formData	: _card.register,
			},
			tabGroupObj	: param.tabGroupObj || undefined,
			navGroupObj	: param.navGroupObj || undefined,
			effect		: param.style || undefined,
			type		: param.type || undefined,
		};
		//----  Load & Render View with viewParam ----
		var winObj = App.Helper.loadView('card/CardRegisterWindow', viewParam);   //TODO [ZJ] : CardRegisterWindow
		var proxy = winObj.getEventProxy();
		winObj.render();
		//----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.formSubmit, _doRegister);
		proxy.addEventListener(proxy.EVENT.submitLater, _doLater);
		//----------  Return rendered View  ---------- 
		return winObj;
		
		//---------------------  Event Handlers  ---------------------
		function _doRegister(evt) {
			$.app.doNotDisturb(true);
			$.app.showBusy({message: App.Helper.L('hint_wait')});
			
			if('undefined' != typeof _form.resource && 'undefined' != typeof _form.resource.messages){
				var stupidReport = false;
			} else {
				var stupidReport = true;
			}
			
			self.submitRegister({
				cardID		: evt.cardID,
				formData	: evt.formData,
				callback	: function() {
//					$.app.hideBusy();
					winObj.home({animated : true});
					proxy.fireEvent(self.EVENT.cardAccepted, {action : 'Registered'});
				},
				errorback	: function(r) {
					$.app.hideBusy();
					
					if(!stupidReport){
						if(-1 == r.code){
							$.app.showAlert(App.Helper.L('net_required'), App.Helper.L('net_error_title'));								
						} else { 
							var messages = _form.resource.messages;
							var langID = App.Config.LanguagesTag[App.Config.LANG];
							if('undefined' != typeof messages[r.code+langID]){
								var msg =  messages[r.code+langID];
								$.app.showAlert(msg);
							} else if('undefined' != typeof messages[r.code]){
								var msg =  messages[r.code];
								$.app.showAlert(msg);
							} else {
								var msg = self.getErrorString(r.code);
								$.app.showAlert(msg);
							}
						}
					}
					
					// winObj.home({animated : true});
				},
				report : stupidReport,
			});
		}
		function _doLater(evt) {
			$.app.doNotDisturb(true);

			var _card = $.dataMgr.newModel('WOCard', evt.cardID);
			_card.fetch();
			
			_card.data.isDeleted = 0;
			_card.data.deleted = 0;
			_card.save();
			
			Ti.App.fireEvent($.syncMgr.EVENT.doSyncObjects, {
	        	objTypes	: [ 'WOCard' ]
	        });
	        
			winObj.home({animated : true});
			proxy.fireEvent(self.EVENT.cardAccepted, {action : 'Later'});
			
			Ti.App.fireEvent(self.EVENT.cardUpdated, { data: _card.p2l() });
		}
	},

	animateCardAdd : function(cardWidget){
		var self = this;
 		if(App.Config.IS_ANDROID)		return;
 		
 		$.app.doNotDisturb(true, 3333);
 		
 		var animateView = cardWidget.getView();
 		cardWidget.parent.remove(animateView);		// get card's Parent View to detach the card
 
 		var t1 	= Ti.UI.create2DMatrix().scale(0.1);
		var anim1 = Ti.UI.createAnimation({top:'15dp', duration: 300});
		
		var _animTop = ($.rootWindow.tabGroup.getRect().bottom-138)+'dp';
		var anim2 = Ti.UI.createAnimation({ top: _animTop, left: '-125dp', transform: t1 });
		anim2.duration 	= 700;
		
		anim1.addEventListener('complete', function(){
			newWin.animate(anim2);
		});
		anim2.addEventListener('complete', function(){
			newWin.remove(animateView);
			animateView = null;
			newWin.close();
			$.rootWindow.setBadge({
				tab		: 0,
				action	: 'add',
				count	: 1,
			});
			
			$.app.doNotDisturb(false, -3333);
		});
		if (App.Config.IS_ANDROID) {
			var newWin = createAndroidWindow ({
				top:'44dp' , width:'320dp', height:'214dp',
			});
		} else {
			var newWin = Ti.UI.createWindow({
				top:'44dp' , width:'320dp', height:'214dp',
			});			
		}
		newWin.add(animateView);
		
		newWin.open();
		newWin.animate(anim1);
		
		cardWidget = null;
	},
	
	share : function(param){
		var self = this;

		//---  Authorization to Access Address Book  ---
		var winObj = null;
		if (Ti.Contacts.contactsAuthorization == Ti.Contacts.AUTHORIZATION_AUTHORIZED){
		    winObj = self._shareCard(param);
		}else if (Ti.Contacts.contactsAuthorization == Ti.Contacts.AUTHORIZATION_UNKNOWN){
		    Ti.Contacts.requestAuthorization(function(evt){
		        if (evt.success) {
		            var winObj = self._shareCard(param);
		        }else{
		            _shareDisabled();
		        }
		    });
		}else{
		    _shareDisabled();
		}
		return winObj;
	},

	_shareCard : function(param){
		var self = this;
	
		$.app.doNotDisturb(true);
		
		var ALL_CONTACT_GROUPID = 0;
		
		//--- Shared memory space between windows, managed by Controller
		self.SHARED = {
			selectedCount		: 0,		// total contacts selected
			contactRowCache		: {},		// cache of Ti.UI.TableRow objects
			contactSelected		: {},		// contacts that have been selected
		};
		
	    //---  Get Groups from Contacts  ---------------
	    if(App.Config.IS_ANDROID){
	    	self.allGroups = [];
	    }else{
			self.allGroups = Ti.Contacts.getAllGroups();
		    self.allGroups.sort(function(a,b){
				return a.name>b.name ? 1 : -1;
		    });
	    }

		//---  implement "All Contacts" group
	    self.allContactGroup = { name: App.Helper.L('shareCard_allContact'), recordId: ALL_CONTACT_GROUPID, cached: null };
	    self.allContactGroup.getName = function() {
			return  App.Helper.L('shareCard_allContact');
	    };
	    self.allContactGroup.sortedMembers = function(order) {
	    	if(!self.allContactGroup.cached){
	    		// WARNING: getAllPeopleSorted() is a PATCH required in Contacts.m & Contacts.h !!!
	    		if(Ti.Contacts.getAllPeopleSorted)
	    			self.allContactGroup.cached = Ti.Contacts.getAllPeopleSorted();
	    		else{
	    			self.allContactGroup.cached = Ti.Contacts.getAllPeople();
	    			Ti.API.error('=======  Ti.Contacts.getAllPeopleSorted() NOT patched  ======');
	    		}
	    	}
			return self.allContactGroup.cached;
	    };
	    self.allContactGroup.members = function() {
			return Ti.Contacts.getAllPeople();
	    };
	    self.allGroups.unshift(self.allContactGroup);

		//--- Fetch Card object -----
		var sharedCard = $.dataMgr.get('WOCard', param.data.cardID);

		//  FIXME  WARNING:  this section of code NOT working... for future reference
		
		//----  Skip over Group Window if only 1 Contact Group
		// if(self.allGroups.length == 1){
			// _doViewGroup({
				// groupId		: self.allGroups[0].recordId,
				// groupName	: self.allGroups[0].name,
			// });
			// return null;	//  window already opened, MUST not return to route()
		// }

		//---  Note that we are managing 2 windows that need to be closed depending on conditions
		var shareWinObj = null;
		var contactWinObj = null;

		//-----------  Setup Data Package  -----------
		var viewParam = {
			DATA		: {
				allGroups		: self.allGroups,
				allContactGpID	: ALL_CONTACT_GROUPID,
				card			: sharedCard,
			},
			SHARED		: self.SHARED,
			tabGroupObj	: param.tabGroupObj || undefined,
			navGroupObj	: param.navGroupObj || undefined,
		};
		//----  Load & Render View with viewParam ----
		if (App.Config.IS_ANDROID) {			
			shareWinObj = App.Helper.loadView('share/ShareWindowAndroid', viewParam);		
		} else {
			shareWinObj = App.Helper.loadView('share/ShareWindow', viewParam);		
		}
		var proxy = shareWinObj.getEventProxy();
		shareWinObj.render();
		//----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.viewGroup, _doViewGroup);
		proxy.addEventListener(proxy.EVENT.doShareCard, _doShareCard);
		proxy.addEventListener(proxy.EVENT.shareCancel, _doShareCancel);
		//----------  Return rendered View  ----------
		return shareWinObj;
		
		//------------------  Event Handlers  ------------------
		function _doViewGroup(evt){
			
			//--------  Fetch Contacts  -----------
			if(evt.groupId == ALL_CONTACT_GROUPID){
     			var theGroup = self.allContactGroup;
			}else{
				var theGroup = Ti.Contacts.getGroupByID(evt.groupId);
			}
			//-----------  Setup Data Package  -----------
			var viewParam = {
				navGroupObj	: $.currentNavGroupObj,
				eventProxy	: proxy,
				DATA		: {
					group		: theGroup,
					groupID		: evt.groupId,
					lazyRender	: (evt.groupId == ALL_CONTACT_GROUPID),
				},
				SHARED		: self.SHARED,
			};
			//----  Load & Render View with viewParam ----
			if (App.Config.IS_ANDROID) {
				viewParam.DATA.contacts = self.allContactGroup.sortedMembers(Ti.Contacts.CONTACTS_SORT_LAST_NAME);
				contactWinObj = App.Helper.loadView('share/ContactSelectorAndroid', viewParam);			
			} else {
				contactWinObj = App.Helper.loadView('share/ContactSelector', viewParam);			
			}
			contactWinObj.render();
			//----------  Setup Event handlers  ----------
			//----------  Return rendered View  ---------- 
			contactWinObj.open();
		}
		function _doShareCard(evt) {
			/*
			 * evt.share_mode
			 * evt.mobiles (Json string)
			 * evt.total
			 */
			shareWinObj.close();

			var callParam = {
		    	action 	: App.Action.SHARECARD,
		    	data 	: {
		    		cins_id			: param.data.cardID,
					share_method	: evt.share_mode,
					mobiles			: evt.mobiles,
		    	},
		    	callback  : _callback,
		    	errorback : _callback,
		    };
		    self.callServer(callParam);

		    $.app.doNotDisturb(false);
		    _disposeSharedObjects();

		    //-------------------  Callback Handlers  -----------------
		    function _callback(result) {
		    	var msg = (1 == evt.total) ? App.Helper.L('shareCard_cardShared1') : String.format(App.Helper.L('shareCard_cardShared2'), evt.total);
	    		$.app.notify({ message : msg });
				if(param.data.callback)		param.data.callback(result);
		    }
		    function _errorCallback(result){
		    	Ti.API.error('callserver error = '+JSON.stringify(result));
		    }
		}
		function _doShareCancel(evt){
			shareWinObj.close();
			$.app.doNotDisturb(false);
			_disposeSharedObjects();
		}
		function _disposeSharedObjects(){
			delete self.SHARED.contactRowCache;
			delete self.SHARED.contactSelected;	
			delete self.allContactGroup.cached;	
			delete self.allGroups;
		}
	},
	
	_shareDisabled : function(param){
		var self = this;
		
		$.app.showAlert(App.Helper.L('ALERT_AllowToShare'));
	},
	
	//-------------------------  Protected Methods  ------------------------	
	_getCardListdata : function(order){
		var self = this;

        var card = $.dataMgr.newModel('WOCard');
        card.fetchList();
        return card.p2l();
	},
	
	_cardRegisterSubmit : function(param){
		var self = this;
    
    	var callParam = {
	    	// action : App.Action.VALIDATECARD,
	    	action : 'card_validate_registration',
	    	report : param.report,
	    	data:{
				cardMID		: param.mid,
				formData	: JSON.stringify(param.register)
	    	},
	    	callback  : param.callback,
	    	errorback : param.errorback
	    };
	    $.syncMgr.callServer(callParam);
	},
	
	dispose : function(param){
		var self = this;
		
		self.smsDialog = null;
		self.allGroups = null;
	    self.allContactGroup = null;
		self.shareWindowObj.dispose();
		
		self.base();
	},
	
});

module.exports = CardController;

//---  End of file: CardController.js
