/**
 * NotificationController:
 *
 * @author young@waveo.com
 */
Ti.include('/globals.js');
App.Helper.requireController('WONotificationManager');

var NotificationController = App.Class(App.Controller.WONotificationManager, {
	className	: 'NotificationController',

	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	//------------------  Views for Message Types  --------------------
	_show_pushMsg : function(message, callback){
		var self = this;

		var actionHandled = false;
		if(message.content.showDialog){
			self._showMessageDialog(message, callback);
			actionHandled = true;
		}

		//---  trigger Perkd Message if specified  ---
		if(message.content.perkMsgID){
			setTimeout(function(evt){
				Ti.App.fireEvent(self.EVENT.doPerkd, {
					msgID		: message.content.perkMsgID
				});
			}, 2000);
			
			actionHandled = true;
		}
		return actionHandled;
	},

	_show_perkMsg : function(message, callback){
        var self = this;
        
		if(! $.app.canDisturb())	return false;	// actionHandled
		$.app.doNotDisturb(true);
        //-----------  Setup Data Package  -----------
		var viewParam = {
			DATA		: message.content,
			effect		: App.Const.WINDOW.effect.flip,
		};
		viewParam.DATA.msgID = message.id;
		
		switch(message.subType){
			case self.SUBTYPE.perkd.issueCard:
				var windowName = 'perk/IssueCard';
				break;
			case self.SUBTYPE.perkd.issueStamp:
				var windowName = 'perk/IssueStamp';
				break;
			case self.SUBTYPE.perkd.issueOffer:
				var windowName = 'perk/IssueOffer';
				break;
			default:
				$.watch.dog(self, '_show_perkMsg', 'unsupported Perk Message sub-type: '+message.subType);
				return true;
		}
        //----  Load & Render View with viewParam ----
		var winObj = App.Helper.loadView(windowName, viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.perkdDone, _doPerkdDone);

		App.loadHelper('playSound').play('notify');
		winObj.open();
		
		//--- actionHandled always TRUE
		return true;		

		//--------------------  Event Handlers  ------------------------        
        function _doPerkdDone(evt){
            winObj.close({
            	animated	: evt.animated || false,
            	effect		: evt.effect || undefined,
            });            
            if(callback)		callback({ hasRead: true });
            
            var dnd = 'undefined' != typeof evt.dnd ? evt.dnd : false; 
            if(!dnd){
	            //--- make sure animations are completed before allowing interruption
	            setTimeout(function(evt){
					$.app.hideBusy();
					$.app.doNotDisturb(false);					
	            }, 1500);
            }
        }
	},

	_show_reminder : function(message, callback){
		var self = this;

		var actionHandled = false;
		if(message.content.showDialog){
			self._showMessageDialog(message, callback);
			actionHandled = true;
		}
		return actionHandled;
	},

	_show_pullMsg : function(message, callback){
		var self = this;

		var actionHandled = false;
		if(message.content.showDialog){
			self._showMessageDialog(message, callback);
			actionHandled = true;
		}		
		return actionHandled;
	},

	_show_perkMail : function(message, callback){
		//---  To be implemented  ------
	},

    //----------------------------  View Controllers  ------------------------------
	showStartUpPerk : function(param){
		var self = this;
		
		//---  Ready to process notifications  ---
		self.isActive = true;
		$.app.doNotDisturb(false);

		var showParam = {
			msgTypes	: self.TYPE.perkMsg,
			callback	: function(evt){

				if($.currentNavGroupObj){
					$.currentNavGroupObj.close({animated: false});
					$.currentNavGroupObj = null;
				}
				self.isActive = false;
				Ti.App.fireEvent($.app.EVENT.doStart);
			},
		}
		if(! self.showNextMessage(showParam)){
			self.isActive = false;
			Ti.App.fireEvent($.app.EVENT.doStart);
		}
	},

    //-----------------------  Reminder Notification Services  ---------------------
	setupReminders : function(){
		var self = this;

		var reminderMgr = App.Helper.newController('ReminderManager');
        reminderMgr.cancelReminders();		// clear all existing reminders
        reminderMgr.setCardReminders();
		reminderMgr.setOfferReminders();
		reminderMgr.setRewardReminders();
	},

    //-----------------------  Remote Notification Services  -----------------------
	_createMessageFromPush : function(param){
		var self = this;

		var message = $.dataMgr.newModel('WOMessage');
		message.data.type = ('undefined' != typeof param[self.PUSH.type]) ? param[self.PUSH.type] : self.TYPE.pushMsg;
		message.data.subType = '';
		message.data.content = {
			title			: param.title || App.Helper.L('TITLE_MESSAGE'),
			alertBody		: ('undefined' != typeof param.alert) ? param.alert : param.message || '',
			options			: ('undefined' != typeof param[self.PUSH.options]) ? _extractOptions(param[self.PUSH.options]) : {},
			sound 			: param.sound,
		};

		//---  Extract Data payload  ------------
		var hasAction = false;
		if(App.Config.IS_ANDROID){		//  FIXME [ZJ]  due to BUG in GCM module that we are using???
			var payload = ('undefined' != typeof param[self.PUSH.data]) ? JSON.parse(param[self.PUSH.data]) : {};
		}else{
			var payload = ('undefined' != typeof param[self.PUSH.data]) ? param[self.PUSH.data] : {};	
		}

		for(data in payload){

			switch(data){
			case self.DATA.perkMsg:	
				hasAction		= true;
				var theAction	= self.ACTION.viewPerk;
				var actionData	= { perkMsgID: payload[data] };		// perkd message ID

				//---  Pre-fetch Perkd Message Object  ---
				var perkMsg = $.dataMgr.newModel('WOMessage', payload[data]);
				if(! perkMsg.fetch()){
					Ti.App.fireEvent($.syncMgr.EVENT.doSyncObjects);//, { objTypes: ['WOMessage'] });	
				}else{
					Ti.API.warn('handlePushNotification - PerkMsg already prefetched');
				}
				break;

			case self.DATA.cardID:
				hasAction = true;
				var theAction = self.ACTION.viewCard;
				var actionData	= { cardID: payload[data] };		// card instance ID

				//---  Pre-fetch Card Instance Object  ---
				App.Helper.requireModel('WOCard');
				var card = new App.Model.WOCard(payload[data]);
				if(! card.fetch()){
					Ti.App.fireEvent($.syncMgr.EVENT.doSyncObjects, { objTypes: ['WOCard'] });
				}else{
					Ti.API.warn('handlePushNotification - Card Instance already prefetched');
				}
				break;

			case self.DATA.URL:
				hasAction = true;
				var theAction = self.ACTION.gotoURL;
				var actionData	= { url: $.configMgr.get(payload[data], payload[data]) };	// URL lookup
				break;

			case self.DATA.buttons:
				message.data.content.buttons = payload[data];		// array of button labels
				//  [0] == left most button, usually is 'Cancel' when 2 buttons
				break;

			default:
				message.data.content[data] = payload[data];			// unknown payload, pass it on
				Ti.API.warn('handlePushNotification - unknown Data Payload: ['+data+']');
				break;
			}
		}
		
		//--- setup Buttons & Actions  --------------
		if('undefined' == typeof message.data.content.buttons){
			//--- default 'OK' button
			message.data.action = [{ label: App.Helper.L('BUTTON_OK') }];
		}else{
			message.data.action = [];
			for(var i=0; i<message.data.content.buttons.length; i++){
				message.data.action.push({ label: message.data.content.buttons[i] });
			}
		}
		// NOTE:  Action ALWAYS associated with last (rightmost) button
		if(hasAction){
			message.data.action[message.data.action.length-1].name = theAction;
			message.data.action[message.data.action.length-1].data = actionData;
		}
			
		//--- Suppress display of message when (i) mute specified, or (ii) message received outside of app
		// (iii) echo will override startup, ie. always display message in app
		var _mute = message.data.content.options.mute;
		var _echo = message.data.content.options.echo;
		message.data.content.showDialog = (!_mute && !param.isStartUp) || _echo;

		//--- Save Message Object to be displayed later by showNextMessage() ----------
		var msgID = message.save();
		var result = {
		    message     : message,
		    msgID       : msgID,
		};
		return result;
		
		function _extractOptions(optionValue){
			return {
				reset	: (optionValue & self.OPTION.reset) == self.OPTION.reset,
				sync	: (optionValue & self.OPTION.sync) == self.OPTION.sync,
				echo	: (optionValue & self.OPTION.echo) == self.OPTION.echo,
				mute	: (optionValue & self.OPTION.mute) == self.OPTION.mute,
			};
		}
	},

	_postPushMessageProcess : function(message){
		var self = this;
		
		//--- Perform 'Silent' Actions specified in Options of Push Msg  ---
		if(message.data.content.options.sync){
			Ti.App.fireEvent($.syncMgr.EVENT.doSyncObjects);	// sync all objects
		}
	},
	
	//-----------------------  Local Notification Services  ------------------------

	//---  translate Local notify message & save to Message Queue  ---
	_createMessageFromLocalNotify : function(param){
		var self = this;

		var message = $.dataMgr.newModel('WOMessage');
		message.data.type = param.userInfo.messageType;
		
		switch(message.data.type){
		case self.TYPE.reminder:
			message.data.startUp = param.isStartUp;
			message.data.subType = param.userInfo.type;
			message.data.content = {
				showDialog		: !param.isStartUp,	// suppress dialog if already seen before launching app
				title			: App.Helper.L('TITLE_REMINDER'),
				date			: param.date,
				alertBody		: param.alertBody,
				alertAction		: param.alertAction,
				timezone		: param.timezone,
				badge			: param.badge,
				userInfo		: param.userInfo,
			};
			message.data.action = [
				{ label:	'Close' },
				{ label:	param.alertAction || 'View', name: self.ACTION.viewCard,
							data: { cardID: param.userInfo.objID }
				}
			];
			break;

		default:
			//  TODO  support other local notifications?
			break;
		}

		//--- Save Message Object to be displayed later by showNextMessage() ----------
		var msgID = message.save();
		var result = {
		    message     : message,
		    msgID       : msgID,
		};
		return result;
	},

});

module.exports = NotificationController;

//---  End of file: NotificationController.js
