
Ti.include('/globals.js');

var PerkController = App.Class(App.Controller.WOManager, {
	className : 'PerkController',
    alias     : 'perk', 
	
	EVENT		: {
		//--- Handles these events:
		//--- Produces these events:
		goAddCard		: 'PerkController.goAddCard',
		issueCard		: 'PerkController.issueCard',
		issueStamp		: 'PerkController.issueStamp',
		issueOffer		: 'PerkController.issueOffer',
		
		deductStamp		: 'PerkController.deductStamp',
    },

	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
		self.lastScanned = 0;
	},
	
	//-------------------------  View Controllers  --------------------------
	main : function(param){
		var self = this;

        //-----------  Setup Data Package  -----------		
		var viewParam = {
			tabGroupObj		: param.tabGroupObj,
		};
        //----  Load & Render View with viewParam ----
		var winObj = App.Helper.loadView('perk/PerkcodeWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.scanQR, _doScanQR);
        //----------  Return Window Object  ----------
        return winObj;
        
		//--------------------  Event Handlers  ------------------------
		function _doScanQR(evt){
			self.scan();
		}
	},
	
	scan : function(param){
		var self = this;
		$.app.doNotDisturb(true);
		var scanner = App.loadHelper('Zebra');
		Ti.App.addEventListener(scanner.EVENT.scannedCode, _handleScanResult);
		
		scanner.scan({symbols:{"QR-Code":true}});	// only scan for QR Code
		
		function _handleScanResult(evt){
			Ti.App.removeEventListener(scanner.EVENT.scannedCode, _handleScanResult);
			
			var now = new Date().getTime();
			if((now - self.lastScanned) < 2000) {
				self.lastScanned = now;
				return;
			}
			self.lastScanned = now;
			$.app.doNotDisturb(false);
			
			switch(evt.result) {
				case scanner.RESULT.success :
					var QRCode = evt.number;
					var perkCode = self._isPerkCode(QRCode);
					if(!perkCode) {
						var URL = self._isURL(QRCode);
						if(!URL) self._alertUnsupportedMsg();
						else Ti.Platform.openURL(URL);
					} else {
						self.doPerk(perkCode);
					}
					break;
				case scanner.RESULT.failed :
					self._alertUnsupportedMsg();
					break;
				case scanner.RESULT.cancelled :
					// alert('User cancelled');
					break;
				default :
					self._alertUnsupportedMsg();
					Ti.API.error('Scan QR-Code result error, cannot match result value: '+evt.result);
			}
		}
	},
	
	doPerk : function(perkCode) {
		var self = this;
		var perkData = $.dataMgr.get('WOPerkCode', perkCode, false);

		if(false === perkData) {
			if(!Ti.Network.online){
				$.app.showAlert(App.Helper.L('net_error_scan'), App.Helper.L('TITLE_SCAN'));
				return;
        	} else {
          		$.watch.dog(self, 'doPerk', 'Not found PerkCode from local database: '+ perkCode);
          		// Ti.App.fireEvent($.syncMgr.EVENT.doSyncObjects);
          		
          		//alert('Unable to connect to Perkd servers. Please retry again.');     //<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
		        
		        //---------------  Call Server  -------------
		        $.app.showBusy({message:App.Helper.L('hint_requesting')});
		        var payload = {
					perkcode	: perkCode,
		        };
		        var callParam = {
		            action		: 'perkcode_scan',
		            data		: payload,
		            report      : true,
            		title       : App.Helper.L('TITLE_SCAN'),
		            callback	: _scanCallback,
		            errorback 	: _scanErrorback,
		        };
		        self.callServer(callParam);
          		
          		function _scanCallback(result) {
          			if(result.objects)  $.syncMgr.updateSyncDownObjects(result.objects);
          			$.app.hideBusy();
          			if(result.action && 'addCard' == result.action.name) {
          				var cardMaster = $.dataMgr.get('WOCardMaster', result.action.cardMID);
	          			var dialog = Ti.UI.createAlertDialog({
							title		: App.Helper.L('TITLE_SCAN'),
							message		: String.format(App.Helper.L('ALERT_NeedAddCardForReward'), cardMaster.fullName),
							buttonNames	: [App.Helper.L('BUTTON_CANCEL'), App.Helper.L('BUTTON_ADDCARD')],
						});
						//---  Handle Action button  ---
						dialog.addEventListener('click', function(evt){
							if(1 == evt.index) {
								$.app.route({
									url			: 'card/addCard',
									tabGroupObj	: $.rootWindow,
									data		: {
										cardMaster	: $.dataMgr.get('WOCardMaster', result.action.cardMID),
										// form		: result.form.def,
										authState	: $.authMgr.getAuthState(),
										// form		: JSON.decode(result.form.defn),
									},
								});
							}
						});
						dialog.show();
					} else if(result.action && 'regCard' == result.action.name) {
						
						var _card = $.dataMgr.get('WOCard', result.action.cardID);
						var dialog = Ti.UI.createAlertDialog({
							title		: App.Helper.L('TITLE_SCAN'),
							message		: String.format(App.Helper.L('ALERT_NeedRegCardForReward'), _card.fullName),
							buttonNames	: [App.Helper.L('BUTTON_CANCEL'), App.Helper.L('BUTTON_REGISTER')],
						});
						//---  Handle Action button  ---
						dialog.addEventListener('click', function(evt){
							if(1 == evt.index) {
								$.app.route({
									url		: 'card/register', 
									tabGroupObj	: $.rootWindow,
									// effect	: App.Const.WINDOW.effect.flip,
									data	: { cardID : result.action.cardID }
								});
							}
						});
						dialog.show();
          			} else if(result.action && 'retry' == result.action.name) {
          				perkData = $.dataMgr.get('WOPerkCode', perkCode, false);
          				if(perkData) {
          					self._doPerkd(perkData);
          				}
          				else {
          					self._alertUnsupportedMsg();
          					$.watch.dog(self, 'doPerk', 'Retry failed, still not found perkCode data.');
          				}
          			} else if(result.action && 'upgrade' == result.action.name) {
          				if(App.Config.IS_ANDROID) {
          					$.app.showAlert(App.Helper.L('ALERT_NeedUpgrade'), App.Helper.L('TITLE_SCAN'));
          				} else {
	          				var dialog = Ti.UI.createAlertDialog({
				                title		: App.Helper.L('TITLE_SCAN'),
				                message		: App.Helper.L('ALERT_NeedUpgrade'),
				                buttonNames	: [App.Helper.L('BUTTON_NO'), App.Helper.L('BUTTON_UPGRADE')],
				            });
				            dialog.addEventListener('click', function(e){
				                if(e.index ==1){
				                	var config = $.dataMgr.get('WOSettings', 'config');
									var upgradeURL = config['@upgrade'] || 'http://perkd.me/upgrade';
									Ti.Platform.openURL(upgradeURL);
				                }
				            });
				            dialog.show();
			           }
			           return;          			
			        }
          		}
          		function _scanErrorback(result) {
          			if(result.objects)  $.syncMgr.updateSyncDownObjects(result.objects);
          			$.app.hideBusy();
          		}
          	}
	    }
	    if(perkData) {
			self._doPerkd(perkData);
		}
	},
	
	_doPerkd : function(perkData) {
		var self = this;
		$.trackMgr.track(self, 'scan', perkData);
		if('undefined'!= typeof perkData.condition && !self._validatePerkCode(perkData.condition)) return;
		if('undefined'!= typeof perkData.action) {
			for(var i=0; i<perkData.action.length; i++) {
				perkData.action[i].data.cardID = perkData.action[i].data.cardID || perkData.cardID;
				//fire event...
				// alert(perkCode.action[i].data)
				Ti.App.fireEvent(self.EVENT[perkData.action[i].name], {data:perkData.action[i].data});
			}
		} else {
			$.watch.dog(self, 'doPerk', 'Unable to parse PerkCode: '+JSON.stringify(perkData));
		}
	},

	//---- check QR-code condition ----	
	_validatePerkCode : function(param){
		var self = this;
		
		for(var method in param) {
			var validateMethod = '_validate_'+method;
			if('function' != typeof self[validateMethod]){
				// alert('ERROR: Cannot find method '+validateMethod);
				return false;
			}
			if(!self[validateMethod](param[method])) return false;
		}
		return true;
	},
	
	_validate_validDate : function(param){
		var self = this;
		
		if(!param.length) return true;
		
		var timeNow = Math.round(new Date().valueOf() / 1000);
		for(var i=0; i<param.length; i++) if(timeNow >= param[i].begin && timeNow <=  param[i].end) return true;
		return false;
	},
	
	_validate_location : function(param){
		var self = this;
		return true;
	},
	
	_isPerkCode : function(QRCode){
		var self = this;
		var x_servers = $.configMgr.get('perkd_server', ['http://perkd.me/x/']);
		var perkcode = false;
		for(var i=0; i<x_servers.length && !perkcode; i++){
			var server = x_servers[i];
			var codePrefix = QRCode.substr(0, server.length);
			if(codePrefix == server) perkcode = QRCode.substring(server.length);
		}
		return perkcode;
	},
	
	_isURL : function (QRCode) {
		var self = this;
		var http_index = QRCode.toLowerCase().indexOf('http://');
		var https_index = QRCode.toLowerCase().indexOf('https://');
		if(-1 == http_index && -1 == https_index) {
			return false;
		} else if(-1 == http_index){
			return QRCode.substring(https_index);
		} else {
			return QRCode.substring(http_index);
		}
	},
	
	_alertUnsupportedMsg : function() {
		var self = this;
		$.app.showAlert(App.Helper.L('err_wpk_perkcode_101'), App.Helper.L('TITLE_SCAN'));
	},
});

module.exports = PerkController;


// perkcode structure.  example
// var _perkCode_definition = { 
	// codeID : '1q2w3e',
	// cardID : null || 'q1w2e3', //scan under card
	// condition : {
		// validDate : [
			// { begin : 123456789, end : 234567890 },
			// { begin : 323456789, end : 434567890 },
		// ],
		// location : [
			// {},
			// {}
		// ],
	// },
	// action : [
		// {
			// name : 'issueCard',
			// data : {
				// cardMID : '1q2w3e',
			// }
		// },
		// {
			// name : 'issueStamp',
			// data : {
				// rewardsetMID : '1q2w3e',
				// rewardsetMAX : 10,
				// stampID	: 's5',
				// stateID	: 1,
				// quantity : 2,
			// }
		// },
	// ],
// };


//---  End of file: PerkController.js