
Ti.include('/globals.js');

var PreferencesController = App.Class(App.Controller.WOManager, {
	className : 'PreferencesController',
    alias     : 'pref',

	//---  Constructor  ---------------
	initialize : function(param) {
		var self = this;
		self.base(param);
		self.listenForEvents();
	},

	listenForEvents : function(){
		var self = this;

		Ti.App.addEventListener('WOSettings.preference.'+$.syncMgr.EVENT.syncUpdate, function(){
			if(self.preferenceWinObj){
				var proxy = self.preferenceWinObj._eventProxy;
				proxy.fireEvent(proxy.EVENT.refresh, { data: $.dataMgr.get('WOSettings', 'preference') });
				Ti.App.fireEvent($.cardLibMgr.EVENT.updateCategory, {});
			}
		});
	},
    
    get : function(name, defaultValue) {
        var self = this;
        
        defaultValue = 'undefined' == typeof defaultValue ? null : defaultValue;
        var prefData = $.dataMgr.get('WOSettings', 'preference');
        if('undefined' == typeof prefData[name] || null == prefData[name]) return defaultValue;
        return prefData[name];
    },
    
    set : function(name, value) {
        var self = this;
        App.Helper.requireModel('WOSettings');
        
        var preference = new App.Model.WOSettings('preference');
        preference.fetch();
        preference.data[name] = value;
        preference.save();
    },
    
    getCityAware : function() {
    	var self = this;
        
        var cityInfo = self.get('city', []);
        var keywords = [];
        for(var i=0; i<cityInfo.length; i++) {
        	if(null == cityInfo[i]) continue;
        	if('undefined' != typeof cityInfo[i].id && '' != cityInfo[i].id && null != cityInfo[i].id) keywords.push(cityInfo[i].id);
        	if('undefined' != typeof cityInfo[i].cityCode && '' != cityInfo[i].cityCode && null != cityInfo[i].cityCode) keywords.push(cityInfo[i].cityCode);
        	if('undefined' != typeof cityInfo[i].provinceCode && '' != cityInfo[i].provinceCode && null != cityInfo[i].provinceCode) keywords.push(cityInfo[i].provinceCode);
        	if('undefined' != typeof cityInfo[i].countryCode && '' != cityInfo[i].countryCode && null != cityInfo[i].countryCode) keywords.push(cityInfo[i].countryCode);
        }
        return keywords;
    },

	//---------------------  Preferences View Controllers  --------------------
	main : function(param) {
		var self = this;
		App.Helper.requireModel('WOSettings');

        //-----------  Setup Data Package  -----------
        var preferences = new App.Model.WOSettings('preference');
        preferences.fetch();
        var account = new App.Model.WOSettings('account');
		account.fetch();

		var viewParam = { 
			DATA		: {
				preferences		: preferences.p2l(),
				supportScan		: Ti.Media.isCameraSupported || App.Config.IS_SIMULATOR,
				mobileNo		: account.data.mobileNo,
			},
			tabGroupObj	: param.tabGroupObj,
		};
        //----  Load & Render View with viewParam ----
		var winObj = App.Helper.loadView('preference/PreferencesWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.change, _doUpdate);
        //----------  Return Window Object  ---------- 
        self.preferenceWinObj = winObj;
        return winObj;
        
		//--------------------  Event Handlers  ------------------------
		function _doUpdate(evt){
			var preferences = $.dataMgr.newModel('WOSettings', 'preference');
			preferences.fetch();
			preferences.data[evt.field.name] = evt.value;
			preferences.save();
			
			Ti.App.fireEvent($.syncMgr.EVENT.doSyncObjects, {objTypes : ['WOSettings']});
			//---  Post-preference change actions  ---
			switch(evt.field.name){
				case 'city':
					Ti.App.fireEvent($.cardLibMgr.EVENT.updateCategory);
					break;
				case 'scan':
				case 'langRegion':
					Ti.App.fireEvent($.app.EVENT.doRestart);
					break;
				case 'cardRemindDays':
					Ti.App.fireEvent($.notifyMgr.EVENT.doSetupReminders, { type: 'card' });
					break;
				case 'offerRemindDays':
					Ti.App.fireEvent($.notifyMgr.EVENT.doSetupReminders, { type: 'offer' });
					break;
			}
		}
	},   

});

module.exports = PreferencesController;

//---  End of file: PreferencesController.js
