/**
 * Implements Registration Services and View Controllers
 *
 * @author young@waveo.com
 */
Ti.include('/globals.js');

var RegistrationController = App.Class(App.Controller.WOManager, {
	className	: 'RegistrationController',

	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

    //--------------------------  Registration Services  --------------------------
    register : function(param){
        var self = this;
        
        var accountInfo = {};
        var skippedReg = param.skip;
        
        if(! skippedReg){
            accountInfo = {
                mobileNo    : param.mobileNo,
                familyName  : param.familyName,
                givenName   : param.givenName,
                gender      : param.gender,
                birth       : param.birth,
                email		: param.email,
            };
        }

        //-------------  Call Server  -------------
        var payload = {
            action          : App.Action.REGISTER,
            skip            : skippedReg ? 'Y' : 'N',
            account         : JSON.stringify(accountInfo),
            //--- extended payloads ---
            appInfo         : JSON.stringify($.configMgr.getAppInfo()),
            deviceInfo      : JSON.stringify($.configMgr.getDeviceInfo()),
            carrierInfo     : JSON.stringify($.configMgr.getCarrierInfo()),
        };
        var callParam = {
            action      : App.Action.REGISTER,
            report      : true,
            title       : App.Helper.L('title_register'),
            data        : payload,
            callback    : _registerSuccess,
            errorback   : param.errorback,
        };
        self.callServer(callParam);

        //-----------------  Callback Handlers  -----------------
        function _registerSuccess(result){		
			//--- update Account info with version from server
            $.syncMgr.updateSyncDownObjects(result.objects); // save settings.
            
			//--- update states of Authentication Mgr ----
            $.authMgr.setAuthState({
                state   : skippedReg ? App.Const.AUTH_SKIPPED : App.Const.AUTH_REGISTER,
                token   : result.token,
            });
            $.authMgr.autoLoggedIn = true;
            
            if(!skippedReg) $.accountMgr.setRegisterMobileNum(accountInfo.mobileNo);
            
            // App.Helper.requireModel('WOSettings');
            // var account = new App.Model.WOSettings('account');
            // account.fetch();
            
            if(param.callback)
                param.callback({ mobileNo : $.accountMgr.getRegisterMobileNo() });
        }
    },

    //----------------------  Registration View Controllers  ----------------------
    searchCountry : function(param){
    	var self = this;
    	var viewParam = {
			tabGroupObj	: param.tabGroupObj || undefined,
			navGroupObj	: param.navGroupObj || undefined,
		};
    	var winObj = App.Helper.loadView('components/CountrySearchWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();

        return winObj;
    },
    
    signup : function(param){
        var self = this;
        App.Helper.requireModel('WOSettings');

		//---  Make sure we are not interrupted, esp. we skipped Reg & notifyMgr is active
		$.notifyMgr.isActive = false;
		$.app.doNotDisturb(true);

        //----------  Setup Data Package  ------------
        if(!$.authMgr.isSkippedReg()) $.accountMgr.resetAccount(); //Just incase
        
        var carrierInfo = $.configMgr.getCarrierInfo();
        var country = App.Helper.getCountryISO(carrierInfo.isoCountryCode);
        
        Ti.App.Properties.removeProperty('tmp_reg_form');
        
        if($.authMgr.isSkippedReg()) {
        	var account = $.dataMgr.get('WOSettings', 'account');
        	var tmp_reg_form = Ti.App.Properties.getObject('tmp_reg_form', {});
        	tmp_reg_form = {
        		familyName 	: account.familyName,
				givenName 	: account.givenName,
				birth 		: account.birth,
				gender 		: account.gender,
				email 		: account.email,
				countryCode : country.code,
				countryName	: country.name,
        	}
        	Ti.App.Properties.setObject('tmp_reg_form', tmp_reg_form);
        }
		
		var viewParam = {
			DATA: Ti.App.Properties.getObject('tmp_reg_form', {
				familyName : '',
				givenName : '',
				birth : '',
				gender : '2',
				email : '',
				countryCode : country.code,
				countryName	: country.name,
			}),
			navGroupObj	: param.navGroupObj,
		};
        //----  Load & Render View with viewParam ----
		var winObj = App.Helper.loadView('registration/SignupWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
        proxy.addEventListener(proxy.EVENT.searchCountry, _goSearchCountry);
		proxy.addEventListener(proxy.EVENT.signup, _doSignup);
        //----------  Return Window Object  ---------- 
        return winObj;
        
		//--------------------  Event Handlers  ------------------------
		function _goSearchCountry(evt){
			var _winObj = $.app.route({
				url			: 'RegistrationController/searchCountry',
				eventProxy  : proxy,
				navGroupObj	: winObj.navGroupObj,
			});
			var _proxy = _winObj.getEventProxy();
			_proxy.addEventListener(_proxy.EVENT.countrySelected, function(evt){
				winObj.updateCountry(evt);
				_winObj.close({animated:true});
			});
		}
        function _doSignup(evt){
            $.app.showBusy({message:App.Helper.L('SHOW_REGISTERING')});
            
            Ti.App.Properties.setObject('tmp_reg_form', evt.profile);
			var regParam = App.Helper.clone(evt.profile);
            regParam.callback   = _registerDone;
            regParam.errorback  = function(result){
                $.app.hideBusy();
            }
            
            self.register(regParam);
        }
        //----------------  Callback Handlers  ---------------
        function _registerDone(result){
            $.app.hideBusy();
            //--- Get Push permission, then proceed to StartUp page
            $.notifyMgr.getPushPermission({
            	navGroupObj	: winObj.navGroupObj,
                callback 	: function(){
                    $.app.route({
                        url     	: 'auth/startup',
                        navGroupObj	: winObj.navGroupObj,
                    });
                }
            });         
        }
    },
    	
	later : function(param){
		var self = this;

        //-----------  Setup Data Package  -----------
		var viewParam = { 
			DATA		: {
				isSkipped	: ('undefined' == typeof param.data.isSkipped) ? false : param.data.isSkipped,
			},
			navGroupObj	: param.navGroupObj,
		};
        //----  Load & Render View with viewParam ----
		var winObj = App.Helper.loadView('registration/LaterWindow', viewParam);
		var proxy = winObj.getEventProxy();
        winObj.render();
        //----------  Setup Event handlers  ----------
		proxy.addEventListener(proxy.EVENT.signup, _doSignup);
		proxy.addEventListener(proxy.EVENT.skipRegister, _doSkipRegister);  
        //----------  Return Window Object  ---------- 
        return winObj;

        //------------------  Event Handlers  ---------------
		function _doSignup(evt){
			$.app.route({
				url			: 'RegistrationController/signup',
				navGroupObj	: winObj.navGroupObj,
			});
        }        
		function _doSkipRegister(evt){
            $.app.showBusy({message:App.Helper.L('hint_requesting')});
            self.register({
                skip        : true,
                callback    : _skipDone,
                errorback   : function(){
                    $.app.hideBusy();
                }
            });
        }
        //----------------  Callback Handlers  ---------------
        function _skipDone(result){
            $.app.hideBusy();           
            //--- Get Push permission
			$.notifyMgr.getPushPermission({
				navGroupObj	: winObj.navGroupObj,
				callback	: function(){
					//---  Setup Card Library  ---
					_doRestoreCards();
				}
			});
        }
		function _doRestoreCards(){
            $.app.route({
                url			: 'cardLib/restore',
                navGroupObj	: winObj.navGroupObj,
            });
        };
	},
   
});

module.exports = RegistrationController;

//---  End of file: RegistrationController.js
