/**
 *  we inherit majority of the behaviors from WOUpgradeManager
 *  just need to implement view controllers
 * 
 * @author zhoujun@waveo.com
 */
Ti.include('/globals.js');
App.Helper.requireController('WOUpgradeManager');

var UpgradeController = App.Class(App.Controller.WOUpgradeManager, {
	className : 'UpgradeController',

	EVENT		: {
		//--- Handles these events:
		cardsRestored	: 'RestoreCard.cardsRestored',	// mirror in RestoreCard window
    },

	//---  Constructor  ---------------
	initialize: function(param){
		var self = this;
		self.base(param);
	},

	//---------------  View Controllers  -----------------
	upgrade : function(param){
		var self = this;

        //-----------  Setup Data Package  -----------
        //----  Load & Render View with viewParam ----
 		var winObj = App.Helper.loadView('upgrade/UpgradeWindow', {});
        winObj.render();
		self.doUpgrade({
			callback	: _upgradeDone,
		});
		return winObj;
		
		//----------------  Callback Handlers  -----------
		function _upgradeDone(result){
			
	        var authState = $.authMgr.getAuthState();
	        var hadLoggedIn = (authState.state == App.Const.AUTH_LOGGED_IN) || (authState.state == App.Const.AUTH_SKIPPED);
				
			if(hadLoggedIn){
				$.app.route({
	                url     	: 'cardLib/restore',
	                navGroupObj	: winObj.navGroupObj,
	                data		: {
		                callback	: param.data.callback,
	                },
	            });
			}else{
				param.data.callback(result);
			}
		}
	},
	
	doUpgrade : function (param){
		var self = this;
		
		var lastRunVersion = $.app.getLastRunVersion();
		self.pushLocalUpdates({
			version		: lastRunVersion.version,
			type		: lastRunVersion.type,
			callback	: _postUpgrade,
			errorback	: _postUpgrade,
		});
		
		return;
		
		//---------------  Callback Handlers  ---------------
		function _postUpgrade() {
			
			if(param.callback) param.callback();
		}
	},
		
	pushLocalUpdates : function(param){
		var self = this;
		
		var lastRunVersion = $.app.getLastRunVersion();
		
		if(lastRunVersion.version) {
			switch (lastRunVersion.type) {
				case '1.x' :
				case '2.x' :
					var updatesData = self._getLocalUpdatesData_oldVersion(lastRunVersion);
					break;

				default : 
					var updatesData = self._getLocalUpdatesData(lastRunVersion);
			}

			var authState = $.authMgr.getAuthState();
            updatesData.token = authState.token;

		    var callParam = {
		    	action 		: 'perkd_upgrade',
		    	data 		: updatesData,
		    	callback 	: param.callback,
			    errorback 	: param.errorback
		    };
		    self.callServer(callParam);
	    
	    } else {

	    	if(param.callback) param.callback();
	    }
	},
	
	_getLocalUpdatesData : function(param) {
		var self = this;
	},
	
	//for 1.x-2.2.2
	_getLocalUpdatesData_oldVersion : function(param){
		var self = this;
		
		if('1.x' == param.type){
			var db_account		= 'perkpop';
			var db_data 		= 'perkpop';
			var db_source		= 'perkpop';
			// var card_stamp 		= Number($.configMgr.getOld('card_stamp', '1.x'));
			var update_field	= 'update';
			
			//profile
			var profileQuery 	= 'select * from profile';
			var profileData 	= App.DB2.query(profileQuery, db_account, db_account);
			//preference
			var prefQuery 	= 'select * from preferences ';
			var prefData 	= App.DB2.query(prefQuery, db_account, db_account);
			//cards
			var cardQuery 	= 'select * from card';
			var cardData 	= App.DB2.query(cardQuery, db_data, db_source);
			
		} else { // 2.x  <=2.2.2
			var db_data 		= ('2.2.2' == param.version) ? "data_2.2" : "data_"+param.version;
			var db_source		= 'data';
			var db_account		= 'account';
			// var card_stamp 		= Number($.configMgr.getOld('WOCardInstance_stamp'), '2.x');
			var update_field	= 'updated';
			
			//profile
			var profileData 	= [];
			//preference
			var prefQuery 	= 'select * from variable ';
			var prefData 	= App.DB2.query(prefQuery, db_account, db_account);
			//cards
			var cardQuery 	= 'select * from card';
			var cardData 	= App.DB2.query(cardQuery, db_data, db_source);
		}
		
		var combineData = {
			card:cardData,
			profile:profileData,
			pref:prefData,
		}
		
	    var data = {};
		data.appver     = param.version;
		data.data     	= JSON.stringify(combineData);
		
		return data;
	},
});

module.exports = UpgradeController;

/**
 * End of file: UpgradeController.js
 */