/**
 * Form Field Factory: barcodeField
 */

Ti.include('/globals.js');

var barcodeField = App.Class(App.WOFieldFactory, {
	className : 'barcodeField',
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		data = data || {'type' : 0, 'number' : null};
		var widgetParam = self.base(param, data);
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.hasSubview = true;
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};        // contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}
		widgetParam.fieldView	= ('fieldView' in param.properties) ? param.properties.fieldView : {};		// contains {style}		

		return widgetParam;
	},
	
	_attachProperties : function(_Widget_){
		var self = this;
		self.base(_Widget_);
		
		_Widget_.label = self.widgetParam.label.value;
		_Widget_.hint = self.widgetParam.hint.value;
		_Widget_.fieldView = self.widgetParam.fieldView;
	},
		
	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.display.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite', touchEnabled:false,
		});
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.display.view.add(label);
		}
		//---  Hint  --------------------
		if(self.widgetParam.hint){
			var aHint = Ti.UI.createLabel(self.widgetParam.hint.style);
			if(!self.widgetParam.data.number||self.widgetParam.data.number==='') {
				aHint.text = self.widgetParam.hint.value;
			}else {
				aHint.text = '';
			}
			self.widgetParam.modes.display.view.add(aHint);
			self.widgetParam.modes.display.hint = aHint;
		}
		//---  Date Field  ---------------		
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		aField.text = self.widgetParam.data.number;
		
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;
		
		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		//--- view of this mode ---
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite', touchEnabled:false,
		});	
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}
		//---  Hint  --------------------
		if(self.widgetParam.hint){
			var aHint = Ti.UI.createLabel(self.widgetParam.hint.style);
			if(!self.widgetParam.data.number||self.widgetParam.data.number==='') {
				aHint.text = self.widgetParam.hint.value;
			}else {
				aHint.text = '';
			}
			self.widgetParam.modes.edit.view.add(aHint);
			self.widgetParam.modes.edit.hint = aHint;
		}
		//---  Date Field  ---------------
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		aField.text = self.widgetParam.data.number;
		aField.touchEnabled=false;

		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
		self.widgetParam.modes.edit.fieldValue = self.widgetParam.data;
		
		return self.widgetParam.modes.edit.view;
	},	

	//----------------  Template/Prototype Methods for Widget object  -------------------	
	_setValue_ : function(data) {
		var self = this;
		data = data || {
			number 	: '',
			type 	: 0,
		}
		self.data = data;
		if(data.number===''){
			self.modes.edit.hint.text = self.hint;
		}else{
			self.modes.edit.hint.text = '';
		}
		self.modes.edit.fieldValue = data;
		self.modes.edit.field.text = data.number;
		self.modes.display.field.text = data.number;
	},
	
	_getValue_ : function() {
		var self = this;
		return self.modes.edit.fieldValue;
	},

	_setMode_ : function(mode){
		var self = this;
		self.base(mode);
		
		if(self._row){
			switch (mode){
			case 'display':
				self._row.touchEnabled = false;
				self._row.hasChild = false;
				break;
			case 'edit':
				self._row.touchEnabled = true;
				self.modes.edit.field.touchEnabled = true;
				self._row.hasChild = true;
				self._row.backgroundSelectedColor = '#0272ED';
				break;
			}
		}
	},
	
	_blur_ : function(){
	},
	
	_renderFieldView_ : function(fieldWindow) {
		var self = this;

		var windowTitle = Ti.UI.createLabel(self.fieldView.style.windowTitle);
		windowTitle.text =  self.fieldView.title || self.label || self.hint || '';

		fieldWindow.setTitleControl(windowTitle);

		var titlebarShadowContainer = Ti.UI.createView(self.fieldView.style.titlebarShadowContainer);
		var titlebarShadow = Ti.UI.createView(self.fieldView.style.titlebarShadow);
		titlebarShadowContainer.add(titlebarShadow);
		fieldWindow.add(titlebarShadowContainer);


		var canvas = Ti.UI.createView(self.fieldView.style.view.canvas);
		fieldWindow.add(canvas);

		var view_codeNo = Ti.UI.createView(self.fieldView.style.view.codeNo);

		var F_CARDNO = Ti.UI.createTextField(self.fieldView.style.textfield.cardNo);
		F_CARDNO.value = self.modes.edit.fieldValue.number,
		view_codeNo.add(F_CARDNO);
		
		
		//--------------  Scan Button  --------------
		var view_scan = Ti.UI.createView(self.fieldView.style.view.scanCode);
									
		var img_scan = Ti.UI.createImageView(self.fieldView.style.img.scan);
		var lbl_scan = Ti.UI.createLabel(self.fieldView.style.label.btnScan);
		lbl_scan.text = App.Helper.L('formfieldwindow_cardno_scan');
		
		var btn_scan = Ti.UI.createButton(self.fieldView.style.btn.scan);
		
		if(Ti.Media.isCameraSupported || App.Config.IS_SIMULATOR) {
			view_scan.add(btn_scan);
			view_scan.add(img_scan);
			view_scan.add(lbl_scan);
		}
		
		btn_scan.addEventListener('touchstart', function(){
				btn_scan.backgroundGradient = self.fieldView.style.buttonSelectGradient_scan;
			});
			btn_scan.addEventListener('touchend', function(){
				btn_scan.backgroundGradient = self.fieldView.style.buttonGradient_scan;
			});
			btn_scan.addEventListener('click', function(){
				Ti.UI.Android.hideSoftKeyboard();

				var scanner = App.loadHelper('Zebra');
				Ti.App.addEventListener(scanner.EVENT.scannedCode, _handleScanResult);
				$.app.doNotDisturb(true);
				scanner.scan();
				
				//----------------  Event Handlers  ------------------
				function _handleScanResult(evt){
					Ti.App.removeEventListener(scanner.EVENT.scannedCode, _handleScanResult);	// remove myself from listening to the scan event
					$.app.doNotDisturb(false);
	
					switch(evt.result) {
						case  scanner.RESULT.success:		// success
							if(!App.Helper.inArray(evt.code, App.Config.SUPPORT_CTYPE)){
								$.app.showAlert(App.Helper.L('ALERT_UnsupportBarcode'), App.Helper.L('TITLE_SCANBARCODE'));
								$.watch.dog(self, 'barcode scan', 'unsupported barcode type: '+evt.type);
								return;
							}
							self.modes.edit.fieldValue = {type:evt.code, number:evt.number};
							if(evt.number===''){
								self.modes.edit.hint.text = self.hint;
							} else {
								self.modes.edit.hint.text = '';
							}
							self.modes.edit.field.text = evt.number;
							F_CARDNO.value = evt.number;
							self.doChanged(evt);
							self._eventProxy.fireEvent(self._eventProxy.EVENT.closeWindow, {});
							break;
						case  scanner.RESULT.failed:	// error
							$.watch.dog(self, '_handleScanResult', JSON.stringify(evt.error));
							break;
						//  result == 0  -> cancel, do nothing
					}
				}
			});
		
		var lbl_tip = Ti.UI.createLabel(self.fieldView.style.label.tip);
		lbl_tip.text = App.Helper.L('formfieldwindow_cardno_tip');
		
		canvas.add(view_codeNo);
		canvas.add(view_scan);
		
		if(Ti.Media.isCameraSupported || App.Config.IS_SIMULATOR) canvas.add(lbl_tip);
		
		//--------------  Done Button  --------------
		var btnDone = Ti.UI.createButton(self.fieldView.style.systemButton.DONE);
		var btnCancel = Ti.UI.createButton(self.fieldView.style.systemButton.CANCEL);
		btnDone.addEventListener('click', function(evt){
			Ti.UI.Android.hideSoftKeyboard();
			self.modes.edit.fieldValue = {type:0, number:F_CARDNO.value};
			if(F_CARDNO.value===''){
				self.modes.edit.hint.text = self.hint;
			} else {
				self.modes.edit.hint.text = '';
			}
			self.modes.edit.field.text = F_CARDNO.value;
			self.doChanged(evt);
			self._eventProxy.fireEvent(self._eventProxy.EVENT.closeWindow, {});
		});
		btnCancel.addEventListener('click', function(evt){
			Ti.UI.Android.hideSoftKeyboard();
			self._eventProxy.fireEvent(self._eventProxy.EVENT.closeWindow, {});
		});

		fieldWindow.setRightNavButton(btnDone);
		fieldWindow.setLeftNavButton(btnCancel);
	},
});

module.exports = barcodeField;

//---  End of file: barcodeField.js
