/**
 * Form Field Factory: dateField
 */

Ti.include('/globals.js');
if (App.Config.IS_ANDROID){	Ti.include('/waveo/core/WOAndroidUtils.js');	};
var helper = App.loadHelper('default'); //TODO [ZJ]

var dateField = App.Class(App.WOFieldFactory, {
	className : 'dateField',
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		data = data || null;
		data = {value:data};
		var widgetParam = self.base(param, data);
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.hasSubview = true;
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}
		widgetParam.fieldView	= ('fieldView' in param.properties) ? param.properties.fieldView : {};		// contains {style}
				
		widgetParam.config  = param.properties.config || {
			min	:  null,
			max	:  null,
			type : 'all',
		};
		if('all' == widgetParam.config.type) widgetParam.config = {
			min : null,
			max : null,
			type : 'all',
		};
		if('birth' == widgetParam.config.type) widgetParam.config = {
			min : null,
			max : Math.round((new Date()).valueOf() / 1000),
			type : 'birth',
		};
		if('join' == widgetParam.config.type) widgetParam.config = {
			min : null,
			max : Math.round((new Date()).valueOf() / 1000),
			type : 'join',
		};
		if('cardExpiry' == widgetParam.config.type) widgetParam.config = {
			min : null,
			max : null,
			cardRemindDays: $.prefMgr.get('cardRemindDays'),
			// 0 day   ->  cardExpiryReminnder = 'on the date of '
			//	1 day   ->  cardExpiryReminnder = '1 day before'
			type: 'cardExpiry',
		};			
		return widgetParam;
	},

	_attachProperties : function(_Widget_){
		var self = this;
		self.base(_Widget_);
		
		_Widget_.label = self.widgetParam.label.value;
		_Widget_.fieldView = self.widgetParam.fieldView;
		_Widget_.config = self.widgetParam.config;
	},
	
	//----------------  Template/Prototype Methods for Widget object  -------------------
	_setupEventHandlers_ : function(){
		var self = this;
		self.base();

		self.modes.edit.view.addEventListener('click', function(e){
			self.doClick(e);
		});
		
	},

	//----------------  Basic Create View for the row  -------------------
	_createView : function(){
		var self = this;

		var view = Ti.UI.createView(self.widgetParam.style);

		//---  Views for each display mode
		for(mode in self.widgetParam.modes){
			var modeView = self._drawView(mode);
			view.add(modeView);
		}
		
		//---  Wrap Main View as a Row if specified
		if(self.widgetParam.useRow){
			self.widgetParam._row = self._createRow();
			if (App.Config.IS_ANDROID) {
				for(mode in self.widgetParam.modes){
					var modeView1 = self._drawView(mode);
					modeView1.left = self.widgetParam.style.left;
					modeView1.right = self.widgetParam.style.right;
					self.widgetParam._row.add(modeView1);
				}
			} else {
				self.widgetParam._row.add(view);			
			}
		}
		return view;
	},

	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.display.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',  touchEnabled:false,
		});

		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.display.view.add(label);
		}
		//---  Date Field  ---------------		
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		if(self.widgetParam.data.value) {
			aField.text = App.Moment(new Date(self.widgetParam.data.value*1000.0)).format('LL');
		}
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;
		
		aField.setTouchEnabled(false);
		self.widgetParam.modes.display.view.setTouchEnabled(false);
		
		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		//--- view of this mode ---
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',  touchEnabled:false,
		});
	
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}
		//---  Date Field  ---------------
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		aField.touchEnabled=false;
		self.widgetParam.modes.edit.fieldValue = self.widgetParam.data.value;
		if(self.widgetParam.modes.edit.fieldValue) {
			var aDate = new Date(self.widgetParam.modes.edit.fieldValue*1000);
			aField.text = App.Moment(aDate).format('LL');
		}
		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
		
		return self.widgetParam.modes.edit.view;
	},

	//----------------  Template/Prototype Methods for Widget object  -------------------	
	_setValue_ : function(data){
		var self = this;
		if(data) {
			self.data.value = data;
			self.modes.edit.fieldValue = data;
			var aDate = new Date(data*1000.0);
			self.modes.edit.field.text = App.Moment(aDate).format('LL');
			self.modes.display.field.text = self.modes.edit.field.text;
		} else {
			Ti.API.info('CTK - null zero value');
			self.data.value = null;
			self.modes.edit.fieldValue = null;
			self.modes.edit.field.text = '';
			self.modes.display.field.text = '';
		}
	},
	
	_getValue_ : function(){
		var self = this;
		return self.modes.edit.fieldValue;
	},

	_setMode_ : function(mode){
		var self = this;
		self.base(mode);
		
		Ti.API.info('CTK row = '+self._row);
		if(self._row){
			Ti.API.info('CTK mode ='+mode);
			switch (mode){
			case 'display':
				self._row.setHasChild(false);
				self.modes.edit.field.setTouchEnabled(false);
				self.modes.edit.view.visible = false;
				
				self._row.setFocusable (false);				
				self._row.backgroundSelectedColor = '#16000000';
				break;
			case 'edit':
				self._row.setHasChild(true);
				self.modes.edit.field.setTouchEnabled(true);
				self.modes.display.view.visible = false;

				self._row.setFocusable (true);				
				self._row.setTouchEnabled(true);
				self._row.backgroundSelectedColor = '#0272ED';
				break;
			}
		}
	},
	

	_blur_ : function(){
	},

	_renderFieldView_ : function(fieldWindow){
		var self = this;
		
		var windowTitle = Ti.UI.createLabel(self.fieldView.style.windowTitle);
		windowTitle.text =( ( self.fieldView.title || self.label ) || self.hint )|| '';

		fieldWindow.setTitleControl(windowTitle);

		var titlebarShadowContainer = Ti.UI.createView(self.fieldView.style.titlebarShadowContainer);
		var titlebarShadow = Ti.UI.createView(self.fieldView.style.titlebarShadow);
		titlebarShadowContainer.add(titlebarShadow);
		fieldWindow.add(titlebarShadowContainer);		

		var lbl_tip = Ti.UI.createLabel(self.fieldView.style.label.tip);
		lbl_tip.text = self.fieldView.label || '';

		fieldWindow.add(lbl_tip);	
		

		//---  Card Expiry  ----------------------
		if(self.config.type == 'cardExpiry'){
			var cardRemindDays = self.config.cardRemindDays;
			var prompt = App.Helper.L('formfieldwindow_cardExpiry_default');			
			if(cardRemindDays >= 0){		// not == do not remind
				switch(cardRemindDays) {
					case 0 : 
						prompt = App.Helper.L('formfieldwindow_cardExpiryOnTheDay');
						break;
					case 1 :
						prompt = App.Helper.L('formfieldwindow_cardExpiry1DayBefore');
						break;					
					default :
						prompt =  String.format(App.Helper.L('formfieldwindow_cardExpiryNDaysBefore'), cardRemindDays);
				}
			}
			lbl_tip.text = prompt;
		} else {
			lbl_tip.text = self.fieldView.label || '';
		}
		fieldWindow.add(lbl_tip);
		
		var view_datePicker = Ti.UI.createView(self.fieldView.style.datePickerArea);
		
		var datePicker = Ti.UI.createPicker(self.fieldView.style.datePicker);
		datePicker.value = null;
		datePicker.minDate = self.config.min ? new Date(self.config.min*1000) : null;
		datePicker.maxDate = self.config.max ? new Date(self.config.max*1000) : null;
		if(self.modes.edit.fieldValue) {
			datePicker.value = (new Date(self.modes.edit.fieldValue*1000));
		} else {
			datePicker.value = new Date();			
		}
		datePicker.addEventListener('change', function(){}); //this will make datePicker.value be updated.


		var btnDone = Ti.UI.createButton(self.fieldView.style.systemButton.DONE);
		var btnClear = Ti.UI.createButton(self.fieldView.style.systemButton.CLEAR);
		var flexSpace = Ti.UI.createButton(self.fieldView.style.systemButton.FLEXIBLE_SPACE);

		if (App.Config.IS_ANDROID) {
			var toolBar = createAndroidToolbar (self.fieldView.style.toolBar,{items: [btnClear, btnDone]});
		} else {	
			var toolBar = Ti.UI.iOS.createToolbar(self.fieldView.style.toolBar);
	 		toolBar.items = [btnClear,flexSpace, btnDone];
		}
  		toolBar.addEventListener('click', function(){});
  		

		btnDone.addEventListener('click', function(evt){
			Ti.UI.Android.hideSoftKeyboard();
			datePicker.value = datePicker.value || new Date();
			self.modes.edit.fieldValue = Math.round(datePicker.value.valueOf() / 1000);
			self.modes.edit.field.text = App.Moment(datePicker.value).format('LL');
			self._eventProxy.fireEvent(self._eventProxy.EVENT.closeWindow, {});
			self.doChanged(evt);
		});
		btnClear.addEventListener('click', function(evt){
			datePicker.value = new Date();
			self.modes.edit.fieldValue = null;
			self.modes.edit.field.text = '';
			self._eventProxy.fireEvent(self._eventProxy.EVENT.closeWindow, {});
			self.doChanged(evt);
		});

		fieldWindow.add(toolBar);
		view_datePicker.add(datePicker);
		fieldWindow.add(view_datePicker);

	},
});

module.exports = dateField;

//---  End of file: dateField.js