/**
 * Form Field Factory: langField
 */

Ti.include('/globals.js');
if (App.Config.IS_ANDROID){	Ti.include('/waveo/core/WOAndroidUtils.js');	};
var helper = App.loadHelper('default'); //TODO [ZJ]

var langField = App.Class(App.WOFieldFactory, {
	className : 'langField',
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		// data = data || App.Config.LANG_RG;
		data = App.Config.LANG_RG;
		data = {value:data};
		var widgetParam = self.base(param, data);
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.hasSubview = true;
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}
		widgetParam.fieldView	= ('fieldView' in param.properties) ? param.properties.fieldView : {};		// contains {style}
		widgetParam.config  = param.properties.config;
		
		return widgetParam;
	},
	
	_attachProperties : function(_Widget_){
		var self = this;
		self.base(_Widget_);
		
		_Widget_.label = self.widgetParam.label.value;
		_Widget_.fieldView = self.widgetParam.fieldView;
		_Widget_.config = self.widgetParam.config;
	},
	
	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---
		self.widgetParam.modes.display.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite', 
		});
	
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.display.view.add(label);
		}
		//---  Date Field  ---------------		
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		if(self.widgetParam.data.value) aField.text = self.widgetParam.config.langRegion[self.widgetParam.data.value];
		
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;
		
		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		//--- view of this mode ---
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite', 
		});
	
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}
		//---  Date Field  ---------------
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		self.widgetParam.modes.edit.fieldValue = self.widgetParam.data.value;
		aField.text = self.widgetParam.config.langRegion[self.widgetParam.data.value];
		
		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
		
		return self.widgetParam.modes.edit.view;
	},

	//----------------  Template/Prototype Methods for Widget object  -------------------	
	_setValue_ : function(data){
		var self = this;
		
		self.data.value = data;
		self.modes.edit.fieldValue = data;

		self.modes.edit.field.text = self.config.langRegion[App.Config.LANG_RG];
		self.modes.display.field.text = self.config.langRegion[App.Config.LANG_RG];
	},
	
	_getValue_ : function(){
		var self = this;
		return self.modes.edit.fieldValue;
	},

	_setMode_ : function(mode){
		var self = this;
		self.base(mode);
		
		if(self._row){
			switch (mode){
			case 'display':
				self._row.touchEnabled = false;
				self._row.hasChild = false;
				break;
			case 'edit':
				self._row.touchEnabled = true;
				self._row.hasChild = true;
				self.modes.edit.field.touchEnabled = true;
				break;
			}
		}
	},

	_renderFieldView_ : function(fieldWindow){
		var self = this;
		
		var windowTitle = Ti.UI.createLabel(self.fieldView.style.windowTitle);
		windowTitle.text =  self.fieldView.title || self.label || self.hint || '';
		
		fieldWindow.setTitleControl(windowTitle);

		var titlebarShadowContainer = Ti.UI.createView(self.fieldView.style.titlebarShadowContainer);
		var titlebarShadow = Ti.UI.createView(self.fieldView.style.titlebarShadow);
		titlebarShadowContainer.add(titlebarShadow);
		fieldWindow.add(titlebarShadowContainer);
		
		var canvas = Ti.UI.createView(self.fieldView.style.view.canvas);
		fieldWindow.add(canvas);
		
		var datas = [];
		for(var code in self.config.langRegion) {
			var row = Titanium.UI.createTableViewRow(self.fieldView.style.tableRow);
			var langLabel = Ti.UI.createLabel(self.fieldView.style.label);
			langLabel.text = self.config.langRegion[code]
			row.add(langLabel);
			row.code = code;
			row.checkIcon = Ti.UI.createImageView(self.fieldView.style.img.checkIcon);
			row.add(row.checkIcon);
			row.checkIcon.visible = false;
		
			if(row.code == App.Config.LANG_RG){
				//row.hasCheck = true;
				row.checkIcon.visible = true;
			}
			datas.push(row);
		}
		
		var tableview = Ti.UI.createTableView(self.fieldView.style.tableView);
		tableview.data = datas;
		
		tableview.addEventListener('click', function(evt){
			
			if(evt.row.checkIcon.visible) return;
			
			var waitHint = Ti.UI.createAlertDialog({
				message:App.Helper.L('hint_wait'),
			});
			// event data
			var index = evt.index;
			var section = evt.section;
			
			var dialog = Ti.UI.createAlertDialog({
				message:App.Helper.L('ALERT_CONFIRMSWITCH'),
				title:App.Helper.L('TITLE_CONFIRM'),
				buttonNames:[App.Helper.L('BUTTON_CANCEL'), App.Helper.L('BUTTON_OK')],
			});
			dialog.addEventListener('click', function(e){
				if(e.index==1){
					waitHint.show();
					for (var i=0;i<section.rows.length;i++) section.rows[i].checkIcon = false;
					section.rows[index].checkIcon = true;
					self.modes.edit.fieldValue = section.rows[index].code;
					self.modes.edit.field.text = self.config.langRegion[self.data.value];
					$.configMgr.setLanguageRegion(section.rows[index].code);
					self.doChanged();
				}
			});
			dialog.show(); 
		});
		
		fieldWindow.add(tableview);
	},
});

module.exports = langField;

//---  End of file: langField.js
