/**
 * Form Field Factory: linkField
 */

Ti.include('/globals.js');

var linkField = App.Class(App.WOFieldFactory, {
	className : 'linkField',

	EVENT		: {
		//--- Controller to handle:
		doCall	: 'linkField.doCall',
		//--- events from Controller:
    },	

	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		data = {value:data};
		var widgetParam = self.base(param, data);
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}

		return widgetParam;
	},

	//----------------  Basic Create View for the row  -------------------
	_createView : function(){
		var self = this;

		var view = Ti.UI.createView(self.widgetParam.style);

		//---  Views for each display mode
		for(mode in self.widgetParam.modes){
			var modeView = self._drawView(mode);
			view.add(modeView);
		}
		
		//---  Wrap Main View as a Row if specified
		if(self.widgetParam.useRow){
			self.widgetParam._row = self._createRow();
			if (App.Config.IS_ANDROID) {
				for(mode in self.widgetParam.modes){
					var modeView1 = self._drawView(mode);
					modeView1.left = self.widgetParam.style.left;
					modeView1.right = self.widgetParam.style.right;
					self.widgetParam._row.add(modeView1);
				}
			} else {
				self.widgetParam._row.add(view);			
			}
		}
		return view;
	},

	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.display.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});		
		//---  Icon  --------------------
		var icon = Ti.UI.createImageView({
			left: 0, width:'24dp', top: 0, bottom:0,
		});
		icon.image = App.Config.IS_ANDROID ? ('/assets/images/icons/url@2x.png'):('/assets/images/icons/url.png');
		self.widgetParam.modes.display.view.add(icon);
		
		//---  Label  --------------------
		if(self.widgetParam.label){
			var aLabel = Ti.UI.createLabel(self.widgetParam.label.style);
			aLabel.text = self.widgetParam.label.value;
			self.widgetParam.modes.display.view.add(aLabel);
			self.widgetParam.modes.display.label = aLabel;
		}
		
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);		
		aField.text = self.widgetParam.data.value;
		
		if(self.widgetParam.label){
			aField.visible = false;
		} else {
			aField.visible = true;
		}
		
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;
		
		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		// --- view of this mode ---
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});
		
		//---  Icon  --------------------
		var icon = Ti.UI.createImageView({
			left: 0, width:'24dp', top: 0, bottom:0,
		});
		icon.image = App.Config.IS_ANDROID ? ('/assets/images/icons/url.png'):('/assets/images/icons/url@2x.png');
	
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}
		//---  Hint  --------------------
		var hintText = '';
		if(self.widgetParam.hint && self.widgetParam.hint.value){
			hintText = self.widgetParam.hint.value;
		}
		//---  Text Field  ---------------			
		var aField = Ti.UI.createTextField(self.widgetParam.field.style);
		aField.value = self.widgetParam.data.value;
		
		self.widgetParam.modes.edit.view.add(icon);
		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
		
		return self.widgetParam.modes.edit.view;
	},
	
	//----------------  Template/Prototype Methods for Widget object  -------------------
	_setupEventHandlers_ : function(){
		var self = this;
		self.base();

		self.modes.edit.field.addEventListener('blur', (function(that) {
			return (function(evt){
				that.doChanged(evt);
			})
		})(self) );

		if(self.modes.display.label) {
			var clickLabel = self.modes.display.label;
		} else {
			var clickLabel = self.modes.display.field;
		}
		
		clickLabel.addEventListener('click', (function(value){ 
			return (function(evt){
				Ti.Platform.openURL(value);
			});
		})(self.data.value) );
	},

	_setValue_ : function(data){
		var self = this;

		var strValue = ('string' == typeof data) ? data : '';
		self.data = {value:strValue};
		self.modes.edit.field.value = strValue;
		self.modes.display.field.text = strValue;
	},
	
	_getValue_ : function(){
		var self = this;
		return self.modes.edit.field.value;
	},

	_setMode_ : function(mode){
		var self = this;

		self.base(mode);
		self.VIEW.touchEnabled = true;
		
		if(self._row){
			switch (mode){
			case 'display':
				self._row.touchEnabled = false;
				break;
			case 'edit':
				self._row.touchEnabled = true;
				self.modes.edit.field.touchEnabled = true;
				break;
			}
		}
	},
	
	_blur_ : function(){
		var self = this;
		self.modes.edit.field.blur();
	},

});

module.exports = linkField;

//---  End of file: linkField.js
