/**
 * Form Field Factory: nameOrderField
 */

Ti.include('/globals.js');
if (App.Config.IS_ANDROID){	Ti.include('/waveo/core/WOAndroidUtils.js');	};
var helper = App.loadHelper('default'); //TODO [ZJ]

var nameOrderField = App.Class(App.WOFieldFactory, {
	className : 'nameOrderField',
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		data = data || { displayID:param.properties.config.defaultRow, givenName:'', familyName:'' }; //Default selected Row 0, givenName & familyName both are empty.
		data.familyName = data.familyName.replace(/(^\s*)|(\s*$)/g,"");
		data.givenName = data.givenName.replace(/(^\s*)|(\s*$)/g,"");
		
		data = {value:data};
		var widgetParam = self.base(param, data);
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.hasSubview = true;
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}
		widgetParam.fieldView = ('fieldView' in param.properties) ? param.properties.fieldView : {};		// contains {style}		
		widgetParam.config  = param.properties.config;
		
		return widgetParam;
	},
	
	_attachMethods : function(_Widget_){
		var self = this;	
		self.base(_Widget_);

		//---  attach object behaviors  ---
		_Widget_.prototype.setDisplay = self._setDisplay_;
	},
	
	_attachProperties : function(_Widget_){
		var self = this;
		self.base(_Widget_);
		
		_Widget_.label = self.widgetParam.label.value;
		_Widget_.fieldView = self.widgetParam.fieldView;
		_Widget_.config = self.widgetParam.config;
	},
	
	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.display.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',  touchEnabled:false,
		});

		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.display.view.add(label);
		}
		//---  Date Field  ---------------		
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		var display = App.Helper.getDisplayName(self.widgetParam.data.value.displayID, self.widgetParam.data.value.familyName, self.widgetParam.data.value.givenName);
		aField.text = display;
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;
		
		aField.setTouchEnabled(false);
		self.widgetParam.modes.display.view.setTouchEnabled(false);
		
		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		//--- view of this mode ---
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',   touchEnabled:false,
		});
	
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}
		//---  Date Field  ---------------
		self.widgetParam.modes.edit.fieldValue = {};
		self.widgetParam.modes.edit.fieldValue.displayID = self.widgetParam.data.value.displayID;
		self.widgetParam.modes.edit.fieldValue.givenName = self.widgetParam.data.value.givenName;
		self.widgetParam.modes.edit.fieldValue.familyName = self.widgetParam.data.value.familyName;
		
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		aField.touchEnabled=false;
		var display = App.Helper.getDisplayName(self.widgetParam.modes.edit.fieldValue.displayID, self.widgetParam.modes.edit.fieldValue.familyName, self.widgetParam.modes.edit.fieldValue.givenName);
		aField.text = display;
		
		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
		
		return self.widgetParam.modes.edit.view;
	},

	//----------------  Template/Prototype Methods for Widget object  -------------------
	_update_ : function(data){
		var self = this;
		var display = App.Helper.getDisplayName(self.modes.edit.fieldValue.displayID, data.familyName, data.givenName);
		self.modes.edit.field.text = display;
		var displayID = self.modes.edit.fieldValue.displayID;
		self.modes.edit.fieldValue = {};
		self.modes.edit.fieldValue.displayID = displayID;
		self.modes.edit.fieldValue.familyName = data.familyName;
		self.modes.edit.fieldValue.givenName = data.givenName;
	},
	
	_setValue_ : function(data){
		var self = this;
		// var data = {
			// displayID : 0/1/2...
			// givenName : 'xxx',
			// familyName : 'yyy',
		// };
		var display = App.Helper.getDisplayName(data.displayID, data['familyName'], data['givenName']);
		
		self.data.value = {
			displayID : data.displayID,
			givenName : data.givenName,
			familyName : data.familyName,
		};
		self.modes.edit.fieldValue = data;
		self.modes.edit.field.text = display;
		self.modes.display.field.text = display;
	},
	
	_getValue_ : function(){
		var self = this;
		return self.modes.edit.fieldValue.displayID;
	},

	_setMode_ : function(mode){
		var self = this;
		self.base(mode);
		
		if(self._row){
			switch (mode){
			case 'display':
				self._row.setHasChild(false);
				self.modes.edit.field.setTouchEnabled(false);
				self.modes.edit.view.visible = false;
				
				self._row.setFocusable (false);			
				self._row.backgroundSelectedColor = '#16000000';
				break;
			case 'edit':
				self._row.setHasChild(true);
				self.modes.edit.field.setTouchEnabled(true);
				self.modes.display.view.visible = false;

				self._row.setFocusable (true);				
				self._row.setTouchEnabled(true);
				self._row.backgroundSelectedColor = '#0272ED';
				break;
			}
		}
	},

	_renderFieldView_ : function(fieldWindow){
		var self = this;
		var windowTitle = Ti.UI.createLabel(self.fieldView.style.windowTitle);
		windowTitle.text =  App.Helper.L('formLabel_preferredName');

		fieldWindow.setTitleControl(windowTitle);

		var canvas = Ti.UI.createView(self.fieldView.style.view.canvas);
		fieldWindow.add(canvas);

		
		var desc_label = Ti.UI.createLabel(self.fieldView.style.label.tip);
		desc_label.text = App.Helper.L('formLabel_nameOrderDesc');
		canvas.add(desc_label);
		
		picker = Ti.UI.createPicker(self.fieldView.style.picker);
		
		var data = [];
		for(var i=0; i<self.config.rows.length; i++) data.push(Ti.UI.createPickerRow(self.config.rows[i]));

		picker.add(data);
		picker.selectionIndicator = true;
		
		picker.addEventListener('change', function(evt){
			var display = App.Helper.getDisplayName(evt.rowIndex, self.modes.edit.fieldValue.familyName, self.modes.edit.fieldValue.givenName);
			self.modes.edit.field.text = display;
			self.modes.edit.fieldValue.displayID = evt.rowIndex;
		});
		
		canvas.add(picker);

		var titlebarShadowContainer = Ti.UI.createView(self.fieldView.style.titlebarShadowContainer);
		var titlebarShadow = Ti.UI.createView(self.fieldView.style.titlebarShadow);
		titlebarShadowContainer.add(titlebarShadow);
		fieldWindow.add(titlebarShadowContainer);
		
		picker.setSelectedRow(0, self.modes.edit.fieldValue.displayID, false);
	},
});

module.exports = nameOrderField;

//---  End of file: nameOrderField.js
