/**
 * Form Field Factory: tabbedBarField
 */

Ti.include('/globals.js');
if (App.Config.IS_ANDROID){
Ti.include('/waveo/core/WOAndroidUtils.js');	
}

var tabbedBarField = App.Class(App.WOFieldFactory, {
	className : 'tabbedBarField',
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		data = data || param.properties.config.defaultValue;
		data = {value:data};
		var widgetParam = self.base(param, data);
		
		var index = 0;
		widgetParam.properties.config.labels = [];
		widgetParam.properties.config.values = {};
		for(var label in widgetParam.properties.config.options){
			widgetParam.properties.config.labels.push(App.Helper.L(label, 'fieldLabel'));
			widgetParam.properties.config.values[index++] = widgetParam.properties.config.options[label];
		}
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.hasSubview = false;
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}

		return widgetParam;
	},

	_attachProperties : function(_Widget_){
		var self = this;
		self.base(_Widget_);
		
		_Widget_.config = self.widgetParam.properties.config;
	},

	//----------------  Template/Prototype Methods for Widget object  -------------------
	_setupEventHandlers_ : function(){
		var self = this;
		self.base();

		function onClick_(that) {
			return function(evt){
				if (evt.index==undefined||evt.index==null) return; // Android fix.
				that.doChanged(evt);
			}
		}

		self.modes.edit.field.addEventListener('click', onClick_(self));
		
		self.modes.edit.field.addEventListener('postlayout', function(e) {
			Ti.API.info('CTKT-postlayout:'+self.name+';'+
				e.source.getRect().width+'x'+e.source.getRect().height);
			e.source.setVisible(true);
		});
	},

	_createView : function(){
		var self = this;

		var view = Ti.UI.createView(self.widgetParam.style);

		//---  Views for each display mode
		for(mode in self.widgetParam.modes){
			var modeView = self._drawView(mode);
			view.add(modeView);
		}
		
		//---  Wrap Main View as a Row if specified
		if(self.widgetParam.useRow){
			self.widgetParam._row = self._createRow();
			if (App.Config.IS_ANDROID) {
				
				for(mode in self.widgetParam.modes){
					var modeView1 = self._drawView(mode);
					modeView1.left = self.widgetParam.style.left;
					modeView1.right = self.widgetParam.style.right;
					modeView1.top = self.widgetParam.style.top;
					modeView1.height = self.widgetParam.style.height;
					self.widgetParam._row.add(modeView1);
					self.widgetParam.modes[mode].rowView = modeView1;
				}
				
				//self.widgetParam._row.add(view);			
			} else {
				self.widgetParam._row.add(view);			
			}
		}
		return view;
	},

	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.display.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});
		//---  Label  --------------------
		if(self.widgetParam.properties.label){
			var label = Ti.UI.createLabel(self.widgetParam.properties.label.style);
			label.text = self.widgetParam.properties.label.value;
			self.widgetParam.modes.display.view.add(label);
		}
		
		for(var index in self.widgetParam.properties.config.values) 
		if(self.widgetParam.properties.config.values[index] == self.widgetParam.data.value) break;
		
		var aField = Ti.UI.createLabel(self.widgetParam.properties.field.style);

		aField.text = self.widgetParam.properties.config.labels[index];
	//	aField.index = 0;
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;

		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});		//---  Label  --------------------
		if(self.widgetParam.properties.label){
			var label = Ti.UI.createLabel(self.widgetParam.properties.label.style);
			label.text = self.widgetParam.properties.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}
		for(var index in self.widgetParam.properties.config.values) {
			if(self.widgetParam.properties.config.values[index] == self.widgetParam.data.value) break;
		}

		if (App.Config.IS_ANDROID) {
			var _theStyle = App.Helper.clone (self.widgetParam.properties.config.style);
			_theStyle.labels = self.widgetParam.properties.config.labels;
			_theStyle.index = index;
			var aField = createAndroidTabbedBar(_theStyle);
		} else {
			var aField = Titanium.UI.iOS.createTabbedBar(self.widgetParam.properties.config.style);
			aField.labels = self.widgetParam.properties.config.labels;
			aField.index = index;
		}

		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
		
		return self.widgetParam.modes.edit.view;
	},
	
	_setValue_ : function(data){
		var self = this;
		
		for(var index in self.config.values) if(self.config.values[index] == data) break;
		self.data = {value:data};	
		if (App.Config.IS_ANDROID) {
			self.modes.edit.field.setIndex(index);
		} else {
			self.modes.edit.field.index = index;		
		}
		self.modes.display.field.labels = [self.config.labels[index]];
		self.modes.display.field.text = self.config.labels[index];
	},
	
	_getValue_ : function(){
		var self = this;
		return self.config.values[self.modes.edit.field.index];
	},


	_blur_ : function(){
		var self = this;
	},

/*
	_setMode_ : function(mode){
		var self = this;
	
		Ti.API.info('CTKT-setMode:'+self.name+':'+self.modes.edit.rowView.visible);
		
		self.base(mode);
		if (self._row) {
			for(var m in self.modes){
				self.modes[m].rowView.setVisible(true);
			}							
		} else {
			for(var m in self.modes){
				self.modes[m].field.setVisible(m==mode);
			}				
		}
	}
	*/
		
});

module.exports = tabbedBarField;

//---  End of file: tabbedBarField.js