/**
 * Form Field Factory: tcField
 */

Ti.include('/globals.js');
if (App.Config.IS_ANDROID){	Ti.include('/waveo/core/WOAndroidUtils.js');	};
var helper = App.loadHelper('default'); //TODO [ZJ]

var tcField = App.Class(App.WOFieldFactory, {
	className : 'tcField',
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		data = {value:data};
		var widgetParam = self.base(param, data);
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.hasSubview = true;
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}
		widgetParam.fieldView	= ('fieldView' in param.properties) ? param.properties.fieldView : {};		// contains {style}
		widgetParam.config  = param.properties.config
				
		return widgetParam;
	},
	
	_attachProperties : function(_Widget_){
		var self = this;
		self.base(_Widget_);
		
		_Widget_.label = self.widgetParam.label.value;
		_Widget_.fieldView = self.widgetParam.fieldView;
		_Widget_.config = self.widgetParam.config;
	},
	
	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.display.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite', 
		});

		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.display.view.add(label);
		}
		//---  Date Field  ---------------		
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		aField.text = 'undefined' != typeof self.widgetParam.config.values[self.widgetParam.data.value] ? self.widgetParam.config.values[self.widgetParam.data.value].label : '';
		
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;
		
		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		//--- view of this mode ---
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite', 
		});
	
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}
		//---  Date Field  ---------------
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		self.widgetParam.modes.edit.fieldValue = self.widgetParam.data.value;
		aField.text = 'undefined' != typeof self.widgetParam.config.values[self.widgetParam.data.value] ? self.widgetParam.config.values[self.widgetParam.data.value].label : '';

		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
		
		return self.widgetParam.modes.edit.view;
	},

	//----------------  Template/Prototype Methods for Widget object  -------------------	
	_setValue_ : function(data){
		var self = this;
		
		var label = self.config.values[data] || {label:''};
		self.data.value = data;
		self.modes.edit.fieldValue = data;
		self.modes.edit.text = label.label;
		self.modes.display.field.text = label.label;
	},
	
	_getValue_ : function(){
		var self = this;
		return self.modes.edit.fieldValue;
	},

	_setMode_ : function(mode){
		var self = this;
		self.base(mode);
		
		if(self._row){
			switch (mode){
			case 'display':
				self._row.touchEnabled = false;
				self._row.hasChild = false;
				break;
			case 'edit':
				self._row.touchEnabled = true;
				self._row.hasChild = true;
				self.modes.edit.field.touchEnabled = true;
				break;
			}
		}
	},

	_renderFieldView_ : function(fieldWindow){
		var self = this;
		
		var windowTitle = Ti.UI.createLabel(self.fieldView.style.windowTitle);
		windowTitle.text =  self.fieldView.title || self.label || self.hint || '';
		
		fieldWindow.setTitleControl(windowTitle);

		var titlebarShadowContainer = Ti.UI.createView(self.fieldView.style.titlebarShadowContainer);
		var titlebarShadow = Ti.UI.createView(self.fieldView.style.titlebarShadow);
		titlebarShadowContainer.add(titlebarShadow);
		fieldWindow.add(titlebarShadowContainer);
		
		//Render Toolbar
		var buttons = []
		for(var i=0; i<self.config.buttons.length; i++){
			var buttonDefn = self.config.buttons[i];
			if('undefined' != typeof buttonDefn.style){
				var buttonStyle = self.fieldView.style[buttonDefn.style];
			} else {
				var buttonStyle = self.fieldView.style.button;
			}
			var button = Ti.UI.createButton(buttonStyle);
			button.title = buttonDefn.label;
			button.ID = i;
			button.width = 150;
			button.backgroundColor = 'transparent';
			buttons.push(button);
			buttons[buttons.length-1].addEventListener('click', _doClickButton);
		}
		
		var platformWidth = Ti.Platform.displayCaps.platformWidth * 160 / Titanium.Platform.displayCaps.dpi
		var toolBar =createAndroidToolbar({
			width: platformWidth +'dp',
			height: '44dp',
 			bottom : 0, barColor : '#000000',
		}, {items:[]});
		buttons[0].top = '8dp';
		buttons[0].left = '20dp';
		toolBar.add(buttons[0]);
		buttons[1].top = '8dp';
		buttons[1].right = '20dp';
		toolBar.add(buttons[1]);
	 	
	 	//Render Webview
	 	var webView = Ti.UI.createWebView(self.fieldView.style.view.webViewCanvas);
		if(self.config.webview.substr(0, 4) == 'http') webView.url = self.config.webview;
		else webView.html = self.config.webview;
		
		var closePageBtn = Ti.UI.createButton({
	 		title : App.Helper.L('BUTTON_CLOSE'),
	 	});
	 	
	 	var firstLoadDone = false;
		webView.addEventListener('beforeload', function(evt){
			Ti.API.error('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>beforeload:'+evt.url);
			if(!self.config.useBrowser){
	
			} else {
				if(firstLoadDone && Ti.Platform.canOpenURL(evt.url)){
					webView.stopLoading();
					if(webView.canGoBack()) webView.goBack();
					Ti.Platform.openURL(evt.url);
				}
			}
		});
		webView.addEventListener('load', function(evt){
			Ti.API.error('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>load:'+evt.url);
			if(!self.config.useBrowser){ // internal view
				if(webView.canGoBack()){
					if(!fieldWindow.rightNavButton) fieldWindow.setRightNavButton(closePageBtn);
				} else {
					fieldWindow.setRightNavButton(null);
				}
			} else { // browser view
				if(webView.canGoBack()){
					webView.goBack();
				}
			}
			if(!firstLoadDone) firstLoadDone = true;
		});

	 	closePageBtn.addEventListener('click', function(evt){
	 		if(webView.canGoBack()) webView.goBack();
	 		else {
	 			self._eventProxy.fireEvent(self._eventProxy.EVENT.closeWindow, {});
	 			self._eventProxy.fireEvent(self._eventProxy.EVENT.blur, {});
	 		}
	 	});
	 	
		fieldWindow.add(webView);
		fieldWindow.add(toolBar);
		
		function _doClickButton(evt){
			var buttonID = evt.source.ID;
			if('undefined' != typeof self.config.buttons[buttonID].dialog){
				var dialogConfig = self.config.buttons[buttonID].dialog;
				var buttonNames = [];
				for(var i=0; i<dialogConfig.buttons.length; i++){
					buttonNames.push(dialogConfig.buttons[i].label);
				}
				var dialog = Ti.UI.createAlertDialog({
					title		: dialogConfig.title,
					message		: dialogConfig.message,
					buttonNames	: buttonNames,
				});
				//---  Handle Action button  ---
				dialog.addEventListener('click', function(evt){
					if('undefined' != typeof dialogConfig.buttons[evt.index].value){
						self.modes.edit.field.text = self.config.values[dialogConfig.buttons[evt.index].value].label;
						self.modes.edit.fieldValue = dialogConfig.buttons[evt.index].value;
						self._eventProxy.fireEvent(self._eventProxy.EVENT.closeWindow, {});
						self.doChanged(evt);
						self._eventProxy.fireEvent(self._eventProxy.EVENT.blur, {});
					}
				});
				dialog.show();
			} else {
				if('undefined' != typeof self.config.buttons[buttonID].value){
					self.modes.edit.field.text = self.config.values[self.config.buttons[buttonID].value].label;
					self.modes.edit.fieldValue = self.config.buttons[buttonID].value;
					self._eventProxy.fireEvent(self._eventProxy.EVENT.closeWindow, {});
					self.doChanged(evt);
					self._eventProxy.fireEvent(self._eventProxy.EVENT.blur, {});
				}
			}
		}
	},
});

module.exports = tcField;

//---  End of file: tcField.js
