/**
 * Form Field Factory: textField
 */

Ti.include('/globals.js');

var textField = App.Class(App.WOFieldFactory, {
	className : 'textField',
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		if(null == data)  data = '';
		
		data = {value:data};
		var widgetParam = self.base(param, data);
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}

		return widgetParam;
	},

	_attachProperties : function(_Widget_){
		var self = this;
		self.base(_Widget_);
		
		_Widget_.hint = self.widgetParam.hint.value;
	},

	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.display.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite', 
		});
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.display.view.add(label);
		};
		//---  Hint  --------------------
		if(self.widgetParam.hint){
			var aHint = Ti.UI.createLabel(self.widgetParam.hint.style);
			if(!self.widgetParam.data.value||self.widgetParam.data.value==='') {
				aHint.text = self.widgetParam.hint.value;
			}else {
				aHint.text = '';
			}
			self.widgetParam.modes.display.view.add(aHint);
			self.widgetParam.modes.display.hint = aHint;
		}
		//----- Text Field ----------------
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);	
		aField.text = self.widgetParam.data.value;
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;


		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		// --- view of this mode ---
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});	
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}
		//---  Hint  --------------------
		if(self.widgetParam.hint){
			var aHint = Ti.UI.createLabel(self.widgetParam.hint.style);
			if(!self.widgetParam.data.value||self.widgetParam.data.value==='') {
				aHint.text = self.widgetParam.hint.value;
			}else {
				aHint.text = '';
			}
			self.widgetParam.modes.edit.view.add(aHint);
			self.widgetParam.modes.edit.hint = aHint;
		}
		//---  Text Field  ---------------			
		var aField = Ti.UI.createTextField(self.widgetParam.field.style);
		aField.left = '-10dp';
		aField.right = '-10dp';
		aField.value				= self.widgetParam.data.value;
		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
	
		return self.widgetParam.modes.edit.view;
	},
	
	//----------------  Template/Prototype Methods for Widget object  -------------------
	_setupEventHandlers_ : function(){
		var self = this;
		self.base();

		self.modes.edit.field.addEventListener('blur', (function(that) {
			return (function(evt){
				evt.source.isFocused = false;
				that.doChanged(evt);
			})
		})(self) );
		
		self.modes.edit.field.addEventListener('focus', (function(that) {
			return (function(evt){
				evt.source.isFocused = true;
			})
		})(self) );
		
		self.modes.edit.field.addEventListener('change', (function(that) {
			return (function(e){
				if (!that.modes.edit.field.value){	  
					that.modes.edit.hint.text = that.hint;
				}
				else{
					that.modes.edit.hint.text = ' ';
				}
				if (e.source.isFocused) {
					that.shadow_value = e.value;
				} else {
					that.modes.edit.field.value = that.shadow_value;
				}
			});
		})(self));
	},

	_setValue_ : function(data){
		var self = this;

		var strValue = ('string' == typeof data) ? data : '';
		self.data = {value:strValue};
		if(strValue===''){
			self.modes.edit.hint.text = self.hint;
			self.modes.display.hint.text = self.hint;
		} else {
			self.modes.edit.hint.text ='';
			self.modes.display.hint.text = '';
		}
		self.modes.edit.field.value = strValue;
		self.modes.display.field.text = strValue;
	},
	
	_getValue_ : function(){
		var self = this;
		return self.modes.edit.field.value;
	},

	_setMode_ : function(mode){
		var self = this;

		self.base(mode);
			Ti.API.info('CTKI-self.modes.display.hint.text: '+self.modes.display.hint.text);
			Ti.API.info('CTKI-self.modes.edit.hint.text: '+self.modes.edit.hint.text);
			Ti.API.info('CTKI-self.modes.display.field.text: '+self.modes.display.field.text);
			Ti.API.info('CTKI-self.modes.edit.field.value: '+self.modes.edit.field.value);
		if(self._row){
			switch (mode){
			case 'display':
				self._row.touchEnabled = false;
				self.modes.edit.view.visible = false;
				
				break;
			case 'edit':
				self._row.touchEnabled = true;
				self.modes.edit.field.touchEnabled = true;
				self.modes.display.view.visible = false;
			
				break;
			}
		}
	},
	
	_blur_ : function(){
		var self = this;
		self.modes.edit.field.isFocused = false;
		setTimeout(function() {self.modes.edit.field.blur();}, 100);    //FIXME [ZJ] unexplained as to why....but this is dismissing the keyboard.
	},
	


});

module.exports = textField;

//---  End of file: textField.js
