/**
 * Form Field Factory: webViewField
 */

Ti.include('/globals.js');
var helper = App.loadHelper('default'); //TODO [ZJ]

var webViewField = App.Class(App.WOFieldFactory, {
	className : 'webViewField',
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		data = data || '';
		data = {value:data};
		var widgetParam = self.base(param, data);
		//--- Field-specific Properties in Field Definition ---
		widgetParam.hasSubview = true;
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}
		widgetParam.fieldView	= ('fieldView' in param.properties) ? param.properties.fieldView : {};		// contains {style}

		return widgetParam;
	},
	
	_attachProperties : function(_Widget_){
		var self = this;
		self.base(_Widget_);
		
		_Widget_.label = self.widgetParam.label.value;
		_Widget_.fieldView = self.widgetParam.fieldView;
	},
	
	//----------------  Template/Prototype Methods for Widget object  -------------------
	_setupEventHandlers_ : function(){
		var self = this;
		self.base();

		self.modes.edit.view.addEventListener('click', (function(that){
			return (function(e){
				that.doClick(e);
			});
		})(self) );
		self.modes.display.view.addEventListener('click', (function(that){
			return (function(e){
				that.doClick(e);
			});
		})(self) );
	},
	
	//----------------  Basic Create View for the row  -------------------
	_createView : function(){
		var self = this;

		var view = Ti.UI.createView(self.widgetParam.style);

		//---  Views for each display mode
		for(mode in self.widgetParam.modes){
			var modeView = self._drawView(mode);
			view.add(modeView);
		}
		
		//---  Wrap Main View as a Row if specified
		if(self.widgetParam.useRow){
			self.widgetParam._row = self._createRow();
			if (App.Config.IS_ANDROID) {
				for(mode in self.widgetParam.modes){
					var modeView1 = self._drawView(mode);
					modeView1.left = self.widgetParam.style.left;
					modeView1.right = self.widgetParam.style.right;
					self.widgetParam._row.add(modeView1);
				}
			} else {
				self.widgetParam._row.add(view);			
			}
		}
		return view;
	},

	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		// self.widgetParam.modes.display.view = Ti.UI.createView({
			// layout 	: 'horizontal',
			// width	: '100%',
			// height	: 40,
		// });
		self.widgetParam.modes.display.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});			
		//---  Icon  --------------------
		var icon = Ti.UI.createImageView({
			left: 0, width:'24dp', top: 0, bottom:0,
		});
		icon.image = App.Config.IS_ANDROID ? ('/assets/images/icons/info@2x.png'):('/assets/images/icons/info.png');
		self.widgetParam.modes.display.view.add(icon);

		//---  Label  --------------------
		if(self.widgetParam.label){
			var aLabel = Ti.UI.createLabel(self.widgetParam.label.style);
			aLabel.text = self.widgetParam.label.value;
			self.widgetParam.modes.display.view.add(aLabel);
			self.widgetParam.modes.display.label = aLabel;
		}
		//---  Date Field  ---------------
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		aField.text = 'WebViewFieldData';
		
		if(self.widgetParam.label){
			aField.visible = false;
		} else {
			aField.visible = true;
		}
		
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;
		
		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		//--- view of this mode ---
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});	
		
		//---  Icon  --------------------
		var icon = Ti.UI.createImageView({
			left: 0, width:'24dp', top: 0, bottom:0,
		});
		icon.image = App.Config.IS_ANDROID ? ('/assets/images/icons/info@2x.png'):('/assets/images/icons/info.png');
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}
		//---  Date Field  ---------------		
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		// aField.text = self.widgetParam.data.value.xxx;
		//aField.text = 'WebViewFieldData';
		
		self.widgetParam.modes.edit.view.add(icon);
		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
		
		return self.widgetParam.modes.edit.view;
	},

	//----------------  Template/Prototype Methods for Widget object  -------------------	
	_setValue_ : function(data){
		var self = this;
		self.data.value = data;
	},
	
	_getValue_ : function(){
		var self = this;
		
		if(self.data.value) return self.data.value;
		return null;
	},
	
	_setMode_ : function(mode){
		var self = this;

		self.base(mode);
		self.VIEW.touchEnabled = true;
		
		if(self._row){
			switch (mode){
				case 'display':
					self._row.touchEnabled = true;
					self.modes.display.field.touchEnabled = true;
					break;
				case 'edit':
					self._row.touchEnabled = true;
					self.modes.edit.field.touchEnabled = true;
					break;
			}
		}
	},
	
	_blur_ : function(){
	},

	_renderFieldView_ : function(fieldWindow){
		var self = this;

		var windowTitle = Ti.UI.createLabel(self.fieldView.style.windowTitle);
		if(self.hint) windowTitle.text = self.hint;
		if(self.label) windowTitle.text = self.label;

		fieldWindow.setTitleControl(windowTitle);

		var titlebarShadowContainer = Ti.UI.createView(self.fieldView.style.titlebarShadowContainer);
		var titlebarShadow = Ti.UI.createView(self.fieldView.style.titlebarShadow);
		titlebarShadowContainer.add(titlebarShadow);
		fieldWindow.add(titlebarShadowContainer);

		var webView = Ti.UI.createWebView(self.fieldView.style.view.webViewCanvas);
		
		if(self.data.value.substr(0, 4) == 'http') webView.url = self.data.value;
		else webView.html = self.data.value;
		
		// TODO [ZJ] - if not nav , need to add btnDone button.
		
		// var btnDone = Ti.UI.createButton({
			// systemButton: Ti.UI.iPhone.SystemButton.DONE
		// });
// 
		// btnDone.addEventListener('click', function(evt){
			// self._eventProxy.fireEvent(self._eventProxy.EVENT.closeWindow, {});
		// });
		
		// fieldWindow.setRightNavButton(btnDone);
		fieldWindow.add(webView);
	},
});

module.exports = webViewField;

//---  End of file: webViewField.js