/**
 * Form Field Factory: cityField
 */

Ti.include('/globals.js');
if (App.Config.IS_ANDROID){	Ti.include('/waveo/core/WOAndroidUtils.js');	};
var helper = App.loadHelper('default'); //TODO [ZJ]

var cityField = App.Class(App.WOFieldFactory, {
	className : 'cityField',
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		data = data || [];
		data = {value:data};
		var widgetParam = self.base(param, data);
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.hasSubview = true;
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}
		widgetParam.fieldView	= ('fieldView' in param.properties) ? param.properties.fieldView : {};		// contains {style}
				
		return widgetParam;
	},
	
	_attachProperties : function(_Widget_){
		var self = this;
		self.base(_Widget_);
		
		_Widget_.label = self.widgetParam.label.value;
		_Widget_.fieldView = self.widgetParam.fieldView;
		_Widget_.config = self.widgetParam.config;
	},
	
	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.display.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite', 
		});

		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.display.view.add(label);
		}
		//---  Date Field  ---------------		
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		if(self.widgetParam.data.value) aField.text = '';
		
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;
		
		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		//--- view of this mode ---
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite', 
		});
	
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}
		//---  Date Field  ---------------
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		self.widgetParam.modes.edit.fieldValue = self.widgetParam.data.value;
		var cityNum = self.widgetParam.modes.edit.fieldValue.length;
		if(cityNum) {
			if('undefined' != typeof self.widgetParam.data.value[0].name[App.Config.LANG]) {
				var cityInfo = self.widgetParam.data.value[0].name[App.Config.LANG];
			} else {
				var cityInfo = self.widgetParam.data.value[0].name.en;
			}
			if('undefined' == typeof cityInfo) cityInfo = {};
			switch(self.widgetParam.data.value[0].type) {
				case 'city':
					var cityName = cityInfo.city || '';
					break;
				case 'state':
					var cityName = cityInfo.state || '';
					break;
				case 'country':
					var cityName = cityInfo.country || '';
					break;
				default :
					var cityName = cityInfo.city || '';
			}
			aField.text = cityNum > 1 ? String.format(App.Helper.L('preference_cards_cities'), cityNum): cityName;
		}

		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
		
		return self.widgetParam.modes.edit.view;
	},

	//----------------  Template/Prototype Methods for Widget object  -------------------	
	_setValue_ : function(data){
		var self = this;
		
		self.data.value = data;
		self.modes.edit.fieldValue = data;
		
		var cityNum = data.length;
		if(cityNum) {
			if('undefined' != typeof data[0].name[App.Config.LANG]) {
				var cityInfo = data[0].name[App.Config.LANG];
			} else {
				var cityInfo = data[0].name.en;
			}
			if('undefined' == typeof cityInfo) cityInfo = {};
			switch(data[0].type) {
				case 'city':
					var cityName = cityInfo.city || '';
					break;
				case 'state':
					var cityName = cityInfo.state || '';
					break;
				case 'country':
					var cityName = cityInfo.country || '';
					break;
				default :
					var cityName = cityInfo.city || '';
			}
			
			self.modes.edit.field.text = cityNum>1 ? String.format(App.Helper.L('preference_cards_cities'), cityNum): cityName;
			self.modes.display.field.text = cityNum>1 ? String.format(App.Helper.L('preference_cards_cities'), cityNum): cityName;
		} else {
			self.modes.edit.field.text = '';
			self.modes.display.field.text = '';
		}
	},
	
	_getValue_ : function(){
		var self = this;
		return self.modes.edit.fieldValue;
	},

	_setMode_ : function(mode){
		var self = this;
		self.base(mode);
		
		if(self._row){
			switch (mode){
			case 'display':
				self._row.touchEnabled = false;
				self._row.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.NONE;
				self._row.hasChild = false;
				break;
			case 'edit':
				self._row.touchEnabled = true;
				self._row.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.BLUE;
				self._row.hasChild = true;
				self.modes.edit.field.touchEnabled = true;
				break;
			}
		}
	},

	_renderFieldView_ : function(fieldWindow){
		var self = this;
		
		var windowTitle = Ti.UI.createLabel(self.fieldView.style.windowTitle);
		windowTitle.text =  self.fieldView.title || self.label || self.hint || '';
		
		fieldWindow.setTitleControl(windowTitle);

		var titlebarShadowContainer = Ti.UI.createView(self.fieldView.style.titlebarShadowContainer);
		var titlebarShadow = Ti.UI.createView(self.fieldView.style.titlebarShadow);
		titlebarShadowContainer.add(titlebarShadow);
		fieldWindow.add(titlebarShadowContainer);
		
		var btnCancel = _drawCancelBtn();
		var btnDone = _drawDoneBtn();
 		btnCancel.addEventListener('click', function(evt){
			self._eventProxy.fireEvent(self._eventProxy.EVENT.closeWindow, {});
		});
    	btnDone.addEventListener('click', function(evt){
    		var newCityList = self.cityList.getRowsObjectData();
    		var changed = false;
    		if(newCityList.length == self.data.length) {
    			for(var i=0; i<newCityList.length; i++){
    				if(newCityList[i].id != self.modes.edit.fieldValue[i].id) {
    					changed = true;
    					break;
    				}
    			}
    		} else {
    			changed = true;
    		}
    		
    		if(changed) {
    			self.data.value = newCityList;
	    		self.modes.edit.fieldValue = newCityList;
	    		var cityNum = newCityList.length;
	    		
	    		if(cityNum) {
				    if('undefined' != typeof newCityList[0].name[App.Config.LANG]) {
						var cityInfo = newCityList[0].name[App.Config.LANG];
					} else {
						var cityInfo = newCityList[0].name.en;
					}
					if('undefined' == typeof cityInfo) cityInfo = {};
					switch(newCityList[0].type) {
						case 'city':
							var cityName = cityInfo.city || '';
							break;
						case 'state':
							var cityName = cityInfo.state || '';
							break;
						case 'country':
							var cityName = cityInfo.country || '';
							break;
						default :
							var cityName = cityInfo.city || '';
					}
	    			self.modes.edit.field.text = cityNum>1 ? String.format(App.Helper.L('preference_cards_cities'), cityNum): cityName;
	    		}
	    		else self.modes.edit.field.text = '';

	    		self.doChanged(evt);
    		}
			self._eventProxy.fireEvent(self._eventProxy.EVENT.closeWindow, {});
		});

    	fieldWindow.setRightNavButton(btnDone);
		fieldWindow.setLeftNavButton(btnCancel);

		
		var canvas = Ti.UI.createScrollView(self.fieldView.style.view.canvas);
		fieldWindow.add(canvas);
		
		var lbl_discoverCards  = Ti.UI.createLabel(self.fieldView.style.label.desc);
		lbl_discoverCards.text = App.Helper.L('formfieldwindow_city_desc'),
		canvas.add(lbl_discoverCards);

		self.cityListView = Ti.UI.createView();
		self.cityListViewHeight = ((self.data.value.length)*55+45)+'dp';
		self.cityListView.height = self.cityListViewHeight;
		canvas.add(self.cityListView);
		
		_drawCityAwareView(self.cityListView);	

		var btnAdd = _drawAddCityBtn(canvas);
		btnAdd.addEventListener('click', function(evt){
			btnAdd.backgroundGradient = self.fieldView.style.buttonGradient_mainSelected;
			var maxCity = $.configMgr.get('maxCity', 4);
			if(self.cityList.getRowsLength() >= maxCity){
				$.app.showAlert(String.format(App.Helper.L('ALERT_MAXCITY'), maxCity), App.Helper.L('TITLE_ADDCITY'));
				btnAdd.backgroundGradient = self.fieldView.style.buttonGradient_main;
				return;
			}
			
			var win = Ti.UI.createWindow(self.fieldView.style.window);
			
			var viewParam = {
				eventProxy	: self._eventProxy,
			};
			var citySearch = App.Helper.loadView('discover/CitySearchView', viewParam);
			citySearch.render();
			
			var proxy = citySearch.getEventProxy();
			proxy.addEventListener(proxy.EVENT.doCancel, function(evt){
				win.close({modal:true});
			});
			proxy.addEventListener(proxy.EVENT.rowClicked, function(evt){
				win.close({modal:true});
				var city = evt.row.objectData;
				
				if(self.cityList.DATA.length > 0) {
					
					var index = self.cityList.searchIndex(city.id);
					if(null == index) {
						self.cityList.appendRow(city);					
						self.cityListViewHeight = ((self.cityList.getRowsLength())*55+45)+'dp';
						self.cityListView.setHeight(self.cityListViewHeight);
					}
				} else {
					self.cityList.setData([city]);
					self.cityListViewHeight = ((self.cityList.getRowsLength())*55+45)+'dp';
					self.cityListView.setHeight(self.cityListViewHeight);
				}
			});
			win.add(citySearch.getView());
			win.open({modal:true});
			btnAdd.backgroundGradient = self.fieldView.style.buttonGradient_main;
		});

		
		// _drawLocalAwareView(canvas); //[TODO] 2.3.6
		
		function _drawLocalAwareView(canvas){
			var view = Ti.UI.createView(self.fieldView.style.view.location);

			var location = Ti.UI.createView(self.fieldView.style.view.toggle);
	
			var label = Ti.UI.createLabel(self.fieldView.style.label.location);
			label.text = App.Helper.L('formfieldwindow_city_location');
			
			var switcher = Ti.UI.createSwitch(self.fieldView.style.switcher);
			switcher.value = true;
			
			switcher.addEventListener('change', function(evt){
				alert('Coming soon');
				// Ti.App.fireEvent($.configMgr.EVENT.doGetLocPermission);
				// var geo = App.loadHelper('locationInfo');
				// if(geo.hasPermission()){
					// // Ti.App.addEventListener(geo.EVENT.locationUpdated, _pushLocation);
					// alert('need permission');
					// geo.getLocation();
				// } else {
					// alert('here');
					// geo.getLocation();
				// }
			});

			var desc = Ti.UI.createLabel(self.fieldView.style.label.tips);
			desc.text = App.Helper.L('formfieldwindow_city_locationdesc');
			
			location.add(label);
			location.add(switcher);
			view.add(location);
			view.add(desc);
			
			if(canvas) canvas.add(view);
			return view;
		}
		
		function _drawCityAwareView(canvas){
			//---  Render subview  ---
			var viewParam = {
				DATA		: self.data.value,
				eventProxy	: self._eventProxy,
			};
			self.cityList = App.Helper.loadView('discover/CityList', viewParam);
			var proxy = self.cityList.getEventProxy();
			self.cityList.render();
			
			//---  Set Data & Render Rows  ---
			
			//---  Setup event handlers  ---
			proxy.addEventListener(proxy.EVENT.rowDeleted, function(evt){
				self.cityListViewHeight = ((self.cityList.getRowsLength())*55+45)+'dp';
				setTimeout(function(){
					self.cityListView.setHeight(self.cityListViewHeight);
				}, 100);
			});
			var cityListView = self.cityList.getView();
			
			if(canvas) canvas.add(cityListView);
			return cityListView;
		}

		
		function _drawAddCityBtn(canvas){
			var btnAdd = Ti.UI.createButton(self.fieldView.style.btn.add);
			if(canvas) canvas.add(btnAdd);
			return btnAdd;
		}

		function _drawCancelBtn(canvas){
			var btnCancel = Ti.UI.createButton(self.fieldView.style.systemButton.CANCEL);
			return btnCancel;
		}	
		function _drawDoneBtn(canvas){
			var btnDone = Ti.UI.createButton(self.fieldView.style.systemButton.DONE);
			return btnDone;
		}
	
	},
	
});

module.exports = cityField;

//---  End of file: cityField.js
