/**
 * Form Field Factory: emailDialogField
 */

Ti.include('/globals.js');
var helper = App.loadHelper('default'); //TODO [ZJ]

var emailDialogField = App.Class(App.WOFieldFactory, {
	className : 'emailDialogField',

	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		data = data || null;
		data = {value:data};
		var widgetParam = self.base(param, data);
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.hasSubview = false;
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}
		widgetParam.config  = param.properties.config || {
			subject : null,
			recipients : null,
			message : null,
		};

		return widgetParam;
	},
	
	_attachProperties : function(_Widget_){
		var self = this;
		self.base(_Widget_);
		
		_Widget_.label = self.widgetParam.label.value;
		_Widget_.config = self.widgetParam.config;
	},
	
	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.display.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});	
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.display.view.add(label);
		}
		//---  Date Field  ---------------		
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		if(self.widgetParam.data.value) aField.text = null;
		
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;
		
		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		//--- view of this mode ---
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});	
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}

		//---  Date Field  ---------------
		var aField =  Ti.UI.createEmailDialog();
		aField.setSubject(self.widgetParam.config.subject);
		aField.setToRecipients(self.widgetParam.config.recipients);//['cards@perkd.me']);
		if (App.Config.DEVICE_NAME == 'iPhone OS') {
			aField.setMessageBody(self.widgetParam.config.message);
			aField.setHtml(true);
			aField.setTitle(self.widgetParam.config.subject);
		} else {
			aField.setMessageBody(self.widgetParam.config.message);
		}
		self.widgetParam.modes.edit.field = aField;
		
		self.widgetParam.modes.edit.view.addEventListener('click', function(evt){
			var emailDialog = self.widgetParam.modes.edit.field;
			if (!emailDialog.isSupported()) {
				Ti.UI.createAlertDialog({
					title: App.Helper.L('preference_email_not_avaliable_title'),
					message:App.Helper.L('preference_email_not_avaliable'),
				}).show();
				return;
			}
			else {
				emailDialog.open();
			}
		})
		
		return self.widgetParam.modes.edit.view;
	},

	//----------------  Template/Prototype Methods for Widget object  -------------------	
	_setValue_ : function(data){
		var self = this;
	},
	
	_getValue_ : function(){
		var self = this;
		return undefined;
	},

	_setMode_ : function(mode){
		var self = this;
		self.base(mode);
		
		if(self._row){
			switch (mode){
			case 'display':
				self._row.touchEnabled = false;
				self._row.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.NONE;
				self._row.hasChild = false;
				break;
			case 'edit':
				self._row.touchEnabled = true;
				self._row.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.BLUE;
				self._row.hasChild = true;
				self.modes.edit.field.touchEnabled = true;
				break;
			}
		}
	},
	
	_blur_ : function(){
	},

	_renderFieldView_ : function(fieldWindow){
		var self = this;

		var email = self.modes.edit.field;
		email.open();
		fieldWindow.add(email);
	},
});

module.exports = emailDialogField;

//---  End of file: emailDialogField.js