/**
 * Form Field Factory: emailField
 */

Ti.include('/globals.js');

var emailField = App.Class(App.WOFieldFactory, {
	className : 'emailField',
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		data = {value:data};
		var widgetParam = self.base(param, data);
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}

		return widgetParam;
	},

	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.display.view = Ti.UI.createView(self.widgetParam.style.view_formfield);
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.display.view.add(label);
		};

		//----- Text Field ----------------
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);	
		aField.text = self.widgetParam.data.value;
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;

		//---  Hint Text  --------------------
		var hintStyle = {};
		var hintValue = '';
		if(self.widgetParam.hint&&aField.text===''){
				hintStyle = self.widgetParam.hint.style;
				hintValue = self.widgetParam.hint.value;
		};
		var aHint = Ti.UI.createLabel(hintStyle);	
			aHint.text = hintValue;
		self.widgetParam.modes.display.view.add(aHint);
		self.widgetParam.modes.display.hint = aHint;

		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		// --- view of this mode ---
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});	
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}

		//---  Text Field  ---------------			
		var aField = Ti.UI.createTextField(self.widgetParam.field.style);
		aField.value				= self.widgetParam.data.value;
		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
	
		if(self.widgetParam.data.value===''&&self.widgetParam.hint){
			var hint = Ti.UI.createLabel(self.widgetParam.hint.style);
			hint.text = self.widgetParam.hint.value;
			self.widgetParam.modes.edit.view.add(hint);
		};

		return self.widgetParam.modes.edit.view;
	},
	
	//----------------  Template/Prototype Methods for Widget object  -------------------
	_setupEventHandlers_ : function(){
		var self = this;
		self.base();

		self.modes.edit.field.addEventListener('blur', function(evt){
			self.doChanged(evt);
		});
	},

	_setValue_ : function(data){
		var self = this;

		var strValue = ('string' == typeof data) ? data : '';
		self.data = {value:strValue};
		if(strValue!=''){
			self.modes.display.hint.text = '';
		}
		self.modes.edit.field.value = strValue;
		self.modes.display.field.text = strValue;

	},
	
	_getValue_ : function(){
		var self = this;
		return self.modes.edit.field.value;
	},

	_setMode_ : function(mode){
		var self = this;

		self.base(mode);
		
		if(self._row){
			switch (mode){
			case 'display':
				self._row.touchEnabled = false;
				break;
			case 'edit':
				self._row.touchEnabled = true;
				self.modes.edit.field.touchEnabled = true;
				break;
			}
		}
	},
	
	_blur_ : function(){
		var self = this;
		self.modes.edit.field.blur();
	},

});

module.exports = emailField;

//---  End of file: emailField.js
