/**
 * Form Field Factory: sliderField
 */

Ti.include('/globals.js');
var helper = App.loadHelper('default'); //TODO [ZJ]

var sliderField = App.Class(App.WOFieldFactory, {
	className : 'sliderField',
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		data = data || 0;
		data = {value:data};
		var widgetParam = self.base(param, data);
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.hasSubview = true;
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}
		widgetParam.fieldView	= ('fieldView' in param.properties) ? param.properties.fieldView : {};		// contains {style}
		
		widgetParam.config 	= param.properties.config;
		widgetParam.config.label = widgetParam.config.label || {};
		widgetParam.config.desc = widgetParam.config.desc || {};
		for(var key in widgetParam.config.label) if(!(key in widgetParam.config.desc)) widgetParam.config.desc[key] = widgetParam.config.label[key];

		return widgetParam;
	},
	
	_attachProperties : function(_Widget_){
		var self = this;
		self.base(_Widget_);
		
		_Widget_.label = self.widgetParam.label.value;
		_Widget_.fieldView = self.widgetParam.fieldView;
		_Widget_.config = self.widgetParam.config;
	},
	
	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.display.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.display.view.add(label);
		}
		//---  Slider Field  ---------------		
		var aField = Ti.UI.createLabel();
		var sliderValue = self.widgetParam.data.value;
		if(sliderValue+'' in self.widgetParam.config.label) {
			aField.text = self.widgetParam.config.label[sliderValue+''];
		} else {
			aField.text = sliderValue>1 ? sliderValue+' '+self.widgetParam.config.units : sliderValue+' '+self.widgetParam.config.unit;
		}
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;
		
		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}
		//---  Slider Field  ---------------
		var aField = Ti.UI.createLabel(self.widgetParam.field.style);
		var sliderValue = self.widgetParam.data.value;
		self.widgetParam.modes.edit.fieldValue = sliderValue;
		if(sliderValue+'' in self.widgetParam.config.label) {
			aField.text = self.widgetParam.config.label[sliderValue+''];
		} else {
			aField.text = sliderValue>1 ? sliderValue+' '+self.widgetParam.config.units : sliderValue+' '+self.widgetParam.config.unit;
		}
		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
		
		return self.widgetParam.modes.edit.view;
	},

	//----------------  Template/Prototype Methods for Widget object  -------------------	
	_setValue_ : function(data){
		var self = this;
		
		var sliderValue = data;
		self.data.value = data;
		self.modes.edit.fieldValue = data;
		
		if(sliderValue+'' in self.config.label) {
			self.modes.edit.field.text = self.config.label[sliderValue+''];
			self.modes.display.field.text = self.config.label[sliderValue+''];
		} else {
			self.modes.edit.field.text = data>1 ? data+' '+self.config.units : data+' '+self.config.unit;
			self.modes.display.field.text = data>1 ? data+' '+self.config.units : data+' '+self.config.unit;
		}
	},
	
	_getValue_ : function(){
		var self = this;
		return self.modes.edit.fieldValue;
	},

	_blur_ : function(){
	},

	_renderFieldView_ : function(fieldWindow){
		var self = this;	

		var windowTitle = Ti.UI.createLabel(self.fieldView.style.windowTitle);
		windowTitle.text =  self.fieldView.title || self.label || self.hint || '';

		//App.Config.IS_ANDROID ? fieldWindow.add(windowTitle) : fieldWindow.setTitleControl(windowTitle) ;
		fieldWindow.setTitleControl(windowTitle);

		var titlebarShadowContainer = Ti.UI.createView(self.fieldView.style.titlebarShadowContainer);
		var titlebarShadow = Ti.UI.createView(self.fieldView.style.titlebarShadow);
		titlebarShadowContainer.add(titlebarShadow);
		fieldWindow.add(titlebarShadowContainer);
					
		var btnDone = Ti.UI.createButton(self.fieldView.style.systemButton.DONE);
		var btnCancel = Ti.UI.createButton(self.fieldView.style.systemButton.CANCEL);
		
		var canvas = Ti.UI.createView(self.fieldView.style.view.canvas);
		fieldWindow.add(canvas);

		var lbl_desc = Ti.UI.createLabel(self.fieldView.style.label.desc);		
		lbl_desc.text = self.fieldView.label;

		var lbl_data = Ti.UI.createLabel(self.fieldView.style.label.data);

		var lbl_tip = Ti.UI.createLabel(self.fieldView.style.label.tip);
		lbl_tip.text = self.label;
		
		var slider = Ti.UI.createSlider(self.fieldView.style.slider);
			
		canvas.add(lbl_desc);
		canvas.add(lbl_data);
		canvas.add(lbl_tip);
		canvas.add(slider);
		
		slider.min = self.config.min;
		slider.max = self.config.max;
		slider.value = self.modes.edit.fieldValue;
		
		slider.addEventListener('change', function(evt){
			var value = Math.round(slider.value);
			if(value+'' in self.config.desc) 
				lbl_data.text = self.config.desc[value+''];
			else 
				lbl_data.text = value>1 ? value+' '+self.config.units : value+' '+self.config.unit;
			
			if(value+'' in self.config.tips)
				lbl_tip.text = self.config.tips[value+''];
			else 
				lbl_tip.text = self.config.tip;
		});

		btnDone.addEventListener('click', function(evt){
			var value = Math.round(slider.value);
			self.modes.edit.fieldValue = value;
			if(value+'' in self.config.label) 
				self.modes.edit.field.text = self.config.label[value+''];
			else
				self.modes.edit.field.text = value>1 ? value+' '+self.config.units : value+' '+self.config.unit;
				
			self.doChanged(evt);
			self._eventProxy.fireEvent(self._eventProxy.EVENT.closeWindow, {});
		});
		btnCancel.addEventListener('click', function(evt){
			self._eventProxy.fireEvent(self._eventProxy.EVENT.closeWindow, {});
		});

		fieldWindow.setRightNavButton(btnDone);
		fieldWindow.setLeftNavButton(btnCancel);
		fieldWindow.add(canvas);

	},
});

module.exports = sliderField;

//---  End of file: sliderField.js