/**
 * Form Field Factory: switchField
 */

Ti.include('/globals.js');

var switchField = App.Class(App.WOFieldFactory, {
	className : 'switchField',
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		data = data || false;
		data = {value:data};
		var widgetParam = self.base(param, data);
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}
		
		return widgetParam;
	},
	
	_attachProperties : function(_Widget_){
		var self = this;
		self.base(_Widget_);
		
		_Widget_.label = self.widgetParam.label.value;
	},
	
	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.display.view = Ti.UI.createView({
			width: '100%',
			height: 40,
		});
		
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.display.view.add(label);
		}
		
		var aField = Ti.UI.createSwitch();
		aField.value = self.widgetParam.data.value;
		
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;
		
		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			width: '100%',
			height: 40,
		});
		
		//---  Label  --------------------
		if(self.widgetParam.label){
			var label = Ti.UI.createLabel(self.widgetParam.label.style);
			label.text = self.widgetParam.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}
		
		var aField = Ti.UI.createSwitch({
			right : 5,
		});
		aField.value = self.widgetParam.data.value;
		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
		
		return self.widgetParam.modes.edit.view;
	},
	
	//----------------  Template/Prototype Methods for Widget object  -------------------
	_setupEventHandlers_ : function(){
		var self = this;
		self.base();

		self.modes.edit.field.addEventListener('change', function(evt){
			self.doChanged(evt);
		});
	},
	
	_setValue_ : function(data){
		var self = this;
		self.data = {value:data};
		self.modes.edit.field.value = data;
		self.modes.display.field.text = data;
	},
	
	_getValue_ : function(){
		var self = this;
		return self.modes.edit.field.value;
	},

	_setMode_ : function(mode){
		var self = this;

		self.base(mode);
		
		if(self._row){
			switch (mode){
			case 'display':
				self._row.touchEnabled = false;
				break;
			case 'edit':
				self._row.touchEnabled = true;
				self.modes.edit.field.touchEnabled = true;
				break;
			}
		}
	},
	
	_blur_ : function(){
		var self = this;
	},

});

module.exports = switchField;

//---  End of file: switchField.js
