/**
 * Form Field Factory: tabbedBarField
 */

Ti.include('/globals.js');
if (App.Config.IS_ANDROID){
Ti.include('/waveo/core/WOAndroidUtils.js');	
}

var tabbedBarField = App.Class(App.WOFieldFactory, {
	className : 'tabbedBarField',
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},

	_getDefinition : function(param, data){
		var self = this;
		
		data = data || param.properties.config.defaultValue;
		data = {value:data};
		var widgetParam = self.base(param, data);
		
		var index = 0;
		widgetParam.properties.config.labels = [];
		widgetParam.properties.config.values = {};
		for(var label in widgetParam.properties.config.options){
			widgetParam.properties.config.labels.push(App.Helper.L(label, 'fieldLabel'));
			widgetParam.properties.config.values[index++] = widgetParam.properties.config.options[label];
		}
		
		//--- Field-specific Properties in Field Definition ---
		widgetParam.hasSubview = false;
		widgetParam.label 	= ('label' in param.properties) ? param.properties.label : {};		// contains {value} + {style}
		widgetParam.hint	= ('hint' in param.properties) ? param.properties.hint : {};		// contains {value} + {style}
		widgetParam.field	= ('field' in param.properties) ? param.properties.field : {};		// contains {style}

		return widgetParam;
	},

	_attachProperties : function(_Widget_){
		var self = this;
		self.base(_Widget_);
		
		_Widget_.config = self.widgetParam.properties.config;
	},

	//-----------  Render 'display' mode  -----------
	_draw_displayView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.display.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});
		//---  Label  --------------------
		if(self.widgetParam.properties.label){
			var label = Ti.UI.createLabel(self.widgetParam.properties.label.style);
			label.text = self.widgetParam.properties.label.value;
			self.widgetParam.modes.display.view.add(label);
		}
		
		for(var index in self.widgetParam.properties.config.values) 
		if(self.widgetParam.properties.config.values[index] == self.widgetParam.data.value) break;
		
		var aField = Ti.UI.createLabel(self.widgetParam.properties.field.style);

		aField.text = self.widgetParam.properties.config.labels[index];
	//	aField.index = 0;
		self.widgetParam.modes.display.view.add(aField);
		self.widgetParam.modes.display.field = aField;

		return self.widgetParam.modes.display.view;
	},

	//-----------  Render 'edit' mode  -----------
	_draw_editView : function(){
		var self = this;
		
		//--- view of this mode ---	
		self.widgetParam.modes.edit.view = Ti.UI.createView({
			left: 0, right:0, top: 0, bottom:0, 
			layout: 'composite',
		});		//---  Label  --------------------
		if(self.widgetParam.properties.label){
			var label = Ti.UI.createLabel(self.widgetParam.properties.label.style);
			label.text = self.widgetParam.properties.label.value;
			self.widgetParam.modes.edit.view.add(label);
		}
		for(var index in self.widgetParam.properties.config.values) {
			if(self.widgetParam.properties.config.values[index] == self.widgetParam.data.value) break;
		}

		if (App.Config.IS_ANDROID) {
			var _theStyle = App.Helper.clone (self.widgetParam.properties.config.style);
			_theStyle.labels = self.widgetParam.properties.config.labels;
			_theStyle.index = index;
			var aField = createAndroidTabbedBar(_theStyle);
		} else {
			var aField = Titanium.UI.iOS.createTabbedBar(self.widgetParam.properties.config.style);
			aField.labels = self.widgetParam.properties.config.labels;
			aField.index = index;
		}

		self.widgetParam.modes.edit.view.add(aField);
		self.widgetParam.modes.edit.field = aField;
		
		return self.widgetParam.modes.edit.view;
	},
	
	//----------------  Template/Prototype Methods for Widget object  -------------------
	_setupEventHandlers_ : function(){
		var self = this;
		self.base();

		self.modes.edit.field.addEventListener('click', function(evt){
			self.doChanged(evt);
		});
	},

	_setValue_ : function(data){
		var self = this;
		
		for(var index in self.config.values) if(self.config.values[index] == data) break;
		self.data = {value:data};	
		self.modes.edit.field.index = index;
		self.modes.display.field.labels = [self.config.labels[index]];
		self.modes.display.field.text = self.config.labels[index];
	},
	
	_getValue_ : function(){
		var self = this;
		return self.config.values[self.modes.edit.field.index];
	},

});

module.exports = tabbedBarField;

//---  End of file: tabbedBarField.js