/**
 * Produces Form Objects from: form definition + form data + styles
 * 
 * @author zhoujun@waveo.com
 */
Ti.include('/globals.js');
if (App.Config.IS_ANDROID) {
Ti.include('/waveo/core/WOAndroidUtils.js');
}

var CardIntroForm = App.Class(App.WOForm, {
	className : 'CardIntroForm',
	
	getHeaderView : function(header){
		var self = this;

		var headerView = Ti.UI.createView(self.STYLE.view_formHeader);

		var signupTopView = self._addSignupTopView(headerView);
		self._addCardFace(signupTopView);
		self._addCardName(signupTopView);
		
		//---  Sign-up button  ---------
		var btn_signup = self._addCardAddButton(signupTopView);
		var btn_signup_label = self._addCardAddButtonLabel(signupTopView);
		
		btn_signup.addEventListener('touchstart', function(){
			btn_signup.backgroundGradient = self.STYLE.buttonSelectGradient;
		});
		btn_signup.addEventListener('touchend', function(){
			btn_signup.backgroundGradient=self.STYLE.buttonGradient;
		});	

		//---  Description  ------------
		if(self.DATA.cardMaster.desc.slice(0, 20).indexOf('<!DOCTYPE') > -1) { // ok, webview

			var descView = Ti.UI.createWebView(self.STYLE.webView.cardDesc_android);
			descView.html	= self.DATA.cardMaster.desc;

			if(!App.Config.IS_ANDROID){
	
				headerView.updateLayout({
					height: (parseInt(signupTopView.height) + parseInt(signupTopView.top) + parseInt(signupTopView.bottom) )+'dp' ,
				});
				
				var descView = self.getWebView({
				html		: self.DATA.cardMaster.desc,
				style		: self.STYLE.webView.cardDesc,
				callback	: function(evt){
							var newHeight = (parseInt(headerView.height)+evt.height)+'dp';
							headerView.updateLayout({
								height: newHeight,
							});
					}			
				});
			}
		} else {
				var descView = (App.Config.IS_ANDROID)? Ti.UI.createLabel(self.STYLE.label.cardDesc_android) : Ti.UI.createLabel(self.STYLE.label.cardDesc);
				descView.text	= self.DATA.cardMaster.desc;
		}
				
		if(descView) headerView.add(descView);
	
		if(self.DATA.cardMaster.isDigital && self.DATA.authState.state != App.Const.AUTH_LOGGED_IN) {
			btn_signup.addEventListener('click', function(evt){
				if (App.Config.IS_ANDROID) {
					if (checkIfButtonWasClickedThenDisableForShortPeriod
						(evt.source,1000)) {
							return;
						}
				}
			
				Ti.App.fireEvent(self._eventProxy.EVENT.goPerkdIDView);
			});
		} else {
			btn_signup.addEventListener('click', function(evt){
				if (App.Config.IS_ANDROID) {
					if (checkIfButtonWasClickedThenDisableForShortPeriod
						(evt.source,1000)) {
							return;
						}
					disableButtonUntilEventOccurs(evt.source,'SIGNUP_BUTTON_RESET_EVENT');
				}
			
				self._eventProxy.fireEvent(self._eventProxy.EVENT.addCard, {});
			});
		}

		return headerView;
	},

	//--- Draw UI Components ---
	_addSignupTopView : function(theView){
		var self = this;
		
		var signupView = Ti.UI.createView(self.STYLE.view.cardSinup);
		if(theView) theView.add(signupView);
		return signupView;
	},
	
	_addCardFace : function(theView){
		var self = this;
		
		//--- Card Definition ---------
		var cardDefn = {
			name		: self.DATA.cardMaster.name,
			style		: self.STYLE.view.cardFace,
			properties 	: {
				cardObj			: self.DATA.cardMaster,
				format			: $.cardMgr.factory.FORMAT.cardIntro,
				cardFront		: true,
				shadow			: true,
			},
		};
		var cardData = {};

		var card = $.cardMgr.factory.make(cardDefn, cardData);
		card.setMode('display');
		var _cardFace = card.getView();
		
		self.CARD = card;
		if(theView) theView.add(_cardFace);
		return _cardFace;
	},
	
	_addCardName : function(theView){
		var self = this;
		
		var view_name = Ti.UI.createView(self.STYLE.view.cardName);		
		var lbl_name = Ti.UI.createLabel(self.STYLE.label.cardName);
		lbl_name.text = self.DATA.cardMaster.name;
		view_name.add(lbl_name);
		if(theView) theView.add(view_name);
		return view_name;
	},
	
	_addCardAddButtonLabel : function(theView){
		var self = this;

		var btn_signup_label = Ti.UI.createLabel(self.STYLE.btn.signupLabel);
		
		var buttonLabel = self.DATA.cardMaster.isDigital ? App.Helper.L('BUTTON_SIGNUP').toUpperCase() : App.Helper.L('BUTTON_ADD').toUpperCase() ;
		btn_signup_label.setText(buttonLabel);

				
		if(theView) theView.add(btn_signup_label);
		return btn_signup_label;
	},
	
	_addCardAddButton : function(theView){
		var self = this;

		var btn_signup = Ti.UI.createButton(self.STYLE.btn.signup);
		
		if(theView) theView.add(btn_signup);
		return btn_signup;
	},


});

module.exports = CardIntroForm;

//---  End of file: CardIntroForm.js