/**
 * AccountWindow
 */
Ti.include('/globals.js');
if (App.Config.IS_ANDROID) {
	Ti.include('/waveo/core/WOAndroidUtils.js');
}

var AccountWindow = App.Class(App.View.WOWindow, {
	className 	: 'AccountWindow',
	stylePath 	: 'AccountMgtStyle',

	//--- Event Name constants -----
	EVENT		: {
		//--- Controller to handle:
		logout			: 'AccountWindow.logout',
    },
    
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
		
		self.WINDOW.hideTabBar = true;		
		self.WINDOW.title = App.Helper.L('win_title_account');
	},
	
	draw : function(param){
		var self = this;
		App.Helper.requireModel('WOSettings');
		
		self.goNextpage = false;

		var titlebarShadowContainer = Ti.UI.createView(self.STYLE.titlebarShadowContainer);
		var titlebarShadow = Ti.UI.createView(self.STYLE.titlebarShadow);
		titlebarShadowContainer.add(titlebarShadow);
		
		self.VIEW.add(titlebarShadowContainer);
			
		var canvas = Ti.UI.createView(self.STYLE.view.canvas);
		
		//---  Mobile Number  -------------------
		var lbl_registerNo = Ti.UI.createLabel(self.STYLE.label.mobileLabel);
		lbl_registerNo.text = App.Helper.L('account_registeredMobileNumber');
		
		var account = new App.Model.WOSettings('account');
		account.fetch();

		var lbl_mobile = Ti.UI.createLabel(self.STYLE.label.mobileNo);
        lbl_mobile.text = account.data.mobileNo;

		canvas.add(lbl_registerNo);
	 	canvas.add(lbl_mobile);	
	

		//---  The Form  ------------------------
		var tableView_manage = Ti.UI.createTableView(self.STYLE.tableView);

		var row_updatePassword = Ti.UI.createTableViewRow(self.STYLE.tableRow);
		
		var rowLabel_updatePassword = Ti.UI.createLabel(self.STYLE.label.manageAction);
		rowLabel_updatePassword.text = App.Helper.L('account_changePassword');

		row_updatePassword.add(rowLabel_updatePassword);
		row_updatePassword.addEventListener('click', function(){
			self.goNextpage = true;
			$.app.route({
				url			: 'account/updatePassword',
				tabGroupObj	: self.tabGroupObj,
			});
		});
		
		tableView_manage.appendRow(row_updatePassword);	
		
		var row_updateMobile =  Ti.UI.createTableViewRow(self.STYLE.tableRow);
		
		var rowLabel_updateMobile = Ti.UI.createLabel(self.STYLE.label.manageAction);
		rowLabel_updateMobile.text = App.Helper.L('account_updateMobileNumber');

		row_updateMobile.add(rowLabel_updateMobile);
		row_updateMobile.addEventListener('click', function(){
			self.goNextpage = true;
			var winObj = $.app.route({
				url			: 'account/updateMobile',
				tabGroupObj	: self.tabGroupObj,
			});
			var proxy = winObj.getEventProxy();
			proxy.addEventListener(proxy.EVENT.mobileVerified, function(evt){
				lbl_mobile.text = evt.mobileNo;
			});
		});
		
		tableView_manage.appendRow(row_updateMobile);
		
		canvas.add(tableView_manage);
		
		//---  Description  ----------------------
		var label_desc = Ti.UI.createLabel(self.STYLE.label.tips);
		label_desc.text = App.Helper.L('account_mobileRegisterDesc');
		canvas.add(label_desc);

		//---  Logout Button  --------------------
		var flexFooter = Ti.UI.createView(self.STYLE.view.flexFooter);

		self.btnLogout = Ti.UI.createButton(self.STYLE.btn.logout);
		self.btnLogout.title = App.Helper.L('BUTTON_LOGOUT');
	
		self.btnLogout.addEventListener('touchstart', function(){
			self.btnLogout.backgroundGradient = self.STYLE.buttonSelectGradient;
		});
		self.btnLogout.addEventListener('touchend',  function(){
			self.btnLogout.backgroundGradient = self.STYLE.buttonGradient;
		});
		self.btnLogout.addEventListener('click', _doLogout);
		
		flexFooter.add(self.btnLogout);
		canvas.add(flexFooter);
		
		self.VIEW.add(canvas);
		
		self.VIEW.addEventListener('blur', function(e){
			if(!self.goNextpage && self.VIEW && !App.Config.IS_ANDROID){
					titlebarShadowContainer.hide();
					self.VIEW.backgroundImage = null; 
					self.VIEW.backgroundColor = 'transparent';
			}
			self.goNextpage = false;
		});
		
		if(App.Config.IS_ANDROID) {
			self.btnLogout.visible = true;
			self.btnLogout.borderColor = 'transparent';
		}

		//---------------------  Controller Event Handlers  ----------------------	
		function _doLogout(evt){

			if (App.Config.IS_ANDROID) {
				if (checkIfButtonWasClickedThenDisableForShortPeriod
					(evt.source,1000)) {
						return;
					}
			}

            var dialog = Ti.UI.createAlertDialog({
                message		: App.Helper.L('logout_alert'),
                title		: App.Helper.L('TITLE_CONFIRM'),
                buttonNames	: [App.Helper.L('BUTTON_CANCEL'), App.Helper.L('BUTTON_LOGOUT')],
            });
            dialog.addEventListener('click', function(e){
                if(e.index ==1){
					//---  Call Controller  ---
					self.goNextpage = true;
					self._eventProxy.fireEvent(self.EVENT.logout, {});
                }
//                evt.source.clicked = false;
            });
            dialog.show();
        };
	},
	
	dispose : function(){
		var self = this;
		self.base();
		
		// $.app.viewingAccount = false;
	}
	
});

module.exports = AccountWindow;

//---  End of file: AccountWindow.js