/**
 * CardListWindow
 */
Ti.include('/globals.js');
if (App.Config.IS_ANDROID){
	Ti.include('/waveo/core/WOAndroidUtils.js');	
}

var CardListWindow = App.Class(App.View.WOWindow, {
	className 	: 'CardListWindow',
	stylePath 	: 'CardListStyle',
	
	EVENT		: {
		//--- Handles these events:
		doFlipCard		: 'WOCardFactory.doFlipCard',
		//--- Produces these events:
		goPerkdIDView	: 'Nav.goPerkdIDView',
		addCard			: 'CardListWindow.addCard',
		viewCard		: 'CardListWindow.viewCard',
		deleteCard		: 'CardListWindow.deleteCard',
		moveCard        : 'CardListWindow.moveCard',
		fetchData		: 'CardListWindow.fetchData',
		
		orderList		: 'CardListWindow.orderList',
		cardListRendered: 'CardListWindow.cardListRendered',
    },

    SUBVIEWS		: [
    	{
    		name		: 'card',
    		viewFile	: App.Config.IS_ANDROID?'components/CardListViewBigAndroid':'components/CardListViewBig2',
    	},
    	{
    		name		: 'list',
    		viewFile	: App.Config.IS_ANDROID? 'components/CardListView' : 'components/CardListView2',
    	},
    ],
    
	PROPERTY	: {
		cardView		: 'custom_cardView',
	},
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);

		self.lazyRender = true;
		self.settingListData = true;

		if (!App.Config.IS_ANDROID) {
			self.WINDOW.hideNavBar = true;
		}
		//---  Initialize subviews  ---
		self.subviewList = [];
		for(var i=0; i<self.SUBVIEWS.length; i++){
			self.subviewList[i] = {
				name	: self.SUBVIEWS[i].name,
				view	: null,
			};
		}
		//---  Determine default subview  ---
		if(null == Ti.App.Properties.getInt(self.PROPERTY.cardView)){
			Ti.App.Properties.setInt(self.PROPERTY.cardView, 0);
		}
		self.currentView = Ti.App.Properties.getInt(self.PROPERTY.cardView);
	},
	
	preDraw : function(param){
		var self = this;
		self.base(param);
		
		self.canvas = Ti.UI.createView(self.STYLE.view.canvas);

		
		//---  Title bar  ----------
		//---  Logo  ---------------
		self.btn_Perkd = Ti.UI.createView(self.STYLE.perkd_btn);		
		
		if (App.Config.IS_ANDROID) {
			self.VIEW.setLeftNavButton(self.btn_Perkd);
		} else {
			self.customTitlebar = Ti.UI.createView(self.STYLE.titlebar);
			self.canvas.add(self.customTitlebar);

			self.customTitlebar.add(self.btn_Perkd);			
		}

		self.titlebarShadowContainer = Ti.UI.createView(self.STYLE.titlebarShadowContainer);
		self.titlebarShadow = Ti.UI.createView(self.STYLE.titlebarShadow);
		self.titlebarShadowContainer.add(self.titlebarShadow);
		self.canvas.add(self.titlebarShadowContainer);
		
		self.tableViewCanvas = Ti.UI.createView(self.STYLE.view.tableViewCanvas);
		self.canvas.add(self.tableViewCanvas);

		self.VIEW.add(self.canvas);
	},
	
	draw_test : function(){
		var self = this;
		
		//---  Perkd Button  ---------
		self.btn_Perkd.addEventListener('click', function(){
			self._eventProxy.fireEvent(self.EVENT.goPerkdIDView, {});
		});		
		self.btnAdd = Ti.UI.createButtonBar(self.STYLE.btn.add);
		self.customTitlebar.add(self.btnAdd);
		self.btnAdd.addEventListener('click', function(e){
			if (e.index==undefined||e.index==null) return; // Quick fix for Android multiple event on single click. Does not happen on iphone			
			self._eventProxy.fireEvent(self.EVENT.addCard, {});
		});
		// var view1 = Ti.UI.createView({
			// backgroundColor : 'red',
		// });
		// var view2 = Ti.UI.createView({
			// backgroundColor : 'green',
		// });
		
		var viewParam = {
				DATA		: self.DATA.cards,
				eventProxy	: self._eventProxy,
			};
		var newSubView1 = App.Helper.loadView(self.SUBVIEWS[0].viewFile, viewParam);
		var view1 = newSubView1.render();
		// newSubView1.setData(self.DATA.cards);
		var newSubView2 = App.Helper.loadView(self.SUBVIEWS[1].viewFile, viewParam);
		var view2 = newSubView2.render();
		// newSubView2.setData(self.DATA.cards);
		
		//---  Setup event handlers  ---
		self._eventProxy.addEventListener(newSubView1.EVENT.rowClicked, function(evt){
			self._eventProxy.fireEvent(self.EVENT.viewCard, {cardID: evt.data.cardID});
		});
		self._eventProxy.addEventListener(newSubView1.EVENT.rowDeleted, function(evt){
			self._eventProxy.fireEvent(self.EVENT.deleteCard, {cardID: evt.data.cardID});
		});
		self._eventProxy.addEventListener(newSubView1.EVENT.rowMoved, function(evt){
			self.moveCard({
				cardID: evt.data.cardID,
				fromIndex : evt.fromIndex,
				toIndex : evt.index,
			});
		});
		//---  Setup event handlers  ---
		self._eventProxy.addEventListener(newSubView2.EVENT.rowClicked, function(evt){
			self._eventProxy.fireEvent(self.EVENT.viewCard, {cardID: evt.data.cardID});
		});
		self._eventProxy.addEventListener(newSubView2.EVENT.rowDeleted, function(evt){
			self._eventProxy.fireEvent(self.EVENT.deleteCard, {cardID: evt.data.cardID});
		});
		self._eventProxy.addEventListener(newSubView2.EVENT.rowMoved, function(evt){
			self.moveCard({
				cardID: evt.source.cardID,
				fromIndex : evt.fromIndex,
				toIndex : evt.index,
			});
		});
		
		self.subviewList[0].view = newSubView1;
		self.subviewList[1].view = newSubView2;
		
		var scrollableView = Ti.UI.createScrollableView({
		  views:[view1,view2],
		  // showPagingControl:true,
		  // pagingControlColor : '00000000',
		});
		// scrollableView.addEventListener('scrollEnd', _scrollEndHandler);
		
		// view1.setBackgroundColor('red');		
		
		self.tableViewCanvas.add(scrollableView);
		
		// function _scrollEndHandler(evt){
			// scrollableView.removeEventListener('scrollEnd', _scrollEndHandler);
			// scrollableView.setShowPagingControl(false);
		// }
	},
	
	draw : function(){
		var self = this;
		
		//---  Perkd Button  ---------
		self.btn_Perkd.addEventListener('click', function(){
			self._eventProxy.fireEvent(self.EVENT.goPerkdIDView, {});
		});
		
		//---  Add Card button  ------
		if(App.Config.IS_ANDROID){
			self.btnAdd = createAndroidTabbedBar(self.STYLE.btn.add_android);
			self.btnAdd.setIndex (0);
			self.VIEW.setRightNavButton(self.btnAdd);			
		}else{
			self.btnAdd = Ti.UI.createButtonBar(self.STYLE.btn.add);
			self.customTitlebar.add(self.btnAdd);
		}
		self.btnAdd.addEventListener('click', function(e){
			if (e.index==undefined||e.index==null) return; // Quick fix for Android multiple event on single click. Does not happen on iphone			
			self._eventProxy.fireEvent(self.EVENT.addCard, {});
		});

		//---  Switch View buttons  ------		
		if(App.Config.IS_ANDROID){
			self.btnbarSwitch = createAndroidTabbedBar(self.STYLE.tabbedBar.cardListSwitcher_android);
			self.btnbarSwitch.setIndex (self.currentView);
			self.VIEW.setTitleControl(self.btnbarSwitch);
			self.VIEW.showNavBar();
		}else{
			self.btnbarSwitch = Ti.UI.iOS.createTabbedBar(self.STYLE.tabbedBar.cardListSwitcher);
			self.btnbarSwitch.index = self.currentView;
			self.customTitlebar.add(self.btnbarSwitch);			
		}

		self.btnbarSwitch.addEventListener('click', function(evt){
			if (evt.index==undefined||evt.index==null) return; // Quick fix for Android multiple event on single click. Does not happen on iphone
			Ti.API.info('CTK: --- '+evt.index);
			self.currentView = evt.index;
			self.switchView(evt.index);
		});
			
		//---  Display Current View  ---
		self.switchView(self.currentView);
		//self.switchView(0);
	},
	
	switchView : function(toSubView){
		var self = this;

		//---  Load subview if necessary  ---
		if(null == self.subviewList[toSubView].view){
			var viewParam = {
				DATA		: self.DATA.cards,
				eventProxy	: self._eventProxy,
			};
			var newSubView = App.Helper.loadView(self.SUBVIEWS[toSubView].viewFile, viewParam);
			var _theView = newSubView.render();
			
			//---  Add subview to our container
			self.tableViewCanvas.add(_theView);
			
			self.subviewList[toSubView].view = newSubView;
			
			if(App.Config.IS_ANDROID){
				setTimeout(function(e){
					self.subviewList[toSubView].view.setData(self.DATA.cards);
				}, 300);
			}

			//---  Setup event handlers  ---
			self._eventProxy.addEventListener(newSubView.EVENT.rowClicked, function(evt){
				self._eventProxy.fireEvent(self.EVENT.viewCard, {cardID: evt.data.cardID});
			});
			self._eventProxy.addEventListener(newSubView.EVENT.rowDeleted, function(evt){
				self._eventProxy.fireEvent(self.EVENT.deleteCard, {cardID: evt.data.cardID});
			});
			self._eventProxy.addEventListener(newSubView.EVENT.rowMoved, function(evt){
				self.moveCard({
					cardID: evt.rowData.objectID,
					fromIndex : evt.fromIndex,
					toIndex : evt.index,
				});
			});
			
			self._eventProxy.addEventListener(newSubView.EVENT.orderCards, function(evt){
				// alert('order?');
				self._eventProxy.fireEvent(self.EVENT.orderList, evt);
				// if(self.subviewList[0].view) self.subviewList[0].view.setData();
			});
			
			self._eventProxy.addEventListener(newSubView.EVENT.listRendered, function(evt){
				self._eventProxy.fireEvent(self.EVENT.cardListRendered, {});
			});
			
			// self._eventProxy.addEventListener('9999', function(evt){alert('9999');
				// if(self.subviewList[0].view) self.subviewList[0].view.setData();
				// if(self.subviewList[1].view) self.subviewList[1].view.setData();
			// });

			// self._eventProxy.fireEvent(self.EVENT.fetchData, {});
		}

		//---  Hide non-current subviews  ---
		for(var i=0; i<self.subviewList.length; i++){
			if(self.subviewList[i].view){
				self.subviewList[i].view.hide();
			}
		}
		//---  Show Current Subview  ---
		self.currentView = toSubView;
		self.subviewList[self.currentView].view.show();
		Ti.App.Properties.setInt(self.PROPERTY.cardView, self.currentView);
	},
	
	addCard : function(theCard){
		var self = this;
		
		self.DATA.cards.unshift(theCard);
		for(var i=0; i<self.subviewList.length; i++){
			if(self.subviewList[i].view){
				self.subviewList[i].view.insertRow(theCard, 0);
				self.subviewList[i].view.scrollToTop(0);
			}
		}
	},
	
	deleteCard : function(param){
		var self = this;
		for(var i=0; i<self.DATA.cards.length; i++){
			if(self.DATA.cards[i].cardID == param.cardID) {
				self.DATA.cards.splice(i, 1);
				break;
			}
		}
		for(var i=0; i<self.subviewList.length; i++){
			if(self.subviewList[i].view){
				var index = self.subviewList[i].view.searchIndex(param.cardID);
				self.subviewList[i].view.removeRow(index);
			}
		}
	},
	
	moveCard : function(param){
		var self = this;
		if(self.subviewList[0].view) self.subviewList[0].view.moveRow(param.fromIndex, param.toIndex);

		var card = self.DATA.cards.splice(param.fromIndex, 1);
		self.DATA.cards.splice(param.toIndex, 0, card[0]);
	},
	
	setListData : function(param){
		var self = this;
		
		self.settingListData = true;
		self.DATA.cards = param.cards;
		for(var i=0; i<self.subviewList.length; i++){
			if(self.subviewList[i].view){
				self.subviewList[i].view.setData(param.cards);				
			}
		}
		self.settingListData = false;
		
		setTimeout(function(e){
			Ti.API.error('setListData... _doMessages()');
            _doMessages();
        }, 3000);
	},
	
	updateList : function(param){
		var self = this;

		for(var i=0; i<self.subviewList.length; i++){
			if(self.subviewList[i].view){
				self.subviewList[i].view.updateRow(param.cards);
			}
		}
	},
	
	onFocus : function(evt){
		var self = this;
		self.base(evt);
				
		if(!self.settingListData) {
			Ti.API.error('CardListWindow...On Focus _doMessages()');
			_doMessages();
		}
	},
	
});

function _doMessages() {
	$.app.doNotDisturb(false); //FIXME [ZJ] : temporary solution v2.3.1
	$.notifyMgr.isActive = true;
	$.app.hideBusy();
 	Ti.App.fireEvent($.notifyMgr.EVENT.doMessages);
}

module.exports = CardListWindow;

//---  End of file: CardListWindow.js
