/**
 * Restoring Cards Window
 */
Ti.include('/globals.js');

var RestoreCard = App.Class(App.View.WOWindow, {
	className : 'RestoreCard',
	stylePath : 'AuthStyle',

	EVENT		: {
		//--- Handles these events:
		//--- Produces these events:
		cardsRestored	: 'RestoreCard.cardsRestored',
    },

	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
		
		self.WINDOW.hideNavBar = true;
	},

	draw : function(param){
		var self = this;
		self.base(param);
		
		self.startLoading();

        //----  Display task title  ----
		self.L_Restore = Ti.UI.createLabel(self.STYLE.Restore.text);
		self.L_Restore.text = App.Helper.L('auth_SETTINGLIBRARY');
		self.VIEW.add(self.L_Restore);

        //----  Sync Cards  ----
        Ti.App.addEventListener($.syncMgr.EVENT.syncDone, _syncImages);
        Ti.App.fireEvent($.syncMgr.EVENT.doSyncObjects);		// sync all objects
        //----------------  Callback handlers  ----------------
        function _syncImages(){
            Ti.App.removeEventListener($.syncMgr.EVENT.syncDone, _syncImages);
            self.stopLoading();
            self._restoreImages();
        }
	},

    //-----------  Protected methods  ---------
    _restoreImages : function(){
        var self = this;
        App.Helper.requireModel('WOCard');
        
        var card = new App.Model.WOCard();
        var urls = card.getMissingImages();
    	$.syncMgr.queueImages({ urls: urls });

        var total = $.syncMgr.imagesQueued();
        if(total<=0){
            self._eventProxy.fireEvent(self.EVENT.cardsRestored, {});
            return;
        }
        
   		self.L_Restore.text = App.Helper.L('auth_RESTORINGCARD');
   		
   		if(App.Config.IS_ANDROID) {
	  		 self.progressBar = Ti.UI.createSlider(self.STYLE.Restore.slider);
   		} else {
         	self.progressBar = Ti.UI.createProgressBar(self.STYLE.Restore.progressBar);
   		}

        self.progressBar.min = 0;
        self.progressBar.max = total;

        self.VIEW.add(self.progressBar);

        self.progressBar.show();
        self.progressBar.setValue(0);
        
        
        // var progress = Ti.UI.createView({
	        // width: 220,
	        // height: 'auto',
	        // backgroundColor: 'transparent',
	        // bottom: 5
	    // });
    
	    
    
        $.syncMgr.syncImages({
            pcallback   : _doUpdateProgress,
            scallback   : _doRestoreDone,
        });
        //----------------  Callback Handlers  ------------------
        function _doUpdateProgress(doneCount){
        	if(doneCount > self.progressBar.getValue()){
            	self.progressBar.setValue(doneCount);            		
        	}
        }
        function _doRestoreDone(doneCount){
        	self.progressBar.setValue(self.progressBar.max);
            self._eventProxy.fireEvent(self.EVENT.cardsRestored, {});
        }
    }

});

module.exports = RestoreCard;

//---  End of file: RestoreCard.js
