/**
 * CardListView
 */
Ti.include('/globals.js');

var CardListView = App.Class(App.View.WOList, {
	className 	: 'CardListView',
	stylePath 	: 'CardListStyle',

	EVENT		: {
		//--- Handles these events:
		//--- Produces these events:
		rowClicked		: 'CardListView.rowClicked',
		rowDeleted		: 'CardListView.rowDeleted',
    },
    
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);

		self.ITEM_KEY		= 'cardID';
		self.ALTERNATE		= true;
		self.renderbyTimer	= true;
		self.renderInterval	= 5;
	},
	
	/**
	 * Row render
	 * Should be implement by sub-class (card list view: list/face/grid, default:list)
	 */
	renderRow : function(param, index){
		var self = this;
		var theCard = param;

		//--- Card Definition ---------
		var cardDefn = {
			name		: theCard.masterName,
			style		: {touchEnabled: false},
			properties 	: {
				cardObj			: theCard,
				format			: $.cardMgr.factory.FORMAT.singleRow,
				cardFront		: true,
				shadow			: true,
				row		: {
					style	: self.STYLE.tableRow2,
				},
			},
			useRow		: true,
			eventProxy	: self._eventProxy,
		};
		var card = $.cardMgr.factory.make(cardDefn, theCard);
		
		var row = card.getRow();
		row.cardID = theCard.cardID;
		row.backgroundColor = (index%2==0)?'#16000000':'transparent';

		return row;
	},
	
	alternateUpdateRow : function(row, index){
		var self = this;
		var bgColor = (index%2==0)?'#16000000':'transparent';
		row.backgroundColor = bgColor;
	},

	postDraw : function(param){
		var self = this;
		
		var view_hiddenTip = Ti.UI.createView(self.STYLE.view.hiddenTip);
    	var text = App.Helper.L('cardlist_hiddenTip');

		if(self.total>1){
			text = App.Helper.L('cardlist_hiddenTip1')+(self.total)+App.Helper.L('cardlist_hiddenTip2');
		}
		self.lbl_desc = Ti.UI.createLabel(self.STYLE.desc.L1b);
		self.lbl_desc.text = text;
		view_hiddenTip.add(self.lbl_desc);
		self.tableView.headerPullView = view_hiddenTip;
	},
	
	hookSetData  :function(){
		var self = this;
    	var text = App.Helper.L('cardlist_hiddenTip');
		if(self.total>1){
			text = App.Helper.L('cardlist_hiddenTip1')+(self.total)+App.Helper.L('cardlist_hiddenTip2');
		}
		self.lbl_desc.text = text;
	}
	

});

module.exports = CardListView;

//---  End of file: CardListView.js
