/**
 * CitySearchView
 */
Ti.include('/globals.js');

var CitySearchView = App.Class(App.View.WOList2, {
	className 	: 'CitySearchView',
	stylePath 	: 'CitySearchStyle',

	EVENT		: {
		//--- Handles these events:
		//--- Produces these events:
		rowClicked		: 'CitySearchView.rowClicked',
		hideTitlebar	: 'CitySearchView.hideTitlebar',
		showTitlebar	: 'CitySearchView.showTitlebar',
		doSearch		: 'CitySearchView.doSearch',
		doCancel        : 'CitySearchView.doCancel',
    },

	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
	},
	
	/**
	 * Row render
	 * Should be implement by sub-class
	 */
	renderRow : function(city, index){
		var self = this;
		
		if(!city) return false;
		
		for(var lang in city.name){
			if('undefined' == typeof city.name[lang].city) city.name[lang].city = '-';
			if('undefined' == typeof city.name[lang].state) city.name[lang].state = '';
			if('undefined' == typeof city.name[lang].country) city.name[lang].country = '';
		}
		
		var row = Ti.UI.createTableViewRow(self.STYLE.tableRow);
		row.objectID = city.id;
		row.objectData = city;

      	switch(city.type){
      		case 'city' :
  		      	var cityName = Ti.UI.createLabel(self.STYLE.label.city);
		      	cityName.touchEnabled	= false;
		      	var stateName = Ti.UI.createLabel(self.STYLE.label.state);
		      	stateName.touchEnabled	= false;
		      	var countryName = Ti.UI.createLabel(self.STYLE.label.countrystate);
		      	countryName.touchEnabled	= false;
      			if('undefined' != typeof city.name[App.Config.LANG]){
      				cityName.text = city.name[App.Config.LANG].city;
      				stateName.text = city.name[App.Config.LANG].state;
      				countryName.text = city.name[App.Config.LANG].country;
      			} else {
      				cityName.text = city.name.en.city;
      				stateName.text = city.name.en.state;
      				countryName.text = city.name.en.country;
      			}
      			row.add(cityName);
		      	row.add(stateName);
		      	row.add(countryName);
      			break;
      		case 'state':
      			var cityName = Ti.UI.createLabel(self.STYLE.label.city);
		      	cityName.touchEnabled	= false;
		      	var countryName = Ti.UI.createLabel(self.STYLE.label.country);
		      	countryName.touchEnabled	= false;
      		    if('undefined' != typeof city.name[App.Config.LANG]){
      				cityName.text = city.name[App.Config.LANG].state;
      				countryName.text = city.name[App.Config.LANG].country;
      			} else {
      				cityName.text = city.name.en.state;
      				countryName.text = city.name.en.country;
      			}
      			row.add(cityName);
		      	row.add(countryName);      			
      			break;
      		case 'country':
      			var cityName = Ti.UI.createLabel(self.STYLE.label.city);
		      	cityName.touchEnabled	= false;
      		    if('undefined' != typeof city.name[App.Config.LANG]){
      				cityName.text = city.name[App.Config.LANG].country;
      			} else {
      				cityName.text = city.name.en.country;
      			}
  				row.add(cityName);
      			break;
      	}
      	
		row.backgroundColor = (index%2==0)?'#16000000':'transparent';
		return row;
	},
	
	drawHeaderView : function(){
		var self = this;
		
		self.searchBar = Ti.UI.createSearchBar(self.STYLE.searchBar);
		self.searchBar.hintText = App.Helper.L('formfieldwindow_city_search');
		//---  Attach Event Handlers  -----

		self.searchBar.addEventListener('cancel', function(evt){
			self.searchBar.setValue('');
			self.searchBar.blur();			
			self.searching = false;
			self.setData([]);
			self.hideNoResult();
			self._eventProxy.fireEvent(self.EVENT.doCancel, {});
		});
		self.searchBar.addEventListener('return', function(evt){
			if(self.locked) return;
			var _keyword = self.searchBar.value;
			if(_keyword == '') return;
			self.setData([]);
			self.hideNoResult();
			self.startLoading();
			self.searchBar.blur();			
			self.searching = true;
	        //---------------  Call Server  -------------
	        var payload = {
				string		: _keyword,
	            //--- extended payloads ---
	            // appInfo     : JSON.stringify($.configMgr.getAppInfo()),
	            // deviceInfo  : JSON.stringify($.configMgr.getDeviceInfo()),
	        };
	        var callParam = {
	            action		: 'country_search',
	            data		: payload,
	            report		: true,
	            callback	: _doSearchCallback,
	            errorback 	: _doSearchErrorback,
	        };
	        $.syncMgr.callServer(callParam);
	        
	        function _doSearchCallback(result) {
	        	self.stopLoading();
	        	self.searching = false;
	        	Ti.API.error(result);
	        	if(result.data.length > 0) {
	        		self.hideNoResult();
	        		self.setData(result.data);
	        	} else {
	        		self.setData([]);
	        		self.showNoResult();
	        	}
	        }
	        
	        function _doSearchErrorback(result) {
	        	self.stopLoading();
	        	self.searching = false;
	        	Ti.API.error(result);
	        }
			
			// self._eventProxy.fireEvent(self.EVENT.doSearch, {
				// keyword		: _keyword,
				// callback	: function(result){
					// if(result.length>0){
						// self.hideNoResult();
					// }else{
						// self.showNoResult();
					// }
					// self.setData(result);
				// }
			// });	

			
		});
		
		
		self.searchBar.focus();
		return (self.searchBar);
	},
	
	setRowBackgroundColor : function(row, index){
		var self = this;
		var bgColor = (index%2==0)?'#16000000':'transparent';
		row.backgroundColor = bgColor;
	},
	
	postDraw : function(param){
		var self = this;
		self._eventProxy.addEventListener(self.EVENT.rowClicked, function(evt){
//			if(self.searching){
				self.searchBar.blur();
				self._eventProxy.fireEvent(self.EVENT.showTitlebar, {});
				self.searching = false;
//			}
		});
	},
	
	drawNoResultView : function(param){
		var self = this;
		
		self.noResultView = Ti.UI.createView(self.STYLE.view.noResultFrame);

		var view_noResultCanvas = Ti.UI.createView(self.STYLE.view.noResultCanvas);

		var lbl_oops = Ti.UI.createLabel(self.STYLE.label.title);
		lbl_oops.text = App.Helper.L('formfieldwindow_searchResult_oops');

		var lbl_desc = Ti.UI.createLabel(self.STYLE.label.desc);
		lbl_desc.text = App.Helper.L('formfieldwindow_searchResult_desc');
		
		view_noResultCanvas.add(lbl_oops);
		view_noResultCanvas.add(lbl_desc);
		self.noResultView.add(view_noResultCanvas);
		
		self.noResultView.hide();
		
		return self.noResultView;
	}
	
});

module.exports = CitySearchView;

//---  End of file: CitySearchView.js
