/**
 * DiscoverWindow
 */
Ti.include('/globals.js');
if (App.Config.IS_ANDROID){
Ti.include('/waveo/core/WOAndroidUtils.js');	
}

var DiscoverWindow = App.Class(App.View.WOWindow, {
	className 	: 'DiscoverWindow',
	stylePath 	: 'DiscoverStyle',

	EVENT		: {
		//--- Handles these events:
		//--- Produces these events:
		addCustomCard	: 'DiscoverWindow.addCustomCard',
		viewCategory	: 'DiscoverWindow.viewCategory',
		doSearch		: 'DiscoverWindow.doSearch',
		viewCardMaster	: 'DiscoverWindow.viewCardMaster',
    },
    
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
		
		self.channelSwitcher 	= null;
		self.channels			= [];
		
		self.CH_NEW				= 'new';
		self.CH_CATEGORIES		= 'categories';
		self.CH_SEARCH			= 'search';
		
		self.channelData		= 
		[
			//{title:'New',	 		channel:self.CH_NEW 		},
			{title:App.Helper.L('categories'),	channel:self.CH_CATEGORIES	},
			{title:App.Helper.L('search'),		channel:self.CH_SEARCH		}
		];

		self.locked = false;
		
	    self.WINDOW.title		= '';
	},

	draw : function(param){
		var self = this;

		var titlebarShadowContainer = Ti.UI.createView(self.STYLE.titlebarShadowContainer);
		var titlebarShadow = Ti.UI.createView(self.STYLE.titlebarShadow);
		titlebarShadowContainer.add(titlebarShadow);

		var canvas = Ti.UI.createView(self.STYLE.view.canvas);

		// //---  tabber bar
		for( var index = 0; index < self.channelData.length; index ++ ){
			var channelName	= self.channelData[index]['channel'];
			var channelView = self.getChannel(channelName, param);
			channelView.visible = false;
			canvas.add(channelView);
		
			self.channels.push(channelView);
		}
	
		if (App.Config.IS_ANDROID){
			var aStyle = App.Helper.clone(self.STYLE.channel_switcher_android);
			aStyle.index = 0;
			aStyle.labels = self.channelData;
			self.channelSwitcher = createAndroidTabbedBar(aStyle);
		}else{		
			self.channelSwitcher = Ti.UI.iOS.createTabbedBar(self.STYLE.channel_switcher);
			self.channelSwitcher.labels = self.channelData;
			self.channelSwitcher.index = 0;
		}

		if (App.Config.IS_ANDROID){
			self.VIEW.setTitleControl(self.channelSwitcher);
		}else{		
			self.VIEW.titleControl = self.channelSwitcher;
		}
		
		self.channelSwitcher.addEventListener('click', function(e){
			if(e.index == null || e.index==undefined) return;
			if(e.index ===1) {
				titlebarShadowContainer.hide();
			} else {
				titlebarShadowContainer.show();
			};
			self.switchChannel(e.index);
		});

		self.VIEW.add(titlebarShadowContainer);
		self.VIEW.add(canvas);
		self.switchChannel(0);
	},

	switchChannel : function(index){
		var self = this;
		
		for( var i = 0; i < self.channels.length; i ++ ){
			if(i == index){
				self.channels[i].visible = true;
			}else{
				self.channels[i].visible = false;
			}
		}
		Ti.App.fireEvent('blur.all');		// FIXME  [ZJ]
		// if(self.cardSearch){
			// self._eventProxy.fireEvent(self.cardSearch.EVENT.showTitlebar, {});
		// }
	},
	
	getChannel : function(channelName, param){
		var self = this;

		switch(channelName){
		case self.CH_NEW:
			return Ti.UI.createView({width:'100%', height:'100%', backgroundColor:'red'});
			break;
			
		case self.CH_CATEGORIES:
			//---  Render subview  ---
			var viewParam = {
				DATA		: self.DATA.categories,
				eventProxy	: self._eventProxy,
				tabGroupObj	: self.tabGroupObj,
			};
			self.categoryList = App.Helper.loadView('discover/CategoryList', viewParam);
			self.categoryList.render();
			
			//---  Set Data & Render Rows  ---
			self.categoryList.setData(self.DATA.categories);
			
			//---  Setup event handlers  ---
			self._eventProxy.addEventListener(self.categoryList.EVENT.rowClicked, function(evt){
				if(evt.data.category){
					self._eventProxy.fireEvent(self.EVENT.viewCategory, { category: evt.data.category });
				}else{
					if(evt.data.addCustomCard){
						self._eventProxy.fireEvent(self.EVENT.addCustomCard, {});
					}
				}
			});
			return self.categoryList.getView();
			break;
			
		case self.CH_SEARCH:
			//---  Render subview  ---
			var viewParam = {
				eventProxy	: self._eventProxy,
			};
			self.cardSearch = App.Helper.loadView('discover/SearchView', viewParam);
			self.cardSearch.render();

			//---  Setup event handlers  ---
			self._eventProxy.addEventListener(self.cardSearch.EVENT.hideTitlebar, function(evt){
				if(self.locked) return;
				self.lock();
				self.VIEW.hideNavBar();
				setTimeout(function(){
					self.unlock();
				}, 500);
			});
			self._eventProxy.addEventListener(self.cardSearch.EVENT.showTitlebar, function(evt){
				if(self.locked) return;
				if (self.VIEW == null) return;    // in blur-all, VIEW may have been destroyed.
				self.lock();
				self.VIEW.showNavBar();	
				setTimeout(function(){
					self.unlock();
				}, 500);
			});

			self._eventProxy.addEventListener(self.cardSearch.EVENT.rowClicked, function(evt){
				self._eventProxy.fireEvent(self.EVENT.viewCardMaster, { masterID: evt.data.masterID });
			});
			return self.cardSearch.getView();
			break;
		}
	},
	
	updateCategory : function(param){
		var self = this;
		self.categoryList.setData(param);
	},
	
	lock:function(){
		var self = this;
		//self.locked = true;
		//self.channelSwitcher.touchEnabled = false;
	},
	unlock:function(){
		var self = this;
		self.locked = false;
		self.channelSwitcher.touchEnabled = true;
	},

	onFocus : function(evt){
		var self = this;
		
		$.app.doNotDisturb(false);		//  FIXME  [young]  tmp solution, to remove
		$.notifyMgr.isActive = true;
		
        Ti.App.fireEvent($.notifyMgr.EVENT.doMessages);
		
		self.base(evt);
	},
	
});

module.exports = DiscoverWindow;

//---  End of file: DiscoverWindow.js
