/**
 * SearchView
 */
Ti.include('/globals.js');

var SearchView = App.Class(App.View.WOList, {
	className 	: 'SearchView',
	stylePath 	: 'CardListStyle',

	EVENT		: {
		//--- Handles these events:
		//--- Produces these events:
		rowClicked		: 'SearchView.rowClicked',
		hideTitlebar	: 'SearchView.hideTitlebar',
		showTitlebar	: 'SearchView.showTitlebar',
		doSearch		: 'SearchView.doSearch',
    },

	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);
		
		self.ITEM_KEY		= 'category';
		
		self.ALTERNATE		= true;
		self.searching		= false;
		
		self.renderbyTimer	= true;
		self.renderInterval	= 10;
		
		self.showLoadingIndicator = true;
	},
	
	/**
	 * Row render
	 * Should be implement by sub-class
	 */
	renderRow : function(param, index){
		var self = this;
		
		var cardMaster = param;
		//--- Card Definition ---------
		var cardDefn = {
			name		: cardMaster.name,
			style		: { touchEnabled: false, },
			properties 	: {
				cardObj			: cardMaster,
				format			: $.cardMgr.factory.FORMAT.singleRow,
				cardFront		: true,
				shadow			: true,
				isCardMaster	: true,
				row		: {
					style	: self.STYLE.tableRow2,
				},
			},
			useRow		: true,
			eventProxy	: self._eventProxy,
		};

		var cardData = {};
		var card = $.cardMgr.factory.make(cardDefn, cardData);
		card.setMode('display');
		
		var row = card.getRow();
		row.masterID = cardMaster.id;
		row.backgroundColor = (index%2==0)?'#16000000':'transparent';

		return row;
	},
	
	getHeader:function(){
		var self = this;
		
		self.searchBar = Ti.UI.createSearchBar(self.STYLE.searchBar);
		self.searchBar.hintText = App.Helper.L('cardlist_search');
		//---  Attach Event Handlers  -----
		self.searchBar.addEventListener('focus', function(evt){
			if($.app.viewingMasterInfo) return;
			self.searching = true;
			/*if (!App.Config.IS_ANDROID)*/ {
				self._eventProxy.fireEvent(self.EVENT.hideTitlebar, {});
			}
		});
		
		self.searchBar.addEventListener('click', function(evt){
			if($.app.viewingMasterInfo) return;
		});
		
		self.searchBar.addEventListener('cancel', function(evt){
			self.searchBar.setValue('');
			self.searchBar.blur();
			/*if (!App.Config.IS_ANDROID)*/ {
				self._eventProxy.fireEvent(self.EVENT.showTitlebar, {});
			}
			self.searching = false;
			self.setData([]);
			self.hideNoResult();
		});
		self.searchBar.addEventListener('return', function(evt){
			if(self.locked) return;
			var _keyword = self.searchBar.value;
			if(_keyword == '') return;
			self.setData([]);
			self.hideNoResult();
			self.startLoading();
			
			self._eventProxy.fireEvent(self.EVENT.doSearch, {
				keyword		: _keyword,
				callback	: function(result){
					if(result.length>0){
						self.hideNoResult();
					}else{
						self.showNoResult();
					}
					self.setData(result);
				}
			});	

			self.searchBar.blur();
			self._eventProxy.fireEvent(self.EVENT.showTitlebar, {});
			self.searching = false;
		});
		
		Ti.App.addEventListener('blur.all', function(){
			self.searchBar.blur();
			self._eventProxy.fireEvent(self.EVENT.showTitlebar, {});
			self.searching = false;
		});
		return (self.searchBar);
	},
	
	alternateUpdateRow : function(row, index){
		var self = this;
		var bgColor = (index%2==0)?'#16000000':'transparent';
		row.backgroundColor = bgColor;
	},
	
	postDraw : function(param){
		var self = this;
		self._eventProxy.addEventListener(self.EVENT.rowClicked, function(evt){
//			if(self.searching){
				self.searchBar.blur();
				self._eventProxy.fireEvent(self.EVENT.showTitlebar, {});
				self.searching = false;
//			}
		});
	},
	
	renderNoResult : function(){
		var self = this;
		
		var view_noResultFrame = Ti.UI.createView(self.STYLE.view.noResultFrame);

		var view_noResultCanvas = Ti.UI.createView(self.STYLE.view.noResultCanvas);

		var lbl_oops = Ti.UI.createLabel(self.STYLE.title.L1);
		lbl_oops.text = App.Helper.L('searchResult_oops');

		var lbl_cannotfind = Ti.UI.createLabel(self.STYLE.desc.L1);
		lbl_cannotfind.text = App.Helper.L('searchResult_cannotfind');
	
		var lbl_addCustomCard = Ti.UI.createLabel(self.STYLE.title.L2);
		lbl_addCustomCard.text = App.Helper.L('searchResult_addCustomCard');
	
		var btn_addCustomCard = Ti.UI.createButton(self.STYLE.btn.addCustomCard);
		btn_addCustomCard.title = App.Helper.L('searchResult_btn_addCustomCard');

		btn_addCustomCard.addEventListener('touchstart', function(){
			btn_addCustomCard.backgroundGradient = self.STYLE.buttonSelectGradient;
		});
		btn_addCustomCard.addEventListener('touchend', function(){
			btn_addCustomCard.backgroundGradient=self.STYLE.buttonGradient;
		});
		btn_addCustomCard.addEventListener('click', function(){
			self._eventProxy.fireEvent(self._eventProxy.EVENT.addCustomCard, {});
		});
		
		view_noResultCanvas.add(lbl_oops);
		view_noResultCanvas.add(lbl_cannotfind);
		view_noResultCanvas.add(lbl_addCustomCard);
		view_noResultCanvas.add(btn_addCustomCard);
		view_noResultFrame.add(view_noResultCanvas);
		
		view_noResultFrame.hide();
		return view_noResultFrame;
	}
	
});

module.exports = SearchView;

//---  End of file: SearchView.js
