/**
 * PreferencesWindow
 */
Ti.include('/globals.js');

var PreferencesWindow = App.Class(App.View.WOWindow, {
	className 	: 'PreferencesWindow',
	stylePath : 'PreferenceStyle',

	EVENT		: {
		//--- Controller to handle:
		//--- events from Controller:
    },
	
	initialize : function(param){
		var self = this;
		self.base(param);
		
		self.WINDOW.title = App.Helper.L('win_title_settings');
	},
	
	draw : function(param){
		var self = this;

		var mainView = self._addMainView(self.VIEW);
		var titlebarShadowContainer = Ti.UI.createView(self.STYLE.titlebarShadowContainer);
		var titlebarShadow = Ti.UI.createView(self.STYLE.titlebarShadow);
		titlebarShadowContainer.add(titlebarShadow);

		self.VIEW.add(titlebarShadowContainer);
	},
	
	_addMainView : function(myWindow) {
		var self = this;
		
		self._addForm(myWindow);

		return myWindow;
	},
	
	_addForm : function(theView) {
		var self = this;

		//--- create profile form view ---
		var formParam = {
			formDefn	: self._getDefaultFormDefinition(),
			formData	: self.DATA.preferences,
			stylePath	: self.stylePath,
			DATA 		: self.DATA,
			eventProxy	: self._eventProxy,
			defaultMode	: 'edit',
			tabGroup	: self.tabGroupObj.tabGroup,
		};
		self.FORM = new App.WOForm();
		self.FORM.initialize(formParam);
		var formView = self.FORM.render();		
		
		if(theView) theView.add(formView);
		return formView;

	},
	
	// ------------------------ Get Default Form Definition ------------------------	
	
	_getDefaultFormDefinition : function() { //TODO [ZJ] - pass from self.DATA.
		var self = this;
		
		var theForm = {};
		theForm.form = {
			name : 'perferences',
			style : 'form',
			properties : {
				fieldWindow : {
						style : 'fieldWindow',
				},
			}
		};
		theForm.sections = [{},{},{},{}];
		theForm.sections[0].name = 'remindMe';
		theForm.sections[0].header = {
			type		: 'text',
			style		: 'view_formSectionHeader',
			properties	: {
				value	: App.Helper.L('preference_remind'),
				style	: 'label_formSectionHeader',
			},
		},
		theForm.sections[0].fields = [{},{}];
		theForm.sections[0].fields[0] = {
			name : 'cardRemindDays',
			type : 'slider',
			style : 'view_formfield',
			properties : {
				row : {
					style : 'view_formRow_withWindow',
				},
				label : {
					value : App.Helper.L('preference_cardReminder'),
					style : 'label_fieldLabel',
				},
				field : {
					style : 'label_fieldData',
				},
				fieldView : {
					title : App.Helper.L('preference_cardExpiry'),
					label : App.Helper.L('preference_receiveCardReminder'),
					style : 'fieldWindowElement_slider',
				},
				config : {
					unit	: App.Helper.L('preference_remindUnitDay'),
					units 	: App.Helper.L('preference_remindUnitDays'),
					min		: -1,
					max		: 30,
					label : {
						'-1' : App.Helper.L('preference_remindNever'),
						'0'  : App.Helper.L('preference_remindOnExpiry'),
						'7'	 : App.Helper.L('preference_remind1Week'),
						'14' : App.Helper.L('preference_remind2Weeks'),
						'21' : App.Helper.L('preference_remind3Weeks'),
						'30' : App.Helper.L('preference_remind1Month'),
					},
					desc : {
						'-1' : App.Helper.L('preference_remindDoNot'),
						'0'  : App.Helper.L('preference_remindOnDay'),
					},
					tip : App.Helper.L('preference_remindBeforeCardsExpire'),
					tips : {
						'-1' 	: App.Helper.L('preference_remindMe'),
						'0' 	: App.Helper.L('preference_remindOfCardExpiry'),
					}
				},
			},
		};
		theForm.sections[0].fields[1] = {
			name : 'offerRemindDays',
			type : 'slider',
			style : 'view_formfield',
			properties : {
				row : {
					style : 'view_formRow_withWindow',
				},
				label : {
					value : App.Helper.L('preference_offerReminder'),
					style : 'label_fieldLabel',
				},
				field : {
					style : 'label_fieldData',
				},
				fieldView : {
					title : App.Helper.L('preference_offerExpiry'),
					label : App.Helper.L('preference_receiveOfferReminder'),
					style : 'fieldWindowElement_slider',
				},
				config : {
					unit	: App.Helper.L('preference_remindUnitDay'),
					units 	: App.Helper.L('preference_remindUnitDays'),
					min		: -1,
					max		: 30,
					label : {
						'-1' :  App.Helper.L('preference_remindNever'),       
						'0'  :  App.Helper.L('preference_remindOnExpiry'),    
						'7'	 :  App.Helper.L('preference_remind1Week'),       
						'14' :  App.Helper.L('preference_remind2Weeks'),      
						'21' :  App.Helper.L('preference_remind3Weeks'),      
						'30' :  App.Helper.L('preference_remind1Month'),      
					},
					desc : {
						'-1' : App.Helper.L('preference_remindDoNot'),
						'0'  : App.Helper.L('preference_remindOnDay'),
					},
					tip : App.Helper.L('preference_remindBeforeOffersExpire'),
					tips : {
						'-1' 	: App.Helper.L('preference_remindMe'),
						'0' 	: App.Helper.L('preference_remindOfOfferExpiry'),
					}
				},
			},
		};
		theForm.sections[1].name = 'recommend';
		theForm.sections[1].header = {
			type		: 'text',
			style		: 'view_formSectionHeader',
			properties	: {
				value	: App.Helper.L('preference_recommend'),
				style	: 'label_formSectionHeader',
			},
		},		
		// theForm.sections[1].footer = {
			// type		: 'text',
			// style		: 'view_formSectionFooter',
			// properties	: {
				// value	: App.Helper.L('preference_keepPosted'),
				// style	: 'label_formSectionFooter',
			// },
		// },
		theForm.sections[1].fields = [{}];
		theForm.sections[1].fields[0] = {
			name : 'recommend',
			type : 'switcher',
			style : 'view_formfield',
			properties : {
				row : {
					style : 'view_formRow',
				},
				label : {
					value : App.Helper.L('preference_cardLike'),
					style : 'label_fieldLabel',
				},
				field : {
					style : 'label_fieldData',
				},
			},
		};
		theForm.sections[2] = {};
		theForm.sections[2].name = App.Helper.L('preference_discover');
		theForm.sections[2].header = {
			type		: 'text',
			style		: 'view_formSectionHeader',
			properties	: {
				value	: App.Helper.L('LABEL_DISCOVER'),
				style	: 'label_formSectionHeader',
			},
		},
		theForm.sections[2].fields = [{}];
		theForm.sections[2].fields[0] = {
			name : 'city',
			type : 'city',
			style : 'view_formfield',
			properties : {
				row : {
					style : 'view_formRow_withWindow',
				},
				label : {
					value : App.Helper.L('preference_cards'),
					style : 'label_fieldLabel',
				},
				field : {
					style : 'label_fieldData',
				},
				fieldView : {
					title : App.Helper.L('formfieldwindow_city_title'),
					label : '',
					style : 'fieldWindowElement_city',
				},
				config : {
				},
			},
		};
		theForm.sections[3].name = 'mySuggestions';
		theForm.sections[3].header = {
			type		: 'text',
			style		: 'view_formSectionHeader',
			properties	: {
				value	: App.Helper.L('preference_suggestions'),
				style	: 'label_formSectionHeader',
			},
		},
		// theForm.sections[2].footer = {
			// type		: 'text',
			// style		: 'view_formSectionFooter',
			// properties	: {
				// value	: App.Helper.L('preference_letusknow'),
				// style	: 'label_formSectionFooter',
			// },
		// };
		theForm.sections[3].fields = [{}];
		theForm.sections[3].fields[0] = {
			name : 'feedback',
			type : 'emailDialog',
			style : 'view_formfield',
			properties : {
				row : {
					style : 'view_formRow_withWindow',
				},
				label : {
					value : App.Helper.L('preference_feedback'),
					style : 'label_fieldLabel',
				},
				field : {
					style : 'label_fieldData',
				},
				config : {
					subject : self.DATA.mobileNo ? App.Helper.L('preference_feedback_subject1')+self.DATA.mobileNo+')' : App.Helper.L('preference_feedback_subject2'),
					recipients : ['service@perkd.me'],
					message : "",
				},
			},
		};
		theForm.sections[3].fields[1] = {
			name : 'cardsLibrary',
			type : 'emailDialog',
			style : 'view_formfield',
			properties : {
				row : {
					style : 'view_formRow_withWindow',
				},
				label : {
					value : App.Helper.L('preference_library'),
					style : 'label_fieldLabel',
				},
				field : {
					style : 'label_fieldData',
				},
				config : {
					subject : self.DATA.mobileNo ? App.Helper.L('preference_library_subject1')+self.DATA.mobileNo+')' : App.Helper.L('preference_library_subject2'),
					recipients : ['cards@perkd.me'],
					message : App.Helper.L('preference_library_email'),
				},
			},
		};
		
		if((Ti.Media.isCameraSupported || App.Config.IS_SIMULATOR) && !App.Config.IS_ANDROID) {
			var i = theForm.sections.length;
			theForm.sections[i] = {};
			theForm.sections[i].name = 'QRCode';
			theForm.sections[i].fields = [{}];
			theForm.sections[i].fields[0] = {
				name : 'scan',
				type : 'switcher',
				style : 'view_formfield',
				properties : {
					row : {
						style : 'view_formRow',
					},
					label : {
						value : App.Helper.L('preference_QRCode'),
						style : 'label_fieldLabel',
					},
					field : {
						style : 'label_fieldData',
					},
				},
			};
			// theForm.sections[3].footer = {
			//	type		: 'text',
			//	style		: 'view_formSectionFooter',
			//	properties	: {
			//		value	: App.Helper.L('preference_QRCodeSwitch'),
			//		style	: 'label_formSectionFooter',
			//	},
			//};
			
		};
		
		if('zh-Hant' == Ti.Locale.currentLanguage || 'zh' == Ti.Locale.currentLanguage) {
			var langField = {
				name : 'langRegion',
				type : 'lang',
				style : 'view_formfield',
				properties : {
					row : {
						style : 'view_formRow_withWindow',
					},
					label : {
						value : App.Helper.L('LABEL_LANGREGION'),
						style : 'label_fieldLabel',
					},
					field : {
						style : 'label_fieldData',
					},
					fieldView : {
						title : App.Helper.L('LABEL_LANGREGION'),
						label : '',
						style : 'fieldWindowElement_lang',
					},
					config : {
						langRegion : {
							TW : '台灣',
							HK : '香港, 澳门',
						}
					},
				},
			};
			
			var i = theForm.sections.length-1;
			if('QRCode' == theForm.sections[i].name) {
				theForm.sections[i].fields.push(langField);
			} else {
				i++;
				theForm.sections[i] = {};
				theForm.sections[i].name = 'langRegion';
				theForm.sections[i].fields = [{}];
				theForm.sections[i].fields[0] = langField;
			}
		}
		
		theForm.footer = {
			type : 'text',
			style : 'view_formFooter',
			properties	: {
				value	: App.Helper.L('preference_version')+': '+Ti.App.version,
				style	: 'label_formFooter',
			},
		};
			
		return theForm;
	},
});

module.exports = PreferencesWindow;

//---  End of file: PreferencesWindow.js