/**
 * ProfileWindow
 */
Ti.include('/globals.js');

var ProfileWindow = App.Class(App.View.WOWindow, {
	className 	: 'ProfileWindow',
	stylePath	: 'ProfileStyle',
	
	EVENT		: {
		//--- Controller to handle:
		updateProfile	: 'ProfileWindow.updateProfile',
		//--- events from Controller:
		profileUpdated	: 'ProfileWindow.profileUpdated',
    },

	//---  Constructor  ---------------	
	initialize : function(param) {
		var self = this;
		self.base(param);

	    self.WINDOW.title = App.Helper.L('win_title_profile');

		//---  Data package from Controller ---
		// self.DATA.familyName;
		// self.DATA.givenName;
		// self.DATA.gender;
		// self.DATA.birth;
		// self.DATA.email;
		
		// --- UI Components ---
		self.FORM;
	},
	
	draw : function(param){
		var self = this;

		var mainView = self._addMainView(self.VIEW);

		var editBtn = Ti.UI.createButton(self.STYLE.btn.EDIT);
		var saveBtn = Ti.UI.createButton(self.STYLE.btn.SAVE);
		var cancelBtn = Ti.UI.createButton(self.STYLE.btn.CANCEL);
		editBtn.addEventListener('click', function(evt) {
			self.FORM.setMode('edit');
			self.VIEW.setLeftNavButton(cancelBtn);
			self.VIEW.setRightNavButton(saveBtn);
		});
		saveBtn.addEventListener('click', function(evt) {
			if(App.Config.IS_ANDROID) Ti.UI.Android.hideSoftKeyboard();
			self.FORM.blur();
			var formData = self.FORM.getFieldValues();
			self._eventProxy.fireEvent(self.EVENT.updateProfile, { profile : formData });
			formData.nameOrder = {
				displayID : formData.nameOrder,
				givenName : formData.givenName,
				familyName : formData.familyName,
			}
			self.FORM._eventProxy.fireEvent(self.FORM._eventProxy.EVENT.refresh, { data : formData });
			self.FORM.setMode('display');
			self.VIEW.setLeftNavButton(null);
			self.VIEW.setRightNavButton(editBtn);
		});
		cancelBtn.addEventListener('click', function(evt) {
			if(App.Config.IS_ANDROID) Ti.UI.Android.hideSoftKeyboard();
			self.FORM.blur();
			self.FORM.reset();
			self.FORM.setMode('display');
			self.VIEW.setLeftNavButton(null);
			self.VIEW.setRightNavButton(editBtn);
		});
		
		self.FORM._eventProxy.addEventListener(self.FORM._eventProxy.EVENT.change, function(evt){
			self.FORM.update(self.FORM.getFieldValues())
		});

		self.VIEW.setRightNavButton(editBtn);

		var titlebarShadowContainer = Ti.UI.createView(self.STYLE.titlebarShadowContainer);
		var titlebarShadow = Ti.UI.createView(self.STYLE.titlebarShadow);
		titlebarShadowContainer.add(titlebarShadow);
		
		self.VIEW.add(titlebarShadowContainer);
	},
		
	// ------------------------ Draw UI Components ------------------------
	_addMainView : function(theWindow){
		var self = this;

		self._addForm(theWindow);

		return theWindow;
	},
		
	_addForm : function(theView){
		var self = this;

		var formData = {
			familyName	: self.DATA.familyName,
			givenName	: self.DATA.givenName,
			nameOrder   : {
				displayID   : self.DATA.nameOrder || 0,
				familyName	: self.DATA.familyName,
				givenName	: self.DATA.givenName,
			},
			gender 		: self.DATA.gender,
			email 		: self.DATA.email,
			birth		: self.DATA.birth,
		};
		var formParam = {
			formDefn	: self._getDefaultFormDefinition(),
			formData	: formData,
			stylePath	: self.stylePath,
			DATA		: self.DATA,
			eventProxy	: self._eventProxy,
			defaultMode	: 'display',
			tabGroup	: self.tabGroupObj.tabGroup
		};
		
		self.FORM = new App.WOForm(); //tableView
		self.FORM.initialize(formParam);
		var formView  = self.FORM.render();

		if(theView) theView.add(formView);
		return formView;
	},

	// ------------------------ Get Default Form Definition ------------------------

	_getDefaultFormDefinition : function() {
		var self = this;

		var profileForm = {};
		profileForm.form = {
			name : 'profile',
			style : 'form',
			properties 	: {						// (optional) styles for each display mode
				fieldWindow	: {
					style	: 'fieldWindow',
				},
			},
		};
		profileForm.header = {
			type : 'text',
			style : 'view_formHeader',
			properties	: {
				value	: App.Helper.L('profileform_intro'),
				style	: 'label_formHeader',
			},
		};
		profileForm.sections = [{},{},{}];
		profileForm.sections[0].name = 'name';
		profileForm.sections[0].fields = [{},{}];
		if('zh-Hant' == Ti.Locale.currentLanguage || 'zh-Hans' == Ti.Locale.currentLanguage || 'zh' == Ti.Locale.currentLanguage){
			profileForm.sections[0].fields[0] = {
				name : 'familyName',
				type : 'text',
				style : 'view_formfield',
				properties : {
					row : {
						style : 'view_formRow',
					},
					hint : {
						value : App.Helper.L('formLabel_familyname'),
						style : 'label_fieldHint_name',
					},
					field : {
						style : 'label_fieldData_name',
					},
				},
			};
			profileForm.sections[0].fields[1] = {
				name : 'givenName',
				type : 'text',
				style : 'view_formfield',
				properties : {
					row: {
						style : 'view_formRow',
					},
					hint : {
						value : App.Helper.L('formLabel_givenname'),
						style : 'label_fieldHint_name',
					},
					field : {
						style : 'label_fieldData_name',
					},
				},
			};
		} else {
			profileForm.sections[0].fields[0] = {
				name : 'givenName',
				type : 'text',
				style : 'view_formfield',
				properties : {
					row: {
						style : 'view_formRow',
					},
					hint : {
						value : App.Helper.L('formLabel_givenname'),
						style : 'label_fieldHint_name',
					},
					field : {
						style : 'label_fieldData_name',
					},
				},
			};
			profileForm.sections[0].fields[1] = {
				name : 'familyName',
				type : 'text',
				style : 'view_formfield',
				properties : {
					row : {
						style : 'view_formRow',
					},
					hint : {
						value : App.Helper.L('formLabel_familyname'),
						style : 'label_fieldHint_name',
					},
					field : {
						style : 'label_fieldData_name',
					},
				},
			};
		}
		profileForm.sections[0].fields[2] = {
			name : 'nameOrder',
			type : 'nameOrder',
			style : 'view_formfield',
			properties : {
				row : {
					style : 'view_formRow_withWindow',
				},
				label : {
					value : App.Helper.L('formLabel_displayAs'),
					style : 'label_fieldLabel',
				},
				field : {
					style : 'label_fieldData',
				},
				fieldView : {
					style : 'fieldWindowElement_nameOrder',
				},
				config : {
					rows : [
						{title:App.Helper.L('profileform_nameOrderFG')}, //  <<<< Default row 0.
						{title:App.Helper.L('profileform_nameOrderGF')},
						{title:App.Helper.L('profileform_nameOrderG')},
					],
					defaultRow : 0,
				}
			},
		},
		profileForm.sections[1].name = 'personal';
		profileForm.sections[1].fields = [{},{}];
		profileForm.sections[1].fields[0] = {
			name : 'gender',
			type : 'tabbar',
			style : 'view_formfield',
			properties : {
				row: {
					style : 'view_formRow',
				},
				label : {
					value : App.Helper.L('formLabel_gender'),
					style : 'label_fieldLabel',
				},
				field : {
					style : 'label_fieldData'
				},
				config : {
					style: 'tabbar_gender',
					options : {
						'Male' : 1,
						'Female' : 2,
					},
					defaultValue : 0,
				}
			},
		},
		profileForm.sections[1].fields[1] = {
			name : 'birth',
			type : 'date',
			style : 'view_formfield',
			properties : {
				row : {
					style : 'view_formRow_withWindow',
				},
				label : {
					value : App.Helper.L('formLabel_birthday'),
					style : 'label_fieldLabel',
				},
				field : {
					style : 'label_fieldData',
				},
				fieldView : {
					style : 'fieldWindowElement_date',
				},
				config : {
					type : 'all',
				}
			},
		},
		profileForm.sections[2].name = 'email';
		profileForm.sections[2].fields = [{}];
		profileForm.sections[2].fields[0] = {
			name : 'email',
			type : 'text',
			style : 'view_formfield',
			properties : {
				row : {
					style : 'view_formRow',
				},
				hint : {
					value : App.Helper.L('formLabel_email'),
					style : 'label_fieldHint_email',
				},
				field : {
					style : 'label_fieldData_email',
				},
			},
		},
		profileForm.footer = {
			type : 'text',
			style : 'view_formFooter',
			properties	: {
				value	: App.Helper.L('profileform_tip'),
				style	: 'label_formFooter',
			},
		};
		return profileForm;
	},
});

module.exports = ProfileWindow;

//---  End of file: ProfileWindow.js