/**
 * ContactSelector
 * @copyright WaveO
 */
Ti.include('/globals.js');

var ContactSelector = App.Class(App.View.WOWindow, {
	className 	: 'ContactSelector',
	stylePath 	: 'ContactSelectorStyle',

	EVENT		: {
		//--- Handles these events:
		clearSelection	: 'ContactSelector.clearSelection',
		//--- Produces these events:
    },

	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);

		self.WINDOW.title	= self.DATA.group.name;
		self.lazyRender		= self.DATA.lazyRender;

		self.SHARED			= param.SHARED;		// shared memory space with other windows

		self.sortName   = Ti.Contacts.CONTACTS_SORT_ORDER == 0 ? 'firstName': 'lastName';
		self.unsortName = Ti.Contacts.CONTACTS_SORT_ORDER == 0 ? 'lastName' : 'firstName';
		
		self.rowListeners	= [];				// list of row event listeners
	},

	// --------------------------------------------------------
	draw : function(param) {
		var self = this;

	    self.startLoading();
		self.contactListView = Ti.UI.createTableView(self.STYLE.tableView.contact);
		//--- Search Bar  ------------
		if(! App.Config.IS_ANDROID){
		    self.search = Ti.UI.createSearchBar(self.STYLE.searchBar);
			self.contactListView.search = self.search;
			self.contactListView.filterAttribute = 'searchField';
		}

		self._setupEventHandlers();
	    //--- Contact List  ----------
		self._renderList(self.DATA.group, self.contactListView);

		self.VIEW.add(self.contactListView);
	},

	_setupEventHandlers : function(param){
		var self = this;

		//---  List View Events  --------------
		self.contactListView.addEventListener('click', function(evt){
			self.rowClickHandler(evt.row);
		});
		//---  Search Bar Events  --------------
		if(! App.Config.IS_ANDROID){
		    self.search.addEventListener('cancel', function(evt){
				self.search.searching = false;
				if (self.VIEW) 		self.VIEW.showNavBar();
		    });
		    self.search.addEventListener('focus', function(evt){
				self.search.searching = true;
				if (self.VIEW) 		self.VIEW.hideNavBar();
		    });
		    self.search.addEventListener('return', function(evt){
				if (self.VIEW)		self.VIEW.showNavBar();
		    });
	
		    self.search.addEventListener('blur', function(evt){
				if(self.search.searching == false)		return;
	
				if(!self.search.value || self.search.value == '')
					self.search.searching = false;
				if (self.VIEW) 		self.VIEW.showNavBar();
			});
		}
	},

	_renderList : function(group, contactListView){
	    var self = this;

	    var contacts = group.sortedMembers(Ti.Contacts.CONTACTS_SORT_LAST_NAME /* 3 */) || {};
	    var rows = [];
	    var sections = [];

	    self._eventProxy.fireEvent(self._eventProxy.EVENT.updateToolbar, {
	    	contactCount	: contacts.length || 0,
	    });
	
	    if(contacts.length > 0){
	    	var buildIndex = (contacts.length > 30) ? true : false;

	    	//---  Populate list  ---
			for(var i=0; i<contacts.length; i++){
				
			    var cid = contacts[i][ App.Config.IS_ANDROID ? 'id' : 'recordId' ];
			    
				if(App.Config.IS_ANDROID){
					var aRow = self._renderRow(contacts[i]);
					var key = aRow.contactId + '';
		    		if(self.SHARED.contactSelected[key]){
		    			self._renderSelectedView(aRow);
		    			aRow.doRowSelect(aRow);
		    		}
					//--- WARNING: turning ON caching will CRASH in Android !!!
			  	}else{
				    if(self.SHARED.contactRowCache[cid]){
						var aRow = self.SHARED.contactRowCache[cid];
				    }else{
						var aRow = self._renderRow(contacts[i]);
						self.SHARED.contactRowCache[cid] = aRow;
					}
				}
			    if(aRow.selected){
			    	var key = aRow.contactId + '';
					self.SHARED.contactSelected[key].group[self.DATA.groupID] = true;
			    }

			    if(buildIndex){
					//---  Add row to section  ---
					var name = contacts[i][self.sortName];
					if(!name)	name = contacts[i][self.unsortName];
					if(!name)	name = contacts[i]['fullName'];

					var firstChar = name.substr(0,1).toUpperCase();
					if(!App.Helper.isAlphabet(firstChar))		firstChar = '#';
					if(!sections[firstChar]){
						sections[firstChar] = Ti.UI.createTableViewSection({
							headerTitle		: firstChar,
						});
					}
					sections[firstChar].add(aRow);
				}else{
					rows.push(aRow);
				}
			}

		    //---  Add List Index to list  ----
			if(buildIndex){
				var theIndex = 0;
				sections = sortByKey(sections);

				var listIndex = [{ title: '{search}' }];
				for(key in sections){
					if(key!='#'){
						listIndex.push({ title: key,  index: theIndex });
						theIndex += sections[key].rowCount;
						rows.push(sections[key]);
					}
				}
				if(sections['#']){
					listIndex.push({ title: '#',  index: theIndex });
					rows.push(sections['#']);
				}
				contactListView.index = listIndex;
		    }
		}
	    contactListView.setData(rows);
	    self.stopLoading();

	    //-------------  Private Methods  ---------------
	    function sortByKey(arr){
			// Setup Arrays
			var sortedKeys = new Array();
			var sortedObj = {};

			// Separate keys and sort them
			for(i in arr)
				sortedKeys.push(i);

			sortedKeys.sort();
			// Reconstruct sorted obj based on keys
			for (i in sortedKeys)
				sortedObj[sortedKeys[i]] = arr[sortedKeys[i]];
			return sortedObj;
		}
	},

	_renderRow : function(aContact){
		var self = this;

		var row = Ti.UI.createTableViewRow(self.STYLE.tableRow_Contact);
		row.contactId	= aContact[ App.Config.IS_ANDROID ? 'id' : 'recordId' ];
	    row.name		= aContact.fullName;
	    row.searchField	= aContact.fullName;
		//---  Contact Name  ---------------
		row.nameTxt1 = Ti.UI.createLabel(self.STYLE.label.contactName);
		row.nameTxt1.text = row.name;
		row.add(row.nameTxt1);
		//---  Check box  ---------------
		var unCheckIcon = Ti.UI.createImageView(self.STYLE.img.unCheckIcon);
		row.add(unCheckIcon);

		return row;
	},

	rowClickHandler : function(row){
		var self = this;

		if(!row.rendered){
			self._renderSelectedView(row);
		}
		if(row.selected){
			row.unselectHandler();
		}else{
			self._doSelectPhoneNo(row);
		}
	},

	_renderSelectedView : function(row){
		var self = this;
 
		var selectedView = Ti.UI.createView(self.STYLE.tableRow_SelectedContact);

		row.nameTxt = Ti.UI.createLabel(self.STYLE.label.contactName);
		row.nameTxt.text = row.name;
		row.nameTxt.bottom = '20dp'; 
		selectedView.add(row.nameTxt);

		row.fieldTxt = Ti.UI.createLabel(self.STYLE.label.phoneLabel);
	  	row.numberTxt = Ti.UI.createLabel(self.STYLE.label.phoneNo);

		row.txtView = Ti.UI.createView(self.STYLE.view.phone);
	  	row.txtView.add(row.fieldTxt);
	  	row.txtView.add(row.numberTxt);
		selectedView.add(row.txtView);

		var checkIcon = Ti.UI.createImageView(self.STYLE.img.checkIcon);
		selectedView.add(checkIcon);

		//--- Attach row handler methods  ---
		row.doRowSelect = _doRowSelect;
		row.selectHandler	= function(selectedPhone){
			_doRowSelect(row, selectedPhone);
			_updateSelectedContact(row, true);
		}
		row.unselectHandler	= function(){
			_doRowUnselect(row);
			_updateSelectedContact(row, false);
		};
		row.updatePhone		= function(phoneParam){
			_doUpdatePhone(row, phoneParam);
		};

		//---  Fetch Person's Phone Number  ---
		var person = Ti.Contacts.getPersonByID(row.contactId);
		var phone = person.getPhone();
		if(!phone){
			$.app.showAlert(App.Helper.L('ALERT_PHONENBRNOTFOUND'));
		}else{
			var phones = [];
			for(key in phone){
				var fieldTotal = phone[key].length;
			    for(var m=0;m<fieldTotal;m++){
					var param = { name: row.name, key: key, value: phone[key][m], uid: row.contactId };
					phones.push(param);
			    }
			}
		}

		selectedView.backgroundColor = selectedView.backgroundColor;
		selectedView.visible = false;

		row.add(selectedView);
		row.selectedView = selectedView;
		row.phones = phones;
		row.rendered = true;
		  
		//----------------  Event Handlers  ------------------
		function _clearSelection(){
			if(row.selected){
				row.unselectHandler();
			}
		}
		function _doRowSelect(aRow, selectedPhone){
		    aRow.selectedView.visible = true;
			aRow.selectedView.backgroundColor = aRow.selectedView.backgroundColor;
		    aRow.selected = true;

			var key = aRow.contactId + '';	// coerce to string
		    if(!self.SHARED.contactSelected[key])
		    	self.SHARED.contactSelected[key] = {};
		    if(selectedPhone)
				self.SHARED.contactSelected[key].selected = selectedPhone;

			if(!self.SHARED.contactSelected[key].group)
				self.SHARED.contactSelected[key].group = {};
			self.SHARED.contactSelected[key].group[self.DATA.groupID] = true;

			self._eventProxy.addEventListener(self.EVENT.clearSelection, _clearSelection, self.rowListeners);
		}
		function _doRowUnselect(aRow){
		    aRow.selectedView.visible = false;
		    aRow.selected = false;

			var key = aRow.contactId + '';
		    delete self.SHARED.contactSelected[key];

			self._eventProxy.removeEventListener(self.EVENT.clearSelection, _clearSelection, self.rowListeners);
		}
		function _doUpdatePhone(aRow, phoneParam){
		    aRow.numberTxt.text  = phoneParam.value;
		    aRow.fieldTxt.text   = phoneParam.key;
		}
		function _updateSelectedContact(aRow, isSelected){
			if(isSelected){
				self.SHARED.selectedCount++;
		    }else{
				self.SHARED.selectedCount--;
		    }
	    	//---  Update Share button on Toolbar  ---
			self._eventProxy.fireEvent(self._eventProxy.EVENT.updateToolbar, {
				selectedCount	: self.SHARED.selectedCount,
			});
		}
	},

	_doSelectPhoneNo : function(row){
		var self = this;

		if(('undefined' == typeof row.phones) || (row.phones.length == 0)){
			$.app.showAlert(App.Helper.L('no_mobile'), App.Helper.L('TITLE_SHARE'));
			return;
		}else if(row.phones.length == 1){
			//--- only 1 phone, use it ---
			_updateSelection(row.phones[0]);
		}else{
			//---  multiple phone no., display options dialog to select ---
			var numbersSelect = [];
			for(var i=0;i<row.phones.length;i++){
				numbersSelect.push( row.phones[i].key+' '+row.phones[i].value );
			}
			numbersSelect.push(App.Helper.L('BUTTON_CANCEL'));

			var phoneDialog = Ti.UI.createOptionDialog({
			    title		: App.Helper.L('select_mobile'),
			    options		: numbersSelect,
				cancel		: row.phones.length,
			});
			phoneDialog.addEventListener('click', function(evt){
				if(evt.index < row.phones.length){
				    _updateSelection(row.phones[evt.index]);
				}
			});
			
			phoneDialog.show();
		}

		function _updateSelection(selectedPhone){
			row.updatePhone(selectedPhone);
			row.selectHandler(selectedPhone);
		}
	},

	// --------------------------------------------------------
	dispose : function() {
	    var self = this;

		if(App.Config.IS_ANDROID){
			self._eventProxy.removeListeners(self.rowListeners);
		}
	    self.contactListView = null;
	    self.base();
	},
});

module.exports = ContactSelector;

