/**
 * ShareWindowAndroid
 */
Ti.include('/globals.js');
if (App.Config.IS_ANDROID) {
	Ti.include('/waveo/core/WOAndroidUtils.js');
}

var ShareWindowAndroid = App.Class(App.View.WOWindow, {
	className 	: 'ShareWindowAndroid',
	stylePath 	: 'ContactSelectorStyle',
	
	EVENT : {
		//--- Handles these events:
		updateToolbar	: 'ShareWindowAndroid.updateToolbar',
		updateGroupCount: 'ShareWindowAndroid.updateGroupCount',
		updateContactCount: 'ShareWindowAndroid.updateContactCount',
		clearContactCount: 'ShareWindowAndroid.clearContactCount',
		//--- Produces these events:
		viewGroup	 	: 'ShareWindowAndroid.viewGroup',
		doShareCard 	: 'ShareWindowAndroid.doShareCard',
		shareSMSSent 	: 'ShareWindowAndroid.shareSMSSent',
		shareCancel 	: 'ShareWindowAndroid.shareCancel',
	},
	
	//---  Constructor  ---------------
	initialize : function(param){
		var self = this;
		self.base(param);

		self.WINDOW.type	= App.Const.WINDOW.type.navGroup;
	    self.WINDOW.title	= App.Helper.L('shareCard_contactGroups');

		self.SHARED			= param.SHARED,		// shared memory namespace among subviews
		self.evtHandlers	= [];				// list of event handlers that we add to proxy

	    self.ALL_CONTACT_GROUPID	= self.DATA.allContactGpID;
	},

	draw : function(param){
		var self = this;

		//---  Add Cancel button to Titlebar ----
		var buttonCancel = Ti.UI.createButton(self.STYLE.btn.cancel);
		buttonCancel.addEventListener('click', function(evt){
			self._eventProxy.fireEvent(self.EVENT.shareCancel, {});
		});
		self.VIEW.setRightNavButton(buttonCancel);

		//---  add Toolbar  ---
		self.setToolbar(self._renderToolBar());

		//---  Display group list  ---
		var groupListView = self._renderGroupTableView(self.DATA.allGroups);

		groupListView.addEventListener('click', function(evt){
			if(self.notRapidFire('listViewClick')){
				self._eventProxy.fireEvent(self.EVENT.viewGroup, {
					groupId		: evt.rowData.groupId,
					groupName	: evt.rowData.groupName,
				});
			}
		});

		self._setupEventHandlers();
		self.VIEW.add(groupListView);
	},

	close : function(param){
		var self = this;

		self._eventProxy.removeListeners(self.evtHandlers);
		self.base(param);
	},

	_setupEventHandlers : function(param){
		var self = this;

		self.VIEW.addEventListener('focus', function(evt){
			self.lbl_contactCount.text = '';
		});

		self._eventProxy.addEventListener(self.EVENT.updateToolbar, _doUpdateToolbar, self.evtHandlers);
		self._eventProxy.addEventListener(self.EVENT.updateGroupCount, _doUpdateGroupCount, self.evtHandlers);
		self._eventProxy.addEventListener(self.EVENT.updateContactCount, _doUpdateContactCount, self.evtHandlers);
		self._eventProxy.addEventListener(self.EVENT.clearContactCount, _doClearContactCount, self.evtHandlers);

		//---------  Event Handlers  -------------
		
		function _doClearContactCount(evt){	
			//---  Contacts Count  ---
			self.lbl_contactCount.text = '';
		}

		function _doUpdateContactCount(evt){	
			//---  Contacts Count  ---
			if('undefined' != typeof evt.contactCount){
				if(evt.contactCount>1) {
					self.lbl_contactCount.text = String.format(App.Helper.L('shareCard_contactUnits'), evt.contactCount);
				} else if(1 == evt.contactCount) {
					self.lbl_contactCount.text = String.format(App.Helper.L('shareCard_contactUnit'), evt.contactCount);
				} else {
					self.lbl_contactCount.text = App.Helper.L('shareCard_noContacts');
				}
			}
		}

		function _doUpdateToolbar(evt){
			
			//---  Contacts Count  ---
			if('undefined' != typeof evt.contactCount){
				if(evt.contactCount>1) {
					self.lbl_contactCount.text = String.format(App.Helper.L('shareCard_contactUnits'), evt.contactCount);
				} else if(1 == evt.contactCount) {
					self.lbl_contactCount.text = String.format(App.Helper.L('shareCard_contactUnit'), evt.contactCount);
				} else {
					self.lbl_contactCount.text = App.Helper.L('shareCard_noContacts');
				}
			}
			//---  Selected Count  ---
			if('undefined' != typeof evt.selectedCount){
			    if(evt.selectedCount <= 0) {
					self.buttonShare.title = App.Helper.L('BUTTON_SHARE');
					self.buttonShare.enabled = false;
					self.buttonClear.enabled = false;
				}else{
					self.buttonShare.title = String.format(App.Helper.L('shareCard_shareWith'), evt.selectedCount);
					self.buttonShare.enabled = true;
					self.buttonClear.enabled = true;
			    }
			}
			self._eventProxy.fireEvent(self.EVENT.updateGroupCount, {});
		}
		function _doUpdateGroupCount(param){
			//---  Calculate no. of selected contacts for each group  ---
			var groupCount = {};
			for(contact in self.SHARED.contactSelected){
				var containerGroups = self.SHARED.contactSelected[contact].group;
				for(groupID in containerGroups){
					groupCount[groupID] = ('undefined' == typeof groupCount[groupID]) ? 1 : groupCount[groupID] + 1;
				}
			}
			//---  Update display  --------
			var rows = self.groupRows;
			for(var i=0; i<rows.length; i++){
				var row = rows[i];
				var gid = row.groupId;
				if(gid == self.ALL_CONTACT_GROUPID){
					var countNumber = self.SHARED.selectedCount;
				}else{
					var countNumber = groupCount[gid] || 0;
				}
				if(countNumber > 0) {
			    	row.lbl_count.text = countNumber.toString();
			    	row.view_count.backgroundColor = row.view_count.backgroundColor;
			    	row.view_count.visible = true;
			  	}else{
					row.lbl_count.text = '';
					row.view_count.visible = false;
				}
			}
		}
	},

	_renderToolBar : function(){
		var self = this;
		
		var space = Ti.UI.createButton(self.STYLE.btn.FLEXIBLE_SPACE);
		var space1 = Ti.UI.createButton(self.STYLE.btn.FLEXIBLE_SPACE);
		var space2 = Ti.UI.createButton(self.STYLE.btn.FLEXIBLE_SPACE);
		//---  Clear Button  ------------------
		self.buttonClear = Ti.UI.createButton(self.STYLE.btn.clear);
		self.buttonClear.enabled = false;
		self.buttonClear.addEventListener('click', function(evt){
			self._eventProxy.fireEvent(self._eventProxy.EVENT.clearSelection, {});
			self.clearSelection();
		});
		//---  Contact Count Label  -----------
		self.lbl_contactCount = Ti.UI.createLabel(self.STYLE.label.contactCount);
		//---  Share Button  -------------------
		self.buttonShare = Ti.UI.createButton(self.STYLE.btn.share);
		self.buttonShare.enabled = false;
		self.buttonShare.addEventListener('click', _confirmShare);

		if (App.Config.IS_ANDROID) {
			self.buttonShare.width = '100dp';
			var items = [];
			items.push(self.buttonClear);
			items.push(self.lbl_contactCount);
			items.push(self.buttonShare);
			var toolbar = createAndroidToolbar (self.STYLE.toolBar,{items: items});
		} else {	
			var toolbar = Ti.UI.iOS.createToolbar(self.STYLE.toolBar);
			toolbar.items = [self.buttonClear, space, self.lbl_contactCount, space, self.buttonShare];
		}

		return toolbar;
		
		//----------------  Private Methods  ----------------
		function _confirmShare(evt){
			if(self.notRapidFire(evt.source.title)){
			    // ---  get selected contacts ----
			    var mobiles  = [];
			    var contacts = [];
			    
			    for(contact in self.SHARED.contactSelected){
			    	/*
			    	var name	= self.SHARED.contactSelected[contact].selected.name;
			    	var number	= self.SHARED.contactSelected[contact].selected.value;
					*/
					var name = self.SHARED.contactSelected[contact].selected.name;
					var number = self.SHARED.contactSelected[contact].selected.value;
					//if(in_array(number, mobiles)) continue;
					mobiles.push(number);
					contacts.push({ name: name, mobile: number });
//					Ti.API.info('CTK_=>'+contact+':'+number);
				}
	
			    //--- Share Confirmation Dialog  -------------
			    var shareCount = mobiles.length;
			    var shareConfirmAlert = Ti.UI.createAlertDialog({
					title		: App.Helper.L('poptitle_share'),
					message		: (shareCount > 1) ? String.format(App.Helper.L('poptitle_shareUnits'), shareCount) : String.format(App.Helper.L('poptitle_shareUnit'), shareCount),
					buttonNames	: [App.Helper.L('BUTTON_CANCEL'), App.Helper.L('BUTTON_OK')]
			    });
			    shareConfirmAlert.addEventListener('click', function(evt){
					if(evt.index == 1){
						self.sendShareSMS({
				        	contacts : contacts,
				        	mobiles  : mobiles,
			        	});
					}
				});
			    shareConfirmAlert.show();
			}
		}
	},

	_renderGroupTableView : function(allGroups){
	    var self = this;
	    
	    var grouplist = Ti.UI.createTableView(self.STYLE.tableView.group);
		self.groupRows = [];
	
		Ti.API.info('CTKV-in render group');
	
		for(var i = 0; i < allGroups.length; i++){
			var rowStyle = App.Helper.clone(self.STYLE.tableRow_group);
			rowStyle.groupName = allGroups[i].name;
			rowStyle.groupId = allGroups[i].recordId;
			var row = Ti.UI.createTableViewRow(rowStyle);
	
			var name = Ti.UI.createLabel(self.STYLE.label.groupName);
		    name.text = allGroups[i]['name'];
			row.add(name);
			  
			if(row.groupId == self.DATA.allContactGpID){
			    var countColor = '#3478FA';
			}else{
			    var countColor = '#CCCCCC';
			}
			
			row.view_count = Ti.UI.createView(self.STYLE.view.count);
			row.view_count.backgroundColor = countColor;

			row.lbl_count = Ti.UI.createLabel(self.STYLE.label.count);
			row.lbl_count.text = '';
			
			row.view_count.add(row.lbl_count);
			row.view_count.visible = false;
			row.add(row.view_count);

			self.groupRows[i] = row;
		}
		Ti.API.info('CTKV-in before setData');
	    grouplist.setData(self.groupRows);
		Ti.API.info('CTKV-in after setData');
	
	    return grouplist;
	},

	clearSelection : function(){
		var self = this;

	    self.buttonShare.title = App.Helper.L('BUTTON_SHARE');
	    self.buttonShare.enabled = false;
	    self.buttonClear.enabled = false;

		if(App.Config.IS_ANDROID){
			delete self.SHARED.contactSelected;
			self.SHARED.contactSelected = [];
			self.SHARED.selectedCount = 0;
			self._eventProxy.fireEvent(self.EVENT.updateGroupCount, {});
		}
//	    self.VIEW.fireEvent('focus');
	},

	sendShareSMS : function(param){
		var self = this;
		
		//---  Get invitation message  ---		
		var messageBody = self.DATA.card.shareMsg;
		if(!messageBody || messageBody == ''){
			messageBody = $.configMgr.get('sms_share');
			if(!messageBody || messageBody == ''){
				messageBody = App.Helper.L('shareCard_defaultMsg');
			}
		}
		//---  Merge brand and cardname into message  ---
		var cardname	= self.DATA.card.fullName;
		var brand		= self.DATA.card.brandName;
		messageBody = messageBody.replace(/##CARDNAME##/g, 	cardname); 
		messageBody = messageBody.replace(/##BRAND##/g, 	brand);

		//---  Share it  -------------------
		
		if (App.Config.IS_ANDROID) {
	
			//  determine sharing mode
			var shareMode = (!self.DATA.card.shareMode) ? self.DATA.card.shareMode : 1 //*send sms;
//			if(! smsDialog.canSendSMS())		shareMode = 2;		// don't send sms 
  				
			if(shareMode == 1){
				var dialog = Ti.UI.createAlertDialog({
					cancel: 1,
	    			buttonNames: [App.Helper.L('shareCard_confirm'), App.Helper.L('BUTTON_CANCEL')],
	    			message:  (param.mobiles.length>1) ? String.format(App.Helper.L('ALERT_SHARECONFIRMANDROID_N'), param.mobiles.length) : String.format(App.Helper.L('ALERT_SHARECONFIRMANDROID_1'), param.mobiles.length),
	    			title: App.Helper.L('shareCard_confirm')
	  			});
	  			dialog.addEventListener('click', function(e) {
	  				if (e.index == 0) {
						var smsPhones = "smsto:"+param.mobiles[0];
						for (var n=1; n<param.mobiles.length; n++) {
							smsPhones += ','+param.mobiles[n];
						}
						var intent = Ti.Android.createIntent({
							action: Ti.Android.ACTION_SENDTO,
							data: smsPhones,
						});
						intent.putExtra('sms_body', messageBody);
			
						Ti.Android.currentActivity.startActivityForResult(intent, function(e) {
				            if (e.resultCode == Ti.Android.RESULT_OK
				            	|| e.resultCode == Ti.Android.RESULT_CANCELED){
				            	_doSMSSent(e);
				            }else{
				            	_doError(e);
				            }
						});  					
	  				} else {
			            _doCancel();  					
	  				}
	  			});
	  			dialog.show();
			} else {
     			_doSMSSent(e);
		   	}
								
		} else {
			
			self.hideToolbar(true);
			var smsDialog = App.loadHelper('smsDialog');
	
			//  determine sharing mode
			var shareMode = (!self.DATA.card.shareMode) ? self.DATA.card.shareMode : 1 /*send sms*/;
			if(! smsDialog.canSendSMS())		shareMode = 2;		/* don't send sms */
	
			if(shareMode == 1){
				//---  Display SMS Dialog  -------------------
				var proxy = new App.WOEventProxy();
				proxy.addEventListener(smsDialog.EVENT.cancel, _doCancel);
				proxy.addEventListener(smsDialog.EVENT.error, _doError);
				proxy.addEventListener(smsDialog.EVENT.smsSent, _doSMSSent);
	
			    smsDialog.open({
					eventProxy	: proxy,
			        mobiles     : param.mobiles,
			        message     : messageBody,
			    });
			}else{
				_doSMSSent();
			}
	
		}
		
			//------------------------  Event Handlers  -----------------------------
		function _doSMSSent(evt){
			smsDialog = null;
			self.hideToolbar(false);
					
			self._eventProxy.fireEvent(self.EVENT.shareSMSSent, {});
			self._eventProxy.fireEvent(self.EVENT.doShareCard, {
				mobiles		: JSON.stringify(param.contacts),
				share_mode	: shareMode,
				total		: param.contacts.length,
			});
		}
		function _doCancel(evt){
			smsDialog = null;

			var cancelConfirm = Ti.UI.createAlertDialog({
				title		: App.Helper.L('shareCard_confirm'),
				message		: App.Helper.L('shareCard_confirmMsg'),
				buttonNames	: [App.Helper.L('BUTTON_SHAREWOSMS'), App.Helper.L('BUTTON_BACKTOSHARE'), App.Helper.L('BUTTON_SHARECANCEL')]
			});
			cancelConfirm.addEventListener('click', function(evt){
				if(evt.index == 2){
					//--- Cancel Sharing ---
					self._eventProxy.fireEvent(self.EVENT.shareCancel, {});
				}else if(evt.index == 0){
					//--- Share without sending SMS ---
					_doSMSSent();
				}
				self.hideToolbar(false);
			});
			setTimeout(function(evt){
				cancelConfirm.show();			// MUST have min 500 msec delay to overcome iOS 4 + Ti bug
			}, 500);
		}
		function _doError(evt){
			smsDialog = null;
			self.hideToolbar(false);
			self._eventProxy.fireEvent(self.EVENT.shareCancel, {});
		}
		
	},

});

module.exports = ShareWindowAndroid;

//---  End of file:ShareWindowAndroid.js
