
Ti.include('/globals.js');

exports.query = function(sql, dbName, dbSource){
	var db		= exports.getDB(dbName, dbSource);
	if(App.Helper.isArray(sql)){
		var rows	= db.execute.apply(db, sql);
	}else{
		var rows	= db.execute(sql);
	}
	
	if(rows){
		var result = helper_db2array(rows, '*');
		rows.close();
		db.close();
		return result;
	}
	return [];
};

exports.getDB = function(dbName, dbSource){
	if(!dbName){
		dbName = App.Const.DB_DATA;
	}
	
	if(!dbSource){
		dbSource = Ti.Filesystem.resourcesDirectory + 'assets/database/'+dbName+'.sql';
	}else{
		dbSource = Ti.Filesystem.resourcesDirectory + 'assets/database/'+dbSource+'.sql';
	}
	
	if(dbName == App.Const.DB_DATA){
		dbName = dbName + '_' + App.Config.APP_VERSION;
	}
	
	var db = Ti.Database.install(dbSource, dbName);
	try{
		db.execute.apply = Function.prototype.apply;
	}catch(e){ }
	return db;
};

exports.getRecord = function(table, object, key, fields, operation, dbName){
	var db = exports.getDB(dbName);
	var condition = '';
	if(!operation){
		operation = '=';
		condition = "`"+key+"`='"+object[key]+"'";
	}else if(operation == 'in'){
		condition = "`"+key+"` in "+object[key];
	}
	var query = "select * from "+table+" where "+condition;
	var rows = db.execute(query);
	var result = helper_db2array(rows, fields);
	rows.close();
	db.close();
	if(result.length>0){
		return result;
	}else{
		return false;
	}
}

exports.addRecord = function(table, params, pri_key, dbName){
    var db = exports.getDB(dbName);
    if(pri_key!=null){
      //clear database
      db.execute.apply(db, ["delete from " + table + " where " + pri_key + "=?", params[pri_key]]);
    }
    
    var sql_fields = "(";
    var sql_params = "(";
    var sql_values = [];
  
    for (var prop in params) {
      if (params.hasOwnProperty(prop)) {
        sql_fields += "`" + prop + "`,";
        sql_params += "?,";
        sql_values.push(params[prop]);
      }
    }
    
    sql_fields = sql_fields.substring(0, sql_fields.length - 1);
    sql_params = sql_params.substring(0, sql_params.length - 1);
    sql_fields += ")";
    sql_params += ")";
  
    var sql = "insert into " + table + " " + sql_fields + " values " + sql_params;
    sql_values.unshift(sql);
    var result = db.execute.apply(db, sql_values);
    //result.close();
    db.close();
    return;
}


function helper_db2array(rows, result_fields){
	var result = [];
	if(!rows) return [];
	if(rows.rowCount == 0) return [];
	try{
		rows.isValidRow();
	}catch(e){
		return [];
	}
	
	if(result_fields == '*' || !result_fields){
		var result_fields = [];
		var filedCount = 0;
		if (! App.Config.IS_ANDROID){
			filedCount = rows.fieldCount();
		}else{
			filedCount = rows.fieldCount;
		}
		for(var i=0;i<filedCount;i++){
			result_fields.push(rows.fieldName(i));
		}
	}
	
	while(rows.isValidRow()) {
		var item = {};
		for (var m = 0; m < result_fields.length; m++) {
			var prop = result_fields[m];
			if (undefined == rows.fieldByName(prop) || null == rows.fieldByName(prop)) {
				item[prop] = '';
				continue;
			}
			item[prop] = rows.fieldByName(prop);
		}
		result.push(item);
		rows.next();
	}
	return result;
};
